Package: marray
Version: 1.90.0
Date: 2009-08-15
Title: Exploratory analysis for two-color spotted microarray data
Author: Yee Hwa (Jean) Yang <jeany@maths.usyd.edu.au> with contributions
        from Agnes Paquet and Sandrine Dudoit.
Depends: R (>= 2.10.0), limma, methods
Suggests: tkWidgets
Maintainer: Yee Hwa (Jean) Yang <jean@biostat.ucsf.edu>
Description: Class definitions for two-color spotted microarray data.
        Fuctions for data input, diagnostic plots, normalization and
        quality checking.
License: LGPL
URL: http://www.maths.usyd.edu.au/u/jeany/
biocViews: Microarray, TwoChannel, Preprocessing
Collate: maClasses.R maGet.R maSet.R maPrint.R maSubset.R maBind.R
        maComp.R maDots.R maInput.R maNorm.R maOutput.R maWidget.R
        maPlots.R maAnnotate.R maRankGenes.R maSMA.R maSearch.R
        maWrap.R
LazyLoad: yes
git_url: https://git.bioconductor.org/packages/marray
git_branch: RELEASE_3_23
git_last_commit: 5983b33
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.23
Date/Publication: 2026-04-28
NeedsCompilation: no
Packaged: 2026-04-28 22:40:37 UTC; biocbuild
Built: R 4.6.0; ; 2026-04-29 04:48:51 UTC; unix

## Edit By Alice Dong [Apr, 07]
##Namespace file for marray package

importFrom(graphics,plot)
importFrom(graphics,boxplot)
importFrom(graphics,image)
importFrom(graphics,points)
importFrom(graphics,text)
importFrom(graphics,lines)
import(methods)

S3method(rbind,marrayInfo)
S3method(cbind,marrayRaw)
S3method(cbind,marrayNorm)
S3method(plot,marrayRaw)
S3method(plot,marrayNorm)
S3method(dim,marrayInfo)
S3method(dim,marrayRaw)

export("controlCode","mapGeneInfo", "widget.mapGeneInfo", "htmlPage", "opVersionID", "table2html", "rbind.marrayInfo", "cbind.marrayRaw", "cbind.marrayNorm", "maNum2Logic",  "maGeneTable", "maCompPlate", "maInd2Coord", "maCoord2Ind", "maCompCoord", "maCompInd", "maCompLayout", "maGenControls", "maDotsMatch", "maPlate<-","maControls<-","read.marrayLayout","read.marrayInfo",
"read.marrayRaw","read.Spot","read.GenePix","read.Agilent","read.Galfile","read.SMD","checkTargetInfo","maNormMain","maNorm","maNormScale","maMed","maNormMed","maLoess","maNormLoess","ma2D","maNorm2D","maMAD","maNormMAD",
"maCompNormEq","maCompNormA","write.marray","write.xls","write.list",
"plot.marrayRaw","plot.marrayNorm","maDotsDefaults","maDefaultPar",
"maBoxplot","maPlot.func","maText","maLowessLines","maLoessLines",
"maLegendLines","maPlot","maImage.func","maImage","maPalette","maColorBar","maTop","maSelectGnames","stat.confband.text","stat.gnames","findID",
"mean.na","var.na","cor.na","length.na","log.na","quantile.na","order.na","scale.na","prod.na","rm.na","widget.marrayLayout","widget.marrayInfo",
"widget.marrayRaw","widget.TwoSamples","maTwoSamples","sum.na",
"URLstring","maInfo","addLines","addText","addPoints","URLstring","read.fname")

exportMethods("maLabels","maLabels<-","maInfo<-","maInfo","maNgc<-",
"maNsr<-","maNsc<-","maNspots<-","maPlate<-","maControls<-","maLayout<-",
"maGnames<-","maTargets<-","maW<-","maGf<-","maRf<-","maGb<-","maGb<-","maRb<-",
"maRb<-","maA<-","maM<-","maMloc<-","maMscale<-","maNotes<-",
"coerce","maNspots","maNgr<-","maNgc","maNsr","maNsc","maSub","maPlate","maNotes","maGridRow","maGridCol","maSpotRow","maSpotCol","maPrintTip","maRf","maGf","maRb","maGb","maW","maLR", "maLG","maM","maA","maLayout","maNgr","maSpotCol","maControls","maNsamples","maGnames", "maTargets","maA","maMloc", "maMscale","maControls","maNormCall","boxplot", "image", "points","text","lines","summary","maNotes","show","[")

exportClasses("marrayInfo","marrayLayout","marrayRaw","marrayNorm","ShowLargeObject")

###########################################################################
## WIDGET for marrayLayout
## setwd("C:\\MyDoc\\Projects\\BioC\\marrayInput\\R")
###########################################################################

widget.marrayLayout<-function(path="",
                              skip=0,
                              sep="\t",
                              quote="",
                              ...)
{
  haveTkW <- require("tkWidgets", character.only=TRUE)
  if (!haveTkW)
    stop("This feature requires tkWidgets")

##  require(tcltk)
##  require(tkWidgets)
  
  ## Functions:
  ## Calling read.marrayLayout
  inputLayout <- function(name.plate,
                          name.controls,
                          skip=skip,
                          sep=sep,
                          quote=quote,
                          ...)
    {
      fname <- tclvalue("galfname")

      ## Name of the new marrayLayout objects in R
      if(tclvalue("layoutName")=="")
        newname <- paste(unlist(strsplit(tclvalue("galfname"), "\\."))[1],
                         "Layout", sep="")
      else
        newname <- tclvalue("layoutName")

      ## Values of notes
      if(tclvalue("notes") =="")
        notes <- fname
      else
        notes <- tclvalue("notes")

      ## automatically look for skip for gal file.
      if(fname != ""){
        x <- unlist(strsplit(fname, "\\."))
        xx <- x[length(x)]
        if(xx == "gal"){
          y <- readLines(fname, n=100)
          skip <- grep("Row", y)[1] - 1
        }
      }
      else
        fname <- NULL

      ## Calling read.marrayLayout
      newLayout <- read.marrayLayout(fname=fname,
                                     ngr=as.numeric(tclvalue("gr")),
                                     ngc=as.numeric(tclvalue("gc")),
                                     nsr=as.numeric(tclvalue("sr")),
                                     nsc=as.numeric(tclvalue("sc")),
                                     pl.col=name.plate,
                                     ctl.col=name.controls,
                                     notes=notes,
                                     skip=skip,
                                     sep=sep,
                                     quote=quote,
                                     ...
                                     )
      ## Assign name to newname)
      assign(newname, newLayout, env = .GlobalEnv)
      cat(paste("\n Finish creating a new marrayLayout: ", newname, "\n", sep=""))
    } ## end inputLayout


  ## List column names from file
  listColnames <- function(fname,
                           skip=skip,
                           sep=sep,
                           quote=quote,
                           ...)
    {
      x <- unlist(strsplit(fname, "\\."))
      xx <- x[length(x)]
      if(xx == "gal"){
        y <- readLines(fname, n=100)
        skip <- grep("Row", y)[1] - 1
      }

      h<-strsplit(readLines(fname, n=skip+1),split=sep)
      h<-as.list(unlist(h[[length(h)]]))
      columnHeadings<-gsub("\"", "", unlist(h))

      base <- tktoplevel()
      tkwm.title(base, "Select Column Information")
      mainfrm <- tkframe(base, borderwidth=2)

      ## Plate
      filevarfr <- tkframe(mainfrm, relief="groove", borderwidth=2)
      tkpack(filevarfr, tklabel(filevarfr, text="Plate"), side='top')
      for(ch in columnHeadings)
        tkpack(filevarfr, tkcheckbutton(filevarfr, text=ch,
                                        variable=paste('pl', ch, sep=".")))

      ## Controls
      file2varfr <- tkframe(mainfrm, relief="groove", borderwidth=2)
      tkpack(file2varfr, tklabel(file2varfr, text="Controls"), side='top')
      for(ch in columnHeadings)
      tkpack(filevarfr, tkcheckbutton(file2varfr, text=ch,
                                      variable=paste('ct', ch, sep=".")))

      tkpack(mainfrm, filevarfr, side="left", fill="both")
      tkpack(mainfrm, filevarfr, side="right", fill="both")

      getResults <- function()
        {
          for(ch in columnHeadings){
            check <- eval(parse(text=tclvalue(paste("pl", ch, sep="."))))
            if(check == "1") name.plate <<- ch
            check <- eval(parse(text=tclvalue(paste("ct", ch, sep="."))))
            if(check == "1") name.controls <<- ch
            tkdestroy(base)
          }
        }
      butfrm <- tkframe(mainfrm, borderwidth= 1, relief="groove")
      b.but <- tkbutton(butfrm, command=getResults, text="Go Back")
      tkpack(butfrm, b.but, side="bottom")
      tkpack(mainfrm, butfrm, side="bottom")
    }

  # Jianhua added this function in
  fileBrow <- function()
    {
      tkdelete(fileEntry, 0, "end")
      temp <- fileBrowser(path)
      temp <- paste(temp, sep = "", collapse = "," )
      tkinsert(fileEntry, 0, temp)
    }

  newname <- newLayout <- name.plate <- name.controls <- NULL

  base <- tktoplevel()
  tkwm.title(base, "marrayLayout Builder")
  mainfrm <- tkframe(base, borderwidth=2)

  # Input Notes info
  notesvarfr <- tkframe(mainfrm, relief="groove", borderwidth=2)
  tkpack(notesvarfr, tklabel(notesvarfr,
                             text="Name of marrayLayout object:"), side='top')
  tkpack(notesvarfr, tkentry(notesvarfr,
                             width=30, textvariable="layoutName"), side='left')
  tkpack(mainfrm, notesvarfr, side="top")

  # Select spot coord
  coordvarfr <- tkframe(mainfrm, relief="groove", borderwidth=2)
  heading <- tklabel(coordvarfr, text="Spot coordinates")

  gr.label <- tklabel(coordvarfr, text="Grid row")
  gc.label <- tklabel(coordvarfr, text="Grid col")
  sr.label <- tklabel(coordvarfr, text="Spot row")
  sc.label <- tklabel(coordvarfr, text="Spot col")

  gr.entry <- tkentry(coordvarfr, width=7)
  gc.entry <- tkentry(coordvarfr, width=7)
  sr.entry <- tkentry(coordvarfr, width=7)
  sc.entry <- tkentry(coordvarfr, width=7)

  tkgrid(heading, columnspan = 4)
  tkgrid(gr.label, gr.entry, gc.label, gc.entry)
  tkgrid(sr.label, sr.entry, sc.label, sc.entry)
  tkgrid.configure(gr.label, gr.entry, gc.label, gc.entry)
  tkgrid.configure(sr.label, sr.entry, sc.label, sc.entry)

  tkconfigure(gr.entry, textvariable = "gr")
  tkconfigure(gc.entry, textvariable = "gc")
  tkconfigure(sr.entry, textvariable = "sr")
  tkconfigure(sc.entry, textvariable = "sc")

  tkpack(mainfrm, coordvarfr, side="top")

  # Select .gal file
  galvarfr <- tkframe(mainfrm, relief="groove", borderwidth=2)
  fileFrame <- tkframe(galvarfr)
  fileLabel <- tklabel(fileFrame, text="Info file:")
  fileEntry <- tkentry(fileFrame, width=30, textvariable="galfname")
  fileBrow <- tkbutton(fileFrame, text = "Browse", command = fileBrow)
  tkgrid(fileLabel, columnspan = 2)
  tkgrid(fileEntry, fileBrow)
  tkpack(fileFrame, side = "top")
  tkpack(mainfrm, galvarfr, side="top")

  # Input Notes info
  notesvarfr <- tkframe(mainfrm, relief="groove", borderwidth=2)
  tkpack(notesvarfr, tklabel(notesvarfr, text="Notes:"), side='top')
  tkpack(notesvarfr, tkentry(notesvarfr, width=30, textvariable="notes"), side='left')
  tkpack(mainfrm, notesvarfr, side="top")

  # Input print-tip and controls column
  butfrm <- tkframe(mainfrm, borderwidth= 1, relief="groove")
  a.but <- tkbutton(butfrm, command=function()
                    {
                      if(length(tclvalue("galfname")) != 0)
                        listColnames(tclvalue("galfname"),
                                     skip=skip, sep=sep, quote=quote, ...)
                    },
                    text="GetInfo")
  b.but <- tkbutton(butfrm, command=function()tkdestroy(base), text="Quit")
  c.but <- tkbutton(butfrm, command=function()
                    inputLayout(name.plate, name.controls,
                                skip=skip, sep=sep, quote=quote, ...),
                    text="Build")
  tkpack(butfrm, a.but, c.but, side="left")
  tkpack(butfrm, b.but, side="right")
  tkpack(mainfrm, butfrm,anchor='w')
  return(invisible())
}


###########################################################################
## WIDGET : marrayInfo
###########################################################################
widget.marrayInfo <- function(path=".",
                              skip=0,
                              sep="\t",
                              quote="",
                              ...)
{
  haveTkW <- require("tkWidgets", character.only=TRUE)
  if (!haveTkW)
    stop("This feature requires tkWidgets")

  ##  require(tcltk)
  ##  require(tkWidgets)
   
  ## Functions:
  ## Calling read.marrayInfo
  inputInfo <- function(skip=skip,
                        sep=sep,
                        quote=quote,
                        ...)
    {
      fname <- tclvalue("Fname")


      ## Name of the new marrayInfo objects in R
      if(tclvalue("infoName")=="")
        newname <- paste(unlist(strsplit(tclvalue("Fname"), "\\."))[1], "Info", sep="")
      else
        newname <- tclvalue("infoName")

      ## Name of the new marrayNotes objects in R
      if(tclvalue("notes") =="")
        notes <- fname
      else
        notes <- tclvalue("notes")

      ## Setting some defaults for GenePix Gal file
      x <- unlist(strsplit(fname, "\\."))
      xx <- x[length(x)]
      if(xx == "gal"){
        y <- readLines(fname, n=100)
        skip <- grep("Row", y)[1] - 1
      }

      info.id <- eval(parse(text=tclvalue("infoID")))
      labels.id <- eval(parse(text=tclvalue("labels")))
      newInfo <- read.marrayInfo(fname=fname,
                                 info.id = info.id,
                                 labels = labels.id,
                                 notes=notes,
                                 skip=skip,
                                 sep=sep,
                                 quote=quote,
                                 ...
                                 )
      cat(paste("\n Finish creating a new marrayInfo: ", newname, "\n", sep=""))
      assign(newname, newInfo, env = .GlobalEnv)
    }

  # Jianhua put this function in
  fileBrow <- function()
    {
      tkdelete(fileEntry, 0, "end")
      temp <- fileBrowser(path)
      temp <- paste(temp, sep = "", collapse = ",")
      tkinsert(fileEntry, 0, temp)
    }

  newName <- NULL
  base <- tktoplevel()
  tkwm.title(base, "MarrayInfo Builder")
  mainfrm <- tkframe(base, borderwidth=2)

  # Input Names info
  notesvarfr <- tkframe(mainfrm, relief="groove", borderwidth=2)
  tkpack(notesvarfr, tklabel(notesvarfr, text="Name of marrayInfo object:"), side='top')
  tkpack(notesvarfr, tkentry(notesvarfr, width=30, textvariable="infoName"), side='left')
  tkpack(mainfrm, notesvarfr, side="top")

  # Select Fname file
  galvarfr <- tkframe(mainfrm, relief="groove", borderwidth=2)
  fileFrame <- tkframe(galvarfr)
  fileLbl <- tklabel(fileFrame, text="File Name:")
  fileEntry <- tkentry(fileFrame, width=30, textvariable="Fname")
  fileBrow <- tkbutton(fileFrame, text = "Browse",
                       command = fileBrow)
  tkgrid(fileLbl, columnspan = 2)
  tkgrid(fileEntry, fileBrow)
  tkpack(fileFrame, side = "top")
  tkpack(mainfrm, galvarfr, side="top")

  # Input ID info
  idvarfr <- tkframe(mainfrm, relief="groove", borderwidth=2)
  tkgrid(idvarfr, columnspan = 2)
  info.lab <- tklabel(idvarfr, text="Info Index")
  info.entry <- tkentry(idvarfr, width=15, textvariable="infoID")
  labels.lab <-  tklabel(idvarfr, text="Labels")
  labels.entry <-  tkentry(idvarfr, width=15, textvariable="labels")
  tkgrid(info.lab, labels.lab)
  tkgrid(info.entry, labels.entry)
  tkgrid.configure(info.lab, labels.lab)
  tkgrid.configure(info.entry, labels.entry)
  tkpack(mainfrm, idvarfr, side="top")

  # Input Notes info
  notesvarfr <- tkframe(mainfrm, relief="groove", borderwidth=2)
  tkpack(notesvarfr, tklabel(notesvarfr, text="Notes:"), side='top')
  tkpack(notesvarfr, tkentry(notesvarfr, width=30, textvariable="notes"), side='left')
  tkpack(mainfrm, notesvarfr, side="top")

  # Input print-tip and controls column
  butfrm <- tkframe(mainfrm, borderwidth= 1, relief="groove")
  a.but <- tkbutton(butfrm, command=function()
                    inputInfo(sep=sep,
                              skip=skip,
                              quote=quote,
                              ...), text="Build")
  b.but <- tkbutton(butfrm, command=function()tkdestroy(base), text="End")

  tkpack(butfrm, a.but, side="left")
  tkpack(butfrm, b.but, side="right")
  tkpack(mainfrm, butfrm,anchor='w')
  return(invisible())
}

###########################################################################
## WIDGET for marrayRaw
##
###########################################################################

widget.marrayRaw<-function(ext=c("spot", "xls", "gpr"),
                           skip=0,
                           sep="\t",
                           quote="\"",
                           ...)

{
  haveTkW <- require("tkWidgets", character.only=TRUE)
  if (!haveTkW)
    stop("This feature requires tkWidgets")
  ##require(tcltk)
  ##require(tkWidgets)
  
  ## Jianhua added this in to catch the "..."
  args <- list(...)

  ## Calling read.marrayLayout
  inputRaw <- function(fnames,
                       skip=skip,
                       sep=sep,
                       quote=quote,
                       ...)
    {
      if(tclvalue("rawName")=="")
        newname <- paste(ext, "data", sep="")
      else
        newname <- tclvalue("rawName")

      if(tclvalue("notes") =="")
        notes <- paste("File from", ext)
      else
        notes <- tclvalue("notes")

      ## Deal with Names
      if(tclvalue("nameW") == "") name.W <- NULL else name.W <- tclvalue("nameW")
      if(tclvalue("nameRf") == "")
        stop("Input Foreground intensities") else name.Rf <- tclvalue("nameRf")
      if(tclvalue("nameGf") == "")
        stop("Input Foreground intensities") else name.Gf <- tclvalue("nameGf")
      if(tclvalue("nameRb") == "") name.Rb <- NULL else name.Rb <- tclvalue("nameRb")
      if(tclvalue("nameGb") == "") name.Gb <- NULL else name.Gb <- tclvalue("nameGb")

      ## Calculate Skip
      y <- readLines(fnames[1], n=100)
      skip <- grep(name.Gf, y)[1] - 1

      ## Call read.marrayRaw
      newRaw <- read.marrayRaw(fnames=fnames,
                               path=NULL,
                               name.Gf=name.Gf,
                               name.Gb=name.Gb,
                               name.Rf=name.Rf,
                               name.Rb=name.Rb,
                               name.W=name.W,
                               layout=eval(parse(text=tclvalue("layout"))),
                               gnames=eval(parse(text=tclvalue("genenames"))),
                               targets=eval(parse(text=tclvalue("targets"))),
                               notes=tclvalue("notes"),
                               skip=skip,
                               sep=sep,
                               quote=quote,
                                ...)

      cat(paste("\n Finish creating a new marrayInfo: ", newname, "\n", sep=""))
      assign(newname, newRaw, env = .GlobalEnv)
    }

  ### Jianhua
  # This is the name I put in to store the files when "Files" is
  # clicked. Change it to whatever you would like to
  # Jean: Name it "fnames"
  fnames <- NULL

  # Jianhua added the following functions in
  filesClick <- function(){
      if(!is.null(args$path))
          fnames <<- fileBrowser(args$path)
      else
          fnames <<- fileBrowser()
  }

  layBrowser <- function(){
      browserToEntry(layoutEntry)
  }

  samBrowser <- function(){
      browserToEntry(targetEntry)
  }

  genBrowser <- function(){
      browserToEntry(geneEntry)
  }

  browserToEntry <- function(anEntry){
      first = TRUE
      temp <- objectBrowser()
      tkdelete(anEntry, 0, "end")
      for(i in 1:length(temp)){
          if(first){
              tkinsert(anEntry, "end", temp[[i]]$name)
              first <- FALSE
          }else{
              toPut <- paste(",", temp[[i]]$name, sep = "")
              tkinsert(anEntry, "end", toPut)
          }
      }
  }

  layoutPressed <- function(){
      if(!is.null(args$path))
          widget.marrayLayout(path = args$path)
      else
          widget.marrayLayout(path="")
  }

  genePressed <- function(){
      if(!is.null(args$path))
          widget.marrayInfo(path = args$path)
      else
          widget.marrayInfo(path="")
  }
  # end of Jianhua's functions

  newName <- NULL

  base <- tktoplevel()
  tkwm.title(base, "MarrayRaw builder")
  mainfrm <- tkframe(base, borderwidth=2)

  # Select files
  filevarfr <- tkframe(mainfrm, relief="groove", borderwidth=2)
  tkpack(filevarfr, tkbutton(filevarfr, text="Files",
                             command = filesClick), side='top')

  # Input Names
  notesvarfr <- tkframe(mainfrm, relief="groove", borderwidth=2)
  tkpack(notesvarfr, tklabel(notesvarfr, text="Name of the marrayRaw object:"),
         side='top')
  tkpack(notesvarfr, tkentry(notesvarfr, width=50, textvariable="rawName"), side='left')
  tkpack(mainfrm, notesvarfr, side="top")

  ## Files Names
  tkpack(mainfrm, filevarfr, side="top")

  # Select fg and bg
  coordvarfr <- tkframe(mainfrm, relief="groove", borderwidth=2)
  heading <- tklabel(coordvarfr, text="Foreground and background intensities")

  gf.label <- tklabel(coordvarfr, text="Green Foreground")
  gb.label <- tklabel(coordvarfr, text="Green Background")
  rf.label <- tklabel(coordvarfr, text="Red Foreground")
  rb.label <- tklabel(coordvarfr, text="Red Background")
  W.label <- tklabel(coordvarfr, text="Weights")

  gf.entry <- tkentry(coordvarfr, width=10, textvariable = "nameGf" )
  gb.entry <- tkentry(coordvarfr, width=10, textvariable = "nameGb" )
  rf.entry <- tkentry(coordvarfr, width=10, textvariable = "nameRf" )
  rb.entry <- tkentry(coordvarfr, width=10, textvariable = "nameRb" )
  W.entry <-  tkentry(coordvarfr, width=10, textvariable = "nameW" )


  tkgrid(heading, columnspan = 4)
  tkgrid(gf.label, gf.entry, gb.label, gb.entry)
  tkgrid(rf.label, rf.entry, rb.label, rb.entry)
  tkgrid(W.label, W.entry)
  tkgrid.configure(gf.label, gf.entry, gb.label, gb.entry)
  tkgrid.configure(rf.label, rf.entry, rb.label, rb.entry)
  tkgrid.configure(W.label, W.entry)

  tkpack(mainfrm, coordvarfr, side="top")

##  Jean: Jianhua, this is not quite working... what should I do?
##  if(ext == "spot")
##    {
##      tkinsert(gf.entry, "0.0", text="Gmean")
##      tkinsert(gb.entry, "0.0", text="morphG")
##      tkinsert(rf.entry, "0.0", text="Rmean")
##      tkinsert(rb.entry, "0.0", text="morphR")
##    }

  # Input Targets, Layout, GeneNames info
  notesvarfr <- tkframe(mainfrm, relief="groove", borderwidth=2)

  # I modified the following to put the browsers in
  ## Layout
  layoutFrame <- tkframe(notesvarfr)
  layoutLbl <- tklabel(layoutFrame, text="Layout:")
  layoutEntry <- tkentry(layoutFrame, width=43, textvariable="layout")
  layoutBrowser <- tkbutton(layoutFrame, text = "Browse",
                            command = layBrowser)
  tkgrid(layoutLbl, columnspan = 2)
  tkgrid(layoutEntry, layoutBrowser)
  tkpack(layoutFrame, side = "top")
  tkpack(mainfrm, notesvarfr, side="top")

  ## Target
  notesvarfr <- tkframe(mainfrm, relief="groove", borderwidth=2)
  targetFrame <- tkframe(notesvarfr)
  targetLbl <- tklabel(targetFrame, text="Target Information:")
  targetEntry <- tkentry(targetFrame, width=43, textvariable="targets")
  targetBrowser <- tkbutton(targetFrame, text = "Browse",
                            command = samBrowser)
  tkgrid(targetLbl, columnspan = 2)
  tkgrid(targetEntry, targetBrowser)
  tkpack(targetFrame, side = "top")
  tkpack(mainfrm, notesvarfr, side="top")

  ## Gene / Probe
  notesvarfr <- tkframe(mainfrm, relief="groove", borderwidth=2)
  geneFrame <- tkframe(notesvarfr)
  geneLbl <- tklabel(geneFrame, text="Gene Information:")
  geneEntry <- tkentry(geneFrame, width=43, textvariable="genenames")
  geneBrowser <- tkbutton(geneFrame, text = "Browse",
                          command = genBrowser)
  tkgrid(geneLbl, columnspan = 2)
  tkgrid(geneEntry, geneBrowser)
  tkpack(geneFrame, side = "top")
  tkpack(mainfrm, notesvarfr, side="top")

  ## Notes
  notesvarfr <- tkframe(mainfrm, relief="groove", borderwidth=2)
  tkpack(notesvarfr, tklabel(notesvarfr, text="Notes:"), side='top')
  tkpack(notesvarfr, tkentry(notesvarfr, width=50, textvariable="notes"), side='left')
  tkpack(mainfrm, notesvarfr, side="top")

  ## Button

  butfrm <- tkframe(mainfrm, borderwidth= 1, relief="groove")
#  a.but <- tkbutton(butfrm, command=function()
#                    {widget.marrayLayout()}, text="Layout")
  a.but <- tkbutton(butfrm, command=layoutPressed, text="Layout")
  b.but <- tkbutton(butfrm, command=function()
                    {widget.marrayInfo()}, text="Target")
#  c.but <- tkbutton(butfrm, command=function()
#                    {widget.marrayInfo()}, text="Genes")
  c.but <- tkbutton(butfrm, command=genePressed, text="Genes")
  d.but <- tkbutton(butfrm, command=function()
                    inputRaw(fnames,
                             sep=sep,
                             skip=skip,
                             quote=quote,
                             ...), text="Build")
  e.but <- tkbutton(butfrm, command=function()tkdestroy(base), text="Quit")
  tkpack(butfrm, a.but, b.but, c.but, d.but, e.but, side="left")
  tkpack(mainfrm, butfrm,anchor='w')

  # Jianhua add this in so that the widget is modal
  ##  tkwait.window(base)
  return(invisible())
}

###########################################################################
#
# Output functions
#
# Date : September 19, 2002
# Modify : October, 14, 2002, April 10 2005 (Agnes)
#
# Runs on R 1.6 and above
#
# This file contains wrapper functions for analysis
#
# source("~/Projects/maTools/R/maInOut.R")
#

write.marray <- function(mraw, file="maRawResults.xls", val="maM", ...)
{
  # Was tmp <- cbind(maGeneTable(mraw), eval(call(val,mraw)), row.names=NULL)
  res <- c()
  for(i in 1:length(val))
    res <- cbind(res, eval(call(val[i],mraw)))
  tmp <- cbind(maGeneTable(mraw), res, row.names=NULL)
  write.table(tmp, file, row.names=FALSE, col.names=TRUE, sep="\t", ...)
  return(NULL)
}
  

## Apr, 09 based on suggestion by
## Matthew Fero <mfero@fhcrc.org>
write.xls <- function(res, file="test.xls", ...)
  {
    ## col.names = NA and row.names = TRUE
    opt = list(...)
    defs <- list(x = res, file = file, row.names=FALSE, col.names=TRUE, sep="\t")  
    write.args <- maDotsMatch(opt, maDotsMatch(defs, formals(args("write.table"))))    
##    print(write.args)
    do.call(write.table, write.args)
    ##    write.table(res, file, row.names=FALSE, col.names=TRUE, sep="\t", ...) 
  }


write.list <- function (x,
                        filename = "data",
                        append = FALSE,
                        closefile = TRUE,
                        outfile
                        ) 
{
  if(!append)
    {
      outfile <- file(filename, "w")
      cat(file=outfile, append= append)
    }
  for(i in 1:length(x))
    {
      cat(paste(names(x)[i], ":"), file = outfile, append = TRUE)
      cat("\n", file = outfile, append=TRUE)
      if(!is.null(x[[i]]))
        {
          switch(data.class(x[[i]]),
                 matrix = write.table(x[[i]], sep="\t", file = outfile, append = TRUE),
                 table = if(!is.null(names(x[[i]])))
                 {
                   write.table(rbind(names(x[[i]]), x[[i]]),file = outfile, 
                               append = TRUE, row.names=FALSE, col.names=FALSE, sep="\t")
                 }
                 else
                 {
                   write(x[[i]], file=outfile, append=TRUE)
                 },
                 list = write.list(x[[i]], outfile=outfile, append=TRUE, closefile=FALSE),
                 if(!is.null(names(x[[i]])))
                 {
                   write.table(rbind(names(x[[i]]), x[[i]]),file = outfile, 
                               append = TRUE, row.names=FALSE, col.names=FALSE, sep="\t")
                 }
                 else
                 {
                   write(x[[i]], ncolumns = length(x[[i]]), file = outfile, append = TRUE)
                 }
                 )
        }
      cat("\n", file = outfile, append=TRUE)
    }
  if(closefile) close(outfile)
}

###########################################################################
# Statistics for Microarray Analysis
##########################################################################

###############################################################################
## NA's
###########################################################################
# Statistics for Microarray Analysis
# Function dealing with NA's
#
# Date : March 19, 2001
#
# Authors: Sandrine Dudoit and Yee Hwa (Jean) Yang.
##########################################################################

##########################################################################
# Basic statistics functions that are able to handle missing values
##########################################################################

########################################################################/**
# \name{na}
#
# \alias{log.na}
# \alias{sum.na}
# \alias{mean.na}
# \alias{var.na}
# \alias{cor.na}
# \alias{quantile.na}
# \alias{length.na}
# \alias{order.na}
# \alias{scale.na}
# \alias{prod.na}
#
# \title{Basic Statistical Functions for Handling Missing Values}
#
# \description{
# Basic statistical functions for handling missing values or NA. \cr
# In \code{log.na}, \code{sum.na}, \code{mean.na} and \code{var.na},
# \code{quantile.na}, \code{length.na}, missing values are omitted
# from the calculation. \cr
# The function \code{cor.na} calls \code{cor} with the argument
# \code{use="pairwise.complete.obs"}. \cr
# The function \code{order.na} only handles vector arguments and not
# lists.  However, it gives the option of omitting the NAs
# (\code{na.last=NA}), of placing the NAs at the start of the ordered
# vector (\code{na.last=F}) or at the end (\code{na.last=T}). \cr
# The function \code{scale.na} is a modified version of
# \code{\link{scale}} which allows NAs in the variance calculation. If
# \code{scale = T}, the function \code{f} in \code{scale.na} uses
# \code{var.na} to perform the variance calculation.
# The function \code{prod.na} is similar to the \code{\link{prod}}
# function with \code{na.rm=TRUE}. This function returns the product of
# all the values present in its arguments, omitting any missing values.
# }
#
# \author{
#   Yee Hwa Yang, \email{yeehwa@stat.berkeley.edu} \cr
#   Sandrine Dudoit, \email{sandrine@stat.berkeley.edu}
# }
#
# \seealso{\code{\link{log}}, \code{\link{sum}}, \code{\link{mean}},
#   \code{\link{var}}, \code{\link{cor}}, \code{\link{order}},
#   \code{\link{scale}}, \code{link{prod}}.}
#
# \keyword{log, sum, mean, variance, correlation, order, scale,
# product, missing values, NA.}
#
#*/#########################################################################


mean.na <- function(x,...)
{
        mean(x[!(is.na(x) | is.infinite(x))])
}

 var.na <- function(x)
{
        res <- NA
        tmp <- !(is.na(x) | is.infinite(x))
        if(sum(tmp) > 1)
                res <- var(x[tmp])
        res
}

cor.na <- function(x)
{
  cor(x, use="pairwise.complete.obs")
}

sum.na <- function(x, ...)
{
  sum(x, na.rm=TRUE, ...)
}


length.na <- function(x, ...)
{
   tmp <- !(is.na(x) | is.infinite(x))
   length(x[tmp],...)
 }

log.na <- function(x, ...)
{
  log(ifelse(x > 0, x, NA), ...)
}


quantile.na <- function(x, ...)
 {
   tmp <- !(is.na(x) | is.infinite(x))
   quantile(x[tmp],...)
 }

order.na <- function (x, na.last = TRUE)
{
    y <- order(x)
    n <- sum(is.na(x))
    tmp <- (length(x) - n + 1):length(x)
    if (!is.na(na.last)) {
        if (na.last)
            res <- y
        if (!na.last)
          {
            if(n == 0)
              res <- y
            else
              res <- c(y[tmp], y[-tmp])
          }
      }
    if (is.na(na.last)) {
        warning("NA's discarded")
        res <- y[-tmp]
    }
    res
}

scale.na<-function(x, center = TRUE, scale = TRUE)
{
  x <- as.matrix(x)
  nc <- ncol(x)

  if (is.logical(center)) {
     if (center)
       x <- sweep(x, 2, applyy(x, 2, mean, na.rm=TRUE))
    }
  else if (is.numeric(center) && (length(center) == nc))
    x <- sweep(x, 2, center)
  else
    stop("Length of center must equal the number of columns of x")

  if (is.logical(scale)) {
    if (scale) {
      f <- function(v) {
        sqrt(var.na(v))
      }
      x <- sweep(x, 2, apply(x, 2, f), "/")
    }
    }
  else if (is.numeric(scale) && length(scale) == nc)
    x <- sweep(x, 2, scale, "/")
  else
    stop("Length of scale must equal the number of columns of x")
    x
}

prod.na <- function (x, ...)
{
##  prod(x[!(is.na(x) | is.infinite(x))])
  prod(x, na.rm=TRUE, ...)
}


rm.na <- function(x)
  {
    ind <- is.na(x) | is.nan(x) | is.infinite(x)
    return(x[!ind])
  }


##########################################################################
#                                End of file
##########################################################################

###################################################################
##
## Date: October 11, 2002
## 
## source("~/Projects/maTools/R/maAnnotate.R")
## 
###################################################################

##########################################################################
## Widget for html page

mapGeneInfo <- function(widget=FALSE, Gnames, Name="pubmed", ID="genbank", ACC="SMDacc",  ...)
  {
    if(widget)
      {
        res <- widget.mapGeneInfo(Gnames)
        return(res)
      }
    else
      {
        opt <- list(...)
        base <- matrix(c("Grid", "Spot", "Row", "Column", "Block",
                         "cood", "cood", "cood", "cood", "cood"), ncol=2)
        rownames(base) <- c("Grid", "Spot", "Row", "Column", "Block")
        newinfo <- rbind(c("Name", Name),
                         c("ID", ID),
                         c("ACC", ACC))
        rownames(newinfo) <- c("Name", "ID", "ACC")
        return(rbind(newinfo, cbind(names(opt), unlist(opt)),base))
      }
  }

widget.mapGeneInfo <- function(Gnames)
  {
    startfun <- function()
      {
        print("The URL choices are:")
        print(names(URLstring))
      }
    haveTkW <- require("tkWidgets", character.only=TRUE)
    if (!haveTkW)
      stop("This feature requires tkWidgets")

##    require(tcltk)
##    require(tkWidgets)
    switch(data.class(Gnames),
           marrayNorm = headings <- colnames(maInfo(maGnames(Gnames))),
           marrayRaw= headings <- colnames(maInfo(maGnames(Gnames))),
           data.frame = headings <- colnames(Gnames),
           headings <- colnames(Gnames)
           )
    
    headings <- headings[-unique(c(grep("Grid", headings),
                                   grep("Spot", headings),
                                   grep("Row", headings),
                                   grep("Column", headings),
                                   grep("Block", headings)))]

    wlist <- list()
    for(hvalue in headings)
      {
        test <- list(Name=hvalue, Value=hvalue,
                     toText=function(x) paste(x,collapse = ","),
                     fromText=NULL, canEdit=TRUE, buttonFun = NULL,
                     buttonText = "Choices")
        wlist <- c(wlist, list(test))
      }
    names(wlist) <- headings
    widget1 <- list(wList = wlist,
                    preFun = startfun)
    res <- widgetRender(widget1, "Map Gene Names")

    resValues <- values.Widget(res)
    base <- matrix(c("Grid", "Spot", "Row", "Column", "Block",
                     "cood", "cood", "cood", "cood", "cood"), ncol=2)
    for(i in 1:length(resValues))
      base <- rbind(base, c(resValues[[i]]$Entry, resValues[[i]]$Value))
    return(base)
  }


##########################################################################
htmlPage <- function(genelist,
                     filename="GeneList.html",
                     geneNames=Gnames,
                     mapURL=SFGL,
                     othernames,
                     title,
                     table.head,
                     table.center=TRUE,
                     disp=c("browser", "file")[1])
{
  switch(class(geneNames),
         data.frame= data <- geneNames,
         marrayRaw = data <- maGeneTable(geneNames),
         marrayNorm = data <- maGeneTable(geneNames),
         marrayInfo = data <- maInfo(geneNames),
         matrix = data <- data.frame(geneNames),
         data <- geneNames)

  if(missing(othernames))
    restable <- data[genelist,] else
  restable <- cbind(data, othernames)[genelist,]
  
  args <- list(filename = filename, mapURL = mapURL,
               table.center = table.center,disp = disp)
  if(!missing(title)) args <- c(args, list(title=title))
  if(!missing(table.head)) args <- c(args, list(table.head=table.head))
  do.call(table2html, c(list(restable), args))
  return()
}
         

#####################################################
## Base Function
##
tablegen <-  function(input)
  {
    HTwrap <-   function (x, tag = "TD") {
      paste("<", tag, ">", x, "</", tag, ">", sep = "")}
    
    HTwrap.matrix <- function(input)
      {
        output <- ""
        for (nm in 1:ncol(input))
          output <- paste(output, HTwrap(input[,nm]), sep = "")
        return(output)
      }

    HTwrap.list <- function(input)
      {
        output <- ""
        for (nm in 1:length(input))
          output <- paste(output, HTwrap(input[[nm]]), sep = "")
        return(output)
      }
    
    switch(data.class(input),
           vector = output <- HTwrap(input),
           matrix = output <- HTwrap.matrix(input),
           list = output <- HTwrap.list(input),
           output <- HTwrap(input)
           )
    return(output)
  }


opVersionID <- function(opID)
  {
    code <- unlist(lapply(strsplit(as.vector(opID), split=""),
                          function(x){paste(x[1:2], collapse="")}))
    tmp <- table(code)
    code2 <- names(tmp)[tmp==max(tmp)]
    switch(code2,
           M2 = res <- "operonm2",
           M0 = res <- "operonm1",
           H2 = res <- "operonh2",
           H0 = res <- "operonh1"
           )
    return(res)
  }


gsubAnchor <-function (id, urlString) 
{
  test <-  function(x){
    if(!is.na(x))
      res <- gsub(pattern="UNIQID", replacement=x, urlString)
    else
      res <- x
    return(res)
  }
  paste("<A HREF=", sapply(as.character(id), test), ">", id, "</A>", sep = "")
}
#####################################################
## Table 2 HTML
## Extention of ll.htmlpage
## Date: Feb 16, 2003
##
#####################################################
table2html <- function (restable, filename = "GeneList.html",
                        mapURL = SFGL, title, table.head, table.center = TRUE, 
                        disp = c("browser", "file")[1]) 
{

  HTwrap <- function(x, tag = "TD") {
    paste("<", tag, ">", x, "</", tag, ">", sep = "")
  }

  ## Open file
  outfile <- file(filename, "w")

  ## Write Header
  cat("<html>", file = outfile)
  cat(HTwrap(HTwrap("Gene Lists", tag = "TITLE"), tag = "head"), file = outfile)
  cat("<body bgcolor=\"#FFFFEF\">", "<H1 ALIGN=CENTER > BioConductor Gene Listing </H1>", 
      file = outfile, sep = "\n")
  if (!missing(title)) 
    cat("<CENTER><H1 ALIGN=\"CENTER\">", title, " </H1></CENTER>\n", 
        file = outfile, sep = "\n")
  if (table.center) 
    cat("<CENTER> \n", file = outfile)

  ## Start TABLE header
  cat("<TABLE BORDER=4>", file = outfile, sep = "\n")
  if (!missing(table.head)) {
    headout <- paste("<TH>", table.head, "</TH>")
    cat("<TR>", headout, "</TR>", file = outfile, sep = "\n")
  }

  ## Check that we have URL mapping information
  if (is.null(mapURL)) 
    mapURL <- widget.mapGeneInfo(restable)

  ## Main part: convert restable to html
  ##
  oldGnamesID <- colnames(restable)
  GnamesID <- rep("none", length(oldGnamesID))
  for (i in 1:nrow(mapURL))
    GnamesID[grep(mapURL[i, 1], oldGnamesID)] <- mapURL[i,2]
  if (sum(GnamesID == "operon") != 0)    ## Special case for operon
    GnamesID[grep("operon", GnamesID)] <- opVersionID(restable[1:100, grep("operon", GnamesID)])

  mainTable <- Headings <- NULL
  for (i in 1:length(GnamesID)) {
    info <- GnamesID[i]
    x <- as.vector(restable[, i])
    if(!is.null(class(x))) if(class(x) == "numeric") x <- round(x, 2)
    if ((info != "") | is.null(info)) {
      switch(info, cood = mainTable <- paste(mainTable, HTwrap(x), sep = ""),
             none = mainTable <- paste(mainTable,  HTwrap(x), sep = ""),
             mainTable <- paste(mainTable,
                                HTwrap(gsubAnchor(x, urlString = URLstring[[info]])), sep = ""))
    }
    Headings <- c(Headings, colnames(restable)[i])
  }
  cat(paste(HTwrap(Headings), collapse = ""), file = outfile)
  cat("\n", file = outfile)
  cat(HTwrap(mainTable, tag = "TR"), file = outfile, sep = "\n")
  ##
  ## END Main part: convert restable to html

  ## End html file
  cat("</TABLE>", "</body>", "</html>", sep = "\n", file = outfile)
  if (table.center) 
    cat("</CENTER> \n", file = outfile)
  close(outfile)
  
  if (disp == "browser") 
    if (interactive()) browseURL(paste("file://", filename, sep = "/"))
  return()
}



###################################################################
## predefine info
###################################################################

URLstring <- list(
 pubmed = "http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?cmd=Search&db=PubMed&term=UNIQID",
 locuslink = "http://www.ncbi.nlm.nih.gov/LocusLink/LocRpt.cgi?l=UNIQID",
 riken = "http://read.gsc.riken.go.jp/chipinfo.php?defkey=&chiprearrayid=UNIQID",
 SMDclid = "http://genome-www4.stanford.edu/cgi-bin/SMD/source/sourceResult?option=CloneID&criteria1=IMAGE:UNIQID&choice=cDNA",
 SMDacc = "http://genome-www4.stanford.edu/cgi-bin/SMD/source/sourceResult?option=Number&criteria=UNIQID&choice=Gene",
 operonh2 = "http://oparray.operon.com/human2/index.php?single_query=UNIQID",
 operonh1 = "http://oparray.operon.com/~operon/human/index.php?single_query=UNIQID",
 operonm2 = "http://oparray.operon.com/mouse2/index.php?single_query=UNIQID",
 operonm1 = "http://oparray.operon.com/~operon/mouse/index.php?single_query=UNIQID",
 operonST= "http://arrays.ucsf.edu/cgi-bin/oligo_db.pl?oligo=UNIQID",
 genbank = "http://www.ncbi.nlm.nih.gov/entrez/viewer.fcgi?DB=nucleotide&val=UNIQID",
 unigeneMm="http://www.ncbi.nlm.nih.gov/UniGene/clust.cgi?ORG=Mm&CID=UNIQID",
 unigeneHS="http://www.ncbi.nlm.nih.gov/UniGene/clust.cgi?ORG=Hs&CID=UNIQID",
 gene="http://www.ncbi.nih.gov/entrez/query.fcgi?db=gene&cmd=Retrieve&dopt=summary&list_uids=UNIQID",
 affyid = "https://www.affymetrix.com/LinkServlet?probeset=UNIQID")

                  
###################################################################
## Some example of Map Info
###################################################################

SFGL <- mapGeneInfo(ID="operonST",
                    ACC="SMDacc",
                    LocusLink="locuslink",
                    Cluster="unigeneMm",
                    LOCUSLINK="locuslink",
                    GenBank="genbank",
                    Name="none")

UCBFGL <- mapGeneInfo(ID="riken",
                      ACC="SMDacc")

###################################################################

##########################################################################
# maGet.R
#
# Accessor methods for two-color cDNA microarrays classes
#
###########################################################################

###########################################################################
# Accessor methods for marrayInfo class

if(!isGeneric("maLabels"))
   setGeneric("maLabels", function(object)
   standardGeneric("maLabels"))

setMethod("maLabels", "marrayInfo", function(object) slot(object, "maLabels"))

if(!isGeneric("maInfo"))
   setGeneric("maInfo", function(object) standardGeneric("maInfo"))
setMethod("maInfo", "marrayInfo", function(object) slot(object, "maInfo"))

if(!isGeneric("maNotes"))
   setGeneric("maNotes", function(object) standardGeneric("maNotes"))
setMethod("maNotes", "marrayInfo", function(object) slot(object,
   "maNotes") )

##########################################################################
# Accessor methods for marrayLayout class

if(!isGeneric("maNspots"))
   setGeneric("maNspots", function(object)
   standardGeneric("maNspots"))

setMethod("maNspots", "marrayLayout",
	function(object) {
		n<-slot(object, "maNspots")
		if(length(n)==0)
		  n<-maNgr(object)*maNgc(object)*maNsr(object)*maNsc(object)
		n
	})

if(!isGeneric("maNgr"))
   setGeneric("maNgr", function(object) standardGeneric("maNgr"))
setMethod("maNgr", "marrayLayout", function(object) slot(object, "maNgr"))

if(!isGeneric("maNgc"))
   setGeneric("maNgc", function(object) standardGeneric("maNgc"))
setMethod("maNgc", "marrayLayout", function(object) slot(object, "maNgc"))

if(!isGeneric("maNsr"))
   setGeneric("maNsr", function(object) standardGeneric("maNsr"))
setMethod("maNsr", "marrayLayout", function(object) slot(object, "maNsr"))

if(!isGeneric("maNsc"))
   setGeneric("maNsc", function(object) standardGeneric("maNsc"))
setMethod("maNsc", "marrayLayout", function(object) slot(object, "maNsc"))

if(!isGeneric("maSub"))
   setGeneric("maSub", function(object) standardGeneric("maSub"))
setMethod("maSub", "marrayLayout", function(object) slot(object, "maSub"))

if(!isGeneric("maPlate"))
   setGeneric("maPlate", function(object) standardGeneric("maPlate"))
setMethod("maPlate", "marrayLayout", function(object)
          {if(length(object@maPlate) == 0)
             maCompPlate(object)
          else
            slot(object, "maPlate")})

if(!isGeneric("maControls"))
   setGeneric("maControls", function(object) standardGeneric("maControls"))
setMethod("maControls", "marrayLayout",
          function(object) slot(object, "maControls"))

if(!isGeneric("maNotes"))
   setGeneric("maNotes", function(object) standardGeneric("maNotes"))
setMethod("maNotes", "marrayLayout", function(object) slot(object, "maNotes"))

#########################
# Methods for quantities that are not slots of marrayLayout

if(!isGeneric("maGridRow"))
   setGeneric("maGridRow", function(object) standardGeneric("maGridRow"))
setMethod("maGridRow", "marrayLayout",
	function(object) {
		ngr<-maNgr(object)
		ngc<-maNgc(object)
		nsr<-maNsr(object)
		nsc<-maNsc(object)
                gr<-numeric(0)
 		if(length(ngr*ngc*nsr*nsc)!=0)
		{
		  gr <- rep(1:ngr, rep(nsr * nsc * ngc, ngr))
		  gr <- gr[as.logical(maSub(object))]
                }
		gr
	})

if(!isGeneric("maGridCol"))
   setGeneric("maGridCol", function(object) standardGeneric("maGridCol"))
setMethod("maGridCol", "marrayLayout",
	function(object) {
		ngr<-maNgr(object)
		ngc<-maNgc(object)
		nsr<-maNsr(object)
		nsc<-maNsc(object)
		gc<-numeric(0)
		if(length(ngr*ngc*nsr*nsc)!=0)
		{
		  gc <- rep(rep(1:ngc, rep(nsr * nsc, ngc)), ngr)
		  gc <- gc[as.logical(maSub(object))]
		}
		gc
   	})


if(!isGeneric("maSpotRow"))
   setGeneric("maSpotRow", function(object) standardGeneric("maSpotRow"))
setMethod("maSpotRow", "marrayLayout",
	function(object) {
		ngr<-maNgr(object)
		ngc<-maNgc(object)
		nsr<-maNsr(object)
		nsc<-maNsc(object)
		sr<-numeric(0)
		if(length(ngr*ngc*nsr*nsc)!=0)
		{
		  sr <- rep(rep(1:nsr, rep(nsc, nsr)), ngc * ngr)
		  sr <- sr[as.logical(maSub(object))]
		}
		sr
	})

if(!isGeneric("maSpotCol"))
   setGeneric("maSpotCol", function(object) standardGeneric("maSpotCol"))
setMethod("maSpotCol", "marrayLayout",
	function(object) {
		ngr<-maNgr(object)
		ngc<-maNgc(object)
		nsr<-maNsr(object)
		nsc<-maNsc(object)
		sc<-numeric(0)
		if(length(ngr*ngc*nsr*nsc)!=0)
		{
  		  sc <- rep(1:nsc, nsr * ngc * ngr)
		  sc <- sc[as.logical(maSub(object))]
		}
		sc
	})


if(!isGeneric("maPrintTip"))
   setGeneric("maPrintTip", function(object) standardGeneric("maPrintTip"))
setMethod("maPrintTip", "marrayLayout",
	function(object) {
		ngr<-maNgr(object)
		ngc<-maNgc(object)
		nsr<-maNsr(object)
		nsc<-maNsc(object)
		pt<-numeric(0)
		if(length(ngr*ngc*nsr*nsc)!=0)
		{
		  gr<- rep(1:ngr, rep(nsr * nsc * ngc, ngr))
		  gc <- rep(rep(1:ngc, rep(nsr * nsc, ngc)), ngr)
		  pt<-(gr-1) * ngc + gc
		  pt<-pt[as.logical(maSub(object))]
		}
		pt
	})


###########################################################################
# Accessor methods for marrayRaw class

if(!isGeneric("maRf"))
   setGeneric("maRf", function(object) standardGeneric("maRf"))
setMethod("maRf", "marrayRaw", function(object) slot(object, "maRf"))

if(!isGeneric("maGf"))
   setGeneric("maGf", function(object) standardGeneric("maGf"))
setMethod("maGf", "marrayRaw", function(object) slot(object, "maGf"))

if(!isGeneric("maRb"))
   setGeneric("maRb", function(object) standardGeneric("maRb"))
setMethod("maRb", "marrayRaw", function(object) slot(object, "maRb"))

if(!isGeneric("maGb"))
   setGeneric("maGb", function(object) standardGeneric("maGb"))
setMethod("maGb", "marrayRaw", function(object) slot(object, "maGb"))

if(!isGeneric("maW"))
   setGeneric("maW", function(object) standardGeneric("maW"))
setMethod("maW", "marrayRaw", function(object) slot(object, "maW"))

#########################
# Methods for quantities that are not slots of marrayRaw

if(!isGeneric("maLR"))
   setGeneric("maLR", function(object) standardGeneric("maLR"))
setMethod("maLR", "marrayRaw",
          function(object)
          {
            r<-maRf(object)
            if(length(maRf(object))!=0)
	    {
              if(length(maRb(object))!=0)
                r<-maRf(object) - maRb(object)
              else
                r<-maRf(object)
              r<-log(ifelse(r>0, r, NA),2)
	    }
      	   r
          }
          )



if(!isGeneric("maLG"))
   setGeneric("maLG", function(object) standardGeneric("maLG"))
setMethod("maLG", "marrayRaw",
          function(object)
          {
            g<-maGf(object)
            if(length(maGf(object))!=0)
	    {
              if(length(maGb(object))!=0)
                g<-maGf(object) - maGb(object)
              else
                g<-maGf(object)
              g<-log(ifelse(g>0, g, NA),2)
	    }
      	   g
          }
          )


if(!isGeneric("maM"))
   setGeneric("maM", function(object) standardGeneric("maM"))
setMethod("maM", "marrayRaw",
	function(object)
	{
           M<-matrix(nr=0,nc=0)
	   if((length(maLR(object))!=0) & (length(maLG(object))!=0))
 	      M<-maLR(object)-maLG(object)
	   M
        }
	)


if(!isGeneric("maA"))
   setGeneric("maA", function(object) standardGeneric("maA"))
setMethod("maA", "marrayRaw",
	function(object)
	{
           A<-matrix(nr=0,nc=0)
           if((length(maLR(object))!=0) & (length(maLG(object))!=0))
             A<-(maLR(object)+maLG(object))/2
	   A
        }
	)


#########################
# maLayout slots

if(!isGeneric("maLayout"))
   setGeneric("maLayout", function(object) standardGeneric("maLayout"))
setMethod("maLayout", "marrayRaw", function(object) slot(object, "maLayout"))

if(!isGeneric("maNspots"))
   setGeneric("maNspots", function(object) standardGeneric("maNspots"))
setMethod("maNspots", "marrayRaw", function(object) maNspots(maLayout(object)))

if(!isGeneric("maNgr"))
   setGeneric("maNgr", function(object) standardGeneric("maNgr"))
setMethod("maNgr", "marrayRaw", function(object) maNgr(maLayout(object)))

if(!isGeneric("maNgc"))
   setGeneric("maNgc", function(object) standardGeneric("maNgc"))
setMethod("maNgc", "marrayRaw", function(object) maNgc(maLayout(object)))

if(!isGeneric("maNsr"))
   setGeneric("maNsr", function(object) standardGeneric("maNsr"))
setMethod("maNsr", "marrayRaw", function(object) maNsr(maLayout(object)))

if(!isGeneric("maNsc"))
   setGeneric("maNsc", function(object) standardGeneric("maNsc"))
setMethod("maNsc", "marrayRaw", function(object) maNsc(maLayout(object)))

if(!isGeneric("maGridRow"))
   setGeneric("maGridRow", function(object) standardGeneric("maGridRow"))
setMethod("maGridRow", "marrayRaw", function(object) maGridRow(maLayout(object)))

if(!isGeneric("maGridCol"))
   setGeneric("maGridCol", function(object) standardGeneric("maGridCol"))
setMethod("maGridCol", "marrayRaw", function(object) maGridCol(maLayout(object)))

if(!isGeneric("maSpotRow"))
   setGeneric("maSpotRow", function(object) standardGeneric("maSpotRow"))
setMethod("maSpotRow", "marrayRaw", function(object) maSpotRow(maLayout(object)))

if(!isGeneric("maSpotCol"))
   setGeneric("maSpotCol", function(object) standardGeneric("maSpotCol"))
setMethod("maSpotCol", "marrayRaw", function(object) maSpotCol(maLayout(object)))

if(!isGeneric("maPrintTip"))
   setGeneric("maPrintTip", function(object) standardGeneric("maPrintTip"))
setMethod("maPrintTip", "marrayRaw", function(object) maPrintTip(maLayout(object)))

if(!isGeneric("maSub"))
   setGeneric("maSub", function(object) standardGeneric("maSub"))
setMethod("maSub", "marrayRaw", function(object) maSub(maLayout(object)))

if(!isGeneric("maPlate"))
   setGeneric("maPlate", function(object) standardGeneric("maPlate"))
setMethod("maPlate", "marrayRaw", function(object) maPlate(maLayout(object)))

if(!isGeneric("maControls"))
   setGeneric("maControls", function(object) standardGeneric("maControls"))
setMethod("maControls", "marrayRaw", function(object) maControls(maLayout(object)))

if(!isGeneric("maNsamples"))
   setGeneric("maNsamples", function(object) standardGeneric("maNsamples"))
setMethod("maNsamples", "marrayRaw", function(object) max(ncol(maRf(object)), ncol(maGf(object))))

#########################
if(!isGeneric("maGnames"))
   setGeneric("maGnames", function(object) standardGeneric("maGnames"))
setMethod("maGnames", "marrayRaw", function(object) slot(object, "maGnames"))

if(!isGeneric("maTargets"))
   setGeneric("maTargets", function(object) standardGeneric("maTargets"))
setMethod("maTargets", "marrayRaw", function(object) slot(object, "maTargets"))

if(!isGeneric("maNotes"))
   setGeneric("maNotes", function(object) standardGeneric("maNotes"))
setMethod("maNotes", "marrayRaw", function(object) slot(object, "maNotes"))

###########################################################################
# Accessor methods for marrayNorm class

if(!isGeneric("maA"))
   setGeneric("maA", function(object) standardGeneric("maA"))
setMethod("maA", "marrayNorm", function(object) slot(object, "maA"))

if(!isGeneric("maM"))
   setGeneric("maM", function(object) standardGeneric("maM"))
setMethod("maM", "marrayNorm", function(object) slot(object, "maM"))

if(!isGeneric("maMloc"))
   setGeneric("maMloc", function(object) standardGeneric("maMloc"))
setMethod("maMloc", "marrayNorm", function(object) slot(object, "maMloc"))

if(!isGeneric("maMscale"))
   setGeneric("maMscale", function(object) standardGeneric("maMscale"))
setMethod("maMscale", "marrayNorm", function(object) slot(object, "maMscale"))

if(!isGeneric("maW"))
   setGeneric("maW", function(object) standardGeneric("maW"))
setMethod("maW", "marrayNorm", function(object) slot(object, "maW"))

##############
# Methods for quantities that are not slots of marrayRaw

if(!isGeneric("maLR"))
   setGeneric("maLR", function(object) standardGeneric("maLR"))
setMethod("maLR", "marrayNorm",
          function(object)
          {
            M <- maM(object)
            A <- maA(object)
            r <- M/2 + A
            r
          }
          )

if(!isGeneric("maLG"))
  setGeneric("maLG", function(object) standardGeneric("maLG"))
setMethod("maLG", "marrayNorm",
          function(object)
          {
            M <- maM(object)
            A <- maA(object)
            g <- A - M/2
            g
          }
          )

#########################
# maLayout slots

if(!isGeneric("maLayout"))
   setGeneric("maLayout", function(object) standardGeneric("maLayout"))
setMethod("maLayout", "marrayNorm", function(object) slot(object, "maLayout"))

if(!isGeneric("maNspots"))
   setGeneric("maNspots", function(object) standardGeneric("maNspots"))
setMethod("maNspots", "marrayNorm", function(object) maNspots(maLayout(object)))

if(!isGeneric("maNgr"))
   setGeneric("maNgr", function(object) standardGeneric("maNgr"))
setMethod("maNgr", "marrayNorm", function(object) maNgr(maLayout(object)))

if(!isGeneric("maNgc"))
   setGeneric("maNgc", function(object) standardGeneric("maNgc"))
setMethod("maNgc", "marrayNorm", function(object) maNgc(maLayout(object)))

if(!isGeneric("maNsr"))
   setGeneric("maNsr", function(object) standardGeneric("maNsr"))
setMethod("maNsr", "marrayNorm", function(object) maNsr(maLayout(object)))

if(!isGeneric("maNsc"))
   setGeneric("maNsc", function(object) standardGeneric("maNsc"))
setMethod("maNsc", "marrayNorm", function(object) maNsc(maLayout(object)))

if(!isGeneric("maGridRow"))
   setGeneric("maGridRow", function(object) standardGeneric("maGridRow"))
setMethod("maGridRow", "marrayNorm", function(object) maGridRow(maLayout(object)))

if(!isGeneric("maGridCol"))
   setGeneric("maGridCol", function(object) standardGeneric("maGridCol"))
setMethod("maGridCol", "marrayNorm", function(object) maGridCol(maLayout(object)))

if(!isGeneric("maSpotRow"))
   setGeneric("maSpotRow", function(object) standardGeneric("maSpotRow"))
setMethod("maSpotRow", "marrayNorm", function(object) maSpotRow(maLayout(object)))

if(!isGeneric("maSpotCol"))
   setGeneric("maSpotCol", function(object) standardGeneric("maSpotCol"))
setMethod("maSpotCol", "marrayNorm", function(object) maSpotCol(maLayout(object)))

if(!isGeneric("maPrintTip"))
   setGeneric("maPrintTip", function(object) standardGeneric("maPrintTip"))
setMethod("maPrintTip", "marrayNorm", function(object) maPrintTip(maLayout(object)))

if(!isGeneric("maSub"))
   setGeneric("maSub", function(object) standardGeneric("maSub"))
setMethod("maSub", "marrayNorm", function(object) maSub(maLayout(object)))

if(!isGeneric("maPlate"))
   setGeneric("maPlate", function(object) standardGeneric("maPlate"))
setMethod("maPlate", "marrayNorm", function(object) maPlate(maLayout(object)))

if(!isGeneric("maControls"))
   setGeneric("maControls", function(object) standardGeneric("maControls"))
setMethod("maControls", "marrayNorm", function(object) maControls(maLayout(object)))

if(!isGeneric("maNsamples"))
   setGeneric("maNsamples", function(object) standardGeneric("maNsamples"))
setMethod("maNsamples", "marrayNorm", function(object) ncol(maM(object)))

#########################
if(!isGeneric("maGnames"))
   setGeneric("maGnames", function(object) standardGeneric("maGnames"))
setMethod("maGnames", "marrayNorm", function(object) slot(object, "maGnames"))

if(!isGeneric("maTargets"))
   setGeneric("maTargets", function(object) standardGeneric("maTargets"))
setMethod("maTargets", "marrayNorm", function(object) slot(object, "maTargets"))

if(!isGeneric("maNotes"))
   setGeneric("maNotes", function(object) standardGeneric("maNotes"))
setMethod("maNotes", "marrayNorm", function(object) slot(object, "maNotes"))

if(!isGeneric("maNormCall"))
   setGeneric("maNormCall", function(object) standardGeneric("maNormCall"))
setMethod("maNormCall", "marrayNorm", function(object) slot(object, "maNormCall"))

##########################################################################
# maClasses.R
#
# Basic class definitions for two-color cDNA microarrays
#
###########################################################################

###########################################################################
# marrayInfo
# Class for describing either samples hybed to array, spots, or other objects
## Jean: Add prototype June-19,03

setClass("marrayInfo",
         representation(maLabels="character",
			maInfo="data.frame",
			maNotes="character"),
         prototype=list(maInfo=data.frame()))

##########################################################################
# marrayLayout
# Class for the microarray layout:  spot coords, grid coords, print-tip group,
# plate, controls (e.g. with expected log-ratio M=0), etc.

setClass("marrayLayout",
         representation(maNgr="numeric",
			maNgc="numeric",
			maNsr="numeric",
                        maNsc="numeric",
			maNspots="numeric",
			maSub="logical",
                        maPlate="factor",
			maControls="factor",
			maNotes="character"),
         prototype=list(maSub=TRUE, maPlate=factor(numeric(0)),
           maControls=factor(numeric(0))))

###########################################################################
# marrayRaw
# Class for pre-normalization intensity data and layout for several arrays

setClass("marrayRaw",
         representation(maRf="matrix",
 			maGf="matrix",
			maRb="matrix",
			maGb="matrix",
                        maW="matrix",
			maLayout="marrayLayout",
                        maGnames="marrayInfo",
			maTargets="marrayInfo",
			maNotes="character"))

###########################################################################
# marrayNorm
# Class used for normalization of intensity data (before and after)
# Use match.call() inside normalization function to return the call

setClass("marrayNorm",
         representation(maA="matrix",
			maM="matrix",
			maMloc="matrix",
			maMscale="matrix",
                        maW="matrix",
			maLayout="marrayLayout",
                        maGnames="marrayInfo",
			maTargets="marrayInfo",
			maNotes="character",
			maNormCall="call")
         )



#########################
# Coerce object from class marrayRaw to marrayNorm
#########################
setAs("marrayRaw", "marrayNorm", function(from)
{
    mnorm<-new("marrayNorm", maA=maA(from), maM=maM(from), maW=maW(from),
        maLayout=maLayout(from), maGnames=maGnames(from),
        maTargets=maTargets(from), maNotes=maNotes(from))
    mnorm
})

dim.marrayRaw <- function(x) dim(x@maRf)
dim.marrayNorm <- function(x) dim(x@maM)
dim.marrayInfo <- function(x) dim(x$maInfo)

###########################################################################
##  Function: read.marrayLayout
##
##  source("~/Projects/madman/Rpacks/marray/R/maInput.R")
##
###########################################################################

read.fname <-
  function(fname,
           skip,
           sep="\t",
           quote="\"",
           ...)
  {
    h<-strsplit(readLines(fname, n=skip+1),split=sep)
    h<-as.list(unlist(h[[length(h)]]))
    names(h)<- unlist(h)
    dat<-scan(fname,quiet=TRUE,what=h, sep=sep, skip=skip+1, quote=quote, ...)
    return(dat)
  }


read.marrayLayout<-
  function(fname=NULL,
           ngr,
           ngc,
           nsr,
           nsc,
           pl.col=NULL,
           ctl.col=NULL,
           sub.col=NULL,
           notes=fname,
           skip,
           sep="\t",
           quote="\"",
           ...)
{
  # Read layout file
  if(!is.null(fname)){
    h<-strsplit(readLines(fname, n=skip+1),split=sep)
    h<-as.list(unlist(h[[length(h)]]))
    names(h)<-unlist(h)
    dat<-scan(fname,quiet=TRUE,what=h, sep=sep, skip=skip+1, fill=TRUE, quote=quote, ...)
  }
  else
    notes <- "No Input File"

  # Generate spot coordinates
  nspots<-ngr * ngc * nsr * nsc
  layout <- new("marrayLayout", maNgr=ngr, maNgc = ngc,
                maNsr=nsr, maNsc=nsc, maNspots=nspots, maNotes=notes)

  if(!is.null(fname)){
  # Get Uneven layout
    if(!is.null(sub.col)){
      subvalue <- intersect(c(1:nspots), as.numeric(dat[[sub.col]]))
      maSub(layout)  <- rep(0, nspots)
      maSub(layout)[subvalue] <- 1
    }

  # Get plate id
    if(!is.null(pl.col))
      maPlate(layout) <- as.factor(dat[[pl.col]])

  # Get control
    if(!is.null(ctl.col))
      maControls(layout) <- as.factor(dat[[ctl.col]])
  }
  return(layout)
}

###########################################################################
# Function: read.marrayInfo
#
###########################################################################
read.marrayInfo <-
  function(fname,
           info.id=NULL,
           labels=NULL,
           notes=fname,
           sep="\t",
           skip=0,
           quote="\"",
           ...)
{
  h<-strsplit(readLines(fname, n=skip+1),split=sep)
  h<-as.list(unlist(h[[length(h)]]))
  names(h)<-gsub("\"", "", unlist(h))
  dat <- read.table(fname, sep=sep, skip=skip+1, fill=TRUE, quote=quote, ...)
  colnames(dat) <- h

  descript <- new("marrayInfo", maNotes=notes)

  ## Enter Description
  if(is.null(info.id)) info.id <- 1:ncol(dat)
  maInfo(descript) <- as.data.frame(dat[,info.id])

  ## Enter Predefine Labels
  if(length(labels) == nrow(dat))
    maLabels(descript) <- as.vector(labels)
  else
    {
      if(is.null(labels))
        labels <- 1
      maLabels(descript) <- as.character(as.vector(dat[,labels]))
    }
  return(descript)
}


###########################################################################
# Function: read.marrayRaw
#
###########################################################################
read.marrayRaw<-
  function(fnames,
           path=".",
           name.Gf=NULL,
           name.Gb=NULL,
           name.Rf=NULL,
           name.Rb=NULL,
           name.W=NULL,
           layout=NULL,
           gnames=NULL,
           targets=NULL,
           notes=NULL,
           skip=NULL,
           sep="\t",
           quote="\"",
           DEBUG=FALSE,
           ...)
{
  if(DEBUG) print("in read.marrayraw")
  if(DEBUG) cat("Path", path, "\n")
  if(is.null(path))
    fullfnames <- fnames
  else
    fullfnames <- file.path(path, fnames)

  fname <- fullfnames[1]

  if(DEBUG) print(skip)

  # Intensity data
  if(is.null(skip))
    {
      if (DEBUG) print("In is.null(skip) ")
      y <- readLines(fullfnames[1], n=100)
      #modify name.Gf for special character
      regName.Gf <- gsub("\\(", "\\\\\\(", name.Gf)
      regName.Gf <- gsub("\\)", "\\\\\\)", regName.Gf)
      skip2 <- grep(regName.Gf, y)[1] - 1
      if(DEBUG) cat(skip2, "done \n")
    }
  else {
    if(DEBUG) print("skip != NULL")
    skip2 <- skip
    if(DEBUG) print(paste("skip2 =", skip2, sep=" "))
  }

  if(is.null(layout))
    {
      if (DEBUG) print("in is.null(layout)")
      nspots <- length(readLines(fullfnames[1])) - skip2 - 1
      if(DEBUG) cat(nspots, "of rows \n")
    }
  else
    {
      if(sum(layout@maSub) == 1)
        nspots  <- layout@maNgr * layout@maNgc * layout@maNsr * layout@maNsc
      else
        nspots <- sum(layout@maSub)
    }
  ##     nspots <- sum(layout@maSub)  ## Aug 3, maSub is default set to 1.

  if (DEBUG) print(paste("nspots = ", nspots, sep=""))

  Y <- matrix(0, nspots, length(fullfnames))
  colnames(Y) <- fullfnames
  if(!is.null(name.Rf)) Rf <- Y
  if(!is.null(name.Gf)) Gf <- Y
  if(!is.null(name.Gb)) Gb <- Y
  if(!is.null(name.Rb)) Rb <- Y
  if(!is.null(name.W)) W <- Y

  for(f in fullfnames)
  {
    cat("Reading ... ", f, "\n")

    ## Calculate Skip
    if(is.null(skip))
      {
        if(DEBUG) print("in is.null(skip), part 2")
        if(DEBUG) cat("Calculating skip  ... ")
        y <- readLines(f, n=100)
        #modify name.Gf for special character
        regName.Gf <- gsub("\\(", "\\\\\\(", name.Gf)
        regName.Gf <- gsub("\\)", "\\\\\\)", regName.Gf)

        skip2 <- grep(regName.Gf, y)[1] - 1
        if(DEBUG) cat(skip2, "done \n")
      }
    else
      {
        if (DEBUG) print("in !is.null(skip) 2")
        skip2 <- skip
      }

    dat <- read.table(f, skip = skip2, header = TRUE,
                      sep = sep, quote = quote, check.names = FALSE,
                      as.is = TRUE, comment.char = "", nrows = nspots, ...)

    if(!is.null(name.Gf)) Gf[,f]<- as.matrix(dat[, name.Gf])
    if(!is.null(name.Gb)) Gb[,f]<- as.matrix(dat[, name.Gb])
    if(!is.null(name.Rf)) Rf[,f]<-as.matrix(dat[, name.Rf])
    if(!is.null(name.Rb)) Rb[,f]<-as.matrix(dat[, name.Rb])
    if(!is.null(name.W)) W[,f] <-as.matrix(dat[, name.W])
  }

  ## Add Notes

  if(is.null(notes)) notes <- ""

  mraw <- new("marrayRaw", maNotes=notes)
  if(!is.null(name.Gf)) mraw@maGf <- Gf
  if(!is.null(name.Gb)) mraw@maGb <- Gb
  if(!is.null(name.Rf)) mraw@maRf <- Rf
  if(!is.null(name.Rb)) mraw@maRb <- Rb
  if(!is.null(layout)) maLayout(mraw) <- layout
  if(!is.null(gnames)) maGnames(mraw) <- gnames
  if(!is.null(targets)) maTargets(mraw) <- targets
  if(!is.null(name.W)) maW(mraw) <- W
  return(mraw)
}

#########################  END FIRST PART
###
### TEST FUNCTION
## read.layout(fname, 4, 4, 20, 20, pl.col="PLATE",
## ctl.col=NULL, gnames.col=c("SUID","LUID","Gene Name"), notes="BLAH, BLAH")
### END FUNCTION
###########################################################################

###########################################################################
#  Read SPOT
#
read.Spot <-  function(fnames = NULL,
                       path=".",
                       name.Gf = "Gmean",
                       name.Gb = "morphG",
                       name.Rf = "Rmean",
                       name.Rb = "morphR",
                       name.W= NULL,
                       layout = NULL,
                       gnames = NULL,
                       targets = NULL,
                       notes=NULL,
                       skip=NULL,
                       sep="\t",
                       quote="\"",
                       DEBUG=FALSE,
                       ...)
  {
    ## If fnames not specified, read everything in the dir
    opt <- list(...)
    if(is.null(fnames))
      {
        if(!is.null(targets))
          fullnames <- as.vector(maInfo(targets)[,1]) ## modified
        else
          {
            if(is.null(path))
              fullnames <- dir(pattern="\\.spot$")
            else
              fullnames <- dir(path, pattern="\\.spot$")
          }
      }
    else
      fullnames <- fnames

    setgal <- FALSE
    if(DEBUG) print(fullnames)
    if(is.null(gnames) | is.null(layout))
      {
        if(DEBUG) print("Reading Galfile ... ")
        galfile <- dir(pattern=".\\gal", path=path)
        defs <- list(galfile = galfile, path=path, info.id = c("ID", "Name"),
                     labels = "ID", sep = sep, quote=quote, fill=TRUE, check.names=FALSE,
                     as.is=TRUE, ncolumns = 4)
        gal.args <- maDotsMatch(maDotsMatch(opt, defs), formals(args("read.Galfile")))
        gal <- do.call(read.Galfile, gal.args)
        if(is.null(gnames)) gnames <- gal$gnames
        if(is.null(layout)) layout <- gal$layout
        if(DEBUG) print("done \n ")
        setgal <- TRUE
      }

    if(DEBUG) print(table(layout@maControls))
    if(DEBUG) cat("Setting up controls status ... ")
    if(length(layout@maControls) == 0)
      {
        if(DEBUG) cat("Generate controls \n")
        GenControls.args <- maDotsMatch(maDotsMatch(opt, list(Gnames=gnames)), formals(args("maGenControls")))
        layout@maControls <- as.factor(do.call(maGenControls, GenControls.args))
        if(DEBUG) print(table(layout@maControls))
      }
    if(DEBUG) cat("done \n ")
    if(is.null(notes)) notes <- "Spot Data"

    if(DEBUG) cat("Calling read.marrayRaw ... \n")
    defs <- list(fnames = fullnames, path=path,
                 name.Gf = name.Gf, name.Gb=name.Gb, name.Rf=name.Rf, name.Rb=name.Rb,
                 name.W=name.W, layout = layout, gnames=gnames, targets=targets,
                 notes = notes, skip=skip, sep=sep, quote=quote, fill=TRUE)
    maRaw.args <- maDotsMatch(maDotsMatch(opt, defs), formals(args("read.marrayRaw")))
    maRaw.args <- c(maRaw.args, defs[names(defs[setdiff(names(defs), names(maRaw.args))])])
    mraw <- do.call(read.marrayRaw, maRaw.args)

    if(setgal)
      {
        ## Checking orders
        mraw <- mraw[gal$neworder,]
        ## make sure the reordering doesn't affect the subset fucntion.
        mraw@maLayout@maSub <- layout@maSub
      }
    return(mraw)
  }



###########################################################################
#  Read GENEPIX
#
read.GenePix <-  function(fnames = NULL,
                          path=NULL,
                          name.Gf = "F532 Median",
                          name.Gb = "B532 Median",
                          name.Rf = "F635 Median",
                          name.Rb = "B635 Median",
                          name.W= "Flags",
                          layout = NULL,
                          gnames = NULL,
                          targets = NULL,
                          notes=NULL,
                          skip=NULL,
                          sep="\t",
                          quote="\"",
                          DEBUG=FALSE,
                          ...)
  {

    opt <- list(...)
    ## If fnames not specified, read everything in the dir
    if(is.null(fnames))
      {
        if(!is.null(targets))
          fullnames <- as.vector(maInfo(targets)[,1]) ## modified
        else
          {
            if(is.null(path))
              fullnames <- dir(pattern=".*\\.gpr$")
            else
              fullnames <- dir(path, pattern=".*\\.gpr$")
          }
      }
    else
      fullnames <- fnames

    setgal <- FALSE
    if(DEBUG) print(fullnames)
    if(is.null(gnames) | is.null(layout))
      {
        if(DEBUG) print("Reading Galfile ... ")
        defs <- list(galfile = fullnames[1], path=path, info.id = c("ID", "Name"),
                     labels = "ID", sep = sep, quote=quote, fill=TRUE, check.names=FALSE,
                     as.is=TRUE, ncolumns = 4, skip=skip)
        gal.args <- maDotsMatch(maDotsMatch(opt, defs), formals(args("read.Galfile")))
        gal <- do.call(read.Galfile, gal.args)
        if(is.null(gnames)) gnames <- gal$gnames
        if(is.null(layout)) layout <- gal$layout
        if(DEBUG) print("done \n ")
        setgal <- TRUE
      }

    if(DEBUG) print(table(layout@maControls))
    if(DEBUG) cat("Setting up controls status ... ")
    if(length(layout@maControls) == 0)
      {
        if(DEBUG) cat("Generate controls \n")
        GenControls.args <- maDotsMatch(maDotsMatch(opt, list(Gnames=gnames)), formals(args("maGenControls")))
        layout@maControls <- as.factor(do.call(maGenControls, GenControls.args))
        if(DEBUG) print(table(layout@maControls))
      }
    if(DEBUG) cat("done \n ")

    if(is.null(notes)) notes <- "GenePix Data"

    if(DEBUG) cat("Calling read.marrayRaw ... \n")
    defs <- list(fnames = fullnames, path=path,
                 name.Gf = name.Gf, name.Gb=name.Gb, name.Rf=name.Rf, name.Rb=name.Rb,
                 name.W=name.W, layout = layout, gnames=gnames, targets=targets,
                 notes = notes, skip=skip, sep=sep, quote=quote, fill=TRUE)
    maRaw.args <- maDotsMatch(maDotsMatch(opt, defs), formals(args("read.marrayRaw")))
    maRaw.args <- c(maRaw.args, defs[names(defs[setdiff(names(defs), names(maRaw.args))])])
    mraw <- do.call(read.marrayRaw, maRaw.args)

    if(setgal)
      {
        ## Checking orders
        mraw <- mraw[gal$neworder,]
        ## make sure the reordering doesn't affect the subset fucntion.
        mraw@maLayout@maSub <- layout@maSub
      }

    return(mraw)
  }




###########################################################################
#  Read Agilent
#
read.Agilent <-  function(fnames = NULL,
                          path=NULL,
                          name.Gf = "gMedianSignal",
                          name.Gb = "gBGMedianSignal",
                          name.Rf = "rMedianSignal",
                          name.Rb = "rBGMedianSignal",
                          name.W= NULL,
                          layout = NULL,
                          gnames = NULL,
                          targets = NULL,
                          notes=NULL,
                          skip=NULL,
                          sep="\t",
                          quote="\"",
                          DEBUG=FALSE,
                          info.id=NULL,
                          ...)
  {

    opt <- list(...)
    ## If fnames not specified, read everything in the dir
    if(is.null(fnames))
      {
        if(!is.null(targets))
          fullnames <- as.vector(maInfo(targets)[,1]) ## modified
        else
          {
            if(is.null(path))
              fullnames <- dir(pattern=".*\\.txt$")
            else
              fullnames <- dir(path, pattern=".*\\.txt$")
          }
      }
    else
      fullnames <- fnames

    if(DEBUG) print(fullnames)
    if(is.null(gnames) | is.null(layout))
      {
        if(DEBUG) cat("Setting up Gene Annotation information  ... ")
        ## Gnames : reading data in (the first one)
        y <- readLines(fullnames[1], n=100)
        if(class(info.id) == "character")
          skip <- intersect(grep(info.id[1], y), grep("Row", y))[1] - 1
        else
          skip <- intersect(grep("Row", y), grep("ProbeName", y))[1] - 1

        defs <- list(file=fullnames[1], path = path, sep=sep, skip=skip,
                     fill = TRUE, quote = "\"", check.names=FALSE,
                     as.is=TRUE, comment.char="", header=TRUE)
        read.args <- maDotsMatch(maDotsMatch(opt, defs), formals(args("read.table")))
        dat <- do.call(read.table, read.args)

        descript <- new("marrayInfo", maNotes = "Agilent")
        if(is.null(info.id))
          {
            info.id <- intersect(c("ProbeName", "ProbeUID", "SystematicName", "Description", "ControlType", "SwissProt", "GenBank", "Primate", "Sequence", "GenPept"), colnames(dat))
          }
        maInfo(descript) <- data.frame(dat[,info.id])
        rownames(descript@maInfo) <- make.names(as.vector(dat[,info.id[1]]), unique=TRUE)
        descript@maLabels <- as.character(dat[,info.id[1]])
        if(DEBUG) cat("done \n")

        ## Layout
        if(DEBUG) cat("Setting up Layout information  ... ")
##        idR<- grep(layout.id["Row"], colnames(dat));  ##Lrow <- dat[,id]
##        idC <- grep(layout.id["Column"], colnames(dat)); ## Lcolumn <- dat[,id]
        layout <- maCompLayout(cbind(1, 1, dat[,c("Row", "Col")]), ncolumns=1)
        tmp <- rep(TRUE, max(as.integer(dat[,"FeatureNum"])))
        tmp[-dat[,"FeatureNum"]] <- FALSE
        maSub(layout) <- tmp
        if(DEBUG) cat("done \n")
      }

    if(DEBUG) cat("Setting up controls status ... ")
    if(length(layout@maControls) == 0) {
      tmp <- as.character(dat[,"ControlType"])
      tmp[tmp == "0"] <- "probes"
      tmp[tmp == "1"] <- "Positive"
      tmp[tmp == "-1"] <- "Negative"
      layout@maControls <- as.factor(tmp)
      rm(tmp)
    }
    if(DEBUG) cat("done \n ")

    if(is.null(notes)) notes <- "Agilent Data"

    if(DEBUG) cat("Calling read.marrayRaw ... \n")
    if(DEBUG) cat("To read", fullnames, "... \n")

    defs <- list(fnames = fullnames, path=path,
                 name.Gf = name.Gf, name.Gb=name.Gb, name.Rf=name.Rf, name.Rb=name.Rb,
                 name.W=name.W, layout = layout, gnames=descript, targets=targets,
                 notes = notes, skip=skip, sep=sep, quote=quote, fill=TRUE,
                 check.names=FALSE,  as.is=TRUE)
    maRaw.args <- maDotsMatch(maDotsMatch(opt, defs), formals(args("read.marrayRaw")))
    mraw <- do.call(read.marrayRaw, maRaw.args)

    return(mraw)
  }




###########################################################################

read.Galfile <- function (galfile,
                          path=".",
                          info.id = c("ID", "Name"),
                          layout.id =c(Block="Block", Row="Row", Column="Column"),
                          labels = "ID",
                          notes = "",
                          sep = "\t",
                          skip = NULL,
                          ncolumns = 4,
                          ...)
{
  opt <- list(...)

  if(!is.null(path))
    f <- file.path(path, galfile)
   else
    f <- galfile
  y <- readLines(f, n=100)

  ## Fix no check whether skip is null or not ##
  ## Pointed out by Dustin Potter,  Feb 9, 2006 ##

  if(is.null(skip)){
    if(class(info.id) == "character")
      skip <- intersect(grep(info.id[1], y), grep(layout.id[1], y))[1] - 1
    else
      skip <- intersect(grep("ID", y), grep("Name", y))[1] - 1
  }
  
  defs <- list(file=f, path = path, sep=sep, skip=skip,
               fill = TRUE, quote = "\"", check.names=FALSE,
               as.is=TRUE, comment.char="", header=TRUE)
  read.args <- maDotsMatch(maDotsMatch(opt, defs), formals(args("read.table")))
  dat <- do.call(read.table, read.args)

  ## Gnames
  descript <- new("marrayInfo", maNotes = notes)
  if (is.null(info.id))
    info.id <- 1:ncol(dat)
  maInfo(descript) <- data.frame(dat[,info.id])

  ## Fix for R2.0 because data.frame needed unique rownames
  rownames(descript@maInfo) <- make.names(as.vector(dat[,info.id[1]]), unique=TRUE)

  if (length(labels) == nrow(dat))
    maLabels(descript) <- as.vector(labels)
  else {
    if (is.null(labels))
      labels <- 1
    maLabels(descript) <- as.character(dat[,labels])
  }

  ## Layout
  idB <- grep(layout.id["Block"], colnames(dat));  ##Lblock <- dat[,id]
  idR<- grep(layout.id["Row"], colnames(dat));  ##Lrow <- dat[,id]
  idC <- grep(layout.id["Column"], colnames(dat)); ## Lcolumn <- dat[,id]
  mlayout <- maCompLayout(dat[,c(idB, idR, idC)], ncolumns)
  newmat <- cbind(gr = ((dat[,idB] - 1)%/%ncolumns) + 1,
                  gc = ((dat[,idB] - 1)%%ncolumns) + 1, sr = dat[,idR], sc = dat[,idC])
  ngr<-maNgr(mlayout); ngc<-maNgc(mlayout)
  nsr<-maNsr(mlayout); nsc<-maNsc(mlayout)
  ind<-(nsr * nsc)* ((newmat[,1] - 1) * ngc + (newmat[,2] - 1)) + (newmat[,3] - 1) * nsc + newmat[,4]
  ## ind provides the proper location of the entry
  ## Assume no NAs and no missing values
  return(list(gnames = descript, layout=mlayout, neworder=order(ind)))
}



##############################################################
## Read SMD, default arguments are for Hs genome
## For other genomes, you will need to modify the info.id field

read.SMD <- function(fnames = NULL, path = NULL,
                     name.Gf = "Ch1 Intensity (Median)",
                     name.Gb = "Ch1 Background (Median)",
                     name.Rf = "Ch2 Intensity (Median)",
                     name.Rb = "Ch2 Background (Median)",
                     name.W = NULL,
                     info.id = c("Name", "Clone ID"),
                     layout = NULL, gnames = NULL, targets = NULL, notes = NULL,
                     skip = NULL, sep = "\t", quote = "\"", DEBUG=FALSE, ...)
{
  ## If fnames is NULL check target, if target is also NULL then read from dir()
  if(is.null(fnames))
    {
      if(!is.null(targets))
        fullnames <- as.vector(maInfo(targets)[,1]) ## modified
      else
        {
          if(is.null(path))
            fullnames <- dir(pattern="\\.xls$")
          else
            fullnames <- dir(path, pattern="\\.xls$")
        }
    }
  else
    fullnames <- fnames

  if(DEBUG) print(fnames)

  ## Jean Yang, April 18, 2007
  ## Add opt = list(...)
  ## Bug pointed out by Michael Gormley <mpg33@drexel.edu>
  opt <- list(...)
  if(is.null(gnames) | is.null(layout))
      {
        if(DEBUG) cat("Generating layout from ", fullnames[1], "\n", sep="")
        opt <- list(...)
        defs <- list(galfile = fullnames[1], path=path, info.id = c("Name", "Clone ID"),
                     layout.id=c(Block="Sector", Row="X Grid Coordinate \\(within sector\\)",
                       Column="Y Grid Coordinate \\(within sector\\)"),
                     labels = "Spot", sep = sep, quote=quote, fill=TRUE, check.names=FALSE,
                     as.is=TRUE, ncolumns = 4)
        gal.args <- maDotsMatch(maDotsMatch(opt, defs), formals(args("read.Galfile")))
        if (DEBUG) print("Reading Gal file")
        gal <- do.call(read.Galfile, gal.args)
        if(is.null(gnames)) gnames <- gal$gnames
        if(is.null(layout)) layout <- gal$layout
        if(DEBUG) print("done \n ")
      }

    if(is.null(targets))
    {
      cat("Generating target sample info from all files\n")
      maLabels <- character(0)
      maInfo <- data.frame()
      for (i in 1:length(fullnames)) {
        #modify name.Gf for special character
        regName.Gf <- gsub("\\(", "\\\\\\(", name.Gf)
        regName.Gf <- gsub("\\)", "\\\\\\)", regName.Gf)
        if (DEBUG) print(regName.Gf)
        comment <- grep(regName.Gf, readLines(fullnames[i], n=100))
        if (DEBUG) print(paste("comment = ", comment, sep=""))
        z <- readLines(fullnames[i], n = comment)
        row <- grep("Exptid", z)[1]
        maLabels <- c(maLabels, strsplit(z[row], "=")[[1]][2])
        row <- grep("Experiment Name", z)[1]
        Experiment <- strsplit(z[row], "=")[[1]][2]
        row <- grep("Channel 1 Description", z)[1]
        Cy3 <- strsplit(z[row], "=")[[1]][2]
        row <- grep("Channel 2 Description", z)[1]
        Cy5 <- strsplit(z[row], "=")[[1]][2]
        row <- grep("SlideName", z)[1]
        SlideName <- strsplit(z[row], "=")[[1]][2]
        maInfo <- rbind(maInfo, data.frame(Experiment = Experiment,
                                           Cy3 = Cy3, Cy5 = Cy5,
                                           SlideName = SlideName))
      }
    }

  rownames(maInfo) <- 1:dim(maInfo)[1]
  targets <- new("marrayInfo", maLabels = maLabels, maInfo = maInfo)

  defs <- list(fnames = fullnames, path=path,
               name.Gf = name.Gf, name.Gb=name.Gb, name.Rf=name.Rf, name.Rb=name.Rb,
               name.W=name.W, layout = layout, gnames=gnames, targets=targets,
               notes = notes, skip=skip, sep=sep, quote=quote, fill=TRUE,
               check.names=FALSE,  as.is=TRUE)
  if(DEBUG) cat("Calling read.marrayRaw ... \n")
  maRaw.args <- maDotsMatch(maDotsMatch(opt, defs), formals(args("read.marrayRaw")))
  mraw <- do.call(read.marrayRaw, maRaw.args)
  return(mraw)
}

###########################################################################
# Function: checkTargetInfo
# ADD:  check that the foreground and backgruond intensities are
# stored in the same order as provided in the first column of target file.
# Date:  Jan 05, 2005
###########################################################################
checkTargetInfo <- function(mraw)
  {
    if(length(mraw@maTargets@maInfo) == 0)
      stop("Missing Target Information \n")
    targetFnames <- as.vector(mraw@maTargets@maInfo[,1])

    if(class(mraw) == "marrayRaw")
      {
        if((length(mraw@maGf) == 0 ) & length(mraw@maRf) == 0)
          stop("Missing intensities information in both channels\n")
        if((length(mraw@maGf) != 0 ))
          colFnames <- colnames(mraw@maGf)
        else
          colFnames <- colnames(mraw@maRf)
      }
    if(class(mraw) == "marrayNorm")
      {
        if((length(mraw@maM) == 0 ))
          stop("Missing Log-ratio information \n")
        colFnames <- colnames(mraw@maM)
      }

    res <- sapply(targetFnames, grep, colFnames) == 1:length(colFnames)
    return(sum(res) == length(colFnames))

  }

############################################
## END OF FILE
############################################

############################################################################
# maPlot.R
# maPlot3.R
#
# S4 methods
# Diagnostic plots for two-color cDNA microarrays
# source("~/Projects/madman/Rpacks/marray/R/maPlots.R")
###########################################################################

## S3 + S4 settings
## March 15, 2004
plot.marrayRaw <-
  function (x, xvar = "maA", yvar = "maM", zvar="maPrintTip", lines.func,text.func,legend.func,...)
  {maPlot(m=x, x=xvar, y=yvar, z=zvar, lines.func=lines.func, text.func=text.func, legend.func=legend.func,...)}

plot.marrayNorm <-
  function (x, xvar = "maA", yvar = "maM", zvar="maPrintTip", lines.func,text.func,legend.func,...)
  {maPlot(m=x, x=xvar, y=yvar, z=zvar, lines.func=lines.func, text.func=text.func, legend.func=legend.func,...)}


###boxplot
if( is.null(getGeneric("boxplot")))  setGeneric("boxplot")
setMethod("boxplot", signature(x="marrayRaw"), function (x, xvar = "maPrintTip", yvar = "maM", ...){
            maBoxplot(m=x, x=xvar, y=yvar, ...)})
setMethod("boxplot", signature(x="marrayNorm"), function (x, xvar = "maPrintTip", yvar = "maM", ...){
              maBoxplot(m=x, x=xvar, y=yvar, ...)})

if( is.null(getGeneric("image")))  setGeneric("image")
setMethod("image", signature(x="marrayRaw"),
          function (x, xvar = "maM", subset = TRUE, col, contours = FALSE,  bar = TRUE, overlay=NULL, ol.col=1, colorinfo = FALSE, ...){
            maImage(m=x, x=xvar, subset=subset, col=col, contours=contours, bar=bar, overlay=overlay, ol.col=ol.col, colorinfo = colorinfo, ... )})

setMethod("image", signature(x="marrayNorm"),
          function (x, xvar = "maM", subset = TRUE, col, contours = FALSE,  bar = TRUE, overlay=NULL, ol.col=1,colorinfo = FALSE, ...){
              maImage(m=x, x=xvar, subset=subset, col=col, contours=contours, bar=bar, overlay=overlay, ol.col=ol.col, colorinfo = colorinfo, ... )}) 

## points, lines, text
if( is.null(getGeneric("points")))  setGeneric("points")
setMethod("points", signature(x="marrayRaw"), function (x, xvar = "maA", yvar = "maM", ...)
          {addPoints(object=x, xvar=xvar, yvar=yvar, ...)})
setMethod("points", signature(x="marrayNorm"), function (x, xvar = "maA", yvar = "maM", ...)
          {addPoints(object=x, xvar=xvar, yvar=yvar, ...)})

if( is.null(getGeneric("text")))  setGeneric("text")
setMethod("text", signature(x="marrayRaw"), function (x, xvar = "maA", yvar = "maM",...)
          {addText(object=x, xvar=xvar, yvar=yvar, ...)})
setMethod("text", signature(x="marrayNorm"), function (x, xvar = "maA", yvar = "maM", ...)
          {addText(object=x, xvar=xvar, yvar=yvar, ...)})

if( is.null(getGeneric("lines")))  setGeneric("lines")
setMethod("lines", signature(x="marrayRaw"), function (x, xvar = "maA", yvar = "maM", zvar="maPrintTip",...)
          {addLines(object=x, xvar=xvar, yvar=yvar, ...)})
setMethod("lines", signature(x="marrayNorm"), function (x, xvar = "maA", yvar = "maM", zvar="maPrintTip",...)
          {addLines(object=x, xvar=xvar, yvar=yvar, ...)})


addText <- function(object, xvar="maA", yvar="maM", subset=NULL, labels = as.character(1:length(subset)), ...)
  {
    text.func <- maText(subset=subset, labels=labels, ...)
    text.func(as.numeric(eval(call(xvar,object))), as.numeric(eval(call(yvar,object))))
  }


addPoints <- function(object, xvar="maA", yvar="maM", subset=TRUE, ...)
  {
    xx <- as.numeric(eval(call(xvar,object)))
    yy <- as.numeric(eval(call(yvar,object)))
    points(xx[subset], yy[subset], ...)
  }

addLines <- function(object, xvar="maA", yvar="maM", zvar="maPrintTip", subset=TRUE, ...)
  {
    defs <- maDefaultPar(m=object,x=xvar,y=yvar,z=zvar)
    lines.func<-do.call(maLoessLines,
                        c(list(subset=subset, loess.args=list(span=0.4, degree=1, family="symmetric",
                                              control=loess.control(trace.hat="approximate",
                                                iterations=5,surface="direct"))),defs$def.lines))
    xx <- as.numeric(eval(call(xvar,object)))
    yy <- as.numeric(eval(call(yvar,object)))
    zz <- as.numeric(eval(call(zvar,object)))
    lines.func(xx, yy, zz)
  }
 

###########################################################################
# Default plotting parameters for microarray objects
###########################################################################
# Compare default and ... plotting parameters and let ... overwrite defaults

maDotsDefaults<-function(dots, defaults)
{
  args<-c(dots,defaults[setdiff(names(defaults),names(dots))])
  return(args)
}

#########################
# Default parameters for microarray objects of class marrayRaw and marrayNorm
maDefaultPar<-function(m,x,y,z)
{
    m<-m[,1]
    main<-as.character(maLabels(maTargets(m)))

    xlab<-ylab<-zlab<-""
    col<-2
    lty<-1
    lwd<-2.5
    las<-1
    names<-""
    def.legend<-list()

    ylab<-strsplit(y,"ma")[[1]]
    if(ylab[1]=="")
     ylab<-ylab[2]

    if(!is.null(x))
    {
      xlab<-strsplit(x,"ma")[[1]]
      if(xlab[1]=="")
        xlab<-xlab[2]
    }

    if(!is.null(z))
    {
      zz<-eval(call(z,m))
      zlab<-strsplit(z,"ma")[[1]]
      if(zlab[1]=="")
        zlab<-zlab[2]

      if(z!="maPrintTip")
      {
        ## names<-paste(zlab,unique(zz),sep=" ")  ## BUG [modified Feb 27, 2004]
        ifelse(is.factor(zz), tmp <- levels(zz), tmp<- unique(zz))
        names<-paste(zlab, tmp, sep=" ")  
        col<-(1:length(names))
        lty<-rep(1,length(names))
        las<-1
        ncol<-1
        ord<-order(lty,col)
        def.legend<-list(legend=names[ord],col=col[ord], lty=lty[ord], lwd=lwd, ncol=ncol)
      }

      if(z=="maPrintTip")
      {
        which<-unique(zz)
        Ig<-maNgr(m)
        Jg<-maNgc(m)
        lg.names<- paste("(",sort(rep(1:Ig,Jg)),",",rep(1:Jg,Ig),")",sep="")
        lg.col<-sort(rep(2:(Ig+1),Jg))
        lg.lty<-rep(1:Jg,Ig)
        names<-lg.names[which]
        col<-lg.col[which]
        lty<-lg.lty[which]
        las<-3
        ncol<-Jg
        ord<-order(lg.lty,lg.col)
        def.legend<-list(legend=lg.names[ord],col=lg.col[ord], lty=lg.lty[ord], lwd=lwd, ncol=ncol)
      }
   }

    def.box<-list(xlab=xlab,ylab=ylab,names=names,col=col,las=las,main=main)
    def.plot<-list(xlab=xlab,ylab=ylab,pch=20,col=1,main=main)
    def.lines<-list(col=col,lty=lty,lwd=lwd)
    def.text<-list(pch=16,col="purple")
    return(list(def.box=def.box,def.plot=def.plot,def.lines=def.lines,
		def.legend=def.legend,def.text=def.text))
}

###########################################################################
# maBoxplot: Boxplot methods
###########################################################################
# Boxplots for a single and multiple arrays

maBoxplot<- function(m, x="maPrintTip", y="maM", ...) {
    opt<-list(...)
    if(maNsamples(m)==1)
    {

      yy<-as.numeric(eval(call(y,m)))

      if(is.null(x))
        xx<-rep(1,length(yy))
      if(!is.null(x))
        xx<-eval(call(x,m))
      # Optional graphical parameter defaults
      def<-maDefaultPar(m,x,y,x)$def.box
      if(!is.null(opt))
        def<-maDotsDefaults(opt,def)

      args<-c(list(yy~xx),def)
      do.call(boxplot,args)
   }
   if(maNsamples(m)>1)
   {
     yy<-as.data.frame(eval(call(y,m)))

     # Optional graphical parameter defaults
     if(length(maLabels(maTargets(m))) != 0)
       def <- list(names=maLabels(maTargets(m)),ylab=strsplit(y,"ma")[[1]][2],col=2)
     else
       def <- list(names=dimnames(yy)[[2]], ylab=strsplit(y,"ma")[[1]][2],col=2)
     if(!is.null(opt))
        def<-maDotsDefaults(opt,def)

      args<-c(list(yy),def)
      do.call(boxplot,args)
    }
##    if(y=="maM") abline(h=0,col="gray",lwd=2.5)
}

###########################################################################
##
# maPlot: Scatter-plot methods with fitted lines and points highlighted
##
###########################################################################
# General function for scatter-plot of y vs. x with fitted lines within
# values of z and subset of points highlighted

maPlot.func<-function(x, y, z,
	lines.func=maLowessLines(subset=TRUE,f=0.3,col=1:length(unique(z)),
		lty=1,lwd=2.5),
	text.func=maText(),
	legend.func=maLegendLines(legend=as.character(unique(z)),
		col=1:length(unique(z)), lty=1,lwd=2.5,ncol=1),
	...)
{
  plot(x,y,...)

  # Plot fitted curves
  if(!is.null(lines.func))
    lines.func(x,y,z)

  # Legend
  if(!is.null(legend.func))
    legend.func(x,y)

  # Label a subset of points
  if(!is.null(text.func))
    text.func(x,y)

}

#########################
# Label a subset of points
# Jean: Modify (Oct 21, 2002)  line "tmp <- length(c(1:length(subset))[subset])"

maText <-function (subset = NULL, labels = as.character(1:length(subset)),
    ...)
{
    function(x, y) {
      tmp <- length(c(1:length(subset))[subset])
      if (tmp > 0) {
            if (length(subset) < length(labels))
                text(x[subset], y[subset], labels[subset], ...)
            if (length(subset) > length(labels))
                text(x[subset], y[subset], labels, ...)
            if ((length(subset) == length(labels)) & is.logical(subset))
                text(x[subset], y[subset], labels[subset], ...)
            if ((length(subset) == length(labels)) & is.numeric(subset))
                text(x[subset], y[subset], labels, ...)
        }
    }
}


#########################
# Plot fitted lines

# Lowess
maLowessLines<-function(subset=TRUE, f=0.3, col=2, lty=1, lwd=2.5,...)
{
  function(x,y,z)
  {
    subset<-maNum2Logic(length(x), subset)
    g<-unique(z[subset])
    if(length(col)<length(g))
      col<-rep(col[1],length(g))
    if(length(lty)<length(g))
      lty<-rep(lty[1],length(g))

    for(i in (1:length(g)))
    {
      which<-z[subset]==g[i]
      xx<-x[subset][which]
      yy<-y[subset][which]
      ind <- is.na(xx) | is.na(yy) | is.infinite(xx) | is.infinite(yy)
      fit<- lowess(xx[!ind], yy[!ind], f=f)
      lines(fit,col=col[i],lty=lty[i],lwd=lwd,...)
    }
  }
}

# Loess

maLoessLines<-function(subset=TRUE, weights=NULL,
                        loess.args=list(span=0.4, degree=1, family="symmetric",
                          control=loess.control(trace.hat="approximate",
                            iterations=5,surface="direct")),col=2, lty=1, lwd=2.5, ...)
{
  function(x,y,z)
  {
    subset<-maNum2Logic(length(x), subset)
    g<-unique(z[subset])
    if(length(col)<length(g))
      col<-rep(col[1],length(g))
    if(length(lty)<length(g))
      lty<-rep(lty[1],length(g))

    for(i in (1:length(g)))
    {
      which<-z[subset]==g[i]
      if(sum(which) >= 1000){
##        print(sum(which));      print(length(which))
        tmp <- sample(1:sum(which), 1000)
        which[which][-tmp] <- FALSE 
##        print(sum(which));        print(length(which))
      }
      xx<-x[subset][which]
      yy<-y[subset][which]
      ww<-weights[subset][which]
      args<-c(list(yy ~ xx, weights=ww),loess.args)
      fit<-do.call(loess,args)
      xf<-seq(quantile(xx,0.005,na.rm=TRUE),quantile(xx,0.995,na.rm=TRUE),length=100)
      yf<-predict(fit,data.frame(xx=xf))
      lines(xf,yf,col=col[i],lty=lty[i],lwd=lwd,...)
    }
  }
}

#########################
# Add legend to existing plot

maLegendLines<-function(legend="", col=2, lty=1, lwd=2.5, ncol=1, ...)
{
  function(x,y)
  {
    a<-min(x[!(is.na(x)|is.infinite(x))])
    b<-max(y[!(is.na(y)|is.infinite(y))])
    legend(a,b,legend=as.character(legend),col=col,lty=lty,lwd=lwd,ncol=ncol,...)
  }
}

###########################################################################
# Methods for microarray objects: wrapper around maPlot.func
##
## Jean April 9,2003 modified default to maLoessLines
##

maPlot <- function(m, x="maA", y="maM", z="maPrintTip",lines.func,text.func,legend.func, ...)
{

  m<-m[,1]
  # Default plotting arguments
  defs<-maDefaultPar(m,x,y,z)

  if(missing(lines.func))
    lines.func<-do.call(maLoessLines, c(list(subset=TRUE,
                                               loess.args=list(span=0.4, degree=1, family="symmetric",
                                                 control=loess.control(trace.hat="approximate",
                                                   iterations=5,surface="direct"))),
                                               defs$def.lines))
  if(missing(text.func))
    text.func<-maText()
  if(missing(legend.func))
    legend.func<-do.call(maLegendLines,defs$def.legend)

  xx<-as.numeric(eval(call(x,m)))
  yy<-as.numeric(eval(call(y,m)))
  if(is.null(z))
     zz<-rep(1,length(xx))
  if(!is.null(z))
    zz<-eval(call(z,m))

  opt<-list(...)
  if(!is.null(opt))
    def.plot<-maDotsDefaults(opt,defs$def.plot)

  do.call(maPlot.func, c(list(x=xx,y=yy,z=zz,lines.func=lines.func,text.func=text.func,legend.func=legend.func),def.plot))
  if(y=="maM") abline(h=0,col="gray",lwd=2.5)
}

###########################################################################
# maImage: image methods for microarray objects
# add overlay, NULL which means no overlay or a subset of information
# Modified March 15, 2004 to add overlay option (contribution from katie)
###########################################################################

maImage.func<-function(x, L, subset=TRUE, col=heat.colors(12), contours=FALSE, overlay=NULL, ol.col=1, ...)
{
  
  ## x is a matrix
  if(missing(L))
    L <- new("marrayLayout", maNgr=1, maNgc=1, maNsr=nrow(x), maNsc=ncol(x))

  ## When only a subset of spots are stored in marray object, pad with NA
  subset<-maNum2Logic(maNspots(L), subset)
  z<-rep(NA,maNspots(L))
  z[maSub(L)][subset]<-x[subset]
 
  # Create a "full layout"
  Ig<-maNgr(L);  Jg<-maNgc(L);  Is<-maNsr(L);  Js<-maNsc(L)
  L0<-read.marrayLayout(ngr=Ig, ngc=Jg, nsr=Is, nsc=Js)
  nr<-Is*Ig;  nc<-Js*Jg
  row.ind<-(maGridRow(L0)-1)*Is+maSpotRow(L0)
  col.ind<-(maGridCol(L0)-1)*Js+maSpotCol(L0)
  ord<-order(row.ind,col.ind)

  z<-matrix(z[ord],nrow=nr,ncol=nc,byrow=TRUE)
  z<-t(z)[,nr:1]

  # Image of spot statistics
  image(1:nc, 1:nr, z, axes=FALSE, col=col, ...)
  axis(3, at = (0:(Jg-1))*Js + Js/2, labels = 1:Jg)
  axis(2, at = (0:(Ig-1))*Is + Is/2, labels = Ig:1,las=1)
  if(contours)
    contour(1:nc,1:nr,z,add=TRUE, ...)
  box(lwd=4)
  abline(v=((1:Jg-1)*Js + 0.5),lwd=3)
  abline(h=((1:Ig-1)*Is + 0.5),lwd=3)

 # Only for overlay
  if(!is.null(overlay))
    {
      zsub<-rep(NA,maNspots(L))
      subset.over<-maNum2Logic(maNspots(L), overlay)
      zsub[maSub(L)][subset] <- subset.over[subset]
      ## This is subset of subset so we assume that subset.over
      ## (or overlay) is a subset of the subset data a subset on the original

      ## Plotting
      zsub<-matrix(zsub[ord],nrow=nr,ncol=nc,byrow=TRUE)
      zsub<-t(zsub)[,nr:1]
      ## we hard code this part for the moment
      points(row(zsub)[zsub],col(zsub)[zsub], pch=22, col=ol.col, cex=0.9)  
    }
 
}

#########################
# Methods for microarray objects: wrapper around maImage.func

# Methods for microarray objects: wrapper around maImage.func
# Modified by Jean : Sept 15, 2002 to include centering of color
# Modified March 15, 2004 to add overlay option (contribution from katie)

maImage <- function(m, x="maM", subset=TRUE, col, contours=FALSE, bar=TRUE, overlay=NULL, ol.col=1,
                    colorinfo = FALSE, ...)
{
  subset<-maNum2Logic(maNspots(m), subset)
  m<-m[,1]
  if(missing(col))
  {
    col<-rainbow(50)
    if(is.element(x,c("maGb","maGf","maLG")))
      col<-maPalette(low="white", high="green", k=50)
    if(is.element(x,c("maRb","maRf","maLR")))
      col<-maPalette(low="white", high="red", k=50)
    if(is.element(x,c("maM","maMloc","maMscale")))
      col<-maPalette(low="blue", mid="gray", high="yellow", k=50)
    if(is.element(x,c("maA")))
      col<-maPalette(low="white", high="blue", k=50)
  }

  xx<-as.numeric(eval(call(x,m)))


  # Set color range [ensure it's centered]
  tmp<-xx[subset]
  tmp<-tmp[!(is.na(tmp)|is.infinite(tmp))]

  zmax <- ceiling(max(tmp))
  zmin <- floor(min(tmp))
  if(zmin < 0){
    ztmp <- max(abs(zmin), zmax)
    zrange <- c(-ztmp, ztmp)
  }
  else
    zrange <- c(zmin, zmax)

  # Optional graphical parameter defaults
  def<-list(xlab="",ylab="",main=paste(maLabels(maTargets(m)), ": image of ",
                              strsplit(x,"ma")[[1]][2], sep=""), zlim=zrange)
  opt<-list(...)
  if(!is.null(opt))
    def<-maDotsDefaults(opt,def)
  args<-c(list(x=xx, L=maLayout(m), subset=subset, col=col, contours=contours, overlay=overlay, ol.col=ol.col),def) 
  x.bar <- seq(args$zlim[1], args$zlim[2], length=41)

  if(!bar)
    do.call(maImage.func,args)

  if(bar)
  {
    layout(matrix(c(1,2),1,2),width=c(8,2))
    par(mar=c(4,4,5,3))
    do.call(maImage.func,args)
    par(mar=c(3,0,3,3))
    maColorBar(x.bar,horizontal=FALSE,col=col,main="")
    layout(1)
    par(mar=c(5,4,4,2) + 0.1)
  }

  print(colorinfo)
  if(colorinfo)
    return(list(x.col=col[1:length(x.bar)], x.bar=x.bar, summary=summary(xx[subset])))
  else
    return()
}

###########################################################################
# Color bar for calibration
###########################################################################

maPalette <- function(low = "white", high = c("green", "red"), mid=NULL, k =50)
{
    low <- col2rgb(low)/255
    high <- col2rgb(high)/255

    if(is.null(mid)){
        r <- seq(low[1], high[1], len = k)
        g <- seq(low[2], high[2], len = k)
        b <- seq(low[3], high[3], len = k)
      }
    if(!is.null(mid)){
      k2 <- round(k/2)
      mid <- col2rgb(mid)/255
      r <- c(seq(low[1], mid[1], len = k2),
             seq(mid[1], high[1], len = k2))
      g <- c(seq(low[2], mid[2], len = k2),
             seq(mid[2], high[2], len = k2))
      b <- c(seq(low[3], mid[3], len = k2),
             seq(mid[3], high[3], len = k2))
    }
    rgb(r, g, b)
}


maColorBar<-function(x, horizontal = TRUE, col=heat.colors(50),
scale=1:length(x), k=10,  ...)
{
  if(is.numeric(x))
  {
    x <- x
    colmap <- col
  }
  else
  {
    colmap <- x
    low<-range(scale)[1]
    high<-range(scale)[2]
    x <- seq(low, high, length=length(x))
  }

  if(length(x)>k)
    x.small<-seq(x[1], x[length(x)],length=k)
  else
    x.small<-x

  if(horizontal)
  {
    image(x, 1, matrix(x,length(x),1), axes=FALSE, xlab="", ylab="", col=colmap, ...)
    axis(1, at=rev(x.small), labels=signif(rev(x.small),2), srt=270)
  }
  if(!horizontal)
  {
    image(1, x, matrix(x,1,length(x)), axes=FALSE, xlab="", ylab="", col=colmap, ...)
    par(las=1)
    axis(4, at=rev(x.small), labels=signif(rev(x.small), 2))
    par(las=0) # Back to default
  }
  box()
}

###########################################################################
# Functions to filter genes, return a logical vector

maTop<-function(x, h=1, l=1)
{
  x<-as.vector(x)
  return((x>=quantile(x, 1-h, na.rm=TRUE)) | (x<=quantile(x, l, na.rm=TRUE)))
}

###########################################################################

###########################################################################
# maSubset.R
#
# Subsetting methods for microarray classes
#
###########################################################################
# marrayInfo class


setMethod("[", "marrayInfo", function(x, i, j, ..., drop=FALSE)
{
  newx<-x
  if(missing(j))
   j<-TRUE
  if(missing(i))
   i<-TRUE
  if(length(maLabels(x))!=0)
    slot(newx,"maLabels")<-maLabels(x)[i]
  if(length(maInfo(x))!=0)
    slot(newx,"maInfo")<-maInfo(x)[i, j, drop=FALSE]
  return(newx)
})

#########################
# marrayLayout class

setMethod("[", "marrayLayout", function(x, i, j, ..., drop=FALSE)
{
  newx<-x
  if(!missing(i))
  {
    if(length(maPlate(x))!=0)
      slot(newx,"maPlate")<-maPlate(x)[i]
    if(length(maControls(x))!=0)
    slot(newx,"maControls")<-maControls(x)[i]
    if(length(maNspots(x))!=0)
    {
      sub<-rep(FALSE,maNspots(x))
      sub[maSub(x)][i]<-TRUE
      slot(newx, "maSub")<-sub
    }
  }
  return(newx)
})

#########################
# marrayRaw class

setMethod("[", "marrayRaw", function(x, i, j, ..., drop=FALSE)
{
  newx<-x
  if(missing(j))
   j<-TRUE
  if(missing(i))
   i<-TRUE

  if(length(maRf(x))!=0)
    slot(newx,"maRf")<-maRf(x)[i,j,drop=FALSE]
  if(length(maGf(x))!=0)
    slot(newx,"maGf")<-maGf(x)[i,j,drop=FALSE]
  if(length(maRb(x))!=0)
    slot(newx,"maRb")<-maRb(x)[i,j,drop=FALSE]
  if(length(maGb(x))!=0)
    slot(newx,"maGb")<-maGb(x)[i,j,drop=FALSE]
 if(length(maW(x))!=0)
    slot(newx,"maW")<-maW(x)[i,j,drop=FALSE]
  slot(newx,"maLayout")<-maLayout(x)[i]
  slot(newx,"maGnames")<-maGnames(x)[i]
  slot(newx,"maTargets")<-maTargets(x)[j]
  return(newx)
})

#########################
# marrayNorm class

setMethod("[", "marrayNorm", function(x, i, j, ..., drop=FALSE)
{
  newx<-x
  if(missing(j))
   j<-TRUE
  if(missing(i))
   i<-TRUE


  if(length(maA(x))!=0)
    slot(newx,"maA")<-maA(x)[i,j,drop=FALSE]
  if(length(maM(x))!=0)
    slot(newx,"maM")<-maM(x)[i,j,drop=FALSE]
  if(length(maMloc(x))!=0)
    slot(newx,"maMloc")<-maMloc(x)[i,j,drop=FALSE]
  if(length(maMscale(x))!=0)
    slot(newx,"maMscale")<-maMscale(x)[i,j,drop=FALSE]
  if(length(maW(x))!=0)
    slot(newx,"maW")<-maW(x)[i,j,drop=FALSE]
  slot(newx,"maLayout")<-maLayout(x)[i]
  slot(newx,"maGnames")<-maGnames(x)[i]
  slot(newx,"maTargets")<-maTargets(x)[j]
  slot(newx,"maNormCall")<-maNormCall(x)
  return(newx)
})

###########################################################################

###########################################################################
# maPrint.R
#
# Print methods for microarray classes
# Change Print to summary
###########################################################################
# marrayLayout class

if( is.null(getGeneric("summary")))  setGeneric("summary")
setMethod("summary", signature(object="marrayLayout"), function(object, ...) {
    cat("Array layout:\t Object of class marrayLayout. \n\n")

    cat("Total number of spots:\t\t\t")
    cat(maNspots(object))
    cat("\n")

    cat("Dimensions of grid matrix:\t\t")
    cat(paste( maNgr(object), "rows by", maNgc(object), "cols"))
    cat("\n")
    
    cat("Dimensions of spot matrices:\t\t")
    cat(paste(maNsr(object), "rows by", maNsc(object), "cols"))
    cat("\n\n")

    if(length(maNspots(object))==1)
      nsub<-length((1:maNspots(object))[maSub(object)])
    else
      nsub<-NULL
    cat(paste("Currently working with a subset of ", nsub, "spots.\n\n", sep=""))

    cat("Control spots: \n")
    if(length(maControls(object))!=0){
      tmp <- table(maControls(object))
      cat(paste("There are  ",  length(tmp),  "types of controls : \n"))
      print(tmp[1:min(10, length(tmp))])
      if(length(tmp) > 10) cat ("...")
    }
    cat("\n\n")
    cat("Notes on layout: \n", maNotes(object), "\n")
  })
          

#########################
# marrayInfo class

setMethod("summary",
          signature(object="marrayInfo"),
          function(object, ...){
  cat("Object of class marrayInfo. \n\n")
            
  nr <- length(maLabels(object))
  ds <- dim(maInfo(object))
    
  if(!is.null(ds))
    {
      if(nr!=0)
        {
          tmp<-data.frame(maLabels(object),maInfo(object))
          dimnames(tmp)[[2]][1]<-"maLabels"
          dimnames(tmp)[[2]][-1]<-dimnames(maInfo(object))[[2]]
        }
      else
	tmp<-maInfo(object)
      print(tmp[1:min(nrow(tmp), 10),])
      if(nrow(tmp) > 10)  cat("... \n")
    }

  if(is.null(ds)&(nr!=0))
    {
      tmp <- as.vector(maLabels(object))
      print(tmp[1:min(length(tmp), 10)])
      if(nr > 10)  cat("... \n")
    }
  
  cat("\nNumber of labels: ", nr, " \n")
  cat("Dimensions of maInfo matrix: ", ds[1], " rows by ", ds[2], " columns\n\n")
  cat("Notes: \n", maNotes(object), "\n")
})

#########################
# marrayRaw class
setMethod("summary", signature(object="marrayRaw"), function(object, ...){
  if((length(maGf(object))==0) | (length(maRf(object))==0))
    cat("Input is empty \n")
  else
    {
      cat("Pre-normalization intensity data:\t Object of class marrayRaw. \n\n")
      
      cat(paste("Number of arrays: \t",ncol(maM(object)), " arrays.\n \n", sep=""))
      cat("A) Layout of spots on the array: \n")
      summary(maLayout(object))

      cat("\n")

      cat("B) Samples hybridized to the array: \n")
      summary(maTargets(object))
      cat("\n")

      cat("C) Summary statistics for log-ratio distribution: \n")
      results <- apply(maM(object),2, function(x){round(summary(x),2)})
      if(is.matrix(results))
        print(t(results))

      if(is.list(results)){
        x.min <- x.1q <- x.med <- x.mean <- x.3q <- x.max <- x.NA <- NULL
        x.names <- names(results)
        if(is.null(x.names)) x.names <- as.character(1:length(results))
        for(i in 1:length(results)){
          x.min <- c(x.min, results[[i]][1])
          x.1q <- c(x.1q, results[[i]][2])
          x.med <- c(x.med, results[[i]][3])
          x.mean <- c(x.mean, results[[i]][4])
          x.3q <- c(x.3q, results[[i]][5])
          x.max <- c(x.max, results[[i]][6])
          x.NA <- c(x.NA, results[[i]][7])
        }
        y <- data.frame(x.names, x.min, x.1q, x.med, x.mean, x.3q, x.max, as.vector(x.NA))
        dimnames(y)[[2]] <-
          c("   ", "Min.", "1st Qu.", "Median", "Mean", "3rd Qu.", "Max", "NA")
        print(y)
      }

      cat("\n")

      cat("D) Notes on intensity data: \n")
      cat(maNotes(object))
      cat("\n")
    }
})

#########################
# marrayNorm class

setMethod("summary",
          signature(object="marrayNorm"),
          function(object, ...)
          {
            if((length(maA(object))==0) & (length(maM(object))==0))
              cat("Input is empty \n")
            else
              {
                cat("Normalized intensity data:\t Object of class marrayNorm.\n \n")
                
                cat("Call to normalization function:\n")
                if(length(maNormCall(object)) > 1)
                  print(maNormCall(object))
                
                cat(paste("\nNumber of arrays: \t",ncol(maM(object)), " arrays.\n", sep=""))

                cat("\nA) Layout of spots on the array: \n")
                summary(maLayout(object))
                cat("\n")

                cat("B) Samples hybridized to the array: \n")
                summary(maTargets(object))
                cat("\n")
                
                cat("C) Summary statistics for log-ratio distribution: \n")
                if(length(maM(object))!=0)
                  results <- apply(maM(object),2, function(x){round(summary(x),2)})
                if(is.matrix(results))
                  print(t(results))

                if(is.list(results)){
                  x.min <- x.1q <- x.med <- x.mean <- x.3q <- x.max <- x.NA <- NULL
                  x.names <- names(results)
                  if(is.null(x.names)) x.names <- as.character(1:length(results))
                  for(i in 1:length(results)){
                    x.min <- c(x.min, results[[i]][1])
                    x.1q <- c(x.1q, results[[i]][2])
                    x.med <- c(x.med, results[[i]][3])
                    x.mean <- c(x.mean, results[[i]][4])
                    x.3q <- c(x.3q, results[[i]][5])
                    x.max <- c(x.max, results[[i]][6])
                    x.NA <- c(x.NA, results[[i]][7])
                  }
                  y <- data.frame(x.names, x.min, x.1q, x.med, x.mean, x.3q, x.max, as.vector(x.NA))
        dimnames(y)[[2]] <-
          c("   ", "Min.", "1st Qu.", "Median", "Mean", "3rd Qu.", "Max", "NA")
                  print(y)
                }
                cat("\n")
                
                cat("D) Notes on intensity data: \n")
                cat(maNotes(object))
                cat("\n")
              }
          })

setClass("ShowLargeObject")
setIs("marrayRaw","ShowLargeObject")
setIs("marrayNorm","ShowLargeObject")
setIs("marrayInfo","ShowLargeObject")
setIs("marrayLayout","ShowLargeObject")

setMethod("show","ShowLargeObject",
#  Print and show method large data objects
#  Modified from Gordon Smyth
#  March 2004
function(object) {
  cat("An object of class \"",class(object),"\"\n",sep="")
  for (what in slotNames(object)) {
    x <- slot(object,what)
    if((length(x) > 0) | !is.null(x)) {
      cat("@",what,"\n",sep="")
      printHead(x)
      cat("\n")
    }
  }
})

##########################################################################
# maSet.R
#
# Assignment methods for two-color cDNA microarrays classes
#
###########################################################################

###########################################################################
# Assignment methods for maLabels slot of marrayInfo class

if( !isGeneric("maLabels<-") )
      setGeneric("maLabels<-", function(object, value)
               standardGeneric("maLabels<-"))

setReplaceMethod("maLabels", signature("marrayInfo","character"),
             function(object, value){
               slot(object, "maLabels")<-value
               object
             })

setReplaceMethod("maLabels", signature("marrayInfo", "numeric"),
             function(object, value){
               info <- maInfo(object)
               if(length(value)==1)
		 labels<-info[,value]
	       else
                 labels <- apply(info[,value], 1, paste, collapse=" ")
               slot(object, "maLabels") <- labels
               object
             })

###########################################################################
# Assignment methods for maInfo slot of marrayInfo class

if( !isGeneric("maInfo<-") )
      setGeneric("maInfo<-", function(object, value)
               standardGeneric("maInfo<-"))

setReplaceMethod("maInfo", signature("marrayInfo","data.frame"),
             function(object, value) {
               slot(object, "maInfo")<-value
               object
             })

###########################################################################
# Assignment methods for maNgr slot of marrayLayout class

if( !isGeneric("maNgr<-") )
      setGeneric("maNgr<-", function(object, value)
               standardGeneric("maNgr<-"))

setReplaceMethod("maNgr", signature("marrayLayout","numeric"),
  	function(object, value) {
     	  slot(object,"maNgr")<- value
	  object
	})

setReplaceMethod("maNgr", signature("marrayRaw","numeric"),
	function(object, value) {
	  maNgr(slot(object,"maLayout"))<- value
	  object
	})

setReplaceMethod("maNgr", signature("marrayNorm","numeric"),
  	function(object, value) {
	  maNgr(slot(object,"maLayout"))<- value
	  object
	})
#########################################################################
# Assignment methods for maNgc slot of marrayLayout class

if( !isGeneric("maNgc<-") )
      setGeneric("maNgc<-", function(object, value)
               standardGeneric("maNgc<-"))

setReplaceMethod("maNgc", signature("marrayLayout","numeric"),
  	function(object, value) {
     	  slot(object,"maNgc")<- value
	  object
	})

setReplaceMethod("maNgc", signature("marrayRaw","numeric"),
	function(object, value) {
	  maNgc(slot(object,"maLayout"))<- value
	  object
	})

setReplaceMethod("maNgc", signature("marrayNorm","numeric"),
  	function(object, value) {
	  maNgc(slot(object,"maLayout"))<- value
	  object
	})

###########################################################################
# Assignment methods for maNsr slot of marrayLayout class

if( !isGeneric("maNsr<-") )
      setGeneric("maNsr<-", function(object, value)
               standardGeneric("maNsr<-"))

setReplaceMethod("maNsr", signature("marrayLayout","numeric"),
  	function(object, value) {
     	  slot(object,"maNsr")<- value
	  object
	})

setReplaceMethod("maNsr", signature("marrayRaw","numeric"),
	function(object, value) {
	  maNsr(slot(object,"maLayout"))<- value
	  object
	})

setReplaceMethod("maNsr", signature("marrayNorm","numeric"),
  	function(object, value) {
	  maNsr(slot(object,"maLayout"))<- value
	  object
	})

###########################################################################
# Assignment methods for maNsc slot of marrayLayout class

if( !isGeneric("maNsc<-") )
      setGeneric("maNsc<-", function(object, value)
               standardGeneric("maNsc<-"))

setReplaceMethod("maNsc", signature("marrayLayout","numeric"),
  	function(object, value) {
     	  slot(object,"maNsc")<- value
	  object
	})

setReplaceMethod("maNsc", signature("marrayRaw","numeric"),
	function(object, value) {
	  maNsc(slot(object,"maLayout"))<- value
	  object
	})

setReplaceMethod("maNsc", signature("marrayNorm","numeric"),
  	function(object, value) {
	  maNsc(slot(object,"maLayout"))<- value
	  object
	})

###########################################################################
# Assignment methods for maNspots slot of marrayLayout class

if( !isGeneric("maNspots<-") )
      setGeneric("maNspots<-", function(object, value)
               standardGeneric("maNspots<-"))

setReplaceMethod("maNspots", signature("marrayLayout","numeric"),
  	function(object, value) {
     	  slot(object,"maNspots")<- value
	  object
	})

setReplaceMethod("maNspots", signature("marrayRaw","numeric"),
	function(object, value) {
	  maNspots(slot(object,"maLayout"))<- value
	  object
	})

setReplaceMethod("maNspots", signature("marrayNorm","numeric"),
  	function(object, value) {
	  maNspots(slot(object,"maLayout"))<- value
	  object
	})

###########################################################################
# Assignment methods for maSub slot of marrayLayout class

if( !isGeneric("maSub<-") )
      setGeneric("maSub<-", function(object, value)
               standardGeneric("maSub<-"))

setReplaceMethod("maSub", signature("marrayLayout","logical"),
             function(object, value) {
               slot(object, "maSub")<-value
               object
             })

setReplaceMethod("maSub", signature("marrayLayout","numeric"),
             function(object, value){
               slot(object, "maSub")<-maNum2Logic(maNspots(object),value)
               object
             })

setReplaceMethod("maSub", signature("marrayRaw"),
             function(object,value) {
                maSub(slot(object, "maLayout")) <- value
                object
             })

setReplaceMethod("maSub", signature("marrayNorm"),
             function(object,value){
                maSub(slot(object, "maLayout")) <- value
                object
             })

###########################################################################
# Assignment methods for maPlate slot of marrayLayout class

if( !isGeneric("maPlate<-") )
      setGeneric("maPlate<-", function(object, value)
               standardGeneric("maPlate<-"))

setReplaceMethod("maPlate", "marrayLayout",
  	function(object, value) {
     	  slot(object,"maPlate")<- factor(value)
	  object
	})

setReplaceMethod("maPlate", "marrayRaw",
	function(object, value) {
	  maPlate(slot(object,"maLayout"))<- factor(value)
	  object
	})

setReplaceMethod("maPlate", "marrayNorm",
  	function(object, value) {
	  maPlate(slot(object,"maLayout"))<- factor(value)
	  object
	})


###########################################################################
# Assignment methods for maControls slot of marrayLayout class

if( !isGeneric("maControls<-") )
      setGeneric("maControls<-", function(object, value)
               standardGeneric("maControls<-"))

setReplaceMethod("maControls", "marrayLayout",
  function(object, value) {
     slot(object,"maControls")<- factor(value)
     object
  })

setReplaceMethod("maControls", "marrayRaw",
  function(object, value) {
     maControls(slot(object,"maLayout"))<- factor(value)
     object
  })

setReplaceMethod("maControls", "marrayNorm",
  function(object, value) {
     maControls(slot(object,"maLayout")) <- factor(value)
     object
  })


###########################################################################
# Assignment methods for maLayout slot of marrayRaw, marrayNorm etc. classes

if( !isGeneric("maLayout<-") )
      setGeneric("maLayout<-", function(object, value)
               standardGeneric("maLayout<-"))

setReplaceMethod("maLayout", signature("marrayRaw", "marrayLayout"),
  function(object, value) {
     slot(object,"maLayout")<- value
     object
  })

setReplaceMethod("maLayout", signature("marrayNorm", "marrayLayout"),
  function(object, value) {
     slot(object,"maLayout")<- value
     object
  })

###########################################################################
# Assignment methods for maGnames slot of marrayRaw, marrayNorm

if( !isGeneric("maGnames<-") )
      setGeneric("maGnames<-", function(object, value)
               standardGeneric("maGnames<-"))

setReplaceMethod("maGnames", signature("marrayRaw", "marrayInfo"),
  function(object, value) {
     slot(object,"maGnames")<- value
     object
  })

setReplaceMethod("maGnames", signature("marrayNorm", "marrayInfo"),
  function(object, value) {
     slot(object,"maGnames")<- value
     object
  })

###########################################################################
# Assignment methods for maTargets slot of marrayRaw, marrayNorm

if( !isGeneric("maTargets<-") )
      setGeneric("maTargets<-", function(object, value)
               standardGeneric("maTargets<-"))

setReplaceMethod("maTargets", signature("marrayRaw", "marrayInfo"),
  function(object, value) {
     slot(object,"maTargets")<- value
     object
  })

setReplaceMethod("maTargets", signature("marrayNorm", "marrayInfo"),
  function(object, value) {
     slot(object,"maTargets")<- value
     object
  })


###########################################################################
# Assignment methods for maW slot of marrayRaw and marrayNorm classes

if( !isGeneric("maW<-") )
      setGeneric("maW<-", function(object, value)
               standardGeneric("maW<-"))

setReplaceMethod("maW", signature("marrayRaw", "matrix"),
  function(object, value) {
     slot(object,"maW")<- value
     object
  })

setReplaceMethod("maW", signature("marrayNorm", "matrix"),
  function(object, value) {
     slot(object,"maW")<- value
     object
  })

###########################################################################
# Assignment methods for maGf slot of marrayRaw class

if( !isGeneric("maGf<-") )
      setGeneric("maGf<-", function(object, value)
               standardGeneric("maGf<-"))

setReplaceMethod("maGf", signature("marrayRaw", "matrix"),
  function(object, value) {
     slot(object,"maGf")<- value
     object
  })

###########################################################################
# Assignment methods for maRf slot of marrayRaw class

if( !isGeneric("maRf<-") )
      setGeneric("maRf<-", function(object, value)
               standardGeneric("maRf<-"))

setReplaceMethod("maRf", signature("marrayRaw", "matrix"),
  function(object, value) {
     slot(object,"maRf")<- value
     object
  })

###########################################################################
# Assignment methods for maGb slot of marrayRaw class

if( !isGeneric("maGb<-") )
      setGeneric("maGb<-", function(object, value)
               standardGeneric("maGb<-"))

setReplaceMethod("maGb", signature("marrayRaw", "matrix"),
  function(object, value) {
     slot(object,"maGb")<- value
     object
  })

setReplaceMethod("maGb", signature("marrayRaw", "NULL"),
  function(object, value) {
     slot(object,"maGb")<- value
     object
  })

###########################################################################
# Assignment methods for maRb slot of marrayRaw class

if( !isGeneric("maRb<-") )
      setGeneric("maRb<-", function(object, value)
               standardGeneric("maRb<-"))

setReplaceMethod("maRb", signature("marrayRaw", "matrix"),
  function(object, value) {
     slot(object,"maRb")<- value
     object
  })

setReplaceMethod("maRb", signature("marrayRaw", "NULL"),
  function(object, value) {
     slot(object,"maRb")<- value
     object
  })

###########################################################################
# Assignment methods for maA slot of marrayNorm class

if( !isGeneric("maA<-") )
      setGeneric("maA<-", function(object, value)
               standardGeneric("maA<-"))

setReplaceMethod("maA", signature("marrayNorm", "matrix"),
  function(object, value) {
     slot(object,"maA")<- value
     object
  })
###########################################################################
# Assignment methods for maM slot of marrayNorm class

if( !isGeneric("maM<-") )
      setGeneric("maM<-", function(object, value)
               standardGeneric("maM<-"))

setReplaceMethod("maM", signature("marrayNorm", "matrix"),
  function(object, value) {
     slot(object,"maM")<- value
     object
  })

###########################################################################
# Assignment methods for maMloc slot of marrayNorm class

if( !isGeneric("maMloc<-") )
      setGeneric("maMloc<-", function(object, value)
               standardGeneric("maMloc<-"))

setReplaceMethod("maMloc", signature("marrayNorm", "matrix"),
  function(object, value) {
     slot(object,"maMloc")<- value
     object
  })

###########################################################################
# Assignment methods for maMscale slot of marrayNorm class

if( !isGeneric("maMscale<-") )
      setGeneric("maMscale<-", function(object, value)
               standardGeneric("maMscale<-"))

setReplaceMethod("maMscale", signature("marrayNorm", "matrix"),
  function(object, value) {
     slot(object,"maMscale")<- value
     object
  })

###########################################################################
# Assignment methods for maNotes slot of various classes

if( !isGeneric("maNotes<-") )
      setGeneric("maNotes<-", function(object, value)
               standardGeneric("maNotes<-"))

setReplaceMethod("maNotes", signature("marrayLayout", "character"),
  function(object, value) {
     slot(object,"maNotes")<- value
     object
  })

setReplaceMethod("maNotes", signature("marrayRaw", "character"),
  function(object, value) {
     slot(object,"maNotes")<- value
     object
  })

setReplaceMethod("maNotes", signature("marrayNorm", "character"),
  function(object, value) {
     slot(object,"maNotes")<- value
     object
  })

setReplaceMethod("maNotes", signature("marrayInfo", "character"),
  function(object, value) {
     slot(object,"maNotes")<- value
     object
  })

###########################################################################
# maNorm.R
#
# Functions for location and scale normalization for two-color cDNA microarrays
#  source("~/Projects/madman/Rpacks/marray/R/maNorm.R")
###########################################################################

############################################################################
# maNormMain
# Main within-slide location and scale normalization function

maNormMain<-function(mbatch,
                     f.loc=list(maNormLoess()),
                     f.scale=NULL,
                     a.loc=maCompNormEq(),
                     a.scale=maCompNormEq(),
                     Mloc=TRUE, Mscale=TRUE, echo=FALSE)
{

    if (is(mbatch, "marrayRaw"))
    {
        mnorm<-as(mbatch, "marrayNorm")
        M<-Ml<-Ms<-NULL
    }
    if (is(mbatch, "marrayNorm"))
        mnorm<-mbatch

  slot(mnorm, "maNormCall") <- match.call()

  if(length(f.loc)>0)
    M<-Ml<-NULL
  if(length(f.scale)>0)
    M<-Ms<-NULL

  for(i in 1:ncol(maM(mbatch)))
  {

    if(echo)
      cat(paste("Normalizing array ", i, ".\n", sep=""))
    m<-mbatch[,i]
    M1<-M2<-NULL
    Mnorm<-maM(m)

    # Location
    if(length(f.loc)>0)
    {
      for(func in f.loc)
        M1<-cbind(M1, func(m))

      if(length(f.loc)>1)
          M1 <- rowSums(M1*a.loc(maA(m),length(f.loc)))
      Ml<-cbind(Ml,M1)
      Mnorm<-(Mnorm - M1)
    }


    # Scale
    if(length(f.scale)>0)
    {
      # Scale computed for location normalized data
      m<-mnorm[,i]
      slot(m, "maM")<-Mnorm
      for(func in f.scale)
        M2<-cbind(M2, func(m))

      if(length(f.scale)>1)
          M2 <- rowSums(M2*a.scale(maA(m),length(f.scale)))
      Ms<-cbind(Ms,M2)
      Mnorm<-Mnorm/M2
    }

    M<-cbind(M, Mnorm)
  }

  slot(mnorm, "maM")<-M
  if(length(f.loc)>0 & Mloc)
    slot(mnorm, "maMloc")<-Ml
  if(length(f.scale)>0 & Mscale)
    slot(mnorm, "maMscale")<-Ms

  return(mnorm)
}

############################################################################
# maNorm
# Wrapper function around maNormMain for simple normalization procedures
#
# From Gordon's Mar 17 E-mail
# loess(y~x,span=0.4,degree=1,family="symmetric",control=loess.control(trace.hat="ap
# proximate",iterations=5,surface="direct"))
#

maNorm<-function(mbatch,
	norm=c("printTipLoess", "none", "median", "loess", "twoD", "scalePrintTipMAD"),
	subset=TRUE, span=0.4, Mloc=TRUE, Mscale=TRUE, echo=FALSE, ...)
{
  ## Set normalization defaults
  opt <- list(...)

  ## norm <- unlist(strsplit(norm, split=""))[1]
  norm.method <- match.arg(norm)
  if(echo)
    cat(paste("Normalization method: ", norm.method, ".\n", sep=""))

  switch(norm.method,
         none = maNormMain(mbatch, f.loc=NULL,
           Mloc=Mloc, Mscale=Mscale, echo=echo),

         median = maNormMain(mbatch,
           f.loc=list(maNormMed(x=NULL, y="maM", subset=subset)),
           Mloc=Mloc, Mscale=Mscale, echo=echo),

         loess = maNormMain(mbatch,
           f.loc=list(maNormLoess(x="maA", y="maM", z=NULL, w=NULL,
             subset=subset, span=span, ...)),
           Mloc=Mloc, Mscale=Mscale, echo=echo),

         twoD = maNormMain(mbatch,
           f.loc=list(maNorm2D(x = "maSpotRow", y = "maSpotCol",
             z = "maM", g = "maPrintTip",
             w = NULL, subset = subset, span = span, ...)),
           Mloc=Mloc, Mscale=Mscale, echo=echo),

         printTipLoess = maNormMain(mbatch,
           f.loc=list(maNormLoess(x="maA", y="maM", z="maPrintTip",
             w=NULL, subset=subset, span=span, ...)),
           Mloc=Mloc, Mscale=Mscale, echo=echo),

         scalePrintTipMAD = maNormMain(mbatch,
           f.loc=list(maNormLoess(x="maA", y="maM", z="maPrintTip",
             w=NULL, subset=subset, span=span, ...)),
           f.scale=list(maNormMAD(x="maPrintTip", y="maM", geo=TRUE,
             subset=subset)),
           Mloc=Mloc, Mscale=Mscale, echo=echo))
}

############################################################################
# maNormScale
# Wrapper function around maNormMain for simple scale normalization procedures

maNormScale<-function(mbatch,
		norm=c("globalMAD", "printTipMAD"),
		subset=TRUE, geo=TRUE, Mscale=TRUE, echo=FALSE)
{
  ## norm <- unlist(strsplit(norm, split=""))[1]

  norm.method <- match.arg(norm)
  if(echo)
    cat(paste("Normalization method: ", norm.method, ".\n", sep=""))

  if(norm.method == "globalMAD")
  {
     # Not a very nice way to do between slide norm
     if(!geo)
       mnorm<-maNormMain(mbatch,
          f.loc=NULL,
	  f.scale=list(maNormMAD(x=NULL, y="maM", geo=geo, subset=subset)),
	  Mscale=Mscale, echo=echo)
     if(geo)
     {
       mnorm<-maNormMain(mbatch,
         f.loc=NULL,
	 f.scale=list(maNormMAD(x=NULL, y="maM", geo=FALSE, subset=subset)),
	 Mscale=TRUE, echo=echo)
         #mgeo<-apply(maMscale(mnorm), 1, function(z) exp(mean(log(z))))
         mgeo<- exp(mean(log(maMscale(mnorm)[1,])))

       maM(mnorm)<-maM(mnorm)*mgeo
       if(Mscale)
         maMscale(mnorm)<-maMscale(mnorm)/mgeo
       if(!Mscale)
         maMscale(mnorm)<-NULL
     }
  }

  if(norm.method == "printTipMAD")
     mnorm<-maNormMain(mbatch,
        f.loc=NULL,
	f.scale=list(maNormMAD(x="maPrintTip", y="maM", geo=geo,
		subset=subset)),
	Mscale=Mscale, echo=echo)

  return(mnorm)
}

###########################################################################
# Median location normalization
###########################################################################
# maMed
# General function: median of y within values of x

maMed<-function(x, y, subset=TRUE)
{
  subset<-maNum2Logic(length(y), subset)
  yfit<-rep(NA,length(y))
  for(i in unique(x))
    yfit[x==i]<-median(y[(x==i)&subset], na.rm=TRUE)
  return(yfit)
}

#########################
# maNormMed
# Function for objects of class marrayRaw

maNormMed<-function(x=NULL, y="maM", subset=TRUE)
{
  function(m)
  {
    if(is.character(x))
      maMed(x=eval(call(x,m)), y=eval(call(y,m)), subset=subset)
    else
      maMed(x=TRUE, y=eval(call(y,m)), subset=subset)
  }
}

###########################################################################
# Loess location normalization
###########################################################################
# maLoess
# General function: regress y on x within values of z
# In our case, x=NA iff y=NA
## Jean modify Nov 5th, April 9, 2003
## Very ugly fix
## Jean modify Sep 14, 2004
## Sample Loess if the number is too large

maLoess <-
function (x, y, z, w = NULL, subset = TRUE, span = 0.4, ...)
{
  opt <- list(...)
  subset <- maNum2Logic(length(y), subset)
  yfit <- rep(NA, length(y))
  good <- !(is.infinite(x) | is.infinite(y) | is.na(x) | is.na(y))
  if(length(z) == 1) info <- z
  else
    info <- z[subset]
  for (i in unique(info))
    {
      which <- z == i
      ##      cat(i, " :: ", sum(which), "::", unique(z), "\n")
      if((sum(which) == 1) | (length(unique(z)) == 1)) {
        samplesub <- rep(TRUE, length(x))
        samplesub[-sample(1:length(x), min(length(x), 2000))] <- FALSE
        ## cat("in if ", sum(samplesub), " :: ", length(samplesub), "\n")
      }
      else
        samplesub <- TRUE

      defs <- list(degree=1,
                   family="symmetric",
                   control=loess.control(trace.hat="approximate",iterations=5, surface="direct"),
                   subset = samplesub & which & subset & good,
                   span=span,
                   na.action = na.omit,
                   weights=w)
      args <- maDotsMatch(c(defs, opt), formals(args("loess")))
      fit <- loess(y ~ x,  weights = args$w, subset = args$subset,
                   span = args$span, na.action = args$na.action,
                   degree = args$degree, family=args$family,
                   control=args$control)
      ##      gc(TRUE)
      ##      fit <- loess(y ~ x, weights = w, subset = which & subset &
      ##                   good, span = span, na.action = na.omit)
      yfit[which & good] <- predict(fit, x[which & good])
    }
    return(yfit)
}


#########################
# maNormLoess
# Function for objects of class marrayRaw

maNormLoess<-function(x="maA", y="maM", z="maPrintTip", w=NULL, subset=TRUE, span=0.4, ...)
{
  function(m)
  {
    if(is.character(z))
      maLoess(x=eval(call(x,m)), y=eval(call(y,m)), z=eval(call(z,m)), w=w, subset=subset, span=span, ...)
    else
      maLoess(x=eval(call(x,m)), y=eval(call(y,m)), z=TRUE, w=w, subset=subset, span=span, ...)
  }
}

###########################################################################
# 2D spatial location normalization
###########################################################################
# ma2D
# General function: regress z on x*y within values of g

ma2D<-function(x, y, z, g, w=NULL, subset=TRUE, span=0.4, ...)
{
  opt <- list(...)
  subset<-maNum2Logic(length(y), subset)
  zfit<-rep(NA,length(z))
  # Deal with NA, NaN, Inf
  good <- !(is.infinite(z) | is.na(z))

  for(i in unique(g))
  {
    which<-g==i
    defs <- list(degree=1,
                 family="symmetric",
                 control=loess.control(trace.hat="approximate",iterations=5, surface="direct"),
                 subset = which & subset & good,
                 span=span,
                 na.action = na.omit,
                 weights=w)
    args <- maDotsMatch(c(defs, opt), formals(args("loess")))
    fit <- loess(z ~ x * y,  weights = args$w, subset = args$subset,
                 span = args$span, na.action = args$na.action,
                 degree = args$degree, family=args$family,
                 control=args$control)
    ##    gc(TRUE)
    ##    fit<-loess(z ~ x * y, weights=w, subset=subset&which&good, span=span, na.action=na.omit, ...)
    zfit[which]<-predict(fit, data.frame(x=x[which],y=y[which]))
   }
   return(zfit)
}

#########################
# maNorm2D
# Function for objects of class marrayRaw

maNorm2D<-function(x="maSpotRow", y="maSpotCol", z="maM", g="maPrintTip", w=NULL, subset=TRUE, span=0.4, ...)
{
  function(m)
    ma2D(x=eval(call(x,m)), y=eval(call(y,m)), z=eval(call(z,m)), g=eval(call(g,m)), w=w, subset=subset, span=span, ...)
}

###########################################################################
# MAD scale normalization
###########################################################################
# maMAD
# General function: MAD of y within values of x

maMAD<-function(x, y, geo=TRUE, subset=TRUE)
{
  subset<-maNum2Logic(length(y), subset)
  yfit<-rep(NA,length(y))
  m<-rep(NA,length(unique(x)))
  for(i in unique(x))
  {
    m[i]<-mad(y[(x==i)&subset], na.rm=TRUE)
    yfit[x==i]<-m[i]
  }
  if(geo)
   yfit<-yfit/exp(mean(log(m)))
  return(yfit)
}

#########################
# maNormMAD
# Function for objects of class marrayRaw

maNormMAD<-function(x=NULL, y="maM", geo=TRUE, subset=TRUE)
{
  function(m)
  {
    if(is.character(x))
      maMAD(x=eval(call(x,m)), y=eval(call(y,m)), geo=geo, subset=subset)
    else
      maMAD(x=TRUE, y=eval(call(y,m)), geo=geo, subset=subset)
  }
}

###########################################################################
# Weights for composite normalization
###########################################################################
# maCompNormEq
# Equal weights for composite normalization

maCompNormEq<-function()
{
  function(x,n)
  {
    return(matrix(1/n,length(x),n))
  }
}

#########################
# maCompNormA
# Weights for composite intensity dependent normalization

maCompNormA<-function()
{
  function(x,n)
  {
    if(n!=2)
      stop("Can only do composite location normalization for 2 functions in f.loc")
    f<-ecdf(x)
    a<-f(x)
    return(as.matrix(cbind(a,1-a)))
  }
}

###########################################################################

###########################################################################
## Date : October 25, 2002
##
## source("~/Projects/maTools/R/maRankGenes.R")
##
###########################################################################

maSelectGnames <- function(statdata,
                           crit1=50,
                           crit2=crit1,
                           sub=TRUE,
                           selectstat,
                           operate=c("intersect", "union"))
  {
    operate.list <- function(x, operate)
      {
        res <- x[[1]]
        for(i in 2:length(x))
          res <- do.call(operate, list(res, x[[i]]))
        return(res)
      }
    
    gene.ID <- function(x)
      {
        x$gnames
      }

    if(is.integer(sub))
      {
        tmp <- rep(FALSE, nrow(statdata))
        tmp[sub] <- TRUE
        sub <- tmp
      }

    if(missing(selectstat)) selectstat <- 1:ncol(statdata)
    Gnames <- 1:nrow(statdata)
    
    list.id <- list()
    for(i in selectstat)
      {
        switch(data.class(statdata),
                           matrix = headings <- colnames(statdata)[i],
                           data.frame = headings <-  dimnames(statdata)[[2]][i],
                           headings <- colnames(statdata)[i]
                           )
        if(headings == "bayesFun"){
          list.id <- c(list.id,
                       list(gene.ID(stat.gnames(statdata[sub,i],
                                                Gnames[sub], crit=crit1))))
        }
        else
          {
            tmp1 <- gene.ID(stat.gnames(statdata[sub,i], Gnames[sub], crit=crit1)) 
            tmp2 <- gene.ID(stat.gnames(-(statdata[sub,i]), Gnames[sub], crit=crit2)) 
            list.id <- c(list.id, list(c(tmp1, tmp2)))
          }
      }
    finalid <- operate.list(list.id, operate[1])
    return(finalid)
  }



###################################################################
## Select values based on intensities binning
###################################################################
stat.confband.text <-
function (M, A, crit1 = 0.025, crit2 = crit1, nclass = 5)
{
    if (crit1 >= 1)
        crit1 <- crit1/length.na(M)
    if (crit2 >= 1)
        crit2 <- crit2/length.na(M)
    txtA <- (rep(FALSE, length(A)))
    Abin <- quantile.na(A, probs = seq(0, nclass, 1)/nclass)
    for (i in 1:nclass) {
        tmpind <- (Abin[i] <= A) & (A < Abin[i + 1])
        xtmp <- M
        xtmp[!tmpind] <- NA
        n1 <- sum.na(tmpind)
        cutoff <- quantile.na(xtmp, probs = c(crit1, (1 - crit2)))
        vals <- ((xtmp < cutoff[1]) | (xtmp > cutoff[2]))
        txtA[vals] <- TRUE
    }
    res <- c(1:length(txtA))[txtA]
    tmp <- res[rev(order(M[res]))]
    return(tmp)
}


###########################################################################
# Statistics for Microarray Analysis
# Exploratory analysis - Mainly preprocessing.
#
# Date : August 9, 2000
# Last update : May 17, 2001
#
# History:
#   May 17, 2001: Fix to norm.scale.func
#   March, 19: Splitting Rarray in to smaller files.  
#              Including Comments at the start of each function.
#   Nov, 20: Change the argument on plot.mva...it's not usable otherwise.
#            Bug fix ma.func
#   Nov, 13: Ben's Bug fix on stat.ma
#   Nov, 10: Change data structure from matrix to list of matrix.  
#   Sept, 28: Bug fix: ma.func
#
# Authors: Sandrine Dudoit and Yee Hwa (Jean) Yang.
##########################################################################


##########################################################################
#  stat.gnames
#  History:  
#     March 19, 2001:  remove infinite values from the ordering.
#
##########################################################################

stat.gnames<-function(x, gnames, crit=50)
{
    ind <- is.infinite(x)
    x <- x[!ind]
    if (crit < 1) {
        which <- rev(order.na(x, na.last = FALSE))[1:(round(length(x) * 
            crit))]
        if (sum(is.na(x)) > (length(x) - round(length(x) * crit))) 
            warning("NA exists under your selection criteria")
    }
    if (crit >= 1) {
        which <- rev(order.na(x, na.last = FALSE))[1:crit]
        if (sum(is.na(x)) > (length(x) - crit)) 
            warning("NA exists under your selection criteria")
    }
    if (is.matrix(gnames) | is.data.frame(gnames)) 
      {
	gnames <- gnames[!ind, ]
        res <- list(gnames = gnames[which, ], t = x[which])
      }
    if (is.vector(gnames)) 
      {
	gnames <- gnames[!ind]
        res <- list(gnames = gnames[which], t = x[which])
      }
    res
}


##########################################################################
#                                End of file
##########################################################################

## Assuming same Target information
rbind.marrayInfo <- function(..., deparse.level=1)
  {
    data <- list(...)
    newx <- data[[1]]
    for(i in 2:length(data))
      {
        x <- data[[i]]
        if(length(maLabels(x))!=0)
          slot(newx,"maLabels") <- c(maLabels(newx), maLabels(x))
        if(length(maInfo(x))!=0)
          slot(newx,"maInfo")<-  rbind(maInfo(newx), maInfo(x))
        if(length(maNotes(x)) != 0)
          slot(newx,"maNotes")<- paste(maNotes(newx), maNotes(x))
      }
    return(newx)
  }

## Assuming same print-run
## Do not cbind genes
cbind.marrayRaw <- function(..., deparse.level = 1)
  {
    data <- list(...)
    newx <- data[[1]]
    for(x in data[2:length(data)])
      {
        if(length(maGf(x))!=0)
          maGf(newx) <- cbind(maGf(newx), maGf(x))
        if(length(maRf(x))!=0)
          maRf(newx) <- cbind(maRf(newx), maRf(x))
        if(length(maGb(x))!=0)
          maGb(newx) <- cbind(maGb(newx), maGb(x))
        if(length(maRb(x))!=0)
          maRb(newx) <- cbind(maRb(newx), maRb(x))
        if(length(maW(x))!=0)
          maW(newx) <- cbind(maW(newx), maW(x))
        maTargets(newx) <- rbind(maTargets(newx), maTargets(x))
        if(length(maNotes(x)) != 0)
          slot(newx,"maNotes")<- paste(maNotes(newx), maNotes(x))
      }
    return(newx)
  }

cbind.marrayNorm <-  function(..., deparse.level = 1)
  {
    data <- list(...)
    newx <- data[[1]]
    for(x in data[2:length(data)])
      {
        if(length(maM(x))!=0)
          maM(newx) <- cbind(maM(newx), maM(x))
        if(length(maA(x))!=0)
          maA(newx) <- cbind(maA(newx), maA(x))
        if(length(maMloc(x))!=0)
          maMloc(newx) <- cbind(newx@maMloc, x@maMloc)
        if(length(maMscale(x))!=0)
          maMscale(newx) <- cbind(maMscale(newx), maMscale(x))
        if(length(maW(x))!=0)
          maW(newx) <- cbind(maW(newx), maW(x))
        maTargets(newx) <- rbind(maTargets(newx), maTargets(x))
        if(length(maNotes(x)) != 0)
          slot(newx,"maNotes")<- paste(maNotes(newx), maNotes(x))
      }
    return(newx)
  }

###################################################################
## Examples
## library(marrayInput)
## data(swirl)
## findID("fb24a09", swirl, ID="ID")
## findID("geno1", swirl)
###################################################################

findID <-
  function(text,
           Gnames=gnames,
           ID = "Name")
{
  switch(data.class(Gnames),
         ExpressionSet = G <- phenoData(Gnames),
         marrayRaw = G <- maGeneTable(Gnames),
         marrayNorm = G <-maGeneTable(Gnames),
         marrayInfo = G <- maInfo(Gnames),
         G <- Gnames
         )
  ind <- grep(ID, colnames(G))
  y <- as.vector(G[,ind])
  x <- grep(text, y)
  return(x)
}

##########################################################################
# maComp.R
#
# Simple computations for cDNA microarray objects
#
###########################################################################
# Convert integer vector of indices to logical vector

maNum2Logic<-function(n=length(subset), subset=TRUE)
{
  if(is.logical(subset))
    return(subset)
  if(is.numeric(subset))
  {
    which<-rep(FALSE,n)
    which[subset]<-TRUE
    return(which)
  }
}

###########################################################################
# Produce a table of with spot coordinates and gene names

maGeneTable <- function(object)
{
  tmp <- data.frame(
  GR=maGridRow(object),
  GC=maGridCol(object),
  SR=maSpotRow(object),
  SC=maSpotCol(object),
  maInfo(maGnames(object)), row.names=NULL)
  rownames(tmp) <- rownames(maInfo(maGnames(object)))
  colnames(tmp) <- c("Grid.R", "Grid.C", "Spot.R", "Spot.C", names(maInfo(maGnames(object))))
  return(tmp)
}

###########################################################################
# Generate plate IDs from dimensions of grid and spot matrices
## Modified Feb 27, 2004

maCompPlate <- function (x, n = 384)
{
    is.int <- function(z) trunc(z)==z
    totalPlate <- maNspots(x)/n
    if (is.int(totalPlate))
      {
        tmp <- n/(maNgr(x) * maNgc(x))
        factor(rep(rep(1:totalPlate, rep(tmp, totalPlate)), (maNgr(x) *
                                                             maNgc(x))))[maSub(x)]
      }
    else
      {
        totalPlate.c <- ceiling(totalPlate)
        totalPlate.f <- floor(totalPlate)
        tmp1 <- n/(maNgr(x) * maNgc(x))
        tmp2 <- ((totalPlate-totalPlate.f) * n ) / (maNgr(x) * maNgc(x))
        factor(rep(rep(1:totalPlate.c, c(rep(tmp1, totalPlate.f),tmp2)),(maNgr(x)* maNgc(x))))[maSub(x)]
      }
  }



##maCompPlate<-function(x, n=384){
##  totalPlate <- maNspots(x) /n
##  tmp <- n / (maNgr(x) * maNgc(x))
##  factor(rep(rep(1:totalPlate, rep(tmp, totalPlate)), (maNgr(x) * maNgc(x))))
##}



###########################################################################
# Partition Agilent spot index to small spatial regions
# written by Nat Throne
# Date: Sept 14, 2004

##maAgilentPartition <- function(AgmaObj, max.r=15, max.c=15){
##  
##  Col <- maSpotCol(AgmaObj)
##  Row <- maSpotRow(AgmaObj)
##  grid.r <- Row %/% max.r + 1
##  grid.c <- Col %/% max.c + 1
##  spot.c <- Col-floor(Col/max.c)*max.c
##  spot.r <- Row-floor(Row/max.r)*max.r
##  newLayout <- new("marrayLayout", maNsr = max.r, maNsc = max.c, maNgr=max(grid.r), maNgc = max(grid.c),
##                   maNspots  = maNspots(AgmaObj), maSub=TRUE)
##  newmapping <- maCoord2Ind(cbind(grid.r, grid.c, spot.r, spot.c), newLayout)
##  return(newmapping)
##}
## reutrn mapping information to use newdata <- olddata[maAgilentPartition,]

###########################################################################
# Convert spot index to grid and spot matrix coordinates (4 coords)
## Bug fix May 8, 05

maInd2Coord <- function (x, L)
{
    coord<-cbind(maGridRow(L), maGridCol(L), maSpotRow(L), maSpotCol(L))[x,]
    if(is.matrix(coord))
      colnames(coord) <- c("Grid.R", "Grid.C", "Spot.R", "Spot.C")
    else
      names(coord) <- c("Grid.R", "Grid.C", "Spot.R", "Spot.C")
    coord
}

# Convert grid and spot matrix coordinates (4 coords) to spot index.
# Works for subsetted arrays
## BUG : Sept: 2004
## co-ordinates has been re-order.

maCoord2Ind <- function (x, L)
{
  ngr<-maNgr(L)
  ngc<-maNgc(L)
  nsr<-maNsr(L)
  nsc<-maNsc(L)
  n<-maNspots(L)
  ind<-(nsr * nsc)* ((x[,1] - 1) * ngc + (x[,2] - 1)) + (x[,3] - 1) *
        nsc + x[,4]
  ord<-order(ind)
  ind<-ind[ord]
  coord<-x[ord,]
  sub<-(1:n)[maSub(L)]
  ind<-intersect(ind, sub)
  ind<-order(sub)[sub %in% ind]
  ind
}

###########################################################################
# Generate grid and spot matrix coordinates from ranges of rows and
# columns for the grid and spot matrices

maCompCoord<-function(grows, gcols, srows, scols)
{
  ngr <- length(grows)
  ngc <- length(gcols)
  nsr <- length(srows)
  nsc <- length(scols)
  t1 <- rep(grows, rep(nsr * nsc * ngc, ngr))
  t2 <- rep(rep(gcols, rep(nsr * nsc, ngc)), ngr)
  t3 <- rep(rep(srows, rep(nsc, nsr)), ngc * ngr)
  t4 <- rep(scols, nsr * ngc * ngr)
  coord<-cbind(t1, t2, t3, t4)
  colnames(coord) <- c("Grid.R", "Grid.C", "Spot.R", "Spot.C")
  coord
}

# Generate spot index from ranges of rows and columns for the grid
# and spot matrices

maCompInd<-function(grows, gcols, srows, scols, L)
{
  coord<-maCompCoord(grows, gcols, srows, scols)
  maCoord2Ind(coord, L)
}


###########################################################################
##
## Added on April 5, 2004
##
###########################################################################

maCompLayout <- function(mat, ncolumns=4)
  {
    if (dim(mat)[2]==3)
      newmat <- cbind(gr=((mat[,1] - 1) %/% ncolumns) + 1, gc=((mat[,1] -1) %% ncolumns) + 1,
                      sr=mat[,2], sc=mat[,3])
    else
      newmat <- mat

    ngr <- max(newmat[,1]);  ngc <- max(newmat[,2])
    nsr <- max(newmat[,3]);  nsc <- max(newmat[,4])
    nspots <- as.integer(ngr) * as.integer(ngc) * as.integer(nsr) *   as.integer(nsc)

    mlayout <- new("marrayLayout", maNgr = as.integer(ngr),
                   maNgc = as.integer(ngc),
                   maNsr = as.integer(nsr),
                   maNsc = as.integer(nsc),
                   maNspots = nspots)
    maSub(mlayout) <- maCoord2Ind(newmat, mlayout)
    return(mlayout)
  }


###########################################################################
##
##  Move controlCode and maGenControls from marrayTools to marrayClasses
##  May 7, 2003
###########################################################################
controlCode <-
structure(c("Buffer", "Empty", "EMPTY", "AT", "NC", "M200009348", "M200012700",
"M200016219", "M200016205", "M200013499", "M200003425", "M200006376",
"M200001318", "M200004477", "M200001732", "M200006590", "M200000829",
"H200000553", "H200000680", "H200001719", "H200001847", "H200007830",
"H200008181", "H200008484", "H200008489", "H200009216", "H200009498",
"H200011103", "H200019704", "18S", "SSC", "mCNR", "NLG", "GABA",
"pNICE", "M13", "Cot-1", "orward", "everse", "Genomic", "p133",
"Blam", "T7/SP6", "ephrin", "Buffer", "Empty", "Empty", "Negative", "Negative",
"Positive", "Positive", "Positive", "Positive", "Positive", "Positive",
"Positive", "Positive", "Positive", "Positive", "Positive", "Positive",
"Positive", "Positive", "Positive", "Positive", "Positive", "Positive",
"Positive", "Positive", "Positive", "Positive", "Positive", "Positive",
"Positive", "Buffer", "con", "con", "con", "con", "con", "Positive",
"Negative", "Negative", "Negative", "Negative", "Negative", "con",
"con"), .Dim = c(44, 2), .Dimnames = list(c("1", "2", "3", "4",
"5", "6", "7", "8", "9", "10", "11", "12", "13", "14", "15",
"16", "17", "18", "19", "20", "21", "22", "23", "24", "25", "26",
"27", "28", "29", "30", "31", "32", "33", "34", "35", "36", "37",
"38", "39", "40", "41", "42", "43", "44"), c("Pattern", "Name")))

## But Fix March 03, 2003
## Replace controlCode by controlcode (1.0.6)
## Add controlcode = controlCode (1.0.9), add defaults for gpTools and spotTools to work

maGenControls <- function(Gnames, controlcode=controlCode, id="ID")
{
    if (is(Gnames, "marrayInfo"))
    {
      ifelse(is.numeric(id), tmp <- id, tmp <- grep(id, colnames(maInfo(Gnames))))
      if(length(tmp) == 0)
        {
          tmp <- 1
          print("Controls are generated from an arbitaray columns\n")
        }
      ID <- as.vector(as.character(maInfo(Gnames[,tmp])[[1]]))
    }
  else
    {
      if(is.null(dim(Gnames)))
        ID <- as.character(Gnames)
      else
        {
          ifelse(is.numeric(id), tmp <- id, tmp <- grep(id, colnames(Gnames)))
          ID <- as.vector(as.character(Gnames[,tmp]))
        }
    }
  Control <- rep("probes", length(ID))
  for(i in 1:nrow(controlcode))
    {
      position <- grep(as.vector(controlcode[i,"Pattern"]), ID)
      if(length(position) > 0)
        Control[position] <- as.vector(controlcode[i, "Name"])
    }
  return(Control)
}

###########################################################################
# Date : September 19, 2002
# Modify : October, 14, 2002
#
# Runs on R 1.5.1 and above
#
# This file contains wrapper functions for analysis
#
# source("~/Projects/maTools/R/maWrap.R")
#
###########################################################################

widget.TwoSamples <- function(output=TRUE)
  {

    haveTkW <- require("tkWidgets", character.only=TRUE)
    if (!haveTkW)
      stop("This feature requires tkWidgets")

    wlist <- list()
    targetfile <- list(Name="Target File", Value=".txt",
                       toText=function(x) paste(x,collapse = ","),
                       fromText=NULL, canEdit=TRUE, buttonFun = fileBrowser,
                       buttonText = "Browse")
    inputdata <- list(Name="Normalized data", Value="data",
                     toText=function(x) paste(x,collapse = ","),
                     fromText=NULL, canEdit=TRUE, buttonFun = NULL,
                     buttonText = NULL)
    info <- c("Trt", "Ctl", "targetID", "slidesID", "dyesID", "NumID")
    infoValues <- c("Trt", "Ctl", "TargetName", "Slides", "Dyes", "5")

    for(i in 1:length(info))
      {
        test <- list(Name=info[i], Value=infoValues[i],
                     toText=function(x) paste(x,collapse = ","),
                     fromText=NULL, canEdit=TRUE, buttonFun = NULL,
                     buttonText = NULL)
        wlist <- c(wlist, list(test))
      }
    names(wlist) <- info
    widget1 <- list(wList = c(targetfile=list(targetfile), inputdata=list(inputdata), wlist))
    res <- widgetRender(widget1, "Two Samples Adjustment")
    resValues <- values.Widget(res)

    args <- list()
    argsName <- c()
    for(i in 1:length(resValues))
      {
        argsName <- c(argsName, resValues[[i]]$Entry)
        args <-  c(args, list(resValues[[i]]$Value))
      }
    names(args) <- argsName
    args$inputdata <- eval(as.name(args$inputdata))
    names(args)[names(args) == "inputdata"] <- "normdata"
    names(args)[names(args) == "NumID"] <- "RedID"
    outdata <- do.call(maTwoSamples, c(args, list(output=output)))
    return(outdata)
  }

maTwoSamples <- function(targetfile,
                         normdata,
                         Trt,
                         Ctl,
                         targetID="TargetName",
                         slidesID="Slides",
                         dyesID="Dyes",
                         RedID=5,
                         path=".",
                         output=TRUE)
  {
    ## normdata is an marrayNorm objects
     dyeflip <- function(x)
      {
        if(!setequal(as.vector(unique(x[,targetID])), c(Trt, Ctl)))
          res <- NULL
        else
          {
            id <- c(1,2)[as.vector(x[,targetID])==Trt]
            if((as.vector(x[id, dyesID])== RedID))
              res <- 1
            if((as.vector(x[id, dyesID])!= RedID))
              res <- -1
          }
        return(res)
      }

     if(missing(normdata)) normdata <- gpTools()

     target <- maInfo(read.marrayInfo(targetfile))

     dyetype <- unique(target[,"Dyes"])
     if(length(dyetype) !=2)
       stop("Error: More than 2 types of dyes")
     
     if(missing(Trt))    Trt <- as.vector(target[1,targetID])
     if(missing(Ctl))    Ctl <- as.vector(target[2,targetID])
     
     sorttarget <- split(target, target[,slidesID])
     dyeswitch <- unlist(lapply(sorttarget, dyeflip))

     switchM <- switchA <- newtarget <- NULL
     fnames <- colnames(maM(normdata))
     for(i in names(dyeswitch))
       {
         M <- maM(normdata)[,fnames == i]
         A <- maA(normdata)[,fnames == i]
         dyeflip <- dyeswitch[names(dyeswitch)==i]
         switchM <- cbind(switchM, M*dyeflip)
         switchA <- cbind(switchA, A)
         id <- as.vector(target[,slidesID]) == i
         newtarget <- rbind(newtarget, data.frame(t(unlist(apply(target[id,], 2, unique)))))
       }
     rownames(newtarget) <-  as.character(c(1:dim(newtarget)[1]))
     newdata <- normdata
     colnames(switchM) <- colnames(switchA) <- names(dyeswitch)
     slot(newdata, "maM") <- switchM
     slot(newdata, "maA") <- switchA
     
     maTargets(newdata) <- new("marrayInfo",
                               maLabels = as.vector(newtarget[,1]),maInfo = newtarget,
                               maNotes="Generate from maTwoSamples")
     fname <- paste(Trt,"over",Ctl, ".xls", sep="") 
     if(output) write.xls(cbind(maGeneTable(newdata), round(switchM, 5)), file=fname)
     print(paste("Write to file", fname, "\n", sep=" "))
     
     return(newdata)
   }





##################################################################
## END OF FILE
##################################################################

%
% NOTE -- ONLY EDIT THE .Rnw FILE!!!  The .tex file is
% likely to be overwritten.
%
% \VignetteIndexEntry{marrayPlots Overview}
% \VignetteDepends{tools, marray}
% \VignetteKeywords{Expression Analysis, Preprocessing}
% \VignettePackage{marray}
\documentclass[11pt]{article}

\usepackage{amsmath,epsfig,fullpage}
\usepackage[authoryear,round]{natbib} 
\usepackage{hyperref}


\newcommand{\Robject}[1]{{\texttt{#1}}}
\newcommand{\Rfunction}[1]{{\texttt{#1}}}
\newcommand{\Rpackage}[1]{{\textit{#1}}}

\parindent 0in

\bibliographystyle{abbrvnat}

\begin{document}

\title{\bf Bioconductor's marray package: Plotting component}


\author{Yee Hwa Yang$^1$ and Sandrine Dudoit$^2$}

\maketitle
\begin{center}
1. Department of Medicine, University of California, San Francisco,
   {\tt jean@biostat.berkeley.edu}\\
2. Division of Biostatistics, University of California, Berkeley,
   \url{http://www.stat.berkeley.edu/~sandrine}
\end{center}



% library(tools) 
% setwd("C:/MyDoc/Projects/madman/Rpacks/marray/inst/doc")
% Rnwfile<-file.path("C:/MyDoc/Projects/madman/Rpacks/marray/inst/doc","marrayPlots.Rnw")
% options(width=65)
% Sweave(Rnwfile,pdf=TRUE,eps=TRUE,stylepath=TRUE,driver=RweaveLatex())

\tableofcontents



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Overview}

This document provides a detailed discussion of the plotting functions
in {\tt marray} package, which is a packages for diagnostic plots of
two-color spotted microarray data.  This docuement provides functions
for diagnostic plots of microarray spot statistics, such as boxplots,
scatter--plots, and spatial color images. Examination of diagnostic
plots of intensity data is important in order to identify printing,
hybridization, and scanning artifacts which can lead to biased inference
concerning gene expression.  We encourage users to read the shorter {\it
overview} quick start guide on this package given in the {\tt inst/doc}
directory.  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Getting started}

To load the {\tt marray} package in your R session, type {\tt
library(marray)}.  We demonstrate the functionality of this R packages
using gene expression data from the Swirl zebrafish experiment. These
data are included as part of the package, hence you will also need to
install this package. To load the swirl dataset, use {\tt data(swirl)},
and to view a description of the experiments and data, type {\tt ?
swirl}. \\

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Diagnostic plots}

Before proceeding to normalization or any higher--level analysis, it is
instructive to look at diagnostic plots of spot statistics, such as red
and green foreground and background log--intensities, intensity
log--ratio, area, etc. Such plots are useful for the purpose of
identifying printing, hybridization, and scanning artifacts as
demonstrated below. Three main types of functions were defined to
operate on pre-- and post--normalization microarray objects: functions
for boxplots, scatter--plots, and spatial images. The main arguments to
these functions are microarray objects of classes {\tt marrayRaw}, {\tt
marrayNorm} and arguments specifying which spot statistics to display
(e.g. Cy3 and Cy5 background intensities, intensity log--ratios) and
which subset of spots to include in the plots. Default graphical
parameters are chosen for convenience using the function {\tt
maDefaultPar} (e.g. color palette, axis labels, plot title), but the
user has the option to overwrite these parameters at any point. Note
that by default the plots are done for the first array in a batch. To
produce plots for other arrays, subsetting methods may be used. For
example, to produce diagnostic plots for the second array in the batch
of zebrafish arrays {\tt swirl}, the argument {\tt swirl[,2]} should be
passed to the plot functions. \\


To read in the data for the Swirl experiment and generate the plate
IDs (see {\tt marrayClasses} and {\tt marrayInput} for greater
details)

<<eval=TRUE,echo=TRUE>>= 
library(marray)
data(swirl) 
maPlate(swirl)<-maCompPlate(swirl,n=384) 
@

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Spatial plots of spot statistics -- {\tt image}}

 The function {\tt image} creates {\it images} of shades of gray or
colors that correspond to the values of a statistic for each spot on an
array. Details on the arguments of the function are given in {\tt ?
maImage}. The statistic can be the intensity log--ratio $M$, a spot
quality measure (e.g. spot size or shape), or a test statistic. This
function can be used to explore whether there are any spatial effects in
the data, for example, print--tip or cover--slip effects. In addition to
existing color palette functions, such as {\tt rainbow} and {\tt
heat.colors}, a new function {\tt maPalette} was defined to generate
color palettes from user supplied low, middle, and high color values. To
create white--to--green, white--to--red, and green--to--red palettes for
microarray images


<<eval=TRUE,echo=TRUE>>= 
Gcol<- maPalette(low="white", high="green",k=50) 
Rcol<- maPalette(low="white", high="red", k=50) 
RGcol<-maPalette(low="green", high="red", k=50) 
@


Useful diagnostic plots are images of the Cy3 and Cy5 background
intensities; these images may reveal hybridization artifacts such as
scratches on the slides, drops, cover--slip effects etc. The following
commands produce images of the Cy3 and Cy5 background intensities for
the Swirl 93 array (third array in the batch) using white--to--green
and white--to--red color palettes, respectively.

<<maImageGb,fig=TRUE,prefix=FALSE,echo=TRUE,include=FALSE>>=
tmp<-image(swirl[,3], xvar="maGb", subset=TRUE, col=Gcol,contours=FALSE, bar=FALSE) 
@

<<maImageRb,fig=TRUE,prefix=FALSE,echo=TRUE,include=FALSE>>=
tmp<-image(swirl[,3], xvar="maRb", subset=TRUE, col=Rcol, contours=FALSE, bar=FALSE) 
@

Note that the same images can be obtained using the default arguments
of the function by the shorter commands 

%%<<eval=FALSE,echo=TRUE>>=
\begin{verbatim}
> image(swirl[,3], xvar="maGb") 
> image(swirl[,3], xvar="maRb") 
\end{verbatim}
%%@

If {\tt bar=TRUE}, a calibration color bar is displayed to the right
of the images. The {\tt image} function returns the values and
corresponding colors used to produce the color bar, as well as a six
number summary of the spot statistics. The resulting images are shown
in Figure \ref{fig:maImageb}. It can be noted that the Cy3 and Cy5
background intensities are not uniform across the slide and are higher
in the top right corner, perhaps due to cover slip effects or tilt of
the slide during scanning. Such patterns were not as clearly visible
in the individual Cy3 and Cy5 TIFF images. Similar displays of the Cy3
and Cy5 foreground intensities do not exhibit such strong spatial
patterns. For other arrays, such as the Swirl 81 array, background
images revealed the existence of a scratch with very high background
in print--tip--groups (3,2) and (3,3). \\

The {\tt image} function may also be used to generate an image of
the pre--normalization log--ratios $M$ (or any other statistic of
interest), using a green--to--red color palette. Figure
\ref{fig:maImageMraw} displays such an image for the Swirl 93 array,
highlighting only those spots with the highest and lowest 10\%
pre--normalization log--ratios $M$. Other options include displaying
contours and altering graphical parameters such as axis labels and
plot title.  Figure \ref{fig:maImageMraw} suggests the existence of
spatial dye biases in the intensity log--ratio, with higher values in
grid (3,3) and lower values in grid column 1 of the array.

<<maImageMraw1,fig=TRUE,prefix=FALSE,echo=TRUE,include=FALSE>>= 
tmp<-image(swirl[,3], xvar="maM", bar=FALSE, main="Swirl array 93: image of pre--normalization M") 
@

<<maImageMraw2,fig=TRUE,prefix=FALSE,echo=TRUE,include=FALSE>>= 
tmp<-image(swirl[,3], xvar="maM", subset=maTop(maM(swirl[,3]), h=0.10,
l=0.10), col=RGcol, contours=FALSE, bar=FALSE,main="Swirl array 93:
image of pre--normalization M for  10 %  tails")  
@

Note that the {\tt image} function (and other functions {\tt
boxplot} and {\tt plot} to be described next) can be used to plot
other statistics than fluorescence intensities. They can be used to
plot layout parameters such as spot coordinates {\tt maSpotRow},
print--tip--group coordinates {\tt maPrintTip}, or plate IDs {\tt
maPlate} (Figure \ref{fig:maImageLayout}).


<<maImageSpotCol,fig=TRUE,prefix=FALSE,echo=TRUE,include=FALSE>>=
tmp<- image(swirl[,3], xvar="maSpotCol", bar=FALSE) 
@

<<maImagePrintTip,fig=TRUE,prefix=FALSE,echo=TRUE,include=FALSE>>=
tmp<- image(swirl[,3], xvar="maPrintTip", bar=FALSE) 
@

<<maImageControls,fig=TRUE,prefix=FALSE,echo=TRUE,include=FALSE>>=
tmp<- image(swirl[,3], xvar="maControls",col=heat.colors(10),bar=FALSE) 
@

<<maImagePlate,fig=TRUE,prefix=FALSE,echo=TRUE,include=FALSE>>= 
tmp<- image(swirl[,3], xvar="maPlate",bar=FALSE) 
@


\clearpage

\begin{figure} 
\begin{center} 
\begin{tabular}{cc}
\includegraphics[width=3in,height=3in,angle=0]{maImageGb} &
\includegraphics[width=3in,height=3in,angle=0]{maImageRb} \\
(a) & (b) 
\end{tabular} 
\end{center} 
\caption{Images of background intensities for the Swirl 93 array. Panel (a): Cy3 background
intensities using white--to--green color palette. Panel (b): Cy5
background intensities using white--to--red color palette.}
\protect\label{fig:maImageb} 
\end{figure}


\begin{figure} 
\begin{center} 
\begin{tabular}{cc}
\includegraphics[width=3in,height=3in,angle=0]{maImageMraw1} &
\includegraphics[width=3in,height=3in,angle=0]{maImageMraw2} \\
(a) & (b) 
\end{tabular} 
\end{center} 
\caption{Images of the pre--normalization intensity log--ratios $M$ for the Swirl 93 array,
using a green--to--red color palette. Panel (a): All spots are
displayed. Panel (b): only spots with the highest and lowest 10\%
log--ratios are highlighted.}  
\protect\label{fig:maImageMraw}
\end{figure}

\begin{figure}
\begin{center} 
\begin{tabular}{cc}
\includegraphics[width=2in,height=2in,angle=0]{maImageSpotCol} &
\includegraphics[width=2in,height=2in,angle=0]{maImagePrintTip} \\
(a) & (b)\\
\includegraphics[width=2in,height=2in,angle=0]{maImagePlate} &
\includegraphics[width=2in,height=2in,angle=0]{maImageControls} \\
(c) & (d) 
\end{tabular} 
\end{center} 
\caption{Images of layout parameters for the Swirl 93 array. Panel (a): Spot matrix column
coordinate. Panel (b): Print--tip--group. Panel (c): Plate
index. Panel (d): Control status.}   
\protect\label{fig:maImageLayout}
\end{figure}

\clearpage


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Boxplots of spot statistics -- {\tt boxplot}}

 Boxplots of spot statistics by plate, print--tip--group, or slide can
also be useful to identify spot or hybridization artifacts. {\it
Boxplots}, also called {\it box--and--whisker plots}, were first
proposed by Tukey in 1977 as simple graphical summaries of the
distribution of a variable. The summary consists of the median, the
upper and lower quartiles, the range, and, possibly, individual extreme
values. The central box in the plot represents the {\it inter--quartile
range (IQR)}, which is defined as the difference between the 75th
percentile and 25th percentile, i.e., the upper and lower quartiles. The
line in the middle of the box represents the median; a measure of
central location of the data.  Extreme values, greater than 1.5 IQR
above the 75th percentile and less than 1.5 IQR below the 25th
percentile, are typically plotted individually.

The function {\tt boxplot} produces boxplots of microarray spot
statistics for the classes {\tt marrayRaw}, {\tt marrayNorm}.  The
function {\tt boxplot} has three main arguments:
\begin{description} 
\item {{\tt x}:} Microarray object of class {\tt marrayRaw} or {\tt marrayNorm}.
\item {{\tt xvar}:} Name of accessor method for the spot statistic used to
stratify the data, typically a slot name for the microarray layout
object such as {\tt maPlate} or a method such as {\tt maPrintTip}. If
{\tt xvar} is {\tt NULL}, the data are not stratified.  
\item {{\tt yvar}:} Name of accessor method for the spot statistic of
interest, typically a slot name for the microarray object {\tt m}, such
as {\tt maM}.
\end{description}


Figure \ref{fig:maBoxplot1} panel (a) displays boxplots of
pre--normalization log--ratios $M$ for each of the 16 print--tip--groups
for the Swirl 93 array.  This plot was generated by the following
commands
<<maBoxplot1pre,fig=TRUE,prefix=FALSE,echo=TRUE,include=FALSE>>=
boxplot(swirl[,3], xvar="maPrintTip", yvar="maM", main="Swirl array 93: pre--normalization") 
@

The boxplots clearly reveal the need for normalization, since most
log--ratios are negative in spite of the fact that only a small
proportion of genes are expected to be differentially expressed in the
mutant and wild--type zebrafish. As is often the case, this
corresponds to higher signal in the Cy3 channel than in the Cy5
channel even in the absence of differential expression. In addition,
the boxplots show the existence of spatial dye biases in the
log--ratios. In particular, print--tip--group (3,3) clearly stands out
from the remaining ones, as suggested also in the image of Figure
\ref{fig:maImageMraw}. The function {\tt maBoxplot} may also be used
to produce boxplots of spot statistics for all arrays in a batch. Such
plots are useful when assessing the need for between array
normalization, for example, to deal with scale differences among
different arrays. The following command produces a boxplot of the
pre--normalization intensity log--ratios $M$ for each array in the
batch {\tt swirl}.  Figure \ref{fig:maBoxplot2} panel (a) suggest that
different normalizations may be required for different arrays,
including possibly scale normalization.


<<maBoxplot2pre,fig=TRUE,prefix=FALSE,echo=TRUE,include=FALSE>>=
boxplot(swirl, yvar="maM", main="Swirl arrays: pre--normalization") 
@

The function {\tt maNorm} from the {\tt marrayNorm} package can be used
for different types of within-array location normalization.  The
following command normalizes all four arrays in the Swirl experiment
simultaneously. Please refer to the vignette on normalization for more
information.  The following command performs within print-tip group
loesss normalization.

<<eval=TRUE>>= 
swirl.norm <- maNorm(swirl, norm="p")
@

The following commands can be used to produce post--normalization
boxplots of the log--ratios. The plots are shown in panel (b) of
Figures \ref{fig:maBoxplot1} and \ref{fig:maBoxplot2}.

<<maBoxplot1post,fig=TRUE,prefix=FALSE,echo=TRUE,include=FALSE>>=
boxplot(swirl.norm[,3], xvar="maPrintTip", yvar="maM",
	main="Swirl array 93: post--normalization") 
@

<<maBoxplot2post,fig=TRUE,prefix=FALSE,echo=TRUE,include=FALSE>>=
boxplot(swirl.norm, yvar="maM", col="green", main="Swirl arrays: post--normalization") 
@


\clearpage

\begin{figure} 
\begin{center} 
\begin{tabular}{cc}
\includegraphics[width=3in,height=3in,angle=0]{maBoxplot1pre} &
\includegraphics[width=3in,height=3in,angle=0]{maBoxplot1post} \\
(a) & (b) 
\end{tabular} 
\end{center} 
\caption{Boxplots by print--tip--group of the pre-- and post--normalization intensity log--ratios $M$ for the Swirl 93 array. }
\protect\label{fig:maBoxplot1} 
\end{figure}

\begin{figure} 
\begin{center} 
\begin{tabular}{cc}
\includegraphics[width=3in,height=3in,angle=0]{maBoxplot2pre} &
\includegraphics[width=3in,height=3in,angle=0]{maBoxplot2post} \\
 (a) & (b) 
\end{tabular} 
\end{center} 
\caption{Boxplots of the pre--and
post--normalization intensity log--ratios $M$ for the four arrays in
the Swirl experiment. } 
\protect\label{fig:maBoxplot2} 
\end{figure}


\clearpage

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Scatter--plots of spot statistics -- {\tt maPlot} or {\tt plot}}

The function {\tt plot} produces {\it scatter--plots} of microarray spot
statistics for the classes {\tt marrayRaw} and {\tt marrayNorm}. It also
allows the user to highlight and annotate subsets of points on the plot,
and display fitted curves from robust local regression or other
smoothing procedures (see details in {\tt ? maPlot}). The function {\tt
maPlot} has seven main arguments:

\begin{description} 
\item {{\tt x}:} Microarray object of class {\tt marrayRaw} or  {\tt
  marrayNorm}.  
\item {{\tt xvar}:} Name of accessor function for the abscissa spot
statistic, typically a slot name for the microarray object {\tt m},
such as {\tt maA}.
  \item {{\tt yvar}:} Name of accessor function for the
ordinate spot statistic, typically a slot name for the microarray
object {\tt m}, such as {\tt maM}.  
\item {{\tt zvar}:} Name of accessor
method for the spot statistic used to stratify the data, typically a
slot name for the microarray layout object such as {\tt maPlate} or a
method such as {\tt maPrintTip}. If 
{\tt zvar} is {\tt NULL}, the data are not stratified.  

\item {{\tt lines.func}:} Function for computing and plotting smoothed
fits of {\tt yvar} as a function of {\tt xvar}, separately within values
of {\tt zvar}, e.g. {\tt maLoessLines}.  If {\tt lines.func} is {\tt
NULL}, no fitting is performed.  \item {\tt {\tt text.func}:} Function
for highlighting a subset of points, e.g., {\tt maText}. If {\tt
text.func} is {\tt NULL}, no points are highlighted.

\item {{\tt legend.func}:} Function for adding a legend to the plot,
e.g. {\tt maLegendLines}. If {\tt legend.func} is {\tt NULL}, there is
no legend.  
\end{description}

As usual, optional graphical parameters may be supplied and these will
overwrite the default parameters set in the plot functions.  A number
of functions for computing and plotting the fits are provided,
 such as {\tt maLowessLines} and {\tt maLoessLines} for
robust local regression using the R functions {\tt lowess} and {\tt
loess}, respectively (type {\tt ? loess} or {\tt ? lowess} for a brief
description of R functions for robust local regression). Functions are
also provided for highlighting points (e.g. {\tt text}) and adding a
legend to the plot (e.g. {\tt maLegendLines}).\\

{\bf $MA$--plots.} Single--slide expression data are typically
displayed by plotting the log--intensity $\log_2 R$ in the red channel
vs. the log--intensity $\log_2 G$ in the green channel. Such plots
tend to give an unrealistic sense of concordance between the red and
green intensities and can mask interesting features of the data. We
thus recommend plotting the intensity log--ratio $M=\log_2 R/G$
vs. the mean log--intensity $A = \log_2 \sqrt{RG}$. An $MA$--plot
amounts to a $45^o$ counterclockwise rotation of the $(\log_2 G,\log_2
R)$-- coordinate system, followed by scaling of the coordinates. It is
thus another representation of the $(R,G)$ data in terms of the
log--ratios $M$ which directly measure differences between the red and
green channels and are the quantities of interest to most
investigators. We have found $MA$--plots to be more revealing than
their $\log_2 R$ vs.  $\log_2 G$ counterparts in terms of identifying
spot artifacts and for normalization purposes
\citep{Dudoitetal02,Norm,NormNAR}. \\

Figure \ref{fig:maPlot1} panel (a) displays the pre--normalization
$MA$--plots for the Swirl 93 array, with the sixteen lowess fits for
each of the print--tip--groups (using a smoother span $f=0.3$ for the
{\tt lowess} function). The figure was generated with the following
commands

<<maPlot1pre,fig=TRUE,prefix=FALSE,echo=TRUE,include=FALSE>>=
defs<-maDefaultPar(swirl[,3],x="maA",y="maM",z="maPrintTip")

# Function for plotting the legend
legend.func<-do.call("maLegendLines",defs$def.legend)

# Function for performing and plotting lowess fits
lines.func<-do.call("maLowessLines",c(list(TRUE,f=0.3),defs$def.lines))

plot(swirl[,3], xvar="maA", yvar="maM", zvar="maPrintTip",
		      lines.func,
		      text.func=maText(),
		      legend.func,
		      main="Swirl array 93: pre--normalization MA--plot") 
@

<<maPlot1post,fig=TRUE,prefix=FALSE,echo=TRUE,include=FALSE>>=
plot(swirl.norm[,3], xvar="maA", yvar="maM", zvar="maPrintTip",
		      lines.func,
		      text.func=maText(),
		      legend.func,
		      main="Swirl array 93: post--normalization MA--plot") 
@

The same plots can be obtain using the default arguments of the
function by the commands 

\begin{verbatim}
> plot(swirl[,3])
> plot(swirl.norm[,3], legend.func=NULL) 
\begin{verbatim}

To highlight, say, the spots with the highest and lowest 5\%
log--ratios using purple points, or using red symbol {\tt a} use the
following commands 

\begin{verbatim}
> points(swirl.norm[,3], subset=maTop(maM(swirl.norm[,3]),h=0.05,l=0.05),
pch=19, col="purple")
> text(swirl.norm[,3], subset=maTop(maM(swirl.norm[,3]),h=0.05,l=0.05),
labels="a", col="red")
\begin{verbatim}

\begin{figure}
\begin{center}
\begin{tabular}{cc}
\includegraphics[width=3in,height=3in,angle=0]{maPlot1pre} &
\includegraphics[width=3in,height=3in,angle=0]{maPlot1post} \\ 
(a) & (b)
\end{tabular}
\end{center}
\caption{Pre-- and post--normalization $MA$--plot for the Swirl 93
  array, with the lowess fits for individual
  print--tip--groups. Different colors are used to represent lowess
  curves for print--tips from different rows, and different line types
  are used to represent lowess curves for print--tips from different
  columns. } 
\protect\label{fig:maPlot1}
\end{figure}


Figure \ref{fig:maPlot1} illustrates the non--linear dependence of the
log--ratio $M$ on the overall spot intensity $A$ and thus suggests that
an intensity or $A$--dependent normalization method is preferable to a
global one (e.g. median normalization). Also, the lowess fits vary among
print--tip--groups, again revealing the existence of spatial dye
biases. Figure \ref{fig:maPlot1} panel (b) displays the $MA$--plot after
within--print--tip--group loess location normalization.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Wrapper functions for basic sets of diagnostic plots -- {\tt maQualityPlots}}
The following command in another package {\tt arrayQuality} will
generate qualitative diagnostic plots for each arrays in the {\tt
marrayRaw} object and by default, saved it as different png files in the
working directory.  More details of this can be found in the package
{\tt arrayQuality}.


\begin{verbatim}
library(arrayQuality)
maQualityPlots(swirl)
\end{verbatim}



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
{\bf Note: Sweave.} This document was generated using the \Rfunction{Sweave}
function from the R \Rpackage{tools} package. The source file is in the
\Rfunction{/inst/doc} directory of the package \Rpackage{marray}.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\bibliography{marrayPacks}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\end{document}

% NOTE -- ONLY EDIT THE .Rnw FILE!!!  The .tex file is
% likely to be overwritten.
%
% \VignetteIndexEntry{marray Normalization}
% \VignetteDepends{marray}
% \VignetteKeywords{Expression Analysis, Preprocessing}
% \VignettePackage{marray}

\documentclass[11pt]{article}

\usepackage{amsmath,epsfig,fullpage,hyperref}
\parindent 0in

\newcommand{\Robject}[1]{{\texttt{#1}}}
\newcommand{\Rfunction}[1]{{\texttt{#1}}}
\newcommand{\Rpackage}[1]{{\textit{#1}}}

\parindent 0in

\bibliographystyle{abbrvnat}

\begin{document}

\title{\bf Normalization: Bioconductor's marray package}


\author{Yee Hwa Yang$^1$ and Sandrine Dudoit$^2$}
\maketitle
\begin{center}
1. Department of Medicine, University of California, San Francisco,
   {\tt jean@biostat.berkeley.edu}\\
2. Division of Biostatistics, University of California, Berkeley,
   \url{http://www.stat.berkeley.edu/~sandrine}
\end{center}


% library(tools) 
% setwd("C:/MyDoc/Projects/madman/Rpacks/marray/inst/doc")
% Rnwfile<-file.path("C:/MyDoc/Projects/madman/Rpacks/marray/inst/doc","marrayNorm.Rnw")
% Sweave(Rnwfile,pdf=TRUE,eps=TRUE,stylepath=TRUE,driver=RweaveLatex())


\tableofcontents


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Overview}

This document provides a tutorial for the normalization component of the
{\tt marray} package.  Greater details on the packages are given in
\cite{Dudoit&Yang02}.  This package implements robust adaptive location
and scale normalization procedures, which correct for different types of
dye biases (e.g. intensity, spatial, plate biases) and allow the use of
control sequences spotted onto the array and possibly spiked into the
mRNA samples. Normalization is needed to ensure that observed
differences in intensities are indeed due to differential expression and
not experimental artifacts; fluorescence intensities should therefore be
normalized before any analysis which involves comparisons among genes
within or between arrays.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Getting started}

To load the {\tt marray} package in your R session, type {\tt
library(marray)}.  We demonstrate the functionality of this R packages
using gene expression data from the Swirl zebrafish experiment. These
data are included as part of the package, hence you will also need to
install this package. To load the swirl dataset, use {\tt data(swirl)},
and to view a description of the experiments and data, type {\tt ?
swirl}. 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Normalization using robust local regression}

The purpose of normalization is to identify and remove sources of
systematic variation, other than differential expression, in the
measured fluorescence intensities (e.g.  different labeling efficiencies
and scanning properties of the Cy3 and Cy5 dyes; different scanning
parameters, such as PMT settings; print--tip, spatial, or plate
effects). It is necessary to normalize the fluorescence intensities
before any analysis which involves comparing expression levels within or
between slides (e.g. classification, multiple testing), in order to
ensure that differences in intensities are indeed due to differential
expression and not experimental artifacts. The need for normalization
can be seen most clearly in self--self experiments, in which two
identical mRNA samples are labeled with different dyes and hybridized to
the same slide \cite{Dudoitetal02}. Although there is no differential
expression and one expects the red and green intensities to be equal,
the red intensities often tend to be lower than the green
intensities. Furthermore, the imbalance in the red and green intensities
is usually not constant across the spots within and between arrays, and
can vary according to overall spot intensity, location on the array,
plate origin, and possibly other variables. \\


{\bf Location normalization.} We have developed location normalization
methods which correct for intensity, spatial, and other dye biases using
{\it robust locally weighted regression}
\cite{Cleveland79,Norm,NormNAR}. Local regression is a {\it smoothing}
method for summarizing multivariate data using general curves and
surfaces. The smoothing is achieved by fitting a linear or quadratic
function of the predictor variables {\it locally} to the data, in a
fashion that is analogous to computing a moving average. In the lowess
and loess procedures, polynomials are fitted locally using iterated
weighted least squares. {\it Robust} fitting guards against deviant
points distorting the smoothed points. In the context of microarray
experiments, robust local regression allows us to capture the
non--linear dependence of the intensity log--ratio $M=\log_2 R/G$ on the
overall intensity $A = \log_2 \sqrt{RG}$, while ensuring that the
computed normalization values are not driven by a small number of
differentially expressed genes with extreme log--ratios. For details on
the R {\tt loess} function ({\tt modreg} package), type {\tt ? loess}.\\

{\bf Scale normalization.} For scale normalization, a robust estimate of
scale, such as the {\it median absolute deviation (MAD)}, may be used
\cite{Norm,NormNAR}. For a collection of numbers $x_1, \ldots, x_n$,
the MAD is the median of their absolute deviations from the median $m =
{\rm median}\{x_1, \ldots, x_n\}$

$$ MAD = {\rm median}\{ |x_1 - m|,  \ldots, |x_n - m|\}.$$
The R function for MAD is {\tt mad}.\\

Location and scale normalized intensity log--ratios $M$ are given by

$$ M \leftarrow \frac{M - l}{s},$$

where $l$ and $s$ denote the location and scale normalization values,
respectively. The location value $l$ can be obtained, for example, by
robust local regression of $M$ on $A$ within print--tip--group. The
scale value $s$ could be the MAD, within print--tip--group, of location
normalized log--ratios.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Normalization functions}

\subsection{Simple normalization function {\tt maNorm}}
A simple wrapper function {\tt maNorm} is provided for users interested
in applying a standard set of normalization procedures using default
parameters. This function returns an object of class {\tt marrayNorm} and
has seven arguments
\begin{description}
\item
{{\tt mbatch}:} Object of class {\tt marrayRaw}, containing intensity
data for the batch of arrays to be normalized. An object of class {\tt
  marray} may also be passed if normalization is performed in several
steps. 
\item
{{\tt norm}:} Character string specifying the normalization
        procedure. Six normalization procedures are available with this
        function: {\tt none}, for no normalization; {\tt median},  for
        global median location normalization; {\tt loess} for global
        intensity or $A$--dependent location normalization 
        using the {\tt loess} function; {\tt twoD}, for 2D spatial
        location normalization using the {\tt loess} function; {\tt
        printTipLoess}, for within--print--tip--group intensity
        dependent location normalization using the {\tt loess} function;
        and {\tt scalePrintTipMAD}, for within--print--tip--group
        intensity dependent location normalization followed by
        within--print--tip--group scale normalization using the median
        absolute deviation. This argument can be specified using the
        first letter of each method. 
\item
{{\tt subset}:} 
        A logical or numeric vector indicating the subset of points used
        to compute the normalization values. 
\item
{{\tt span}:} The argument span which controls the degree of smoothing
in the loess function. Only used for {\tt loess}, {\tt twoD}, {\tt
  printTipLoess}, and {\tt scalePrintTipMAD} options. 
\item
{{\tt Mloc}:} If {\tt TRUE}, the location normalization values are
         stored in the slot {\tt maMloc} of the object of class {\tt
         marray} returned by the function, if {\tt FALSE}, these values
         are not retained. This option allows to save memory for large datasets.
\item
{{\tt Mscale}:} If {\tt TRUE}, the scale normalization values are stored
         in the slot {\tt maMscale} of the object of class {\tt marray}
         returned by the function, if {\tt FALSE}, these values are not
         retained.
\item
{{\tt echo}:} If {\tt TRUE}, the index of the array currently being normalized is printed.
\end{description}

\subsection{Simple scale normalization function {\tt maNormScale}}
A simple wrapper function {\tt maNormScale} is provided for users
interested in applying a standard set of scale normalization procedures
using default parameters. This function returns an object of class {\tt
marrayNorm} has six arguments

\begin{description}
\item
{{\tt mbatch}:} Object of class {\tt marrayRaw}, containing intensity
data for the batch of arrays to be normalized. An object of class {\tt
  marray} may also be passed if  normalization is performed in several
steps.  
\item
{{\tt norm}:} Character string specifying the normalization
procedure. Two normalization procedures are currently available for this
function: {\tt globalMAD} for global scale normalization using the
median absolute deviation; {\tt printTipMAD} for
within--print--tip--group scale normalization using the median absolute
deviation. This argument can be specified using the first letter of each
method. 
\item
{{\tt subset}:} 
        A logical or numeric vector indicating the subset of points used
        to compute the normalization values. 
\item
{{\tt geo}:} If {\tt TRUE}, the MAD of each group is divided by the
geometric mean of the MADs across groups \cite{NormNAR}. This allows
observations to retain their original units. 
\item
{{\tt Mscale}:} If {\tt TRUE}, the scale normalization values are stored
         in the slot {\tt maMscale} of the object of class {\tt marray}
         returned by the function, if {\tt FALSE}, these values are 
         not retained.
\item
{{\tt echo}:} If {\tt TRUE}, the index of the array currently being
normalized is printed. 

\end{description}

The {\tt globalMad} option, with {\tt geo=TRUE}, allows between slide
scale normalization. 

\subsection{General normalization function {\tt maNormMain}}
Note: We recommand users using {\tt maNorm} and {\tt maNormScale} rather
than this function for performing standard set of normalization
procedures.

This is the main internal function for location and scale normalization
of cDNA microarray data is {\tt maNormMain}; it has eight arguments (see
also {\tt ? maNormMain}):
\begin{description}
\item
{{\tt mbatch}:} Object of class {\tt marrayRaw}, containing intensity
data for the batch of arrays to be normalized. An object of class {\tt
  marrayNorm} may also be passed if normalization is performed in
several steps. 
\item
{{\tt f.loc}:} A list of location normalization functions, e.g., {\tt
  maNormLoess}, {\tt maNormMed}, or {\tt maNorm2D}. 
\item
{{\tt f.scale}:} A list of scale normalization functions, e.g, {\tt maNormMAD}.
\item
{{\tt a.loc}:} For composite normalization, a function for computing the
weights used in combining several location normalization functions,
e.g., {\tt maCompNormA}. 
\item
{{\tt a.scale}:} For composite normalization, a function for computing
the weights used in combining several scale normalization functions. 
\item
{{\tt Mloc}:} If {\tt TRUE}, the location normalization values are
         stored in the slot {\tt maMloc} of the object of class {\tt
         marray} returned by the function, if {\tt FALSE}, these values
         are not retained. This option allows to save memory for large datasets.
\item
{{\tt Mscale}:} If {\tt TRUE}, the scale normalization values are stored
         in the slot {\tt maMscale} of the object of class {\tt marray}
         returned by the function, if {\tt FALSE}, these values are 
         not retained.
\item
{{\tt echo}:} If {\tt TRUE}, the index of the array currently being
normalized is printed. 
\end{description}

 Normalization is performed simultaneously for each array in the batch
 using the location and scale normalization procedures specified by the
 lists of functions {\tt f.loc} and {\tt f.scale}. Typically, only one
 function is given in each list, otherwise composite normalization is
 performed using the weights given by {\tt a.loc} and {\tt a.scale}
 \cite{NormNAR}. The {\tt maNormMain} function returns objects of class
 {\tt marrayNorm}.

The {\tt marray} package contains internal functions for median ({\tt
maNormMed}), intensity or $A$--dependent ({\tt maNormLoess}), and 2D
spatial ({\tt maNorm2D}) location normalization. The R robust local
regression function {\tt loess} is used for intensity dependent and 2D
spatial normalization. The package also contains a function for scale
normalization using the median absolute deviation (MAD) ({\tt
maNormMAD}). These functions have arguments for specifying which spots
to use in the normalization and for controlling the local regression,
when applicable. The functions allow normalization to be done separately
within values of a layout parameter, such as plate or print--tip--group,
and using different subsets of probe sequences (e.g. dilution series of
control probe sequences).


\section{Normalization of Swirl zebrafish microarray data}

To read in the data for the Swirl experiment and generate the plate IDs

<<eval=TRUE,echo=TRUE>>=
library("marray", verbose=FALSE)
data(swirl)
maPlate(swirl)<-maCompPlate(swirl,n=384)
@

The pre--normalization $MA$--plot for the Swirl 93 array in Figure
\ref{fig:maPlot1} illustrates the non--linear dependence of the
log--ratio $M$ on the overall spot intensity $A$ and the existence of
spatial dye biases. Only a small proportion of the spots are expected to
vary in intensity between the two channels. We thus perform
within--print--tip--group loess location normalization using all $8,448$
probes on the array. \\ 

\subsection{Using simple function {\tt maNorm}} 


The following command normalizes all four arrays in the Swirl experiment
simultaneously.  The simple wrapper function could be used to perform
most of the standard normalizations procedures.

<<eval=TRUE>>=
swirl.norm <- maNorm(swirl, norm="p")
summary(swirl.norm)
@

For global median normalization
\begin{verbatim}
> swirl.normm <- maNorm(swirl, norm="median")
\end{verbatim}

\subsection{Using simple function {\tt maNormScale}} 

This simple wrapper function may be used to perform scale normalization
separately from location normalization. The following examples do not
represent a recommended analysis but are simply used for demonstrating
the software functionality. Within--print--tip--group intensity
dependent normalization followed by within--print--tip--group scale
normalization using the median absolute deviation, could be performed in
one step by 

\begin{verbatim}
> swirl.norms <- maNorm(swirl, norm="s")
\end{verbatim}

or sequentially by

<<eval=FALSE,echo=TRUE>>=
swirl.norm1 <- maNorm(swirl, norm="p")
swirl.norm2 <- maNormScale(swirl.norm1, norm="p")
@

For between slide scale normalization using MAD scaled by the geometric
mean of MAD across slides \cite{Norm,NormNAR} 

\begin{verbatim}
swirl.normg <- maNormScale(swirl.norm, norm="g")
\end{verbatim}

\subsection{Using main function {\tt maNormMain}} 

The following command normalizes all four arrays in the Swirl experiment simultaneously

\begin{verbatim}
> swirl.norm <- maNormMain(swirl, 
	f.loc = list(maNormLoess(x = "maA", y = "maM", z = "maPrintTip", 
		w = NULL, subset = TRUE, span = 0.4)),
	f.scale = NULL,
    	a.loc = maCompNormEq(), 
	a.scale = maCompNormEq(),
	Mloc = TRUE, Mscale = TRUE, echo = FALSE)
\end{verbatim}

This is the default normalization procedure in {\tt maNormMain}, thus
the same results could be obtained by calling  

\begin{verbatim}
> swirl.norm <- maNormMain(swirl)
\end{verbatim}


To see the effect of within--print--tip--group location normalization,
compare the pre--and post--normalization boxplots and $MA$--plots in
Figures \ref{fig:maBoxplot1}, \ref{fig:maBoxplot2}, and
\ref{fig:maPlot1}. Normalized log--ratios $M$ are now evenly distributed
about about zero across the range of intensities $A$ for each
print--tip--group. Furthermore, the non--linear location normalization
seems to have eliminated, to some extent, the scale differences among
print--tip--groups and arrays. \\ 

\subsection{Plots}

The plots were produced using the following commands:

<<maBoxplot1pre,fig=TRUE,prefix=FALSE,echo=TRUE,include=FALSE>>=
boxplot(swirl[,3], xvar="maPrintTip", yvar="maM", main="Swirl array 93: pre--normalization")
@

<<maBoxplot2pre,fig=TRUE,prefix=FALSE,echo=TRUE,include=FALSE>>=
boxplot(swirl, yvar="maM", main="Swirl arrays: pre--normalization")
@

<<maBoxplot1post,fig=TRUE,prefix=FALSE,echo=TRUE,include=FALSE>>=
boxplot(swirl.norm[,3], xvar="maPrintTip", yvar="maM", main="Swirl array 93: post--normalization")
@

<<maBoxplot2post,fig=TRUE,prefix=FALSE,echo=TRUE,include=FALSE>>=
boxplot(swirl.norm, yvar="maM", main="Swirl arrays: post--normalization")
@

<<maPlot1pre,fig=TRUE,prefix=FALSE,echo=TRUE,include=FALSE>>=
plot(swirl[,3], main="Swirl array 93: pre--normalization MA--plot")
@

<<maPlot1post,fig=TRUE,prefix=FALSE,echo=TRUE,include=FALSE>>=
plot(swirl.norm[,3], main="Swirl array 93: post--normalization MA--plot")
@

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\begin{figure}
\begin{center}
\begin{tabular}{cc}
\includegraphics[width=3in,height=3in,angle=0]{maBoxplot1pre}, &
\includegraphics[width=3in,height=3in,angle=0]{maBoxplot1post} \\ 
(a) & (b)
\end{tabular}
\end{center}
\caption{Boxplots by print--tip--group of the pre-- and
  post--normalization intensity log-ratios $M$ for the Swirl 93
  array. }
\protect\label{fig:maBoxplot1}
\end{figure}


\begin{figure}
\begin{center}
\begin{tabular}{cc}
\includegraphics[width=3in,height=3in,angle=0]{maBoxplot2pre}, &
\includegraphics[width=3in,height=3in,angle=0]{maBoxplot2post} \\ 
(a) & (b)
\end{tabular}
\end{center}
\caption{Boxplots of the pre--and post--normalization intensity
  log--ratios $M$ for the four arrays in the Swirl experiment. } 
\protect\label{fig:maBoxplot2}
\end{figure}

\newpage


\begin{figure}
\begin{center}
\begin{tabular}{cc}
\includegraphics[width=3in,height=3in,angle=0]{maPlot1pre}, &
\includegraphics[width=3in,height=3in,angle=0]{maPlot1post} \\ 
(a) & (b)
\end{tabular}
\end{center}
\caption{Pre-- and post--normalization $MA$--plot for the Swirl 93
  array, with the lowess fits for individual
  print--tip--groups. Different colors are used to represent lowess
  curves for print--tips from different rows, and different line types
  are used to represent lowess curves for print--tips from different
  columns. }
\protect\label{fig:maPlot1}
\end{figure}


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
{\bf Note: Sweave.} This document was generated using the \Rfunction{Sweave}
function from the R \Rpackage{tools} package. The source file is in the
\Rfunction{/inst/doc} directory of the package \Rpackage{marray}.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\bibliography{marrayPacks}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\end{document}

 

% NOTE -- ONLY EDIT THE .Rnw FILE!!!  The .tex file is
% likely to be overwritten.
%
% \VignetteIndexEntry{marray Overview}
% \VignetteDepends{marray}
% \VignetteKeywords{Expression Analysis, Preprocessing}
% \VignettePackage{marray}

\documentclass[11pt]{article}

\usepackage{amsmath,epsfig,fullpage,hyperref}
\parindent 0in

\newcommand{\Robject}[1]{{\texttt{#1}}}
\newcommand{\Rfunction}[1]{{\texttt{#1}}}
\newcommand{\Rpackage}[1]{{\textit{#1}}}


\begin{document}

\title{\bf Quick start guide for marray}

\author{Yee Hwa Yang}

\maketitle

\begin{center} 
1. Department of Medicine, University of California, San Francisco, \url{http://www.biostat.ucsf.edu/jean}\\ 
\end{center}

\tableofcontents

% library(tools) 
% setwd("C:/MyDoc/Projects/madman/Rpacks/marray/inst/doc")
% Rnwfile<-file.path("C:/MyDoc/Projects/madman/Rpacks/marray/inst/doc","marray.Rnw")
% options(width=65)
% Sweave(Rnwfile,pdf=TRUE,eps=TRUE,stylepath=TRUE,driver=RweaveLatex())

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Overview}

This document provides a brief guide to the \Rpackage{marray} package,
which is packages for diagnostic plots and normalization of cDNA
microarray data.  Information on the other packages can be found in the
other vignettes.

There are three main components to this package.  These are:
\begin{itemize}
\item Reading in data.
\item Perform simple diagnositc plots to access quality.
\item Normalization.
\end{itemize}

After the two main pre--processing tasks, image analysis and
normalization, the next steps in the statistical analysis depend on the
biological question for which the microarray experiment was
designed. Thus, different Bioconductor packages may be applicable. For
example, for identifying differentially expressed genes, functions in
the packages {\tt limma}, {\tt EBarrays}, {\tt siggenes}, and {\tt
multtest} may be used.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Getting started}

To load the {\tt marray} package in your R session, type {\tt
library(marray)}.  We demonstrate the functionality of this R packages
using gene expression data from the Swirl zebrafish experiment which is
included as part of the package. To load the swirl dataset, use {\tt
data(swirl)}, and to view a description of the experiments and data,
type {\tt ?  swirl}.  


\begin{enumerate}

\item To begin, users will create a directory and move all the relevant
image processing output files (e.g. \texttt{.spot} files) and a file
containing target (or samples) descriptions
(e.g. \texttt{SwirlSample.txt} file) to that directory. For this
illustration, the data has been gathered in the data directory {\tt
swirldata}.

\item Start R in the desired working directory and load the
  \Rpackage{marray} packages:

%code 1
<<eval=TRUE, echo=TRUE>>=
library(marray)
dir(system.file("swirldata", package="marray")) 
@

\item {\bf Data input:} Read in the target file containing
information about the hybridization.

%code 2
<<eval=TRUE, echo=TRUE>>=
datadir <- system.file("swirldata", package="marray")
swirlTargets <- read.marrayInfo(file.path(datadir, "SwirlSample.txt"))
@

\item Read in the raw fluorescent intensities data, by default we assume
that the file names are provided in the {\bf first} column of the target
file.
%code 3
<<eval=TRUE, echo=TRUE>>=
mraw <- read.Spot(targets = swirlTargets, path=datadir)
@

If your working directory contains \Rpackage{GenePix} files (\Rfunction{
  .gpr}), run the following command.  By default, the function {\tt
  read.GenePix} will also set up printer layout and probe annotation
  information. 
\begin{verbatim}
> data <- read.GenePix(targets=swirlTargets)
\end{verbatim}

\item Read in the probe annotation information.
%code 4a
<<eval=TRUE, echo=TRUE>>=
galinfo <- read.Galfile("fish.gal", path=datadir)
mraw@maLayout <- galinfo$layout
mraw@maGnames <- galinfo$gnames
@

\item {\bf Array quality assessment:}, the following command generates
     diagnostic plots for a qualitative assessment of slide quality.
     The results are saved as png files in the working directory.  We
     uses the wrapper functions provided in the package {\tt
     arrayQuality}.

%% Code 4
\begin{verbatim}
> library(arrayQuality)
> maQualityPlots(mraw)
\end{verbatim}

In addition, you can perform simple diagnostic plots with
\begin{verbatim}
> image(mraw)
> boxplot(mraw)
> plot(mraw)
\end{verbatim}

\item {\bf Normalization:} Perform print-tip normalization
for each arrays and take a look at the data summary.
%% Code 5
\begin{verbatim}
> normdata <- maNorm(mraw)
> summary(normdata)
\end{verbatim}

\item  Output the normalized log--ratios $M$ data.
%% Code 6
\begin{verbatim}
> write.marray(normdata)
\end{verbatim}

\item {\bf Identify DE genes:} Using the linear model package
\Rpackage{limma} to identify differential expressed (DE) genes between
wildtype and mutant.  Perform fold-chance estimation as well as apply
Bayesian smoothing to the standard errors.
%% Code 8
\begin{verbatim}
> library(limma)
> LMres <- lmFit(normdata, design = c(1, -1, -1, 1), weights=NULL)
> LMres <- eBayes(LMres)
\end{verbatim}

\item Show the top 50 genes and write it out into a clickable html file.
%% Code 9
\begin{verbatim}
> restable <- toptable(LMres, number=50, genelist=maGeneTable(normdata), resort.by="M")
> table2html(restable, disp="file")
\end{verbatim}


\item To utilize other bioconductor packages for downstream analysis, it
  is also possible to convert objects of class {\tt marrayNorm} into
  objects of class {\tt ExpressionSet} (see definition in the {\tt Biobase}
  package), see package {\tt convert} package for more details.
\begin{verbatim}
> library(convert)
> as(normdata, "ExpressionSet")
\end{verbatim}

\end{enumerate}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Other vignettes and packages}

Greater details can be found in other vignettes.  These are: 

\begin{description}
\item {\tt marrayClasses}.  This vignette describes basic class
definitions and associated methods for pre-- and post--normalization
intensity data for batches of arrays.

\item {\tt marrayInput}. This vignette describes functionality for
reading microarray data into R, such as intensity data from image
processing output files (e.g. {\tt .spot} and {\tt .gpr} files for the
{\tt Spot} and {\tt GenePix} packages, respectively) and textual
information on probes and targets (e.g. from gal files and god
lists). {\tt tcltk} widgets are supplied to facilitate and automate data
input and the creation of microarray specific R objects for storing
these data.

\item {\tt marrayPlot}. This vignette provides descriptions to
  functions for diagnostic plots of microarray spot statistics, such as
  boxplots, scatter--plots, and spatial color images. Examination of
  diagnostic plots of intensity data is important in order to identify
  printing, hybridization, and scanning artifacts which can lead to
  biased inference concerning gene expression.

\item {\tt marrayNorm}. This vignette describes various location and
scale normalization procedures, which correct for different types of dye
biases (e.g. intensity, spatial, plate biases) and allow the use of
control sequences spotted onto the array and possibly spiked into the
mRNA samples. Normalization is needed to ensure that observed
differences in intensities are indeed due to differential expression and
not experimental artifacts; fluorescence intensities should therefore be
normalized before any analysis which involves comparisons among genes
within or between arrays. 

\end{description}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

{\bf Note: Sweave.} This document was generated using the \Rfunction{Sweave}
function from the R \Rpackage{tools} package. The source file is in the
\Rfunction{/inst/doc} directory of the package \Rpackage{marray}.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\end{document}
 

This wrapper function \Rfunction{?maQualityPlots} automatically produces
nine plots of  
\begin{itemize}
\item pre-- and post--normalization cDNA microarray data;
\item $MA$--plots of pre-- and post--normalization log--ratios $M$; 
\item color images of pre-- and post--normalization log--ratios $M$; 
\item color images of average log--intensities $A$; 
\item histogram and overlay density of the signal to noise log--ratio
  for Cy5 and Cy3 channels; where the signal to noise ratios is defined
  as the foreground intensity (without background adjustment) over the
  background intensity; and 
\item dot--plots of $M$ and $A$ values for replicate controls probes.
\end{itemize}
In addition, this function automatically saves the figures to a file.
More detailed descriptions of all the arguments and options and be found
the in the arrayQuality package.  Please contact us if you have other
image processing output formats and would like a similar wrapper
functions.

---
jupytext:
  cell_metadata_filter: -all
  formats: md:myst
  text_representation:
    extension: .md
    format_name: myst
    format_version: 0.13
    jupytext_version: 1.16.6
kernelspec:
  display_name: Python 3 (ipykernel)
  language: python
  name: python3
---

# Tutorial

MArray is a package for extending your favorite [Python Array API Standard](https://data-apis.org/array-api/latest/index.html) compatible library with mask capabilities. Motivation for masked arrays can be found at ["What is a masked array?"](https://numpy.org/devdocs/reference/maskedarray.generic.html#what-is-a-masked-array).

MArray is easy to install with `pip`, and it has no required dependencies.

```{code-cell}
# !pip install marray
```

The rest of the tutorial will assume that we want to add masks to NumPy arrays. Note that this is different from using NumPy's built-in masked arrays from the `numpy.ma` namespace because `numpy.ma` is not compatible with the array API standard. Even the base NumPy namespace is not Array API compatible in versions of NumPy prior to 2.0, so we will install a recent version of NumPy to work with.

```{code-cell}
# !pip install --upgrade numpy
```

To create a version of the NumPy namespace with mask support, use Python's `from...import...as` syntax.

```{code-cell}
from marray import numpy as mxp
```

`mxp` exposes all the features of NumPy that are specified in the Array API standard, but adds masks support to them. For example:

```{code-cell}
x = mxp.arange(3)
x
```

Just as `xp.arange(3)` would have created a regular NumPy array with elements [0, 1, 2], `mxp.arange(3)` creates an `MArray` object with these elements. These are accessible via the `data` attribute.

```{code-cell}
x.data
```

The difference is that the `MArray` also has a mask, available via the `mask` attribute.

```{code-cell}
x.mask
```

Because all of the elements of the mask are `False`, this `MArray` will behave just like a regular NumPy array. That's boring. Let's create an array with a nontrivial mask. To do that, we'll use `mxp.asarray`.

```{code-cell}
x = mxp.asarray([1, 2, 3, 4], mask=[False, True, False, True])
x
```

`marray` is intended to be a very light wrapper of the underlying array library. Just as it has only one public function (`get_namespace`), it makes only one modification to the signature of a wrapped library function, which we've used above: it adds a `mask` keyword-only argument to the `asarray` function.

Let's see how the mask changes the behavior of common functions.

## Statistical Functions
For reducing functions, masked elements are ignored; the result is the same as if the masked elements were not in the array.

```{code-cell}
mxp.max(x)  # 4 was masked
```

```{code-cell}
mxp.sum(x)  # 2 and 4 were masked
```

For the only non-reducing statistical function, `cumulative_sum`, masked elements do not contribute to the cumulative sum.

```{code-cell}
mxp.cumulative_sum(x)
```

Note that the elements at indices where the original array were masked remain masked. Because of the limitations of the underlying array library, there will always be values corresponding with masked elements in `data`, *but these values should be considered meaningless*.

## Utility functions
`all` and `any` work like the reducing statistics functions.

```{code-cell}
x = mxp.asarray([False, False, False, True], mask=[False, True, False, True])
mxp.all(x)
```

```{code-cell}
mxp.any(x)
```

Is that last result surprising? Although there is one `True` in `x.data`, it is ignored when computing `any` because it is masked.

You may have noticed that the mask of the result has always been `False` in these examples of reducing functions. This is always the case unless *all* elements of the array are masked. In this case, it is required by the reducing nature of the function to return a 0D array for a 1D input, but there is not an universally accepted result for these functions when all elements are masked. (What is the maximum of an empty set?)

```{code-cell}
x = mxp.asarray(x.data, mask=True)
mxp.any(x).mask
```

## Sorting functions
The sorting functions treat masked values as undefined and, by convention, append them to the end of the returned array.

```{code-cell}
data = [8, 3, 4, 1, 9, 9, 5, 5]
mask = [0, 0, 1, 0, 1, 1, 0, 0]
x = mxp.asarray(data, mask=mask)
mxp.sort(x)
```

Where did those huge numbers come from? We emphasize again: *the `data` corresponding with masked elements should be considered meaningless*; they are just placeholders that allow us respect the mask while doing array operations efficiently.

```{code-cell}
i = mxp.argsort(x)
i
```

Is it surprising that the mask of the array returned by `argsort` is all False? These are the indices that allow us to transform the original array into the sorted result. We can confirm that without a mask, these indices sort the array and keep the right elements masked.

```{code-cell}
y = x[i.data]
y
```

*Gotcha:* Sorting is not supported when the the non-masked data includes the maximum (minimum when `descending=True`) value of the data's `dtype`.

```{code-cell}
z = mxp.asarray(x, mask=mask, dtype=mxp.uint8)
z[0] = 2**8 - 1
# mxp.sort(z)
# NotImplementedError: The maximum value of the data's dtype is included in the non-masked data; this complicates sorting when masked values are present.
# Consider promoting to another dtype to use `sort`.
```

It is often possible to sidestep this limitation by using a different `dtype` for the sorting, then converting back to the original type.

```{code-cell}
z = mxp.astype(z, mxp.uint16)
z_sorted = mxp.sort(z)
z_sorted = mxp.astype(z_sorted, mxp.uint8)
z_sorted
```

## Set functions
Masked elements are treated as distinct from all non-masked elements but equivalent to all other masked elements.

```{code-cell}
res = mxp.unique_counts(x)
```

```{code-cell}
res.values
```

```{code-cell}
res.counts
```

*Gotcha*: set functions have the same limitation as the sorting functions: the non-masked data may not include the maximum value of the data's `dtype`.

+++

## Manipulation functions
Manipulation functions perform the same operation on the data and the mask.

```{code-cell}
mxp.flip(y)
```

```{code-cell}
mxp.stack([y, y])
```

## Creation functions
Most creation functions create arrays with an all-False mask.

```{code-cell}
mxp.eye(3)
```

Exceptions include the `_like` functions, which preserve the mask of the array argument.

```{code-cell}
mxp.zeros_like(y)
```

`tril` and `triu` also preserve the mask of the indicated triangular portion of the argument.

```{code-cell}
import numpy as xp

data = xp.ones((3, 3))
mask = xp.zeros_like(data)
mask[0, -1] = 1
mask[-1, 0] = 1
A = mxp.asarray(data, mask=mask)
A
```

```{code-cell}
mxp.tril(A)
```

## Searching functions
Similarly to the statistics functions, masked elements are treated as if they did not exist.

```{code-cell}
x[[1, -1]] = 0  # add some zeros
x  # let's remember what `x` looks like
```

```{code-cell}
mxp.argmax(x)  # 9 is masked, so 8 (at index 0) is the largest element
```

```{code-cell}
i = mxp.nonzero(x)  # Only elements at these indices are nonzero *and* not masked
i
```

The correct behavior of indexing with a masked array is ambiguous, so use only regular, unmasked arrays for indexing.

```{code-cell}
indices = i[0].data
x[indices]  # nonzero, not masked
```

## Elementwise functions
Elementwise functions (and operators) simply perform the requested operation on the `data`.

For unary functions, the mask of the result is the mask of the argument.

```{code-cell}
x = xp.linspace(0, 2*xp.pi, 5)
x = mxp.asarray(x, mask=(x > xp.pi))
x
```

```{code-cell}
-x
```

```{code-cell}
mxp.round(mxp.sin(x))
```

For binary functions and operators, the mask of the result is the result of the logical *or* operation on the masks of the arguments.

```{code-cell}
x = mxp.asarray([1, 2, 3, 4], mask=[1, 0, 1, 0])
y = mxp.asarray([5, 6, 7, 8], mask=[1, 1, 0, 0])
x + y
```

```{code-cell}
mxp.pow(y, x)
```

Note that `np.ma` automatically masks non-finite elements produced during calculations.

```{code-cell}
import numpy

x = numpy.ma.masked_array(0, mask=False)
with numpy.errstate(divide='ignore', invalid='ignore'):
    y = [1, 0] / x
y
```

`MArray` *does not* follow this convention.

```{code-cell}
x = mxp.asarray(0, mask=False)
with numpy.errstate(divide='ignore', invalid='ignore'):
    y = [1, 0] / x
y
```

This is because masked elements are often used to represent *missing* data, and the results of these operations are not missing. If this does not suit your needs, mask out data according to your requirements after performing the operation.

```{code-cell}
x = mxp.asarray(0, mask=False)
with numpy.errstate(divide='ignore', invalid='ignore'):
    y = [1, 0] / x
mxp.asarray(y.data, mask=xp.isnan(y.data))
```

## Linear Algebra Functions
As usual, linear algebra functions and operators treat masked elements as though they don't exist.

```{code-cell}
x = mxp.asarray([1, 2, 3, 4], mask=[1, 0, 1, 0])
y = mxp.asarray([5, 6, 7, 8], mask=[1, 1, 0, 0])
x @ y  # the last elements of the arrays, 4 and 8, are the only non-masked elements
```

The exception is `matrix_transpose`, which transposes the data and the mask.

```{code-cell}
x = mxp.asarray([[1, 2], [3, 4]], mask=[[1, 1], [0, 0]])
x
```

```{code-cell}
mxp.matrix_transpose(x)
```

## Conclusion
While this tutorial is not exhaustive, we hope it is sufficient to allow you to predict the results of operations with `MArray`s and use them to suit your needs. If you'd like to see this tutorial extended in a particular way, please [open an issue](https://github.com/mdhaber/marray/issues)!

%
% NOTE -- ONLY EDIT THE .Rnw FILE!!!  The .tex file is
% likely to be overwritten.
%
% \VignetteIndexEntry{marrayClasses Tutorial (short)}
% \VignetteDepends{tools}
% \VignetteKeywords{Expression Analysis, Preprocessing}
% \VignettePackage{marray}

\documentclass[11pt]{article}

\usepackage{amsmath,fullpage}
\usepackage[authoryear,round]{natbib}
\usepackage{hyperref}

\newcommand{\Robject}[1]{{\texttt{#1}}}
\newcommand{\Rfunction}[1]{{\texttt{#1}}}
\newcommand{\Rpackage}[1]{{\textit{#1}}}

\parindent 0in

%%%\bibliographystyle{abbrvnat}

\begin{document}

\title{\bf Introduction to the Bioconductor marray package : Classes  structure component (short)}

\author{Yee Hwa Yang$^1$ and Sandrine Dudoit$^2$}

\maketitle

\begin{center}
1. Department of Medicine, University of California, San Francisco, {\tt jean@biostat.berkeley.edu}\\
2. Division of Biostatistics, University of California, Berkeley.\\
\end{center}


% library(tools) 
% setwd("C:/MyDoc/Projects/madman/Rpacks/marray/inst/doc")
% Rnwfile<-file.path("C:/MyDoc/Projects/madman/Rpacks/marray/inst/doc","marrayClassesShort.Rnw")
% options(width=65)
% Sweave(Rnwfile,pdf=TRUE,eps=TRUE,stylepath=TRUE,driver=RweaveLatex())

\tableofcontents


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Overview}

This document provides a short tutorial on the basic class definitions
and associated methods used in the {\tt marray} package.  To load the
{\tt marray} package in your R session, type {\tt library(marray)}. 
<<eval=TRUE, echo=TRUE>>=
library(marray)
data(swirl)
@

\section{Microarray classes}

The three main classes for cDNA microarray data are:

\begin{description}
\item
{{\tt marrayLayout}:} 
This class is used to keep track of important layout parameters for
two--color cDNA microarrays. It contains slots for: the total number of
spotted probe sequences on the array, the 
dimensions of the spot and grid matrices, the plate origin of the
probes, information on spotted control sequences (e.g. probe sequences
which should have equal abundance in the two target samples, such as
housekeeping genes).   

\item
{{\tt marrayRaw}:} This class represents pre--normalization intensity
data for a batch of cDNA microarrays. A {\it batch} of arrays consists
of a collection of arrays with the same layout ("marrayLayout"). The
class contains slots for the green (Cy3) and red (Cy5) foreground and
background intensities, the layout of the arrays, and descriptions of
the target samples hybridized to the arrays and probe sequences spotted
onto the arrays.

\item
{{\tt marrayNorm}:} This class represents post--normalization intensity
data for a batch of cDNA microarrays.  The class contains slots for the
average log--intensities $A = \log_2 \sqrt{RG}$, the normalized
log--ratios $M = \log_2 R/G$, the location and scale normalization
values, the layout of the arrays, and descriptions of the target samples
hybridized to the arrays and probe sequences spotted onto the arrays.
\end{description}

Other classes are {\tt marrayInfo} which can be used to represents the
Target or the Probes information.  The function {\tt slotNames} can be
used to get information on the slots of a formally defined class or an
instance of the class. For example, to get information of the slots for
the {\tt marrayLayout} class or on the slots for the object {\tt swirl}
use

<<eval=TRUE>>=
slotNames("marrayLayout")
slotNames(swirl)
@

\section{Creating and accessing slots of microarray objects} 

{\bf Creating new objects.} The function {\tt new} from the {\tt
methods} package may be used to create new objects from a given
class. For example, to create an object of class {\tt marrayInfo}
describing the target samples in the Swirl experiment, one could use the
following code  

<<eval=TRUE>>=
zebra.RG<-as.data.frame(cbind(c("swirl","WT","swirl","WT"), c("WT","swirl","WT","swirl"))) 
dimnames(zebra.RG)[[2]]<-c("Cy3","Cy5")
zebra.samples<-new("marrayInfo",
		    maLabels=paste("Swirl array ",1:4,sep=""), 
		    maInfo=zebra.RG,
		    maNotes="Description of targets for Swirl experiment")
zebra.samples
@

Slots which are not specified in {\tt new} are initialized to the
prototype for the corresponding class. These are usually "empty", e.g.,
{\tt matrix(0,0,0)}. In most cases, microarray objects can be created
automatically using the input functions and their corresponding widgets
in the {\tt marrayInput} package. These were used to create the object
{\tt swirl} of class {\tt marrayRaw}.

{\bf Accessing slots.} Different components or slots of the microarray
objects may be accessed using the operator {\tt @}, or alternately, the
function {\tt slot}, which evaluates the slot name. For example, to
access the {\tt maLayout} slot in the object {\tt swirl} and the {\tt
maNgr} slot in the layout object {\tt L}:

<<eval=TRUE>>=
L<-slot(swirl, "maLayout")
L@maNgr
@

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Basic microarray methods}\label{smethods}

The following basic methods were defined to facilitate manipulation of
microarray data objects.  To see all methods available for a particular
class, e.g., {\tt marrayLayout}, or just the print methods

<<eval=FALSE>>==
showMethods(classes="marrayLayout")
showMethods("summary",classes="marrayLayout") 
@

\subsection{Printing methods for microarray objects} 

Since there is usually no need to print out fluorescence intensities for
thousands of genes, the {\tt print} method was overloaded for microarray
classes by simple report generators. For an overview of the available
microarray printing methods, type {\tt  methods ? print}, or to see all
print methods for the session 

<<eval=TRUE>>=
showMethods("print")
@

For example, summary statistics for an object of class {\tt marrayRaw},
such as {\tt swirl}, can be obtained by {\tt summary(swirl)} 

<<eval=TRUE>>=
summary(swirl)
@

\subsection{Subsetting methods for microarray objects} 

In many instances, one is interested in accessing only a subset of
arrays in a batch and/or spots in an array. Subsetting methods {\tt "["}
were defined for this purpose.  For an overview of the available
microarray subsetting methods, type {\tt methods ? "["} or to see all
subsetting methods for the session {\tt showMethods("[")}.  When using
the {\tt "["} operator, the first index refers to spots and the second
to arrays in a batch. Thus, to access the first 100 probe sequences in
the second and third arrays in the batch {\tt swirl} use

<<eval=TRUE>>=
swirl[1:100,2:3]
@

\subsection{Methods for accessing slots of microarray objects}


A number of simple methods were defined to access slots of the
microarray classes.  Using such methods is more general than using the
{\tt slot} function or {\tt @} operator. In particular, if the class
definitions are changed, any function which uses the {\tt @} operator
will need to be modified. When using a method to access the data in the
slot, only that particular method needs to be modified. Thus, to access
the layout information for the array batch {\tt swirl} one may also use
{\tt maLayout(swirl)}.\\ 

 In addition, various methods were defined to compute basic statistics
 from microarray object slots. For instance, for memory management
 reasons, objects of class {\tt marrayLayout} do not store the spot
 coordinates of each probe. Rather, these can be obtained from the
 dimensions of the grid and spot matrices by applying methods: {\tt
 maGridRow}, {\tt maGridCol}, {\tt maSpotRow}, and {\tt maSpotCol} to
 objects of class {\tt marrayLayout}. Print--tip--group coordinates are
 given by {\tt maPrintTip}. Similar methods were also defined to operate
 directly on objects of class {\tt marrayRaw} and {\tt marrayNorm}. The
 commands below may be used to display the number of spots on the array,
 the dimensions of the grid matrix, and the print--tip--group
 coordinates.

<<eval=TRUE>>=
swirl.layout<-maLayout(swirl)
maNspots(swirl)
maNspots(swirl.layout)
maNgr(swirl)
maNgc(swirl.layout)
maPrintTip(swirl[1:10,3])
@

\subsection{Methods for assigning slots of microarray objects}

A number of methods were defined to replace slots of microarray objects,
 without explicitly using the {\tt @} operator or {\tt slot}
 function. These make use of the {\tt setReplaceMethod} function from
 the R {\tt methods} package.  As with the accessor methods just
 described, the assignment methods are named after the slots. For
 example, to replace the {\tt maNotes} slot of {\tt swirl.layout}

<<eval=TRUE>>=
maNotes(swirl.layout)
maNotes(swirl.layout)<- "New value"
maNotes(swirl.layout)
@

To initialize slots of an empty {\tt marrayLayout} object

<<eval=TRUE>>=
L<-new("marrayLayout")
L
maNgr(L)<-4
@

Similar methods were defined to operate on objects of class {\tt
marrayInfo}, {\tt marrayRaw} and {\tt marrayNorm}.

\subsection{Methods for coercing microarray objects} 

To facilitate navigation between different classes of microarray
objects, we have defined methods for coercing microarray objects from
one class into another. A list of such methods can be obtained by {\tt
methods ? coerce}. For example, to coerce an object of class {\tt
marrayRaw} into an object of class {\tt marrayNorm}

<<eval=TRUE>>=
swirl.norm<-as(swirl, "marrayNorm")    
@


\subsection{Functions for computing layout parameters}

In some cases, plate information is not stored in {\tt marrayLayout}
objects when the data are first read into R. We have defined a function
{\tt maCompPlate} which computes plate indices from the dimensions of
the grid matrix and number of wells in a plate. For example, the Swirl
arrays were printed from 384--well plates, but the plate IDs were not
stored in the {\tt fish.gal} file. To generate plate IDs (arbitrarily
labeled by integers starting with 1) and store these in the {\tt
maPlate} slot of the {\tt marrayLayout} object use

<<eval=TRUE>>=
maPlate(swirl)<-maCompPlate(swirl,n=384)
@

Similar functions were defined to generate and manipulate spot
coordinates: {\tt maCompCoord}, {\tt maCompInd}, {\tt maCoord2Ind}, {\tt
maInd2Coord}. The function {\tt maGeneTable} produces a table of spot
coordinates and gene names for objects of class {\tt marrayRaw} and{\tt
marrayNorm}.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%\bibliography{marrayPacks}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\end{document}
 

Interested in contributing?

If you're knew to contributing on GitHub, learn the nuts and bolts of the process
at [Contributing to a project](https://docs.github.com/en/get-started/exploring-projects-on-github/contributing-to-a-project).

Typical contributions include:

- Filing [issues](https://github.com/mdhaber/marray/issues)
  (e.g. bug reports and feature requests)
- Triaging (adding new information to facilitate resolution of an issue)
- Code (submitting [pull requests](https://github.com/mdhaber/marray/pulls),
  especially to resolve open issues)
- Improving documentation

Interest in contributing is always appreciated! Note that some potential
contributions are more aligned with project goals than others. Please seek
feedback about your idea in an issue before starting significant work on a
pull request.

# Security Policy

## Supported Versions

| Version | Supported          |
| ------- | ------------------ |
| 1.0     | :white_check_mark: |
| < 1.0   | :x:                |

## Reporting a Vulnerability

Please report potential security vulnerabilities by email to `matt.haberland@gmail.com`.
If the security vulnerability is recognized as a threat, a patch will be crafted
privately to prepare a dedicated bugfix release.

%
% NOTE -- ONLY EDIT THE .Rnw FILE!!!  The .tex file is
% likely to be overwritten.
%
% \VignetteIndexEntry{marrayClasses Overview}
% \VignetteDepends{tools,marray}
% \VignetteKeywords{Expression Analysis, Preprocessing}
% \VignettePackage{marray}
\documentclass[11pt]{article}

\usepackage{amsmath,fullpage}
\usepackage[authoryear,round]{natbib}
\usepackage{hyperref}


\newcommand{\Robject}[1]{{\texttt{#1}}}
\newcommand{\Rfunction}[1]{{\texttt{#1}}}
\newcommand{\Rpackage}[1]{{\textit{#1}}}

\parindent 0in

\bibliographystyle{abbrvnat}

\begin{document}

\title{\bf Introduction to the Bioconductor marray package : Classes structure component}

\author{Sandrine Dudoit$^1$ and Yee Hwa Yang$^2$}

\maketitle

\begin{center}
1. Division of Biostatistics, University of California, Berkeley,
   \url{http://www.stat.berkeley.edu/~sandrine}
2. Department of Medicine, University of California, San Francisco,
   {\tt jean@biostat.berkeley.edu}\\
\end{center}

% library(tools) 
% setwd("C:/MyDoc/Projects/madman/Rpacks/marray/inst/doc")
% Rnwfile<-file.path("C:/MyDoc/Projects/madman/Rpacks/marray/inst/doc","marrayClasses.Rnw")
% options(width=65)
% Sweave(Rnwfile,pdf=TRUE,eps=TRUE,stylepath=TRUE,driver=RweaveLatex())

\tableofcontents

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Overview}

This document provides a tutorial on the class structures used in the
{\tt marray} package.  The {\tt marray} packages contains basic class
definitions and associated methods for pre-- and post--normalization
intensity data for batches of arrays. To load the {\tt marray} package
in your R session, type {\tt library(marray)}.  As with any R package,
detailed information on functions, classes and methods can be obtained
in the help files. For instance, to view the help file for the class
{\tt marrayRaw} in a browser, use {\tt help.start()} followed by {\tt ?
marrayRaw} or alternately the dyadic {\tt class ? marrayRaw}.
Furthermore, se demonstrate the functionality of this collection of R
packages using gene expression data from the Swirl zebrafish
experiment. To load the Swirl dataset, use {\tt data(swirl)}, and to
view a description of the experiments and data, type {\tt ? swirl}.

Getting started:
<<eval=TRUE, echo=FALSE>>=
library("marray")
data(swirl)
@

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Object--oriented programming}\label{soo}

Microarray experiments generate large and complex multivariate datasets,
which contain textual information on probe sequences (e.g. gene names,
annotation, layout parameters) and mRNA target samples (e.g. description
of samples, protocols, hybridization and scanning conditions), in
addition to the primary fluorescence intensity data. Efficient and
coordinated access to these various types of data is an important aspect
of computing with microarray data. To facilitate the management of
microarray data at different stages of the analysis process, a
collection of microarray specific data structures or {\it classes} were
defined (see also the Bioconductor package {\tt Biobase} for microarray
classes and methods for normalized data). The packages rely on the
class/method mechanism provided by John Chambers' R {\tt methods}
package, which allows object--oriented programming in R. Broadly
speaking, {\it classes} reflect how we think of certain objects and what
information these objects should contain. Classes are defined in terms
of {\it slots} which contain the relevant data for the application at
hand. {\it Methods} define how a particular function should behave
depending on the class of its arguments and allow computations to be
adapted to particular classes, that is, data types. For example, a
microarray object should contain intensity data as well as information
on the probe sequences spotted on the array and the target samples
hybridized to it. Useful methods for microarray classes include
specializations of printing, subsetting, and plotting functions for the
types of data represented by these classes.  

The use of classes and methods greatly reduces the complexity of
handling microarray data, by automatically coordinating various sources
of information associated with microarray experiments. 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Microarray classes}\label{sclasses}

The {\it raw data} from a microarray experiment are the image files
produced by the scanner; these are typically pairs of 16--bit tagged
image file format (TIFF) files, one for each fluorescent dye (images
usually range in size from a few megabytes (MB) to 10 or 20 MB for high
resolution scans). Image analysis is required to extract foreground and
background fluorescence intensity measurements for each spotted DNA
sequence. \\ 

Here, we begin our analysis of microarray data with the output files of
image processing packages such as {\tt GenePix} or {\tt Spot}. In what
follows, red and green background intensities are denoted by $R_b$ and
$G_b$, respectively, and red and green foreground intensities by $R_f$
and $G_f$, respectively. Background--corrected red and green
fluorescence intensities are denoted by $R$ and $G$, and $M$ denotes the
corresponding base--2 log--ratio, $M = \log_2 R/G$.  

\subsection{{\tt marrayLayout} class} 

The term {\it array layout} refers to the layout of DNA probe sequences
on the array, as determined by the printing process. In general, probe
sequences are spotted on a glass microscope slide using an arrayer which
has an $ngr \times ngc$ print--head, that is, a regular array of $ngr$
rows and $ngc$ columns of print--tips or pins. The resulting microarrays
are thus partitioned into an $ngr \times ngc$ {\it grid matrix}. The
terms {\it grid}, {\it sector}, and {\it print--tip--group} are used
interchangeably in the microarray literature. Each grid consists of an
$nsr \times nsc$ {\it spot matrix} that was printed with a single
print--tip. DNA probes are usually printed sequentially from a
collection of 384--well plates (or 96--well plates), thus, in some
sense, plates are proxies for time of printing. In addition, a number of
control probe sequences may be spotted on the array for normalization or
other calibration purposes. The term {\it array batch} is used to refer
to a collection of arrays with the same layout. Keeping track of spot
layout information is essential for quality assessment of fluorescent
intensity data and for normalization purposes.\\ 

Important layout parameters are the dimensions of the spot and grid
matrices, and, for each probe on the array, its grid matrix and spot
matrix coordinates. In addition, it is useful to keep track of gene
names, plate origin of the probes, and information on the spotted
control sequences (e.g. probe sequences which should have equal
abundance in the two target samples, such as housekeeping genes). The
class {\tt marrayLayout} was designed to keep track of these various
layout parameters and contains the following slots (the classes of the
slots are listed below the slot names) 

<<R>>=
getClassDef("marrayLayout")
@
\begin{description}
    \item{{\tt maNgr}:}{ Object of class "numeric", number of rows for the grid matrix.}
    \item{{\tt maNgc}:}{ Object of class "numeric", number of columns for the grid matrix.}
    \item{{\tt maNsr}:}{ Object of class "numeric",  number of rows for the spot matrices.} 
    \item{{\tt maNsc}:}{ Object of class "numeric", number of columns for the spot matrices.}
    \item{{\tt maNspots}:}{ Object of class "numeric", total number of
    spots on the array, equal to $maNgr \times maNgc \times maNsr \times
    maNsc$.} 
    \item{{\tt maSub}:}{ Object of class "logical", indicating which
    spots are currently being considered.} 
    \item{{\tt maPlate}:}{ Object of class "factor", recording the plate
    origin of the spotted probe sequences.}
    \item{{\tt maControls}:}{ Object of class "factor", recording the
    control status of the spotted probe sequences.} 
    \item{{\tt maNotes}:}{ Object of class "character",  any notes
    concerning the microarray layout, e.g., printing conditions.} 
\end{description}

In addition, a number of {\it methods} were defined to compute other
important layout parameters, such as print--tip, grid matrix, and spot
matrix coordinates: {\tt maPrintTip}, {\tt maGridRow}, {\tt maGridCol},
{\tt maSpotRow}, and {\tt maSpotCol} (see Section \ref{smethods}). No
slots were defined for these quantities for memory management reasons. 

\subsection{{\tt marrayInfo} class} 

Information on the target mRNA samples co--hybridized to the arrays is
stored in objects of class {\tt marrayInfo}. Such objects may include
the names of the arrays, the names of the Cy3 and Cy5 labeled samples,
notes on the hybridization and scanning conditions, and other textual
information. Descriptions of the spotted probe sequences (e.g. matrix of
gene names, annotation, notes on printing conditions) are also stored in
object of class {\tt marrayInfo}. The {\tt marrayInfo} class is not
specific to the microarray context and has the following definition 

<<R>>=
getClassDef("marrayInfo")
@

\subsection{{\tt marrayRaw} class}

 Pre--normalization intensity data for a batch of arrays are stored in
 objects of class {\tt marrayRaw}, which contain slots for the matrices
 of Cy3 and Cy5 background and foreground intensities ({\tt maGb}, {\tt
 maRb}, {\tt maGf}, {\tt maRf}), spot quality weights ({\tt maW}),
 layout parameters of the arrays ({\tt marrayLayout}), description of
 the probes spotted onto the arrays ({\tt maGnames}) and mRNA target
 samples hybridized to the arrays ({\tt maTargets}).  

<<R>>=
getClassDef("marrayRaw")
@

\begin{description}
  \item{{\tt maRf}:}{ Object of class "matrix", red foreground intensities, rows correspond to spotted probe sequences, columns to arrays in the batch.}
    \item{{\tt maGf}:}{ Object of class "matrix", green foreground intensities, rows correspond to spotted probe sequences, columns to arrays in the batch. }
    \item{{\tt maRb}:}{ Object of class "matrix", red background intensities, rows correspond to spotted probe sequences, columns to arrays in the batch. }
    \item{{\tt maGb}:}{ Object of class "matrix", green background intensities, rows correspond to spotted probe sequences, columns to arrays in the batch. }
    \item{{\tt maW}:}{ Object of class "matrix", spot quality weights, rows correspond to spotted probe sequences, columns to arrays in the batch.}
    \item{{\tt maLayout}:}{ Object of class "marrayLayout", layout parameters for cDNA microarrays.}
    \item{{\tt maGnames}:}{ Object of class "marrayInfo", description of spotted probe sequences.}
    \item{{\tt maTargets}:}{ Object of class "marrayInfo", description of target samples hybridized to the arrays.}
    \item{{\tt maNotes}:}{ Object of class "character", any notes concerning the microarray experiments, e.g. hybridization or scanning conditions.}
\end{description} 

\subsection{{\tt marrayNorm} class} 

Post--normalization intensity data are stored in similar objects of
class {\tt marrayNorm}. These objects store the normalized intensity
log--ratios {\tt maM}, the location and scale normalization values ({\tt
  maMloc} and {\tt maMscale}), and the average log--intensities ({\tt
  maA}). In addition, the {\tt marrayNorm} class has a slot for the
function call used to normalize the data, {\tt maNormCall}. For more
details on the creation of normalized microarray objects, the reader is
referred to the vignette for the {\tt marrayNorm} package. 

<<R>>=
getClassDef("marrayNorm")
@ 

\begin{description}
    \item{{\tt maA}:}{ Object of class "matrix", average log--intensities (base 2) $A$, rows correspond to spotted probe sequences, columns to arrays in the batch.}
    \item{{\tt maM}:}{ Object of class "matrix", intensity log--ratios (base 2) $M$, rows correspond to spotted probe sequences, columns to arrays in the batch.}
    \item{{\tt maMloc}:}{ Object of class "matrix", location normalization values, rows correspond to spotted probe sequences, columns to arrays in the batch.}
    \item{{\tt maMscale}:}{ Object of class "matrix", scale normalization values, rows correspond to spotted probe sequences, columns to arrays in the batch.}
    \item{{\tt maW}:}{ Object of class "matrix", spot quality weights, rows correspond to spotted probe sequences, columns to arrays in the batch.}
    \item{{\tt maLayout}:}{ Object of class "marrayLayout", layout parameters for cDNA microarrays.}
    \item{{\tt maGnames}:}{ Object of class "marrayInfo", description of spotted probe sequences.}
    \item{{\tt maTargets}:}{ Object of class "marrayInfo", description of target samples hybridized to the arrays.}
    \item{{\tt maNotes}:}{ Object of class "character",  any notes concerning the microarray experiments, e.g. hybridization or scanning conditions.}
    \item{{\tt maNormCall}:}{ Object of class "call", function call for normalizing the batch of arrays.}
\end{description}

Most microarray objects contain an {\tt maNotes} slots which may be used to store any string of characters describing the experiments, for examples, notes on the printing, hybridization, or scanning conditions.


\subsection{Creating and accessing slots of microarray objects} 

{\bf Creating new objects.} The function {\tt new} from the {\tt
  methods} package may be used to create new objects from a given
class. For example, to create an object of class {\tt marrayInfo}
describing the target samples in the Swirl experiment, one could use the
following code  

<<eval=TRUE>>=
zebra.RG<-as.data.frame(cbind(c("swirl","WT","swirl","WT"),
c("WT","swirl","WT","swirl")))
dimnames(zebra.RG)[[2]]<-c("Cy3","Cy5")
zebra.samples<-new("marrayInfo",
		    maLabels=paste("Swirl array ",1:4,sep=""), 
		    maInfo=zebra.RG,
		    maNotes="Description of targets for Swirl experiment")
zebra.samples
@

Slots which are not specified in {\tt new} are initialized to the
prototype for the corresponding class. These are usually "empty", e.g.,
{\tt matrix(0,0,0)}. In most cases, microarray objects can be created
automatically using the input functions and their corresponding widgets
in the {\tt marrayInput} package. These were used to create the object
{\tt swirl} of class {\tt marrayRaw}.\\


{\bf Accessing slots.} Different components or slots of the microarray
objects may be accessed using the operator {\tt @}, or alternately, the
function {\tt slot}, which evaluates the slot name. For example, to
access the {\tt maLayout} slot in the object {\tt swirl} and the {\tt
maNgr} slot in the layout object {\tt L}

<<eval=TRUE>>=
L<-slot(swirl, "maLayout")
L@maNgr
@

The function {\tt slotNames} can be used to get information on the slots of a formally defined class or an instance of the class. For example, to get information on the slots for the {\tt marrayLayout} class or on the slots for the object {\tt swirl} use

<<eval=TRUE>>=
slotNames("marrayLayout")
slotNames(swirl)
@

\subsection{Testing the validity of an object}

The function {\tt validObject} from the R package {\tt methods} may be
used to test the validity of an object with respect to its class
definition. This function has two arguments: {\tt object}, the object to
be tested; and {\tt test}. If {\tt test} is TRUE, the function returns a
vector of strings describing the problems, if any.

<<eval=TRUE, echo=TRUE>>=
validObject(maLayout(swirl), test=TRUE)
@

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Basic microarray methods}\label{smethods}


The following basic methods were defined to facilitate manipulation of microarray data objects. 
To see all methods available for a particular class, e.g., {\tt marrayLayout}, or just the print methods

<<eval=FALSE>>==
showMethods(classes="marrayLayout")
showMethods("show",classes="marrayLayout") 
@


\subsection{Printing methods for microarray objects} 

Since there is usually no need to print out fluorescence intensities for thousands of genes, the {\tt print} method was overloaded for microarray classes by simple report generators. For an overview of the available microarray printing methods, type {\tt  methods ? summary}, or to see all summary methods for the session

<<eval=TRUE>>=
showMethods("summary")
@

For example, summary statistics for an object of class {\tt marrayRaw},
 such as {\tt swirl}, can be obtained by {\tt print(swirl)} or simply {\tt swirl}.

<<eval=TRUE>>=
summary(swirl)
@

\subsection{Subsetting methods for microarray objects} 

In many instances, one is interested in accessing only a subset of
arrays in a batch and/or spots in an array. Subsetting methods {\tt "["}
were defined for this purpose.  For an overview of the available
microarray subsetting methods, type {\tt methods ? "["} or to see all
subsetting methods for the session {\tt showMethods("[")}.  When using
the {\tt "["} operator, the first index refers to spots and the second
to arrays in a batch. Thus, to access the first 100 probe sequences in
the second and third arrays in the batch {\tt swirl} use

<<eval=TRUE>>=
swirl[1:100,2:3]
@


\subsection{Methods for accessing slots of microarray objects} 


A number of simple methods were defined to access slots of the
microarray classes.  Using such methods is more general than using the
{\tt slot} function or {\tt @} operator. In particular, if the class
definitions are changed, any function which uses the {\tt @} operator
will need to be modified. When using a method to access the data in the
slot, only that particular method needs to be modified. Thus, to access
the layout information for the array batch {\tt swirl} one may also use
{\tt maLayout(swirl)}.\\ 

 In addition, various methods were defined to compute basic statistics
 from microarray object slots. For instance, for memory management
 reasons, objects of class {\tt marrayLayout} do not store the spot
 coordinates of each probe. Rather, these can be obtained from the
 dimensions of the grid and spot matrices by applying methods: {\tt
 maGridRow}, {\tt maGridCol}, {\tt maSpotRow}, and {\tt maSpotCol} to
 objects of class {\tt marrayLayout}. Print--tip--group coordinates are
 given by {\tt maPrintTip}. Similar methods were also defined to operate
 directly on objects of class {\tt marrayRaw} and {\tt marrayNorm}. The
 commands below may be used to display the number of spots on the array,
 the dimensions of the grid matrix, and the print--tip--group
 coordinates.

<<eval=TRUE>>=
swirl.layout<-maLayout(swirl)
maNspots(swirl)
maNspots(swirl.layout)
maNgr(swirl)
maNgc(swirl.layout)
maPrintTip(swirl[1:10,3])
@

\subsection{Methods for assigning slots of microarray objects}

A number of methods were defined to replace slots of microarray objects,
 without explicitly using the {\tt @} operator or {\tt slot}
 function. These make use of the {\tt setReplaceMethod} function from
 the R {\tt methods} package. 
 As with the accessor methods just described, the assignment methods are
 named after the slots. For example, to replace the {\tt maNotes} slot
 of {\tt swirl.layout} 

<<eval=TRUE>>=
maNotes(swirl.layout)
maNotes(swirl.layout)<- "New value"
maNotes(swirl.layout)
@

To initialize slots of an empty {\tt marrayLayout} object

<<eval=TRUE>>=
L<-new("marrayLayout")
L
maNgr(L)<-4
@


Similar methods were defined to operate on objects of class {\tt
  marrayInfo}, {\tt marrayRaw} and {\tt marrayNorm}.


\subsection{Methods for coercing microarray objects} 

To facilitate navigation between different classes of microarray
objects, we have defined methods for coercing microarray objects from
one class into another. A list of such methods can be obtained by {\tt
  methods ? coerce}. For example, to coerce an object of class {\tt
  marrayRaw} into an object of class {\tt marrayNorm}: 

<<eval=TRUE>>=
swirl.norm<-as(swirl, "marrayNorm")    
@


\subsection{Functions for computing layout parameters}

In some cases, plate information is not stored in {\tt marrayLayout}
objects when the data are first read into R. We have defined a function
{\tt maCompPlate} which computes plate indices from the dimensions of
the grid matrix and number of wells in a plate. For example, the Swirl
arrays were printed from 384--well plates, but the plate IDs were not
stored in the {\tt fish.gal} file. To generate plate IDs (arbitrarily
labeled by integers starting with 1) and store these in the {\tt
  maPlate} slot of the {\tt marrayLayout} object use 

<<eval=TRUE>>=
maPlate(swirl)<-maCompPlate(swirl,n=384)
@

Similar functions were defined to generate and manipulate spot
coordinates: {\tt maCompCoord}, {\tt maCompInd}, {\tt maCoord2Ind}, {\tt
maInd2Coord}. The function {\tt maGeneTable} produces a table of spot
coordinates and gene names for objects of class {\tt marrayRaw} and {\tt
marrayNorm}.


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%\bibliography{marrayPacks}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\end{document}
 

# MArray

MArray[^1] adds masks to your favorite
[Python Array API Standard compatible](https://data-apis.org/array-api/latest/)
array library.

Install with `pip`:

```shell
pip install marray
```

or `conda`:

```shell
conda install -c conda-forge marray-python
```

Use the `from...import...as` syntax to get a masked array namespace.

```python3
# use with any Array API compatible library, installed separately
from marray import numpy as mxp
import numpy as xp  # optional (if the non-masked namespace is desired)
```

The resulting `mxp` namespace has all the features of `xp` that are specified
in the Array API standard, but they are modified to be mask-aware. Typically, the
signatures of functions in the `mxp` namespace match those in the standard;
the one notable exception is the addition of a `mask` keyword argument of `asarray`.

```python3
mxp.asarray([1, 2, 3], mask=[False, True, False])
# MArray(array([1, 2, 3]), array([False,  True, False]))
```

In the spirit of the [Zen of Python](https://peps.python.org/pep-0020/), this is the one
and only obvious way to set the mask of an array.

Documentation provided by attributes of `xp` are exposed in the `mxp`
namespace and are accessible via `help`. For more information, please see
[the tutorial](https://colab.research.google.com/drive/1LaZCK3jvnf40qEjWEhqhhc5e8IWy7vuo?usp=sharing).

[^1]: The MArray logo is a nod to NumPy's logo, but MArray is not affiliated with the NumPy project.

%\VignetteIndexEntry{marrayInput Introduction}
%\VignetteKeywords{Preprocessing, Expression Analysis}
%\VignetteDepends{marray}
%\VignettePackage{marray}
\documentclass[11pt]{article}

\usepackage{amsmath,fullpage, graphicx}
\usepackage[authoryear,round]{natbib}
\usepackage{hyperref}


\newcommand{\Robject}[1]{{\texttt{#1}}}
\newcommand{\Rfunction}[1]{{\texttt{#1}}}
\newcommand{\Rpackage}[1]{{\textit{#1}}}

\parindent 0in

\bibliographystyle{abbrvnat}

\begin{document}

\title{\bf Introduction to the Bioconductor marray package : Input component}


\author{Yee Hwa Yang$^1$ and Sandrine Dudoit$^2$}

\maketitle
\begin{center}
1. Department of Medicine, University of California, San Francisco,\\
   {\tt jean@biostat.berkeley.edu}\\
2. Division of Biostatistics, University of California, Berkeley.
\end{center}


% library(tools) 
% setwd("C:/MyDoc/Projects/madman/Rpacks/marray/inst/doc")
% Rnwfile<-file.path("C:/MyDoc/Projects/madman/Rpacks/marray/inst/doc","marrayInput.Rnw")
% options(width=65)
% Sweave(Rnwfile,pdf=TRUE,eps=TRUE,stylepath=TRUE,driver=RweaveLatex())


\tableofcontents

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Overview}

This document provides a tutorial for the {\bf data input} component of
the {\tt marray} package.  This is similar to the previous {\tt
marrayInput} package which has now been combined with the suite of other
four packages for diagnostic plots and normalization of cDNA microarray
data.  This package relies on object--oriented class/method mechanism,
provided by the R {\tt methods} package, to allow efficient and
systematic representation and manipulation of microarray data. \\

This vignette describeds functionality for reading microarray data into
R, such as intensity data from image processing output files (e.g. {\tt
.spot} and {\tt .gpr} files for the {\tt Spot} and {\tt GenePix}
packages, respectively) and textual information on probes and targets
(e.g. from gal files and god lists). A {\tt tcltk} widget is supplied to
facilitate and automate data input and the creation of microarray
specific R objects for storing these data. 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Getting started}

To load the {\tt marray} package in your R session, type {\tt
library(marray)}.  We demonstrate the functionality of this R packages
using gene expression data from the Swirl zebrafish experiment which is
included as part of the package. To load the swirl dataset, use {\tt
data(swirl)}, and to view a description of the experiments and data,
type {\tt ?  swirl}.  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\section{Case study: Swirl zebrafish microarray experiment}\label{sData} 

We demonstrate the functionality of this collection of R packages using
gene expression data from the Swirl zebrafish experiment. These data
were provided by Katrin Wuennenberg--Stapleton from the Ngai Lab at UC
Berkeley. (The swirl embryos for this experiment were provided by David
Kimelman and David Raible at the University of Washington.)  This
experiment was carried out using zebrafish as a model organism to study
early development in vertebrates. Swirl is a point mutant in the BMP2
gene that affects the dorsal/ventral body axis. Ventral fates such as
blood are reduced, whereas dorsal structures such as somites and
notochord are expanded. A goal of the Swirl experiment is to identify
genes with altered expression in the swirl mutant compared to wild--type
zebrafish. Two sets of dye--swap experiments were performed, for a total
of four replicate hybridizations. For each of these hybridizations,
target cDNA from the swirl mutant was labeled using one of the Cy3 or
Cy5 dyes and the target cDNA wild--type mutant was labeled using the
other dye. Target cDNA was hybridized to microarrays containing 8,448
cDNA probes, including 768 controls spots (e.g. negative, positive, and
normalization controls spots). Microarrays were printed using $4 \times
4$ print--tips and are thus partitioned into a $4 \times 4$ grid
matrix. Each grid consists of a $22 \times 24$ spot matrix that was
printed with a single print--tip. Here, spot row and plate coordinates
should coincide, as each row of spots corresponds to probe sequences
from the same 384 well--plate.\\

Each of the four hybridizations produced a pair of 16--bit images, which
were processed using the image analysis software package {\tt Spot}
\citep{Spot,Image}.  The dataset includes four output files {\tt
swirl.1.spot}, {\tt swirl.2.spot}, {\tt swirl.3.spot}, and {\tt
swirl.4.spot} from the {\tt Spot} package.  Each of these files contains
8,448 rows and 30 columns; rows correspond to spots and columns to
different statistics from the {\tt Spot} image analysis output. The file
{\tt fish.gal} is a gal file generated by the {\tt GenePix} program; it
contains information on individual probe sequences, such as gene names,
spot ID, spot coordinates.  Hybridization information for the mutant and
wild--type target samples is stored in {\tt SwirlSample.txt}. All
fluorescence intensity data from processed images are also included in
this package (see Section \ref{sInput} for greater details).\\

To load the swirl dataset, use {\tt data(swirl)}, and to view a
description of the experiments and data, type {\tt ? swirl}.  Below,
we give step--by--step instructions for reading the swirl data into
R. For convenience, we have also stored the resuls in the object {\tt
swirl} of class {\tt marrayRaw}.

<<eval=TRUE>>=
library(marray)
data(swirl)
@


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\section{Package {\tt marrayInput} --  Reading microarray data into
  R}\label{sInput} 

We begin our analysis of microarray data with the fluorescence
intensities produced by image processing of the microarray scanned
images. These data are typically stored in tables whose rows correspond
to the spotted probe sequences and columns to different spot statistics:
e.g. grid row and column coordinates, spot row and column coordinates,
red and green background and foreground intensities for different
segmentation and background adjustment methods, spot morphology
statistics, etc. For the {\tt GenePix} image processing software, these
are the {\tt .gpr} files, and for {\tt Spot}, these are the {\tt .spot}
files. We also consider probe and target textual information stored, for
example, in {\tt .gal} and {\tt .gdl} (god list) files. The main
functions in the {\tt marrayInput} package are {\tt read.marrayLayout},
{\tt read.marrayInfo}, and {\tt read.marrayRaw}, which create objects of
classes {\tt marrayLayout}, {\tt marrayInfo}, and {\tt marrayRaw},
respectively. Widgets are provided for each of these functions to
facilitate data entry.\\

For the Swirl zebrafish experiment, textual information and fluorescence
intensity data from processed images were included as part of the
package and can be accessed as follows, where {\tt datadir} is the name
of the R package sub--directory containing the data files.

<<eval=TRUE>>=
datadir <- system.file("swirldata", package="marray")
dir(datadir)
@

In general, microarray data consist of three distinct components; probes
(genes) information, target(samples) information and measured gene
expression levels information. Analyzing expression intensities alone
with no corresponding probes or target information is
meaningless. Therefore a data structure \Robject{marrayRaw} is created
to store and link these information together in one R object.

\subsection{ Reading target information} 
We refer to {\it target file} as a file that lists the microarrays
hybridization and describes which RNA samples were hybridized to each
array. A target file is typically a tab-delimited text file which
include at least the complete and {\bf exact} name of each image
processing file you would like to include in the data analysis and the
corresponding names for the Cy3 and Cy5 labeled sample information. It
is also informative to include other variables of interest that are
useful for downstream analysis or for quality assessment. Examples
include subject identification number, gender, age, date of
hybridization, scanning conditions amongst others.\\

The main functions in the {\tt marray} package for this purpose is {\tt
read.marrayInfo}, which will create an R object of class {\tt
marrayInfo}.  Objects of class {\tt marrayInfo} may be used to store
information on probe sequences and target samples. For example, reading
in the target information for the swirl experiment can done with

<<eval=TRUE>>=
swirlTargets <- read.marrayInfo(file.path(datadir, "SwirlSample.txt"))
summary(swirlTargets)
@

\subsection{Reading probes related information}
Probes related information refers to descriptions of the spotted probe
sequences (e.g. matrix of gene names, annotation, notes on printing
conditions). Printing conditions or array fabrication information
include the dimensions of the spot and grid matrices, and, for each
probe on the array, its grid matrix and spot matrix coordinates. In
addition, we also include plate origin of the probes, and information on
the spotted control sequences (e.g. negative controls, housekeeping
genes, spiked in-control probes and many others). These information are
store separately using two objects: an object of class
\Robject{marrayInfo} on the probes annotation information and an object
of class {\tt marrayLayout} to store arrays fabrication information. 

There are two ways to read probes related information: the {\bf first}
method is to use the function {\tt read.Galfile} as follow:
<<eval=TRUE, echo=TRUE>>=
galinfo <- read.Galfile("fish.gal", path=datadir) 
names(galinfo)
@ 
Users can modify the arguments \Rfunction{info.id} and
\Rfunction{layout.id} to specify which column names or index represent
probe annotation and printer layout (array fabrication) information
respectively.  For example the following code reads in the galfile {\tt
fish2.gal} where probe information are stored under the columns {\tt
Gene ID} and {\tt Gene description} and the printeralayout information
is stored under the columns {\tt Grid}, {\tt Row} and {\tt Column}.

%% <<eval=FALSE, echo=TRUE>>=
\begin{verbatim}
> fish2Gal <- read.Galfile(galfile="fish2.txt", 
                           info.id = c("Gene ID", "Gene description"), 
                           layout.id = c(Block="Grid", Row="Row",
	            	   Column="Column"), labels="Gene ID")
\end{verbatim}
%%  @

This function returns a list of 3 components. The first \Robject{gnames}
is an \Robject{marrayInfo} object storing probe annotation information;
the second \Robject{layout} is an \Robject{marrayLayout} object storing
array fabrication (printing) information and lastly a numerical vector
\Robject{neworder} which provides a resorting of data. The probes are
assumed to be ordered and numbered consecutively starting from the top
left grid and the top left spot within each grid.  For most standard
array layout, we typically recommend using this method.\\

{\it Note:} The slot \Robject{maSub} is included to allow importing data
from non-complete arrays. \Robject{maSub} is a "logical" vector
indicating which spots are currently being stored in the slots
containing Cy3 and Cy5 background and foreground fluorescence
intensities.

The {\bf second} method uses both functions {\tt read.marrayLayout} and
{\tt read.marrayInfo} to read and store information on array fabrication
and probe annotation information respectively.  This is usually done for
more complex array structures.  For example, reading in the probe
annotation information for the swirl experiment can done with:

<<eval=TRUE, echo=TRUE>>= 
swirl.gnames <- read.marrayInfo(file.path(datadir, "fish.gal"),
info.id=4:5, labels=5, skip=21) 
summary(swirl.gnames) 
@ 

The following command stores such layout information in the object {\tt
swirl.layout} of class {\tt marrayLayout}.  The location of the control
spots is extracted from the fourth ({\tt ctl.col=4}) column of the file
{\tt fish.gal}.
<<eval=TRUE>>=
swirl.layout <- read.marrayLayout(fname=file.path(datadir, "fish.gal"),
                                  ngr=4, ngc=4, nsr=22, nsc=24,
                                  skip=21,ctl.col=4)
ctl<-rep("Control",maNspots(swirl.layout))
ctl[maControls(swirl.layout)!="control"]  <- "probes"
maControls(swirl.layout)<-factor(ctl)
summary(swirl.layout)
@

\subsection{Reading gene-expression data}
Microarray image processing results are stored in ASCII files and by
default, assumed to be tab-delimited. These can be loaded into R using
\Rfunction{read.marrayRaw} or customized functions like
\Rfunction{read.Spot}, \Rfunction{read.Agilent} and
\Rfunction{read.GenePix} for {\tt Spot}, {\tt Agilent} and {\tt GenePix}
output, respectively. The customized functions are simply ``wrapper''
functions around {\tt read.marrayRaw} which extract relevant spot
statistics for different image processing packages. In addition, these
functions will also setup the probe annotation and array layout
information. The following command illustrate illustrate how to read in
the raw expression data for the swirl data.

<<eval=TRUE, echo=TRUE>>=
mraw <- read.Spot(path=datadir, 
                  layout=galinfo$layout, 
                  gnames=galinfo$gnames, 
                  target=swirlTargets)
summary(mraw)
@

For any arbitrary image analysis output file, we can use the function
{\tt read.marrayRaw}.  The function takes as its main argument a list of
names for files containing the intensity data (e.g. {\tt GenePix} output
files {\tt .gpr}). It also takes as arguments the names of already
created layout, probe, and target description objects, e.g., {\tt
swirl.layout}, {\tt swirl.gnames}, and {\tt swirlTargets} for the Swirl
experiment. The following commands read in all the {\tt Spot} files
residing in the {\tt datadir} directory. The arguments further specify
that the red and green foreground intensities are stored under the
headings {\tt Rmean} and {\tt Gmean}, and that the red and green
background intensities are store under the headings {\tt morphR} and
{\tt morphG}, respectively.

%%<<eval=FALSE>>=
\begin{verbatim}
> fnames <- as.vector(swirlTargets@maInfo[,1])
> swirl.raw <- read.marrayRaw(fnames, path = datadir,
                            name.Gf = "Gmean", name.Gb = "morphG",
                            name.Rf = "Rmean", name.Rb = "morphR",
                            layout = swirl.layout,
                            gnames = swirl.gnames,
                            targets = swirlTargets
                            )
\end{verbatim}
%%@


 \section{Other input functions}

{\bf Widget input functions} \\

To facilitate the creation of microarray
 data objects, each of these three input functions has a corresponding
 {\tt tcltk} widget: {\tt widget.marrayLayout}, {\tt widget.marrayInfo},
 and {\tt widget.marrayRaw}.  A screen--shot of the {\tt marrayRaw}
 widget is shown in Figure \ref{fig:WidgetRaw}; the command to launch
 the widget is as follows (here, {\tt ext} specifies the image output
 file extension):

\begin{verbatim}
>  widget.marrayRaw(path=datadir, ext="spot")
\end{verbatim}


{\bf Wrapper input functions}\\
As mentioned before, for users who prefer command line input for a
specific class of image processing output files, we have defined three
additional functions. The functions {\tt read.Spot}, {\tt read.GenePix},
{\tt read.Agilent} and {\tt read.SMD} automate the creation of {\tt
marrayRaw} objects from {\tt Spot}, {\tt GenePix} and {\tt Agilent}
image analysis files, and from the Stanford Microarray Database (SMD)
raw data files ({\tt .xls}). The main arguments to these functions are a
list of files and the directory path of the files.  The following
commands read two specific files from the {\tt datadir} directory.

%RG: changed to get a build
%% %<<eval=FALSE>>=
\begin{verbatim}
> fnames <- dir(path=datadir,pattern=paste("*", "spot", sep="\."))[1:2]
> swirl <- read.Spot(fnames, path=datadir,
          layout = swirl.layout,
          gnames = swirl.gnames,
          targets = swirl.samples)
\end{verbatim}
%%%@

Alternatively, without specifying any arguments, the functions {\tt
read.spot} and {\tt read.GenePix} by default will read in all {\tt
Spot} or {\tt GenePix} files within a current working directory. One
has the option of setting the layout, probe, and target information
manually at a later stage.

%<<eval=FALSE>>=
\begin{verbatim}
> swirl <- read.Spot()
> test.raw <- read.GenePix()
\end{verbatim}
%@

\begin{figure}[htbp]
\centering
\includegraphics{widget1}
\caption{Screenshot of the widget for creating objects of class {\tt
    marray} from image processing output files.}
    \protect\label{fig:WidgetRaw}
\end{figure}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
{\bf Note: Sweave.} This document was generated using the \Rfunction{Sweave}
function from the R \Rpackage{tools} package. The source file is in the
\Rfunction{/inst/doc} directory of the package \Rpackage{marray}.

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\bibliography{marrayPacks}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
\end{document}

\name{boxplot}
\docType{methods}
\alias{boxplot,marrayRaw-method}
\alias{boxplot,marrayNorm-method}



\title{Boxplots for cDNA microarray spot statistics}

\description{
  The function \code{boxplot} produces boxplots of microarray spot
  statistics for the classes  \code{"\link{marrayRaw}"},
  \code{"\link{marrayNorm}"}.
  We encourage users to use \code{boxplot} rather than \code{maBoxplot}.
  The name of the arguments have changed slightly.
}

\usage{
\S4method{boxplot}{marrayRaw}(x, xvar="maPrintTip", yvar="maM", ...)
\S4method{boxplot}{marrayNorm}(x, xvar="maPrintTip", yvar="maM", ...)
}
%- maybe also `usage' for other objects documented here.

\arguments{
  \item{x}{Microarray object of class \code{"\link{marrayRaw}"}, \code{"\link{marrayNorm}"}}
  \item{xvar}{Name of accessor method for the spot statistic used to
    stratify the data, typically a slot name for the microarray layout
    object (see \code{"\link{marrayLayout}"}) such as \code{maPlate} or
    a method such as \code{maPrintTip}. If \code{x} is NULL, the data
    are not stratified.} 
  \item{yvar}{Name of accessor method for the spot statistic of
    interest, typically a slot name for the microarray object \code{m},
    such as \code{maM}.} 
  \item{\dots}{Optional graphical parameters, see \code{\link{par}}.}
}

\details{
If there are more than one array in the batch, the function produces a
boxplot for each array in the batch. Such plots are useful when
assessing the need for between array normalization, for example, to deal
with scale differences among different arrays. Default graphical
parameters are chosen for convenience using the function
\code{\link{maDefaultPar}} (e.g. color palette,  axis labels,  plot
title) but the user has the option to overwrite these parameters at any
point. 
}

\references{
S. Dudoit and Y. H. Yang. (2002). Bioconductor R packages for
exploratory analysis and normalization of cDNA microarray data. In
G. Parmigiani, E. S. Garrett, R. A. Irizarry and S. L. Zeger, editors,
\emph{The Analysis of Gene Expression Data: Methods and Software},
Springer, New York. 
}

\author{Jean Yang and Sandrine Dudoit}

\seealso{\code{\link{maBoxplot}}, \code{\link{maDefaultPar}}.}

\examples{

# To see the demo type demo(marrayPlots)

# Examples use swirl dataset, for description type ? swirl
data(swirl)

# Boxplots of pre-normalization log-ratios M for each of the 16 
# print-tip-groups for the Swirl 93 array. 
# - Default arguments
boxplot(swirl[,3])  

# All spots 
boxplot(swirl[,3], xvar=NULL, col="green")  

# Boxplots of pre-normalization red foreground intensities for each grid row
# for the Swirl 81 array. 
boxplot(swirl[,1], xvar="maGridRow", yvar = "maRf", main = "Swirl array 81: pre-normalization red foreground intensity") 

# Boxplots of pre-normalization log-ratios for each array in swirl
boxplot(swirl, main="Swirl arrays: pre-normalization log-ratios")
}
\keyword{hplot}% at least one, from doc/KEYWORDS

\name{plot}
\docType{methods}
\alias{plot.marrayRaw}
\alias{plot.marrayNorm}
\alias{addText}
\alias{addLines}
\alias{addPoints}
\alias{text,marrayRaw-method}
\alias{text,marrayNorm-method}
\alias{lines,marrayRaw-method}
\alias{lines,marrayNorm-method}
\alias{points,marrayRaw-method}
\alias{points,marrayNorm-method}
%- Also NEED an `\alias' for EACH other topic documented here.
\title{Scatter-plots for cDNA microarray spot statistics}
\description{The function \code{maPlot} or \code{plot} produces scatter-plots of
  microarray spot statistics for the classes \code{"\link{marrayRaw}"},
  \code{"\link{marrayNorm}"}. It also allows the user to highlight and
  annotate subsets of points on the plot, and display fitted curves from
  robust local regression or other smoothing procedures.}
\usage{
\method{plot}{marrayRaw}(x, xvar = "maA", yvar = "maM", zvar="maPrintTip", lines.func,text.func,legend.func, \dots)
\method{plot}{marrayNorm}(x, xvar = "maA", yvar = "maM", zvar="maPrintTip", lines.func,text.func,legend.func, \dots)
addText(object, xvar="maA", yvar="maM", subset=NULL, labels=as.character(1:length(subset)), \dots)
addPoints(object, xvar="maA", yvar="maM", subset=TRUE, \dots)
addLines(object, xvar="maA", yvar="maM", zvar="maPrintTip", subset=TRUE, \dots)
\S4method{text}{marrayRaw}(x, xvar = "maA", yvar = "maM", \dots)
\S4method{text}{marrayNorm}(x, xvar = "maA", yvar = "maM", \dots)
\S4method{lines}{marrayRaw}(x, xvar = "maA", yvar = "maM", zvar = "maPrintTip", \dots)
\S4method{lines}{marrayNorm}(x, xvar = "maA", yvar = "maM", zvar = "maPrintTip",\dots)
\S4method{points}{marrayRaw}(x, xvar = "maA", yvar = "maM", \dots)
\S4method{points}{marrayNorm}(x, xvar = "maA", yvar = "maM", \dots)
}
%- maybe also `usage' for other objects documented here.
\arguments{
  \item{x}{Microarray object of class \code{"\link{marrayRaw}"}, \code{"\link{marrayNorm}"}.}
  \item{object}{Microarray object of class \code{"\link{marrayRaw}"}, \code{"\link{marrayNorm}"}.}
  \item{xvar}{Name of accessor function for the abscissa spot statistic, typically a slot name for the microarray object \code{x}, such as \code{maA}.}
  \item{yvar}{Name of accessor function for the ordinate spot statistic, typically a slot name for the microarray object \code{x}, such as \code{maM}.}
  \item{zvar}{Name of accessor method for the spot statistic used to stratify the data, typically a slot name for the microarray layout object (see \code{"\link{marrayLayout}"}) such as \code{maPlate} or a method such as \code{maPrintTip}. If \code{zvar} is \code{NULL}, the data are not stratified.}
  \item{lines.func}{Function for computing and plotting smoothed fits of \code{y} as a function of \code{x}, separately within values of \code{zvar}, e.g. \code{\link{maLoessLines}}. If \code{lines.func} is \code{NULL}, no fitting is performed.}
  \item{text.func}{Function for highlighting a subset of points, e.g., \code{\link{maText}}. If \code{text.func} is \code{NULL}, no points are highlighted.}
  \item{legend.func}{Function for adding a legend to the plot,  e.g. \code{\link{maLegendLines}}. If \code{legend.func} is \code{NULL}, there is no legend.}
  \item{subset}{logical vector or numeric values indicating the subset of points to be plotted.}
  \item{labels}{One or more character strings or expressions specifying the text to be written.}
  \item{\dots}{Optional graphical parameters, see \code{\link{par}}.}
}
\details{
This function calls the general function \code{\link{maPlot.func}},
which is not specific to microarray data. If there are more than one
array in the batch, the plot is done for the first array, by
default. Default graphical parameters are chosen for convenience using
the function \code{\link{maDefaultPar}} (e.g. color palette,  axis
labels,  plot title) but the user has the option to overwrite these
parameters at any point.
}
\references{
S. Dudoit and Y. H. Yang. (2002). Bioconductor R packages for exploratory analysis and normalization of cDNA microarray data. In G. Parmigiani, E. S. Garrett, R. A. Irizarry and S. L. Zeger, editors, \emph{The Analysis of Gene Expression Data: Methods and Software}, Springer, New York.
}
\author{Jean Yee Hwa Yang}
\seealso{\code{\link{maPlot.func}}, \code{\link{maDefaultPar}}, \code{\link{maLoessLines}}, \code{\link{maLegendLines}}, \code{\link{maText}}, \code{\link{plot}}, \code{\link{lowess}}, \code{\link{loess}}, \code{\link{legend}}.}
\examples{
# To see the demo type demo(marrayPlots)

# Examples use swirl dataset, for description type ? swirl
data(swirl)

# Pre-normalization MA-plot for the Swirl 93 array, with the lowess fits for 
# individual print-tip-groups. 
# - Default arguments
plot(swirl[,3])

# Lowess fit using all spots
plot(swirl[,3], zvar=NULL, legend.func=NULL)

# Loess fit using all spots
plot(swirl[,3], zvar=NULL, legend.func=maLegendLines(legend="All spots",col="green"), lines.func=maLoessLines(loess.args=list(span=0.3),col="green"))
}
\keyword{hplot}% at least one, from doc/KEYWORDS

\name{cbind}
\alias{cbind.marrayRaw}
\alias{cbind.marrayNorm}
\alias{rbind.marrayInfo}

\title{Combine marrayRaw,  marrayNorm or marrayInfo Objects}
\description{
  Combine a series of \code{marrayRaw}, \code{marrayNorm} and
  \code{marrayInfo} objects.
}

\usage{
\method{cbind}{marrayRaw}(\dots, deparse.level=1)
\method{cbind}{marrayNorm}(\dots, deparse.level=1)
\method{rbind}{marrayInfo}(\dots, deparse.level=1)
}
\arguments{
  \item{\dots}{\code{marrayRaw} objects or \code{marrayNorm} objects}
  \item{deparse.level}{not currently used, see \code{\link[base]{cbind}} in the base package}
}
\details{
\code{cbind} combines data objects assuming the same gene lists but
different arrays. 
\code{rbind} combines data objects assuming equivalent arrays, i.e., the
same RNA targets, but different genes. 

For \code{cbind}, the matrices o f expression data from the individual
objects are cbinded. 
The data.frames of target information, if they exist, are rbinded. 
The combined data object will preserve any additional components or
attributes found in the first object to be combined. 
For \code{rbind}, the matrices of expression data are rbinded while the
target information, in any, is unchanged. 
}
\author{Jean Yang}
\seealso{
  \code{\link[base]{cbind}} in the base package.
}
\keyword{manip}

\name{read.Galfile}
\alias{read.Galfile}

\title{Reading GenePix Gal file}
\description{
  Reading a standard Gal file containing gene information.
}
\usage{
read.Galfile(galfile, path = ".", info.id = c("ID", "Name"),
layout.id =c(Block="Block", Row="Row", Column="Column"),
labels = "ID", notes = "", sep = "\t", skip = NULL,   ncolumns=4, ...)
}
%- maybe also `usage' for other objects documented here.
\arguments{
  \item{galfile}{a character string representing the Gal file.}
  \item{path}{a character string representing the data directory. By
    default this is set to the current working directory ("."). }
  \item{info.id}{the column numbers or names in `fname' that contain the required
    information.}
  \item{layout.id}{the column names in `fname' that specified the
    printer layout information.}
  \item{labels}{the column number in \code{fname} which contains the names that the user would like to use to label spots or arrays (e.g. for default titles in \code{\link{maPlot}}.}
  \item{notes}{object of class character, vector of explanatory text}
  \item{sep}{the field separator character.  Values on each line of the
    file are separated by this character. The
    default is to read a tab delimited file.}
  \item{skip}{the number of lines of the data file to skip before
    beginning to read data.}
  \item{ncolumns}{an integer representing the number of columns of
    sub-array (print-tips) on a slides.}
  \item{\dots}{further arguments to \code{\link{scan}}.}
}

\value{
  \item{gnames}{An object of class \code{\link{marrayInfo}}.}
  \item{layout}{An object of class \code{\link{marrayLayout}}.}
}

\author{Yee Hwa (Jean) Yang}

\seealso{\code{\link{read.marrayInfo}}, \code{\link{read.marrayLayout}}}

\examples{
library(marray)
datadir <- system.file("swirldata", package="marray")
try <- read.Galfile(galfile="fish.gal", path=datadir)
names(try)
try$layout
try$gnames
}

\keyword{file}% at least one, from doc/KEYWORDS

\name{maDefaultPar}
\alias{maDefaultPar}
%- Also NEED an `\alias' for EACH other topic documented here.
\title{Default graphical parameters for microarray objects}
\description{
This function returns default graphical parameters for microarray objects. The parameters may be passed as arguments to the functions \code{maBoxplot} and \code{maPlot}.
} 

\usage{
maDefaultPar(m, x, y, z)
}
%- maybe also `usage' for other objects documented here.

\arguments{
  \item{m}{Microarray object of class \code{"\link{marrayRaw}"} and \code{"\link{marrayNorm}"}.}
  \item{x}{Name of accessor method for the abscissa spot statistic,
    typically a slot name for the microarray object \code{m}, such as
    \code{maA}.} 
  \item{y}{Name of accessor method for the ordinate spot statistic,
    typically a slot name for the microarray object \code{m}, such as
    \code{maM}.} 
  \item{z}{Name of accessor method for the spot statistic used to
    stratify the data, typically a slot name for the microarray layout
    object (see \code{"\link{marrayLayout}"}) such as \code{maPlate} or
    a method such as \code{maPrintTip}.} 
}

\value{
A list with elements
  \item{def.box}{default graphical parameters for \code{\link{maBoxplot}}.}
  \item{def.plot}{default graphical parameters for \code{\link{maPlot}}.}
  \item{def.lines}{default graphical parameters for functions such as \code{\link{maLoessLines}} used in \code{\link{maPlot}}.}  
  \item{def.legend}{default graphical parameters for functions such as \code{\link{maLegendLines}} used in \code{\link{maPlot}}.}   
  \item{def.text}{default graphical parameters for functions such as \code{\link{maText}} used in \code{\link{maPlot}}.} 

}

\references{S. Dudoit and Y. H. Yang. (2002). Bioconductor R packages for exploratory analysis and normalization of cDNA microarray data. In G. Parmigiani, E. S. Garrett, R. A. Irizarry and S. L. Zeger, editors, \emph{The Analysis of Gene Expression Data: Methods and Software}, Springer, New York.
}


\author{Sandrine Dudoit, \url{http://www.stat.berkeley.edu/~sandrine}.}

\seealso{\code{\link{maBoxplot}}, \code{\link{maPlot}},
  \code{\link{maLegendLines}}, \code{\link{maLoessLines}},
  \code{\link{maText}}, \code{\link{maDotsDefaults}}.} 

\examples{
# See examples for maPlot.

}
\keyword{iplot}% at least one, from doc/KEYWORDS
\keyword{dplot}% __ONLY ONE__ keyword per line
\keyword{color}% __ONLY ONE__ keyword per line

\name{na}

\alias{log.na}
\alias{sum.na}
\alias{mean.na}
\alias{var.na}
\alias{cor.na}
\alias{quantile.na}
\alias{length.na}
\alias{order.na}
\alias{scale.na}
\alias{prod.na}

\title{Basic Statistical Functions for Handling Missing Values}

\description{
Basic statistical functions for handling missing values or NA. \cr 
In \code{log.na}, \code{sum.na}, \code{mean.na} and \code{var.na},
\code{quantile.na}, \code{length.na}, missing values are omitted
from the calculation. \cr 
The function \code{cor.na} calls \code{cor} with the argument
\code{use="pairwise.complete.obs"}. \cr 
The function \code{order.na} only handles vector arguments and not
lists.  However, it gives the option of omitting the NAs
(\code{na.last=NA}), of placing the NAs at the start of the ordered
vector (\code{na.last=F}) or at the end (\code{na.last=T}). \cr 
The function \code{scale.na} is a modified version of
\code{\link{scale}} which allows NAs in the variance calculation. If
\code{scale = T}, the function \code{f} in \code{scale.na} uses
\code{var.na} to perform the variance calculation.
The function \code{prod.na} is similar to the \code{\link{prod}}
function with \code{na.rm=TRUE}. This function returns the product of
all the values present in its arguments, omitting any missing values.
}

\author{
  Yee Hwa Yang, \email{jean@biostat.berkeley.edu} \cr
}

\seealso{\code{\link{log}}, \code{\link{sum}}, \code{\link{mean}},
  \code{\link{var}}, \code{\link{cor}}, \code{\link{order}},
  \code{\link{scale}}, \code{\link{prod}}.}

%%\keyword{log, sum, mean, variance, correlation, order, scale, product, missing values, NA.} 
\keyword{arith} 

\name{maGenControls}
\alias{maGenControls}
\alias{controlCode}

\title{Generating a vector recording the control status of the spotted probe sequences. }
\description{
  ControlCode is a matrix representing certain regular expression
  pattern and the control status of the spotted probe sequences.
  This function uses `grep' searches for matches to `pattern' (its first argument)
  within the character vector `x' (second argument). 
}

\usage{
maGenControls(Gnames, controlcode, id = "ID")
}

\arguments{
  \item{Gnames}{An object of class \code{matrix}, \code{data.frame} or \code{marrayInfo}
    which contains  description of spotted probe sequences.}
  \item{controlcode}{A character matrix of n by 2 columns. The first
    column contains a few regular expression of spotted probe sequences
    and the second column contains the corresponding control status.}
  \item{id}{the column number of column name in \code{Gnames} that
    contains description of each spot on the array.}
}

\value{
  A vector of characters recording the control status of the spotted probe sequences.
}

\author{Jean Yee Hwa Yang}

\seealso{\code{\link{grep}}}

\examples{
data(swirl)
maControls(swirl) <- maGenControls(maGnames(swirl), id="Name")
table(maControls(swirl))
}
\keyword{manip}% at least one, from doc/KEYWORDS

\name{Internal functions}
\alias{read.fname}
\alias{stat.gene.cluster}
\alias{plot.gene.cluster}
\alias{tablegen}
\alias{gsubAnchor}

\title{Internal marray functions}
 
\description{Internal marray functions}
 
\details{These are not to be called by the user.}

\keyword{methods}

\name{maText}
\alias{maText}


%- Also NEED an `\alias' for EACH other topic documented here.
\title{Highlight points on a plot}
\description{
This function may be used to highlight a subset of points on an existing
plot, such as a plot produced by \code{\link{plot}},
\code{\link{maPlot}}, or \code{\link{maPlot.func}}. 
}

\usage{
maText(subset=NULL, labels=as.character(1:length(subset)), ...)
}
%- maybe also `usage' for other objects documented here.

\arguments{
  \item{subset}{A "logical" or "numeric" vector indicating the subset of points to
  highlight.}
\item{labels}{One or more character strings or expressions specifying the text to be written.}
  \item{\dots}{Optional graphical parameters, see \code{\link{par}}.}
}

\value{
A function with bindings for \code{subset}, \code{labels}, and \code{\ldots}. This latter function takes as arguments \code{x} and \code{y}, the absissa and ordinates of points on the plot.
}

\references{S. Dudoit and Y. H. Yang. (2002). Bioconductor R packages for exploratory analysis and normalization of cDNA microarray data. In G. Parmigiani, E. S. Garrett, R. A. Irizarry and S. L. Zeger, editors, \emph{The Analysis of Gene Expression Data: Methods and Software}, Springer, New York.
}

\author{Sandrine Dudoit, \url{http://www.stat.berkeley.edu/~sandrine}.}

\seealso{\code{\link{text}}, \code{\link{maPlot}}, \code{\link{maPlot.func}}.}

\examples{
# See examples for maPlot.


}
\keyword{aplot}% at least one, from doc/KEYWORDS

\name{maLegendLines}
\alias{maLegendLines}
%- Also NEED an `\alias' for EACH other topic documented here.
\title{Add a legend to a plot}
\description{
This function may be used to add a legend for lines in plots such as those produced by \code{\link{plot}}, \code{\link{maPlot}}, or \code{\link{maPlot.func}}.
}

\usage{
maLegendLines(legend="", col=2, lty=1, lwd=2.5, ncol=1, ...)
}
%- maybe also `usage' for other objects documented here.

\arguments{
  \item{legend}{A vector of "character" strings to appear in the legend.}
  \item{col}{Line colors for the legend.}
  \item{lty}{Line types for the legend.}
  \item{lwd}{Line widths for the legend.}
  \item{ncol}{The number of columns in which to set the legend items (default is 1, a vertical legend).}
  \item{\dots}{Optional graphical parameters, see \code{\link{par}}.}
}


\value{
A function with bindings for \code{legend}, \code{col}, \code{lty}, \code{lwd}, \code{ncol}, and \code{\ldots}. This latter function takes as arguments \code{x} and \code{y}, the coordinates for the location of the legend on the plot, and it adds the legend to the current plot.
}

\references{S. Dudoit and Y. H. Yang. (2002). Bioconductor R packages for exploratory analysis and normalization of cDNA microarray data. In G. Parmigiani, E. S. Garrett, R. A. Irizarry and S. L. Zeger, editors, \emph{The Analysis of Gene Expression Data: Methods and Software}, Springer, New York.
}


\author{Sandrine Dudoit, \url{http://www.stat.berkeley.edu/~sandrine}.}

\seealso{\code{\link{legend}}, \code{\link{maPlot}}, \code{\link{maPlot.func}}.}

\examples{
# See examples for maPlot.

}
\keyword{aplot}% at least one, from doc/KEYWORDS

\name{read.marrayLayout}
\alias{read.marrayLayout}
\alias{widget.marrayLayout}

\title{Create objects of class marrayLayout}
\description{
  This function creates objects of class \code{\link{marrayLayout}} to store layout parameters for  two-color cDNA microarrays.
}
\usage{
read.marrayLayout(fname = NULL, ngr, ngc, nsr, nsc, pl.col = NULL, ctl.col = NULL, sub.col = NULL, notes = fname, skip, sep = "\t", quote = "\"", ...)
}

%- maybe also `usage' for other objects documented here.
\arguments{
  \item{fname}{the name of the file that stores plate and control
    information. This is usually a file obtained from a database.}
  \item{ngr}{the number of rows of grids per image.}
  \item{ngc}{the number of columns of grids per image.}
  \item{nsr}{the number of rows of spots per grid.}
  \item{nsc}{the number of columns of spots per grid.}
  \item{pl.col}{the column number in \code{fname} that contains plate information.}
  \item{ctl.col}{the column number in \code{fname} that contains control information.}
    \item{sub.col}{the column number in \code{fname} that contains full ID information.}
  \item{notes}{object of class character, vector of explanatory text.}
  \item{skip}{the number of lines of the data file to skip before
    beginning to read data.} 
  \item{sep}{the field separator character.  Values on each line of the
    file are separated by this character. The
    default is to read a tab delimited file.}
  \item{quote}{the set of quoting characters. By default, this is disable
  by setting `quote="\""'.}
  \item{\dots}{further arguments to \code{\link{scan}}.}
}

\value{
  An object of class \code{\link{marrayLayout}}.
}

\references{http://www.bioconductor.org/ }
\author{Jean Yang \email{yeehwa@stat.berkeley.edu}}

\examples{
datadir <- system.file("swirldata", package="marray")

### Reading in control information from file
skip <-  grep("Row", readLines(file.path(datadir,"fish.gal"), n=100)) - 1
swirl.layout <- read.marrayLayout(fname=file.path(datadir,"fish.gal"), ngr=4, ngc=4,
nsr=22, nsc=24, ctl.col=4, skip=skip)

### Setting control information.
swirl.gnames <- read.marrayInfo(file.path(datadir,"fish.gal"), info.id=4:5, labels=5, skip=21)
x <-  maInfo(swirl.gnames)[,1]
y <- rep(0, maNspots(swirl.layout))
y[x == "control"] <- 1
slot(swirl.layout, "maControls") <- as.factor(y)
}

\keyword{file}% at least one, from doc/KEYWORDS
\keyword{connection}

\name{maInd2Coord}
\alias{maInd2Coord}
%- Also NEED an `\alias' for EACH other topic documented here.
\title{Convert spot indices to grid and spot matrix coordinates}
\description{
This functions converts spot indices to grid and spot matrix coordinates (four
coordinates), where spots on the array are numbered consecutively starting
from the top left grid and the top left spot within each grid.
}
\usage{
maInd2Coord(x, L)
}
%- maybe also `usage' for other objects documented here.
\arguments{
  \item{x}{a numeric vector of spot indices.}
  \item{L}{an object of class \code{"\link{marrayLayout}"}.}
}
\value{
a matrix of spot four-coordinates, with rows corresponding to
  spots and columns to grid row, grid column, spot row, and spot column coordinates.
}

\author{Yee Hwa (Jean) Yang, Sandrine Dudoit, \url{http://www.stat.berkeley.edu/~sandrine}.}

\seealso{\code{\link{marrayLayout}}, \code{\link{maCoord2Ind}},
\code{\link{maCompCoord}}, \code{\link{maCompInd}}.}

\examples{

L <- new("marrayLayout", maNgr=4, maNgc=4, maNsr=22, maNsc=24)
maInd2Coord(c(1:10,529:538), L)


}
\keyword{manip}
\keyword{methods}

\name{maColorBar}
\alias{maColorBar}
%- Also NEED an `\alias' for EACH other topic documented here.
\title{Calibration bar for color images}
\description{
This function produces a color image (color bar) which can be used for
the legend to another color image obtained from the functions
\code{\link{image}}, \code{\link{maImage}}, or
\code{\link{maImage.func}}. 
}
\usage{
maColorBar(x, horizontal=TRUE, col=heat.colors(50), scale=1:length(x), k=10, ...)
}
%- maybe also `usage' for other objects documented here.
\arguments{
  \item{x}{If "numeric", a vector containing the "z" values in the
    color image, i.e., the values which are represented in the color
    image. Otherwise, a "character" vector representing colors.}
  \item{horizontal}{If \code{TRUE}, the values of \code{x} are
    represented as vertical color strips in the image, else, the values
    are represented as horizontal color strips.} 
  \item{col}{Vector of colors such as that generated by
    \code{rainbow}, \code{heat.colors},
    \code{topo.colors}, \code{terrain.colors}, or similar
    functions. In addition to these color palette functions, a new
    function  \code{\link{maPalette}} was defined to generate color
    palettes from user supplied low, middle, and high color values.} 
  \item{scale}{A "numeric" vector specifying the "z" values in the color
    image. This is used when the argument \code{x} is a
    "character" vector representing color information.}
  \item{k}{Object of class "numeric", for the number of labels displayed on 
    the bar.}
  \item{\dots}{Optional graphical parameters, see \code{\link{par}}.}
}
\references{
S. Dudoit and Y. H. Yang. (2002). Bioconductor R packages for exploratory analysis and normalization of cDNA microarray data. In G. Parmigiani, E. S. Garrett, R. A. Irizarry and S. L. Zeger, editors, \emph{The Analysis of Gene Expression Data: Methods and Software}, Springer, New York.
}
\author{Sandrine Dudoit, \url{http://www.stat.berkeley.edu/~sandrine}, Yee Hwa (Jean) Yang.}
\seealso{\code{\link{image}}, \code{\link{maImage}}, \code{\link{maImage.func}}, \code{\link{maPalette}}.}
\examples{
par(mfrow=c(3,1))
Rcol <- maPalette(low="white", high="red", k=10)
Gcol <- maPalette(low="white", high="green", k=50)
RGcol <- maPalette(low="green", high="red", k=100)
maColorBar(Rcol)
maColorBar(Gcol, scale=c(-5,5))
maColorBar(1:50, col=RGcol)

par(mfrow=c(1,3))
x<-seq(-1, 1, by=0.01)
maColorBar(x, col=Gcol, horizontal=FALSE, k=11)
maColorBar(x, col=Gcol, horizontal=FALSE, k=21)
maColorBar(x, col=Gcol, horizontal=FALSE, k=51)
}
\keyword{aplot}% at least one, from doc/KEYWORDS
\keyword{hplot}% __ONLY ONE__ keyword per line

\name{stat.confband.text}

\alias{stat.confband.text}

\title{Rank genes according to the value of a statistic.}
  
\description{
  Select values based on intensities binning.
}

\usage{
stat.confband.text(M, A, crit1=0.025, crit2=crit1, nclass=5)
}

\arguments{
 \item{A}{a vector giving the x-coordinates of the points in the scatter
          plot. In the microarray context, this could be a vector of
          average log intensities. ie A} 
 \item{M}{a vector giving the y-coordinates of the points in the scatter
   plot. In the microarray context, this could be a vector of log
   intensity ratios.} 
 \item{crit1}{The number of points to be selected.
   If crit1 < 1, the crit1*100\% spots with the smallest M values
   will be selected. If crit1 >= 1, the crit spots  with the
   smallest M values are selected.}  
 \item{crit2}{Similar to "crit1".   If crit2 < 1, the crit2*100\%
   spots with the largest M values will be selected. If crit2 >= 1,
   the crit2 spots with the largest M values are selected.}  
 \item{nclass}{A single number giving the approximate number of
   intensity depedent groups to consider.} 
% \item{labs}{ one or more character strings or expressions specifying the
%   text to be written.  If this string is not specified, by
%   default the index of the vector `M' will be used.}
% \item{output}{logical, defaulting to `FALSE'. If `TRUE' a vector
%   containning the index to the vector `M' that are  
%   highlighted.} 
}


\value{
  A vector of selected spot index.
}

\examples{
library(marray)
data(swirl)
aveA <- apply(maA(swirl), 1, mean.na)
aveM <- apply(maM(swirl), 1, mean.na)
stat.confband.text(aveM, aveA, crit1=20, crit2=50, nclass=5)
}

\seealso{ \code{\link{stat.gnames}}}

\keyword{manip}

\name{maSelectGnames}
\alias{maSelectGnames}

\title{Select genes according to the values of a few different statistics}
\description{
  Select genes by considering the \code{\link{union}} or
  \code{\link{intersect}} of multiple statistics.
}

\usage{
maSelectGnames(statdata, crit1 = 50, crit2 = crit1, sub = TRUE, selectstat, operate = c("intersect", "union"))
}

\arguments{
  \item{statdata}{A numerical matrix where the rows corresponds to genes
    and the columns corresponds to various statistics corresponding to a
    particular gene.}
  \item{crit1}{The number of points to be selected.
   If crit1 < 1, the crit1*100\% spots with the smallest M values
   will be selected. If crit1 >= 1, the crit spots  with the
   smallest M values are selected.}  
 \item{crit2}{Similar to "crit1".   If crit2 < 1, the crit2*100\%
   spots with the largest M values will be selected. If crit2 >= 1,
   the crit2 spots with the largest M values are selected.}  
 \item{sub}{A "logical" or "numeric" vector indicating the subset of
   genes to be consider.}
 \item{selectstat}{A integer value indicating the statistics where the
   final ranking is based on.}
 \item{operate}{The operation used to combined different rankings}
}
\details{
  This functions calls \code{\link{stat.gnames}} to select say the 100
  most extreme genes from various statistics and combined the different
  gene lists by either union or intersection.
}
\value{
  A vector of numeric values.
}

\author{Jean Yee Hwa Yang}

\seealso{\code{\link{stat.gnames}}, \code{\link{order}}}

\examples{
X <- matrix(rnorm(1000), 100,10)
Xstat <- cbind(mean=apply(X, 1, mean, na.rm=TRUE),
               var=apply(X, 1, var, na.rm=TRUE))
maSelectGnames(Xstat, crit1=50)
}
\keyword{manip}% at least one, from doc/KEYWORDS

\name{checkTargetInfo}
\alias{checkTargetInfo}
\title{Verifying the order between intensities matrix and target file information}
\description{
  Check that the foreground and backgruond intensities are stored in the
  same order as provided in the first column of target file. 
}
\usage{
checkTargetInfo(mraw)
}
\arguments{
  \item{mraw}{Object of class \code{marrayRaw} or \code{marryNorm}.}
}
\value{
  A logical value.  This function returns "TRUE" if the first column from
  the Target information is the same order as the foreground and
  backgruond intensities.  
}

\author{Yee Hwa (Jean) Yang}

\examples{
datadir <- system.file("swirldata", package="marray")
swirl.targets <- read.marrayInfo(file.path(datadir, "SwirlSample.txt"))
data(swirl)
swirl@maTargets <- swirl.targets

checkTargetInfo(swirl)

checkTargetInfo(swirl[, 2:4])

## reorder
swirl@maTargets <- swirl.targets[c(2:4, 1),]
checkTargetInfo(swirl)
}

\keyword{file}% at least one, from doc/KEYWORDS

\name{maNorm}
\alias{maNorm}
\title{Simple location and scale normalization function}
\description{
This function is a simple wrapper function around the main normalization function \code{\link{maNormMain}}. It allows the user to choose from a set of six basic location and scale normalization procedures. The function operates on an object of class \code{"\link{marrayRaw}"} (or possibly \code{"\link{marrayNorm}"}, if normalization is performed in several steps) and returns an object of class \code{"\link{marrayNorm}"}.
}
\usage{
maNorm(mbatch, norm=c("printTipLoess", "none", "median", "loess",
"twoD", "scalePrintTipMAD"), subset=TRUE, span=0.4, Mloc=TRUE,
Mscale=TRUE, echo=FALSE, ...)
}
\arguments{
  \item{mbatch}{Object of class \code{\link{marrayRaw}}, containing intensity 
data for the batch of arrays to be normalized. 
An object of class \code{"\link{marrayNorm}"} may also be passed if 
normalization is performed in several steps.}
\item{norm}{Character string specifying the normalization procedures: 
\describe{
\item{none}{no normalization}
\item{median}{for global median location normalization}
\item{loess}{for global intensity or A-dependent location normalization using 
the \code{\link{loess}} function}
\item{twoD}{for 2D spatial location normalization using the 
\code{\link{loess}} function}
\item{printTipLoess}{for within-print-tip-group intensity dependent location 
normalization using the \code{\link{loess}} function}
\item{scalePrintTipMAD}{for within-print-tip-group intensity dependent 
location normalization followed by within-print-tip-group scale normalization 
using the median absolute deviation (MAD). \cr
}
This argument can be specified using the first letter of each method.}}
  \item{subset}{A "logical" or "numeric" vector indicating the subset of points used to
  compute the  normalization values.}
 \item{span}{The argument \code{span} which controls the degree of smoothing in the \code{\link{loess}} function.}
\item{Mloc}{If \code{TRUE}, the location normalization values are stored in the slot \code{maMloc} of the object of class \code{"\link{marrayNorm}"} returned by the function, if \code{FALSE}, these values are not retained.}
  \item{Mscale}{If \code{TRUE}, the scale normalization values are stored in the slot \code{maMscale} of the object of class \code{"\link{marrayNorm}"} returned by the function, if \code{FALSE}, these values are not retained.}
\item{echo}{If \code{TRUE}, the index of the array currently being
  normalized is printed.}
\item{...}{Misc arguments}
}
\details{
See \code{\link{maNormMain}} for details and also more general procedures.
}
\value{
 \item{mnorm}{An object of class \code{"\link{marrayNorm}"}, containing the normalized intensity data.}
}


\references{S. Dudoit and Y. H. Yang. (2002). Bioconductor R packages for exploratory analysis and normalization of cDNA microarray data. In G. Parmigiani, E. S. Garrett, R. A. Irizarry and S. L. Zeger, editors, \emph{The Analysis of Gene Expression Data: Methods and Software}, Springer, New York.\cr \cr

Y. H. Yang, S. Dudoit, P. Luu, and T. P. Speed (2001). Normalization for cDNA microarray data. In M. L. Bittner, Y. Chen, A. N. Dorsel, and E. R. Dougherty (eds), \emph{Microarrays: Optical Technologies and Informatics}, Vol. 4266 of \emph{Proceedings of SPIE}.\cr \cr

Y. H. Yang, S. Dudoit, P. Luu, D. M. Lin, V. Peng, J. Ngai, and T. P. Speed (2002). Normalization for cDNA microarray data: a robust composite method addressing single and multiple slide systematic variation. \emph{Nucleic Acids Research}, Vol. 30, No. 4.
}

\author{Sandrine Dudoit, \url{http://www.stat.berkeley.edu/~sandrine}.}

\seealso{\code{\link{maNormMain}}, \code{\link{maNormScale}}.}

\examples{
# Examples use swirl dataset, for description type ? swirl
data(swirl)

# Global median normalization for swirl arrays 2 and 3
mnorm<-maNorm(swirl[,2:3], norm="median", echo=TRUE)

# Within-print-tip-group loess location normalization for swirl array 1
mnorm<-maNorm(swirl[,1], norm="p", span=0.45)

}




\keyword{smooth}% at least one, from doc/KEYWORDS
\keyword{methods}% __ONLY ONE__ keyword per line

\name{summary-methods}
\docType{methods}

\alias{summary-methods}
%\alias{print-methods}
%\alias{show-methods}
%\alias{print}

\title{Printing summary methods for microarray objects}

\description{
Print methods were defined for the microarray classes,
\code{"\link{marrayInfo}"}, \code{"\link{marrayLayout}"},
\code{"\link{marrayRaw}"}, \code{"\link{marrayNorm}"}. These methods
produce summaries of the intensity and textual data stored in different
classes of microarray objects. 
}

\section{Methods}{\describe{
\item{x = ANY}{generic print method}
\item{x = marrayLayout}{for an object of class \code{"\link{marrayLayout}"}, the method prints main layout parameters such as the number of spots and the dimensions of the spot and grid matrices.}
\item{x = marrayInfo}{for an object of class \code{"\link{marrayInfo}"}, the method prints the first 10 rows of the \code{"maInfo"} and \code{"maLabels"} slots.}
\item{x = marrayRaw}{for an object of class \code{"\link{marrayRaw}"}, the method prints a short description of the microarray layout \code{"maLayout"} and the target samples hybridized to the arrays \code{"maTargets"}, and a summary of the distribution of the log-ratio statistics \code{"maM"}.}
\item{x = marrayNorm}{for an object of class \code{"\link{marrayNorm}"}, the method prints a short description of the microarray layout \code{"maLayout"} and the target samples hybridized to the arrays \code{"maTargets"}, and a summary of the distribution of the log-ratio statistics \code{"maM"}.}
}}
\keyword{methods}

\name{maTop}
\alias{maTop}
%- Also NEED an `\alias' for EACH other topic documented here.
\title{Identify extreme values}
\description{
This function determines which values in a numeric vector are above or below user supplied cut-offs.
}
\usage{
maTop(x, h=1, l=1)
}
%- maybe also `usage' for other objects documented here.
\arguments{
  \item{x}{A "numeric" vector.}
  \item{h}{A "numeric", upper cut-off.}
  \item{l}{A "numeric", lower cut-off.}
}
\value{A "logical" vector indicating which entries are above or below the cut-offs.}
\references{
S. Dudoit and Y. H. Yang. (2002). Bioconductor R packages for exploratory analysis and normalization of cDNA microarray data. In G. Parmigiani, E. S. Garrett, R. A. Irizarry and S. L. Zeger, editors, \emph{The Analysis of Gene Expression Data: Methods and Software}, Springer, New York.
}
\author{Sandrine Dudoit, \url{http://www.stat.berkeley.edu/~sandrine}.}
\seealso{\code{\link{maPlot}}, \code{\link{maImage}}, \code{\link{quantile}}.}
\examples{
# See examples for maPlot.
}
\keyword{misc}% at least one, from doc/KEYWORDS

\name{htmlPage}
\alias{htmlPage}
\alias{table2html}

\title{Display gene list as a HTML page}
\description{
Given a set of index to a data.frame containing gene names information.
We create a web page with one element per genes that contains URLs links
to various external database links.
E.g Operon oligodatabase , Riken, GenBank and PubMed web sites. 
}

\usage{
htmlPage(genelist, filename = "GeneList.html", geneNames =
                 Gnames, mapURL = SFGL, othernames, title, table.head,
                 table.center = TRUE, disp = c("browser", "file")[1])

table2html(restable, filename = "GeneList.html", mapURL = SFGL,
                 title, table.head, table.center = TRUE, disp =
                 c("browser", "file")[1])
}

\arguments{
  \item{restable}{A data.frame that contains only the information you
    wish to display in the html file.  The rows corresponds to a
    different DNA spots.}
  \item{genelist}{A numeric vector of index to a data.frame}
  \item{filename}{The name of the file to store the HTML in. }
  \item{geneNames}{A data.frame containing the information related the
    each DNA spots. }
  \item{mapURL}{A matrix of characters containing the URL for various
    external database.  E.g \code{\link{SFGL}}.}
  \item{othernames}{A data.frame containing other information.}
  \item{title}{Title of the HTML page}
  \item{table.head}{ A character vector of column labels for the table}
  \item{table.center}{A logical indicating whether the table should be
    centered}
  \item{disp}{Either "File" or "Browser" (default is Browser). File will
    save the information in html file, while Browser will create an html
    files and display information in the user's browser.}
}
\details{
  This function is an extension to \code{ll.htmlpage}
}

\value{
  No value is return, the function produce a html file "filename" and
  output the results in a browser.
}
\author{Yee Hwa Yang }

\seealso{\code{ll.htmlpage}, \code{\link{URLstring}}, \code{\link{widget.mapGeneInfo}}}

\examples{
##library(annotate)
data(swirl)
Gnames <- maGeneTable(swirl)
swirlmap <- mapGeneInfo(Name = "none", ID="genbank")
## htmlPage(100:110, geneNames = Gnames, mapURL = swirlmap, title="Swirl")

moreinfo <- round(maM(swirl), 2)
swirlmap <- mapGeneInfo(Name = "pubmed", ID="genbank")
##htmlPage(100:110, geneNames = Gnames, mapURL = swirlmap, othernames=moreinfo, title="Swirl", disp="file")
}
\keyword{file}% at least one, from doc/KEYWORDS

\name{maPlot}
\alias{maPlot}
%- Also NEED an `\alias' for EACH other topic documented here.
\title{Scatter-plots for cDNA microarray spot statistics}
\description{The function \code{maPlot} produces scatter-plots of
  microarray spot statistics for the classes \code{"\link{marrayRaw}"} and \code{"\link{marrayNorm}"}.
  It also allows the user to highlight and annotate subsets of points on the plot, and display fitted
  curves from robust local regression or other smoothing procedures.
}
\usage{
maPlot(m, x="maA", y="maM", z="maPrintTip", lines.func, text.func, legend.func, \dots)
}
%- maybe also `usage' for other objects documented here.

\arguments{
  \item{m}{Microarray object of class \code{"\link{marrayRaw}"} and \code{"\link{marrayNorm}"}.}
  \item{x}{Name of accessor function for the abscissa spot statistic, typically a slot name for the microarray object \code{m}, such as \code{maA}.}
  \item{y}{Name of accessor function for the ordinate spot statistic, typically a slot name for the microarray object \code{m}, such as \code{maM}.}
  \item{z}{Name of accessor method for the spot statistic used to stratify the data, typically a slot name for the microarray layout object (see \code{"\link{marrayLayout}"}) such as \code{maPlate} or a method such as \code{maPrintTip}. If \code{z} is NULL, the data are not stratified.}
  \item{lines.func}{Function for computing and plotting smoothed fits of \code{y} as a function of \code{x}, separately within values of \code{z}, e.g. \code{\link{maLoessLines}}. If \code{lines.func} is NULL, no fitting is performed.}
  \item{text.func}{Function for highlighting a subset of points, e.g., \code{\link{maText}}. If \code{text.func} is NULL, no points are highlighted.}
  \item{legend.func}{Function for adding a legend to the plot, e.g. \code{\link{maLegendLines}}. If \code{legend.func} is NULL, there is no legend.}
  \item{\dots}{Optional graphical parameters, see \code{\link{par}}.}
}
\details{
This function calls the general function \code{\link{maPlot.func}}, which is not specific to microarray data. If there are more than one array in the batch, the plot is done for the first array, by default. Default graphical parameters are chosen for convenience using the function \code{\link{maDefaultPar}} (e.g. color palette,  axis labels,  plot title) but the user has the option to overwrite these parameters at any point.
}
\references{
S. Dudoit and Y. H. Yang. (2002). Bioconductor R packages for exploratory analysis and normalization of cDNA microarray data. In G. Parmigiani, E. S. Garrett, R. A. Irizarry and S. L. Zeger, editors, \emph{The Analysis of Gene Expression Data: Methods and Software}, Springer, New York.
}
\author{Sandrine Dudoit, \url{http://www.stat.berkeley.edu/~sandrine}.}
\seealso{\code{\link{maPlot.func}}, \code{\link{maDefaultPar}}, \code{\link{maLoessLines}}, \code{\link{maLegendLines}}, \code{\link{maText}}, \code{\link{plot}}, \code{\link{lowess}}, \code{\link{loess}}, \code{\link{legend}}.}
\examples{
# To see the demo type demo(marrayPlots)

# Examples use swirl dataset, for description type ? swirl
data(swirl)

# - Default arguments
maPlot(swirl)

# Lowess fit using all spots
maPlot(swirl, z=NULL, legend.func=NULL)

# Loess fit using all spots
maPlot(swirl, z=NULL, legend.func=maLegendLines(legend="All spots",col="green"), lines.func=maLoessLines(loess.args=list(span=0.3),col="green"))

# Pre-normalization MA-plot for the Swirl 81 array, with the lowess fits for 
# individual grid columns and 1\% tails of M highlighted
defs <- maDefaultPar(swirl[, 1], x = "maA", y = "maM", z = "maGridCol")
legend.func <- do.call("maLegendLines", defs$def.legend)
lines.func <- do.call("maLowessLines", c(list(TRUE, f = 0.3), defs$def.lines))
text.func<-maText(subset=maTop(maM(swirl)[,1],h=0.01,l=0.01), labels="o", col="violet")
maPlot(swirl[, 1], x = "maA", y = "maM", z = "maGridCol", lines.func=lines.func, text.func = text.func, legend.func=legend.func, main = "Swirl array 81: pre-normalization MA-plot")
}
\keyword{hplot}% at least one, from doc/KEYWORDS

\name{maGeneTable}
\alias{maGeneTable}
%- Also NEED an `\alias' for EACH other topic documented here.

\title{Table of spot coordinates and gene names}
\description{
This function produces a table of spot coordinates and gene names for
objects of class \code{"\link{marrayRaw}"} and
\code{"\link{marrayNorm}"}.}

\usage{
maGeneTable(object)
}
%- maybe also `usage' for other objects documented here.
\arguments{
  \item{object}{microarray object of class \code{"\link{marrayRaw}"} and
    \code{"\link{marrayNorm}"}.}
}

\value{
  an object of class \code{\link{data.frame}}, with rows corresponding
  to spotted probe sequences. The first four columns are the grid matrix
  and spot matrix coordinates, and the remaining columns are the spot
  descriptions stored in the \code{maGnames} slot of the microarray
  object. 
}

\author{Yee Hwa (Jean) Yang}


\seealso{\code{\link{marrayInfo}}, \code{\link{marrayLayout}}, \code{\link{marrayRaw}}, \code{\link{marrayNorm}},  \code{\link{maCompCoord}}.}


\examples{
# Example uses swirl dataset, for description type ? swirl

data(swirl)

tab<-maGeneTable(swirl)
tab[1:10,]

}

\keyword{methods}% at least one, from doc/KEYWORDS

\name{maDotsDefaults}
\alias{maDotsDefaults}
%- Also NEED an `\alias' for EACH other topic documented here.
\title{Replace graphical default parameters by user supplied parameters}

\description{
This function may be used to compare default graphical parameters for microarray diagnostic plots to user supplied parameters given in \code{...}. User supplied parameters overwrite the defaults. It is used in \code{\link{maBoxplot}}, \code{\link{maPlot}}, and \code{\link{maImage}}.}


\usage{
maDotsDefaults(dots, defaults)
}
%- maybe also `usage' for other objects documented here.

\arguments{
  \item{dots}{List of user supplied parameters, e.g. from \code{list(...)}.}
  \item{defaults}{List of default parameters, e.g. from the function \code{\link{maDefaultPar}}.}
}

\value{
  \item{args}{List of graphical parameters.}
}

\references{S. Dudoit and Y. H. Yang. (2002). Bioconductor R packages for exploratory analysis and normalization of cDNA microarray data. In G. Parmigiani, E. S. Garrett, R. A. Irizarry and S. L. Zeger, editors, \emph{The Analysis of Gene Expression Data: Methods and Software}, Springer, New York.}

\author{Sandrine Dudoit, \url{http://www.stat.berkeley.edu/~sandrine}.}

\seealso{\code{\link{maDefaultPar}}, \code{\link{maBoxplot}}, \code{\link{maPlot}}, \code{\link{maImage}}.}

\examples{
dots<-list(xlab="X1", ylab="Y1")
defaults<-list(xlab="X1", ylab="Y2", col=2)
pars<-maDotsDefaults(dots, defaults)

do.call("plot",c(list(x=1:10), pars))

}
\keyword{misc}% at least one, from doc/KEYWORDS

\name{marrayInfo-class}
\docType{class}

\alias{marrayInfo-class}
\alias{marrayInfo}

\alias{maLabels}
\alias{maLabels<-}
\alias{maInfo}
\alias{maInfo<-}
\alias{maNotes}
\alias{maNotes<-}

\alias{[,marrayInfo-method}
\alias{maGnames<-,marrayRaw,marrayInfo-method}
\alias{maGnames<-,marrayNorm,marrayInfo-method}
\alias{maGnames<-,marraySpots,marrayInfo-method}
\alias{maInfo<-,marrayInfo,data.frame-method}
\alias{maInfo,marrayInfo-method}
\alias{maLabels<-,marrayInfo,character-method}
\alias{maLabels<-,marrayInfo,numeric-method}
\alias{maLabels,marrayInfo-method}
\alias{maNotes<-,marrayInfo,character-method}
\alias{maNotes,marrayInfo-method}
\alias{maTargets<-,marrayRaw,marrayInfo-method}
\alias{maTargets<-,marrayNorm,marrayInfo-method}
\alias{print,marrayInfo-method}
\alias{rbind,marrayInfo-method}
\alias{summary,marrayInfo-method}

\title{Class "marrayInfo", description of target samples or spotted probe sequences}

\description{This class is used to store information on target samples hybridized to a batch of arrays or probe sequences spotted onto these arrays. It is not specific to the microarray context.}

\section{Objects from the Class}{
Objects can be created by calls of the form \code{  new('marrayInfo',}\cr
\code{    maLabels = ...., # Object of class character}\cr
\code{    maInfo   = ...., # Object of class data.frame}\cr
\code{    maNotes  = ...., # Object of class character}\cr
\code{  )}
}

\section{Slots}{
  \describe{
    \item{\code{maLabels}:}{Object of class \code{"character"}, vector of spot or array labels.}
    \item{\code{maInfo}:}{Object of class \code{"data.frame"}. If the object of class \code{"marrayInfo"} is used to describe probe sequences, rows of \code{maInfo} correspond to spots and columns to various gene identifiers and annotations. 
If the object of class \code{"marrayInfo"} is used to describe target samples hybridized to the arrays, rows of \code{maInfo} correspond to arrays and columns to various descriptions of the hybridizations, e.g., names of Cy3 and Cy5 samples, labels for the arrays etc.}
    \item{\code{maNotes}:}{Object of class \code{"character"}, any notes on the target samples or spotted probe sequences.}
  }
}

\section{Methods}{
  \describe{
    \item{[}{\code{signature(x = "marrayInfo")}: subsetting operator for spots on the array or arrays in the batch, ensures that all slots are subset properly.}
    \item{maGnames<-}{\code{signature(object = "marrayRaw", value = "marrayInfo")}: slot assignment method.}
    \item{maGnames<-}{\code{signature(object = "marrayNorm", value = "marrayInfo")}: slot assignment method.}
    \item{maGnames<-}{\code{signature(object = "marraySpots", value = "marrayInfo")}: slot assignment method.}
    \item{maInfo}{\code{signature(object = "marrayInfo")}: slot accessor method.}
    \item{maInfo<-}{\code{signature(object = "marrayInfo", value = "data.frame")}: slot assignment method.}
    \item{maLabels}{\code{signature(object = "marrayInfo")}: slot accessor method.}
    \item{maLabels<-}{\code{signature(object = "marrayInfo", value = "character")}: slot assignment method.}
    \item{maLabels<-}{\code{signature(object = "marrayInfo", value = "numeric")}: slot assignment method.}
    \item{maNotes}{\code{signature(object = "marrayInfo")}: slot accessor method.}
    \item{maNotes<-}{\code{signature(object = "marrayInfo", value = "character")}: slot assignment method.}
    \item{maTargets<-}{\code{signature(object = "marrayRaw", value = "marrayInfo")}: slot assignment method.}
    \item{maTargets<-}{\code{signature(object = "marrayNorm", value = "marrayInfo")}: slot assignment method.}
    \item{print}{\code{signature(x = "marrayInfo")}: print method for \code{"\link{marrayInfo}"} class.}
  }
}

\references{S. Dudoit and Y. H. Yang. (2002). Bioconductor R packages for exploratory analysis and normalization of cDNA microarray data. In G. Parmigiani, E. S. Garrett, R. A. Irizarry and S. L. Zeger, editors, \emph{The Analysis of Gene Expression Data: Methods and Software}, Springer, New York.
}


\author{Jean Yang and Sandrine Dudoit}

\seealso{\code{\link{marrayLayout}}, \code{\link{marrayRaw}}, \code{\link{marrayNorm}}.}

\examples{
## See marrayRaw
}
\keyword{classes}

\name{coerce-methods}
\docType{methods}
\alias{coerce-methods}
\alias{coerce}
\alias{as}
\title{Coerce an object to belong to a given microarray class}
\description{
Coercing methods were defined to convert microarray objects of one class
into objects of another class, e.g., instances of the
\code{"\link{marrayRaw}"} class into instances of the
\code{"\link{marrayNorm}"} class. 
}
\section{Methods}{\describe{
\item{from = marrayRaw, to = marrayNorm}{convert an object of class
  \code{"\link{marrayRaw}"} into an object of class
  \code{"\link{marrayNorm}"}.} 
}}
\note{Use Package convert to convert object to other data types such as
  \code{ExpressionSet} and \code{MAList}.
}
\keyword{methods}

\name{ma2D}
\alias{ma2D}
%- Also NEED an `\alias' for EACH other topic documented here.
\title{Stratified bivariate robust local regression}
\description{
This function performs robust local regression of a variable \code{z} on predictor variables \code{x} and \code{y}, separately within values of a fourth variable \code{g}. It is used by \code{\link{maNorm2D}} for 2D spatial location normalization.
}
\usage{
ma2D(x, y, z, g, w=NULL, subset=TRUE, span=0.4, ...)
}
%- maybe also `usage' for other objects documented here.
\arguments{
  \item{x}{A numeric vector of predictor variables.}
  \item{y}{A numeric vector of predictor variables.}
  \item{z}{A numeric vector of responses.}
  \item{g}{Variables used to stratify the data.}
  \item{w}{An optional numeric vector of weights.}
  \item{subset}{A "logical" or "numeric" vector indicating the subset of points used to compute the fits. }
  \item{span}{The argument \code{span} which controls the degree of
    smoothing in the  \code{\link{loess}} function.}
  \item{...}{Misc arguments}
}
\details{
\code{z} is regressed on \code{x} and \code{y}, separately within values of \code{g} using the \code{\link{loess}} function.
}
\value{
A numeric vector of fitted values.
}

\references{S. Dudoit and Y. H. Yang. (2002). Bioconductor R packages for exploratory analysis and normalization of cDNA microarray data. In G. Parmigiani, E. S. Garrett, R. A. Irizarry and S. L. Zeger, editors, \emph{The Analysis of Gene Expression Data: Methods and Software}, Springer, New York.
}

\author{Sandrine Dudoit, \url{http://www.stat.berkeley.edu/~sandrine}.}


\seealso{\code{\link{maNormMain}}, \code{\link{maNorm2D}}, \code{\link{loess}}.}


\examples{
# See examples for maNormMain.
}
\keyword{smooth}% at least one, from doc/KEYWORDS

\name{marrayLayout-class}
\docType{class}

\alias{marrayLayout-class}
\alias{marrayLayout}

\alias{maNgr}
\alias{maNgr<-}
\alias{maNgc}
\alias{maNgc<-}
\alias{maNsr}
\alias{maNsr<-}
\alias{maNsc}
\alias{maNsc<-}
\alias{maNspots}
\alias{maNspots<-}
\alias{maSub}
\alias{maSub<-}
\alias{maPlate}
\alias{maPlate<-}
\alias{maControls}
\alias{maControls<-}

\alias{[,marrayLayout-method}
\alias{maControls<-,marrayLayout-method}
\alias{maControls,marrayLayout-method}
\alias{maGridCol,marrayLayout-method}
\alias{maGridRow,marrayLayout-method}
\alias{maLayout<-,marrayRaw,marrayLayout-method}
\alias{maLayout<-,marrayNorm,marrayLayout-method}
\alias{maNgc<-,marrayLayout,numeric-method}
\alias{maNgc,marrayLayout-method}
\alias{maNgr<-,marrayLayout,numeric-method}
\alias{maNgr,marrayLayout-method}
\alias{maNotes<-,marrayLayout,character-method}
\alias{maNotes,marrayLayout-method}
\alias{maNsc<-,marrayLayout,numeric-method}
\alias{maNsc,marrayLayout-method}
\alias{maNspots<-,marrayLayout,numeric-method}
\alias{maNspots,marrayLayout-method}
\alias{maNsr<-,marrayLayout,numeric-method}
\alias{maNsr,marrayLayout-method}
\alias{maPlate<-,marrayLayout-method}
\alias{maPlate,marrayLayout-method}
\alias{maPrintTip,marrayLayout-method}
\alias{maSpotCol,marrayLayout-method}
\alias{maSpotRow,marrayLayout-method}
\alias{maSub<-,marrayLayout,logical-method}
\alias{maSub<-,marrayLayout,numeric-method}
\alias{maSub,marrayLayout-method}
\alias{print,marrayLayout-method}
\alias{summary,marrayLayout-method}

%\alias{maNotes}

\alias{maPrintTip}
\alias{maGridRow}
\alias{maGridCol}
\alias{maSpotRow}
\alias{maSpotCol}


\title{Class "marrayLayout", classes and methods for layout parameters of cDNA microarrays}
\description{This class is used to keep track of important layout
  parameters for two-color cDNA microarrays. It contains slots for: the
  total number of spotted probe sequences on the array, the dimensions
  of the spot and grid matrices, the plate origin of the probes,
  information on spotted control sequences (e.g. probe sequences which
  should have equal abundance in the two target samples, such as
  housekeeping genes). The terms \emph{print-tip-group}, \emph{grid},
  \emph{spot matrix}, and \emph{sector} are used interchangeably and
  refer to a set of spots printed using the same print-tip.} 


\section{Objects from the Class}{
Objects can be created by calls of the form \code{  new('marrayLayout',}\cr
\code{    maNgr      = ...., # Object of class numeric}\cr
\code{    maNgc      = ...., # Object of class numeric}\cr
\code{    maNsr      = ...., # Object of class numeric}\cr
\code{    maNsc      = ...., # Object of class numeric}\cr
\code{    maNspots   = ...., # Object of class numeric}\cr
\code{    maSub      = ...., # Object of class logical}\cr
\code{    maPlate    = ...., # Object of class factor}\cr
\code{    maControls = ...., # Object of class factor}\cr
\code{    maNotes    = ...., # Object of class character}\cr
\code{  )}
}

\section{Slots}{
  \describe{
    \item{\code{maNgr}:}{Object of class \code{"numeric"}, number of rows for the grid matrix.}
    \item{\code{maNgc}:}{Object of class \code{"numeric"}, number of columns for the grid matrix.}
    \item{\code{maNsr}:}{Object of class \code{"numeric"}, number of rows for the spot matrices.}
    \item{\code{maNsc}:}{Object of class \code{"numeric"}, number of columns for the spot matrices.}
    \item{\code{maNspots}:}{Object of class \code{"numeric"}, total number of spots on the array, equal to \eqn{maNgr x maNgc x maNsr x maNsc}.}
    \item{\code{maSub}:}{Object of class \code{"logical"},  indicating which spots are currently being considered.}
    \item{\code{maPlate}:}{Object of class \code{"factor"}, recording the plate origin of the spotted probe sequences.}
    \item{\code{maControls}:}{Object of class \code{"factor"}, recording the control status of the spotted probe sequences.}
    \item{\code{maNotes}:}{Object of class \code{"character"}, any notes concerning the microarray layout, e.g., printing conditions.}
  }
}

\section{Methods}{
  \describe{
    \item{[}{\code{signature(x = "marrayLayout")}: subsetting operator for spots on the array, ensures that all slots are subset properly.}
    \item{maControls<-}{\code{signature(object = "marrayLayout")}: slot assignment method.} 
    \item{maControls}{\code{signature(object = "marrayLayout")}: slot accessor method.}
    \item{maGridCol}{\code{signature(object = "marrayLayout")}: method which computes a vector of grid column coordinates for each spot.}
    \item{maGridRow}{\code{signature(object = "marrayLayout")}: method which computes a vector of grid row coordinates for each spot.}
    \item{maLayout<-}{\code{signature(object = "marrayRaw", value = "marrayLayout")}: slot assignment method.} 
    \item{maLayout<-}{\code{signature(object = "marrayNorm", value = "marrayLayout")}: slot assignment method.} 
    \item{maNgc}{\code{signature(object = "marrayLayout")}: slot accessor method.}
 \item{maNgc<-}{\code{signature(object = "marrayLayout", value = "numeric")}: slot assignment method.}
    \item{maNgr}{\code{signature(object = "marrayLayout")}: slot accessor method.}
\item{maNgr<-}{\code{signature(object = "marrayLayout", value = "numeric")}: slot assignment method.}
    \item{maNotes}{\code{signature(object = "marrayLayout")}: slot accessor method.}
    \item{maNotes<-}{\code{signature(object = "marrayLayout", value = "character")}: slot assignment method.} 
    \item{maNsc}{\code{signature(object = "marrayLayout")}: slot accessor method.}
    \item{maNsc<-}{\code{signature(object = "marrayLayout", value = "numeric")}: slot assignment method.}
    \item{maNspots}{\code{signature(object = "marrayLayout")}: slot accessor method.}
 \item{maNspots<-}{\code{signature(object = "marrayLayout", value = "numeric")}: slot assignment method.}
    \item{maNsr}{\code{signature(object = "marrayLayout")}: slot accessor method.}
\item{maNsr<-}{\code{signature(object = "marrayLayout", value = "numeric")}: slot assignment method.}
    \item{maPlate}{\code{signature(object = "marrayLayout")}: slot accessor method.}
    \item{maPlate<-}{\code{signature(object = "marrayLayout")}: slot assignment method.} 
    \item{maPrintTip}{\code{signature(object = "marrayLayout")}: method which computes a vector of print-tip-group indices for each spot.}
    \item{maSpotCol}{\code{signature(object = "marrayLayout")}: method which computes a vector of spot column coordinates for each spot.}
    \item{maSpotRow}{\code{signature(object = "marrayLayout")}: method which computes a vector of spot row coordinates for each spot.}
    \item{maSub}{\code{signature(object = "marrayLayout")}: slot accessor method.}
    \item{maSub<-}{\code{signature(object = "marrayLayout", value = "logical")}: slot assignment method.} 
    \item{maSub<-}{\code{signature(object = "marrayLayout", value = "numeric")}: slot assignment method.} 
    \item{print}{\code{signature(x = "marrayLayout")}: print method for \code{"\link{marrayLayout}"} class.}
  }
}

\references{S. Dudoit and Y. H. Yang. (2002). Bioconductor R packages for exploratory analysis and normalization of cDNA microarray data. In G. Parmigiani, E. S. Garrett, R. A. Irizarry and S. L. Zeger, editors, \emph{The Analysis of Gene Expression Data: Methods and Software}, Springer, New York.
}

\author{Sandrine Dudoit, \url{http://www.stat.berkeley.edu/~sandrine}.}

\seealso{\code{\link{marrayRaw}}, \code{\link{marrayNorm}},
  \code{\link{marrayInfo}} and \code{\link{[-methods}}.} 

\examples{
## See marrayRaw
}

\keyword{classes}

\name{read.marrayRaw}
\alias{read.marrayRaw}
\alias{read.Spot}
\alias{read.GenePix}
\alias{read.SMD}
\alias{read.Agilent}
\alias{widget.marrayRaw}

\title{Create objects of class "marrayRaw"}
\description{
  This function reads in cDNA microarray data from a directory and creates objects of
  class \code{"\link{marrayRaw}"} from spot quantification data files obtained from image analysis software or databases.
}

\usage{
read.marrayRaw(fnames, path=".", name.Gf=NULL, name.Gb=NULL, name.Rf=NULL,
name.Rb=NULL,name.W=NULL, layout=NULL, gnames=NULL, targets=NULL,
notes=NULL, skip=NULL, sep="	", quote="\"", DEBUG=FALSE, ...)

read.GenePix(fnames = NULL, path = NULL, name.Gf = "F532 Median",
name.Gb ="B532 Median", name.Rf = "F635 Median", name.Rb = "B635 Median",
name.W ="Flags", layout = NULL, gnames = NULL, targets = NULL,
notes = NULL, skip=NULL, sep = "	", quote = "\"", DEBUG=FALSE, ...)

read.SMD(fnames = NULL, path = NULL, name.Gf = "Ch1 Intensity (Median)",
name.Gb = "Ch1 Background (Median)", name.Rf = "Ch2 Intensity (Median)",
name.Rb = "Ch2 Background (Median)", name.W = NULL, info.id = c("Name",
"Clone ID"), layout = NULL, gnames = NULL, targets = NULL, notes = NULL, skip = NULL, sep = "\t", quote = "\"", DEBUG=FALSE, ...)

read.Spot(fnames = NULL, path = ".", name.Gf = "Gmean", name.Gb =
"morphG", name.Rf = "Rmean", name.Rb = "morphR",name.W = NULL, layout =
NULL, gnames = NULL, targets = NULL, notes = NULL, skip = NULL, sep = "\t", quote = "\"", DEBUG=FALSE, ...)

read.Agilent(fnames = NULL, path=NULL, name.Gf = "gMedianSignal", name.Gb = "gBGMedianSignal", name.Rf = "rMedianSignal", name.Rb = "rBGMedianSignal", name.W= NULL, layout = NULL, gnames = NULL, targets = NULL, notes=NULL, skip=NULL, sep="\t", quote="\"", DEBUG=FALSE, info.id=NULL, ...)

widget.marrayRaw(ext = c("spot", "xls", "gpr"), skip = 0, sep = "\t",  quote = "\"", ...)
}

\arguments{
  \item{fnames}{a vector of character strings containing the file names of
    each spot quantification data file.  These typically end in \code{.spot}
    for the software \code{Spot} or \code{.gpr} for the software \code{GenePix}.}
  \item{path}{a character string representing the data directory. By
    default this is set to the current working directory (".").  In the
    case where \code{fnames} contains the full path name, path should be set
    to NULL.}
  \item{name.Gf}{character string for the column header for green foreground intensities.}
  \item{name.Gb}{character string for the column header for green background intensities.}
  \item{name.Rf}{character string for the column header for red foreground intensities.}
  \item{name.Rb}{character string for the column header for red background intensities.}
  \item{name.W}{character string for the column header for spot quality weights.}
  \item{layout}{object of class \code{"\link{marrayLayout}"}, containing microarray layout parameters.}
  \item{gnames}{object of class \code{"\link{marrayInfo}"} containing probe sequence information.}
  \item{targets}{object of class \code{"\link{marrayInfo}"} containing target sample information.}
  \item{notes}{object of class \code{"character"}, vector of explanatory
    text.}
  \item{info.id}{object of class \code{"character"}, vector containing
    the name of the colums of the SMD file containing oligo information
    you want to retrieve. By default, this is set to read Homo sapiens
    data. You may need to modify this argument if your are working on
    another genome.}
  \item{skip}{the number of lines of the data file to skip before beginning to read in data.}
  \item{sep}{the field separator character.  Values on each line of the
    file are separated by this character. The
    default is to read a tab delimited file.}
  \item{quote}{the set of quoting characters. By default, this is
    disabled by setting \code{quote="\""}.}
  \item{ext}{a characters string representing suffix of different image
    analysis output files.}
  \item{DEBUG}{a logical value, if TRUE, a series of echo statements will be printed.}
  \item{\dots}{further arguments to \code{\link{scan}}.}
}
\value{
  An object of class \code{"\link{marrayRaw}"}.
}
\references{\url{http://www.bioconductor.org/}.}
\author{Jean Yang, \email{yeehwa@stat.berkeley.edu}}

\seealso{\code{\link{scan}}, \code{\link{read.marrayLayout}},
  \code{\link{read.marrayInfo}} }

\examples{
datadir <- system.file("swirldata", package="marray")

## Quick guide
swirl.targets <- read.marrayInfo(file.path(datadir, "SwirlSample.txt"))
data <- read.Spot(path=datadir, targets=swirl.targets)

## Alternate commands
skip <-  grep("Row", readLines(file.path(datadir,"fish.gal"), n=100)) - 1

swirl.layout <- read.marrayLayout(ngr=4, ngc=4, nsr=22, nsc=24)

swirl.targets <- read.marrayInfo(file.path(datadir, "SwirlSample.txt"))

swirl.gnames <- read.marrayInfo(file.path(datadir, "fish.gal"),
                                info.id=4:5, labels=5, skip=skip)

x <-  maInfo(swirl.gnames)[,1]
y <- rep(0, maNspots(swirl.layout))
y[x == "control"] <- 1
slot(swirl.layout, "maControls") <- as.factor(y)

fnames <- dir(path=datadir,pattern="spot")
swirl<- read.Spot(fnames, path=datadir,
                       layout = swirl.layout,
                       gnames = swirl.gnames,
                       targets = swirl.targets)

}

\keyword{file}% at least one, from doc/KEYWORDS

\name{write.marray}
\alias{write.marray}

\title{Data Output}
\description{
  Calls the function write.table with predefine argument.   The entries
  in each line (row) are separated by tab. 
}

\usage{
write.marray(mraw, file="maRawResults.xls", val="maM", ...)
}

\arguments{
  \item{mraw}{the object to be written, either a \code{marrayRaw} or
    \code{marrayNorm} object.}
  \item{file}{a character string representing the file name.}
  \item{val}{a character string representing the slotNames to be written.}
  \item{\dots}{further arguments to \code{\link{write.table}}.}
}

\details{
  see \code{\link{write.table}}
}

\author{Jean Yee Hwa Yang}

\seealso{\code{\link{write.table}}, \code{\link{write.list}}}

\examples{
data(swirl)
write.marray(swirl[1:10,])
}

\keyword{file}% at least one, from doc/KEYWORDS

\name{maNum2Logic}
\alias{maNum2Logic}
%- Also NEED an `\alias' for EACH other topic documented here.
\title{Convert a numeric vector of indices to a logical vector}
\description{
This function converts a numeric vector of indices to a logical
vector. It is used for subsetting purposes.
}
\usage{
maNum2Logic(n=length(subset), subset=TRUE)
}
%- maybe also `usage' for other objects documented here.
\arguments{
  \item{n}{the length of the logical vector to be produced.}
  \item{subset}{a numeric vector of indices. A logical vector may also
  be supplied, in which case it is also the value of the function.}
}

\value{
 a logical vector.
}


\author{Sandrine Dudoit, \url{http://www.stat.berkeley.edu/~sandrine}.}


\examples{
maNum2Logic(10, 1:3)
}
\keyword{manip}% at least one, from doc/KEYWORDS

\name{read.marrayInfo}
\alias{read.marrayInfo}
\alias{widget.marrayInfo}

\title{Create objects of class marrayInfo}
\description{
  This function creates objects of class \code{\link{marrayInfo}}. The \code{\link{marrayInfo}} class is used to store
  information regarding the target mRNA samples co-hybridized on the arrays or
  the spotted probe sequences (e.g. data frame of gene names, annotations, and other identifiers).
}
\usage{
read.marrayInfo(fname, info.id=NULL, labels=NULL, notes=fname, sep="\t",skip=0, quote="\"", ...)
}
%- maybe also `usage' for other objects documented here.
\arguments{
  \item{fname}{the name of the file that stores information on target samples or probe sequences. This is usually a file obtained from a database.}
  \item{info.id}{the column numbers in \code{fname} that contain the
    required information.}
  \item{labels}{the column number in \code{fname} which contains the names that the user would like to use to label spots or arrays (e.g. for default titles in \code{\link{maPlot}}.}
  \item{notes}{object of class character, vector of explanatory text}
    \item{sep}{the field separator character.  Values on each line of the
    file are separated by this character. The
    default is to read a tab delimited file.}
\item{skip}{the number of lines of the data file to skip before
  beginning to read data.}
\item{quote}{the set of quoting characters. By default, this is disable
  by setting `quote="\""'.}
  \item{\dots}{further arguments to \code{\link{scan}}.}
}
\value{
  An object of class \code{\link{marrayInfo}}.
}
\references{http://www.bioconductor.org/}
\author{Jean Yang, \email{yeehwa@stat.berkeley.edu}}

\examples{
datadir <- system.file("swirldata", package="marray")

## Reading target sample information
swirl.samples <- read.marrayInfo(file.path(datadir, "SwirlSample.txt"))

## Reading probe sequence information
swirl.gnames <- read.marrayInfo(file.path(datadir, "fish.gal"),
                                info.id=4:5, labels=5, skip=21)
}

\keyword{file}% at least one, from doc/KEYWORDS
\keyword{connection}

\name{mapGeneInfo}
\alias{mapGeneInfo}
\alias{widget.mapGeneInfo}
\alias{URLstring}
\alias{SFGL}
\alias{UCBFGL}

\title{Creating URL strings for external database links }

\description{
  These functions are used with \code{\link{htmlPage}}.
  The function \code{mapGeneInfo}, takes all the arguments and generate
  a character matrix of two columns.  The first columns representing the
  name of the argument and the second columns represents the value of an
  argument. 
  The function \code{widget.mapGeneInfo} allows the user to enter this
  information  interactively. 
}

\usage{
mapGeneInfo(widget = FALSE, Gnames, Name = "pubmed", ID =
                 "genbank", ACC = "SMDacc", ...)
widget.mapGeneInfo(Gnames) 
}

%- maybe also `usage' for other objects documented here.
\arguments{
  \item{widget}{A logical value specifying if widgets should be used.}
  \item{Name}{The external database for spot description, E.g. "pubmed".}
  \item{ID}{The external database for spot ID, E.g. "operon", "Riken", "locuslink".}
  \item{ACC}{The external database for gene accession number, E.g. "genebank".}
  \item{Gnames}{An object of class \code{matrix}, \code{data.frame} or \code{marrayInfo}
    which contains  description of spotted probe sequences.}
  \item{\dots}{Other column names}
}
\details{
  The function \code{mapGeneInfo} generates a character matrix with the
  first column representing the column headings of "Gnames" and the
  second column representing the corresponding names in the list
  \code{URLstring}.  For example, if a particular column in "Gnames"
  with column names "ID" contains genebank accession number, then the
  function \code{mapGeneInfo} generates a row containing "ID" in the
  first column and "genbank" in the second.  Examples are \code{SFGL}
  and \code{UCBFGL}.\cr
  
  \code{URLstring} is a list contains the URL to various external
  database, E.g. operon, Riken, genbank. \cr
  The current choices are: 
  "pubmed",  "locuslink",  "riken",   "SMDclid",    "SMDacc",    "operonh2",  "operonh1" ,
  "operonm2",  "operonm1" and  "genbank" .  
  "SMDclid" and "SMDacc" are links to Stanford Microarray Databases.
 }

 \author{Jean Yee Hwa Yang}

 \examples{
 mapGeneInfo(ID="genebank", ll="locuslink")
 mapGeneInfo(ID="locuslink", Sample.ID="riken")
}

\keyword{manip}% at least one, from doc/KEYWORDS

\name{maImage}
\alias{maImage}

%- Also NEED an `\alias' for EACH other topic documented here.
\title{Color image for cDNA microarray spot statistics}
\description{
  We encourage users calling "image" rather than "maImage".  The name of  the arguments are change slightly.

  The function \code{maImage} creates spatial images of shades of gray or colors that correspond to the values of a statistic for each spot on the array. The statistic can be the intensity log-ratio M, a spot quality measure (e.g. spot size or shape), or a test statistic. This function can be used to explore whether there are any spatial effects in the data, for example, print-tip or cover-slip effects. 
}
\usage{
maImage(m, x="maM", subset=TRUE, col, contours=FALSE, bar=TRUE,
overlay=NULL, ol.col=1, colorinfo=FALSE, ...)
}
%- maybe also `usage' for other objects documented here.
\arguments{
  \item{m}{Microarray object of class \code{"\link{marrayRaw}"} and \code{"\link{marrayNorm}"}.}
  \item{x}{Name of accessor function for the spot statistic of interest, typically a slot name for the microarray object \code{m}, such as \code{maM}.}
  \item{subset}{A "logical" or "numeric" vector indicating the subset of spots to display on the image.}
  \item{col}{List of colors such as that generated by rainbow, heat.colors, topo.colors, terrain.colors, or similar functions. In addition to these color palette functions, a new function \code{\link{maPalette}} was defined to generate color palettes from user supplied low, middle, and high color values. }
  \item{contours}{If \code{contours=TRUE}, contours are plotted, otherwise they are not shown.}
  \item{bar}{If \code{bar=TRUE}, a calibration color bar is shown to the
    right of the image.}
  \item{overlay}{A logical vector of spots to be highlighted on the image plots.}
  \item{ol.col}{Color of the overlay spots.}
  \item{colorinfo}{A logical value indicating whether the function
    should return the color scale information.} 
  \item{\dots}{Optional graphical parameters, see \code{\link{par}}.}
}
\details{
 This function calls the general function \code{\link{maImage.func}}, which is not specific to microarray data. If there are more than one array in the batch, the plot is done for the first array, by default. Default color palettes were set for different types of spot statistics using the \code{\link{maPalette}} function. When \code{x=c("maM", "maMloc", "maMscale")}, a green-to-red color palette is used. When \code{x=c("maGb", "maGf", "maLG")}, a white-to-green color palette is used. When \code{x=c("maRb", "maRf", "maLR")}, a white-to-red color palette is used. The user has the option to overwrite these parameters at any point. 
}

\value{
  If \code{colorinfo} is set to TRUE, the following list with elements will be returned.
  \item{x.col}{vector of colors to be used for calibration color bar.}
  \item{x.bar}{vector of values to be used for calibration color bar.}
  \item{summary}{six number summary of the spot statistics, from the function \code{\link{summary}}.}

}

\references{S. Dudoit and Y. H. Yang. (2002). Bioconductor R packages for exploratory analysis and normalization of cDNA microarray data. In G. Parmigiani, E. S. Garrett, R. A. Irizarry and S. L. Zeger, editors, \emph{The Analysis of Gene Expression Data: Methods and Software}, Springer, New York.
}


\author{Sandrine Dudoit, \url{http://www.stat.berkeley.edu/~sandrine}.}

\seealso{\code{\link{image}}, \code{\link{maImage.func}}, \code{\link{maColorBar}}, \code{\link{maPalette}}, \code{\link{summary}}.}

\examples{

# To see the demo type demo(marrayPlots)

# Examples use swirl dataset, for description type ? swirl
data(swirl)

# Microarray color palettes
Gcol <- maPalette(low = "white", high = "green", k = 50)
Rcol <- maPalette(low = "white", high = "red", k = 50)
RGcol <- maPalette(low = "green", high = "red", k = 50)

# Color images of green and red background and foreground intensities
maImage(swirl[, 3], x="maGb")
maImage(swirl[, 3], x = "maGf", subset = TRUE, col = Gcol, contours = FALSE, bar = TRUE, main="Swirl array 93")
maImage(swirl[, 3], x = "maRb", contour=TRUE)
maImage(swirl[, 3], x = "maRf", bar=FALSE)

# Color images of pre-normalization intensity log-ratios
maImage(swirl[, 1])
maImage(swirl[, 3], x = "maM", subset = maTop(maM(swirl[, 3]), h = 0.1, l = 0.1), col = RGcol, contours = FALSE, bar = TRUE, main = "Swirl array 93: image of pre-normalization M for \% 10 tails")

# Color image of print-tip-group
maImage(swirl[, 1],x="maPrintTip")

}
\keyword{hplot}% at least one, from doc/KEYWORDS

\name{opVersionID}
\alias{opVersionID}

\title{Determine the operon oligo set ID}
\description{
  This functions looks the operon ID and determine whether it belongs to 
  "Human Genome Oligo Set V1", "Human Genome Oligo Set V2", "Mouse
  Genome Oligo Set V1" or "Mouse Genome Oligo Set V2". 
}

\usage{
opVersionID(opID)
}

\arguments{
  \item{opID}{A character strings representing operon ID}
}

\value{
  A value "operonh1", "operonh2", "operonm1" or "operonm2" to represents
  "Human Genome Oligo Set V1", "Human Genome Oligo Set V2", "Mouse
   Genome Oligo Set V1" or "Mouse Genome Oligo Set V2". 
}

\references{\url{http://oparray.operon.com/}}
\author{Jean Yee Hwa Yang}

\seealso{\code{\link{URLstring}}, \code{\link{htmlPage}}}

\examples{
 opVersionID("M000205_01")
 URLstring[opVersionID("M000205_01")]
}

\keyword{manip}% at least one, from doc/KEYWORDS

\name{maLoess}
\alias{maLoess}
%- Also NEED an `\alias' for EACH other topic documented here.
\title{Stratified univariate robust local regression}
\description{
 This function performs robust local regression of a variable \code{y} on predictor variable \code{x}, separately within values of a third variable \code{z}. It is used by \code{\link{maNormLoess}} for intensity dependent location normalization.
}
\usage{
maLoess(x, y, z, w=NULL, subset=TRUE, span=0.4, \dots)
}
%- maybe also `usage' for other objects documented here.
\arguments{
  \item{x}{A numeric vector of predictor variables.}
  \item{y}{A numeric vector of responses.}
  \item{z}{Variables used to stratify the data.}
  \item{w}{An optional numeric vector of weights.}
  \item{subset}{A "logical" or "numeric" vector indicating the subset of points used to compute the fits.}
  \item{span}{The argument \code{span} which controls the degree of smoothing in the \code{\link{loess}} function.}
  \item{\dots}{Misc arguments.}
}
\details{
\code{y} is regressed on \code{x}, separately within values of \code{z} using the \code{\link{loess}} function.  
}
\value{A numeric vector of fitted values.}
\references{S. Dudoit and Y. H. Yang. (2002). Bioconductor R packages for exploratory analysis and normalization of cDNA microarray data. In G. Parmigiani, E. S. Garrett, R. A. Irizarry and S. L. Zeger, editors, \emph{The Analysis of Gene Expression Data: Methods and Software}, Springer, New York.\cr \cr

Y. H. Yang, S. Dudoit, P. Luu, and T. P. Speed (2001). Normalization for cDNA microarray data. In M. L. Bittner, Y. Chen, A. N. Dorsel, and E. R. Dougherty (eds), \emph{Microarrays: Optical Technologies and Informatics}, Vol. 4266 of \emph{Proceedings of SPIE}.\cr \cr

Y. H. Yang, S. Dudoit, P. Luu, D. M. Lin, V. Peng, J. Ngai, and T. P. Speed (2002). Normalization for cDNA microarray data: a robust composite method addressing single and multiple slide systematic variation. \emph{Nucleic Acids Research}, Vol. 30, No. 4.
}
\author{Sandrine Dudoit, \url{http://www.stat.berkeley.edu/~sandrine}.}
\seealso{\code{\link{maNormMain}}, \code{\link{maNormLoess}}, \code{\link{loess}}.}
\examples{
# See examples for maNormMain.
}
\keyword{smooth}% at least one, from doc/KEYWORDS

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This file was automatically generated by Rdoc on 2001-11-08 :05:09.
% Do NOT modify this file, instead modify the source (Rarray.R).
% Rdoc is copyright (C) 2001, Henrik Bengtsson, henrikb@braju.com.
% Download the Rdoc compiler at http://www.braju.com/R/
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\name{stat.gnames}

\alias{stat.gnames}

\title{Sort Genes According to the Value of a Statistic}

\description{
Lists genes and corresponding statistics in decreasing order of the
statistics. This function applies to any type of statistic, including
log ratios, one and two-sample t-statistics, and F-statistics. Missing
values are ignored, as in \code{\link{sort}(..., na.last=NA)}. 
}

\usage{
stat.gnames(x, gnames, crit= 50)
}

\arguments{
 \item{x}{a numeric vector containing the statistics for each
 gene. Missing values (NAs) are allowed. }
 
\item{gnames}{a character vector containing the gene names.}

 \item{crit}{specifies the number of genes to be returned. If crit <
 1, the crit*100\% genes with the largest x values are listed. If crit
 >= 1, the crit genes with the largest x values are listed. }
}

\value{
List containing the following components 
  \item{gnames}{gene names sorted in decreasing order of the
 statistics in x.}
 \item{t}{statistics sorted in decreasing order.}
}

\author{
  Yee Hwa Yang, \email{yeehwa@stat.berkeley.edu} \cr
  Sandrine Dudoit, \email{sandrine@stat.berkeley.edu} }

\seealso{\code{\link{order}}, \code{\link{sort}}.}

\examples{
data(swirl)
aveM <- apply(maM(swirl), 1, mean.na)
Gnames <- maGeneTable(swirl)

stat.gnames(abs(aveM), Gnames, crit=10)
stat.gnames(aveM, Gnames, crit=0.01)

}

\keyword{misc}

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% End of File
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

\name{findID}
\alias{findID}
\title{Find ID when given an accession number}
\description{
  Search gene ID with a vector of accession number from gene names or ID
  values.
}
\usage{
findID(text, Gnames = gnames, ID = "Name")
}
\arguments{
  \item{text}{A character strings of gene names or id names.}
  \item{Gnames}{An objects of \code{marrayRaw}, \code{marrayNorm},
    \code{ExpressionSet} or \code{data.frame} of gene names information.}
  \item{ID}{The column of ID corresponding to 'text'.}
}
\value{A numeric vector the gene ID.}
\author{Yee Hwa (Jean) Yang}
\seealso{\code{\link{grep}}}
\examples{
data(swirl)
findID("fb24a09", swirl, ID="ID")
findID("geno1", swirl)
}
\keyword{manip}% at least one, from doc/KEYWORDS

\name{maNormMain}
\alias{maNormMain}
%- Also NEED an `\alias' for EACH other topic documented here.
\title{Main function for location and scale normalization of cDNA microarray data}
\description{
This is the main function for location and scale normalization of cDNA microarray data. Normalization is performed for a batch of arrays using location and scale normalization procedures specified by the lists of functions \code{f.loc} and \code{f.scale}. Typically, only one function is given in each list, otherwise composite normalization is performed using the weights computed by the functions \code{a.loc} and \code{a.scale}. The function operates on an object of class \code{"\link{marrayRaw}"} (or possibly \code{"\link{marrayNorm}"}, if normalization is performed in several steps) and returns an object of class \code{"\link{marrayNorm}"}. Simple wrapper functions are provided by \code{\link{maNorm}} and \code{\link{maNormScale}}.
}

\usage{
maNormMain(mbatch, f.loc=list(maNormLoess()), f.scale=NULL,
a.loc=maCompNormEq(), a.scale=maCompNormEq(), Mloc=TRUE, Mscale=TRUE, echo=FALSE)
}
%- maybe also `usage' for other objects documented here.

\arguments{
  \item{mbatch}{An object of class \code{"\link{marrayRaw}"}, containing intensity data for the batch of arrays to be normalized. An object of class \code{"\link{marrayNorm}"} may also be passed if normalization is performed in several steps.}
  \item{f.loc}{A list of location normalization functions, e.g., \code{\link{maNormLoess}}, \code{\link{maNormMed}}, or \code{\link{maNorm2D}}.}
  \item{f.scale}{A list of scale normalization functions, .e.g, \code{\link{maNormMAD}}.}
  \item{a.loc}{For composite normalization, a function for computing the weights used in combining several location normalization functions, e.g., \code{\link{maCompNormA}}.}
  \item{a.scale}{For composite normalization, a function for computing
  the weights used in combining several scale normalization functions.}
  \item{Mloc}{If \code{TRUE}, the location normalization values are stored in the slot \code{maMloc} of the object of class \code{"\link{marrayNorm}"} returned by the function, if \code{FALSE}, these values are not retained.}
  \item{Mscale}{If \code{TRUE}, the scale normalization values are stored in the slot \code{maMscale} of the object of class \code{"\link{marrayNorm}"} returned by the function, if \code{FALSE}, these values are not retained.}
  \item{echo}{If \code{TRUE}, the index of the array currently being
  normalized is printed.}
}

\details{
When both location and scale normalization functions (\code{f.loc} and \code{f.scale}) are passed, location normalization is performed before scale normalization. That is, scale values are computed for the location normalized log-rations. The same results could be obtained by two applications of the function \code{maNormMain}, first with only the location normalization function and \code{f.scale=NULL}, and second with only the scale normalization function and \code{f.loc=NULL}.
}

\value{
  \item{mnorm}{An object of class \code{"\link{marrayNorm}"}, containing the normalized intensity data.}
}

\references{S. Dudoit and Y. H. Yang. (2002). Bioconductor R packages for exploratory analysis and normalization of cDNA microarray data. In G. Parmigiani, E. S. Garrett, R. A. Irizarry and S. L. Zeger, editors, \emph{The Analysis of Gene Expression Data: Methods and Software}, Springer, New York.\cr \cr

Y. H. Yang, S. Dudoit, P. Luu, and T. P. Speed (2001). Normalization for cDNA microarray data. In M. L. Bittner, Y. Chen, A. N. Dorsel, and E. R. Dougherty (eds), \emph{Microarrays: Optical Technologies and Informatics}, Vol. 4266 of \emph{Proceedings of SPIE}.\cr \cr

Y. H. Yang, S. Dudoit, P. Luu, D. M. Lin, V. Peng, J. Ngai, and T. P. Speed (2002). Normalization for cDNA microarray data: a robust composite method addressing single and multiple slide systematic variation. \emph{Nucleic Acids Research}, Vol. 30, No. 4.
}

\author{Sandrine Dudoit, \url{http://www.stat.berkeley.edu/~sandrine}.}


\seealso{\code{\link{maNorm}}, \code{\link{maNormScale}}, \code{\link{maNormLoess}}, \code{\link{maLoess}},
\code{\link{maNormMAD}}, \code{\link{maMAD}},
\code{\link{maNormMed}}, \code{\link{maMed}},
\code{\link{maNorm2D}}, \code{\link{ma2D}},
\code{\link{maCompNormA}}, \code{\link{maCompNormEq}}.}

\examples{
# Examples use swirl dataset, for description type ? swirl
data(swirl)

# Within-print-tip-group loess location normalization of batch swirl
# - Default normalization
swirl.norm<-maNormMain(swirl)

boxplot(swirl.norm)
boxplot(swirl.norm[,3])
plot(swirl.norm[,3])

# Global median normalization for arrays 81 and 82
swirl.norm <- maNormMain(swirl[,1:2], f.loc = list(maNormMed(x=NULL,y="maM")))

# Global loess normalization for array 81
swirl.norm <- maNormMain(swirl[,1], f.loc = list(maNormLoess(x="maA",y="maM",z=NULL)))

# Composite normalization as in Yang et al. (2002)
# No MSP controls are available here, so all spots are used for illustration
# purposes
swirl.norm <- maNormMain(swirl[,1], f.loc = list(maNormLoess(x="maA",y="maM",z=NULL),maNormLoess(x="maA",y="maM",z="maPrintTip")), a.loc=maCompNormA())
}
\keyword{smooth}% at least one, from doc/KEYWORDS
\keyword{methods}% __ONLY ONE__ keyword per line

\name{maTwoSamples}
\alias{maTwoSamples}
\alias{widget.TwoSamples}

\title{Changing signs for two sample analysis}
\description{
  Taking target file information and flip the dye swaps experiments.
}
\usage{
maTwoSamples(targetfile, normdata, Trt, Ctl, targetID = "TargetName", slidesID = "Slides", dyesID = "Dyes", RedID = 5, path = ".", output = TRUE)
}
%- maybe also `usage' for other objects documented here.
\arguments{
  \item{targetfile}{A data.frame containing target samples information.}
  \item{normdata}{A R object of class 'marrayNorm'}
  \item{Trt}{A character string representing "treatment" sample.}
  \item{Ctl}{A character string representing "controls" sample.}
  \item{targetID}{A character string representing the column name
    in 'targetfile' containing target samples information.}
  \item{slidesID}{A character string representing the column name
    in 'targetfile' containing the slide label.}
  \item{dyesID}{A character string representing the column name
    in 'targetfile' containing dye labeled information.}
  \item{RedID}{The character use to represent the Cy5 dye.}
  \item{path}{A character string representing the data directory. By
    default this is set to the current working directory ("."). }
  \item{output}{Save and tab delimited file}
}

\value{
  An objects of 'marrayNorm' with the dye assignment adjusted.
}

\author{Yee Hwa (Jean) Yang}

\keyword{manip}% at least one, from doc/KEYWORDS

\name{marrayNorm-class}
\docType{class}

\alias{marrayNorm-class}
\alias{marrayNorm}

\alias{maA}
\alias{maA<-}
\alias{maM}
\alias{maM<-}
\alias{maMloc}
\alias{maMloc<-}
\alias{maMscale}
\alias{maMscale<-}

\alias{[,marrayNorm-method}
\alias{cbind,marrayNorm-method}
\alias{coerce,marrayRaw,marrayNorm-method}
%\alias{coerce,marrayNorm,marraySpots-method}
\alias{maA<-,marrayNorm,matrix-method}
\alias{maA,marrayNorm-method}
\alias{maControls<-,marrayNorm-method}
\alias{maControls,marrayNorm-method}
%\alias{maGnames<-,marrayNorm,marrayInfo-method}
\alias{maGnames,marrayNorm-method}
\alias{maGridCol,marrayNorm-method}
\alias{maGridRow,marrayNorm-method}
%\alias{maLayout<-,marrayNorm,marrayLayout-method}
\alias{maLayout,marrayNorm-method}
\alias{maMloc<-,marrayNorm,matrix-method}
\alias{maMloc,marrayNorm-method}
\alias{maM<-,marrayNorm,matrix-method}
\alias{maM,marrayNorm-method}
\alias{maMscale<-,marrayNorm,matrix-method}
\alias{maMscale,marrayNorm-method}
\alias{maNgc<-,marrayNorm,numeric-method}
\alias{maNgc,marrayNorm-method}
\alias{maNgr<-,marrayNorm,numeric-method}
\alias{maNgr,marrayNorm-method}
\alias{maNormCall,marrayNorm-method}
\alias{maNotes<-,marrayNorm,character-method}
\alias{maNotes,marrayNorm-method}
\alias{maNsamples,marrayNorm-method}
\alias{maNsc<-,marrayNorm,numeric-method}
\alias{maNsc,marrayNorm-method}
\alias{maNspots<-,marrayNorm,numeric-method}
\alias{maNspots,marrayNorm-method}
\alias{maNsr<-,marrayNorm,numeric-method}
\alias{maNsr,marrayNorm-method}
\alias{maPlate<-,marrayNorm-method}
\alias{maPlate,marrayNorm-method}
\alias{maPrintTip,marrayNorm-method}
\alias{maSpotCol,marrayNorm-method}
\alias{maSpotRow,marrayNorm-method}
\alias{maSub<-,marrayNorm-method}
\alias{maSub,marrayNorm-method}
%\alias{maTargets<-,marrayNorm,marrayInfo-method}
\alias{maTargets,marrayNorm-method}
\alias{maW<-,marrayNorm,matrix-method}
\alias{maW,marrayNorm-method}
\alias{maLG,marrayNorm-method}
\alias{maLR,marrayNorm-method}
\alias{print,marrayNorm-method}
\alias{show,marrayNorm-method}
\alias{summary,marrayNorm-method}
\alias{maNormCall}

\title{Class "marrayNorm", classes and methods for post-normalization cDNA microarray intensity data}

\description{
This class represents post-normalization intensity data for a batch of cDNA microarrays. A \emph{batch of arrays} consists of a collection of arrays with the same layout  (\code{"\link{marrayLayout}"}). The class contains slots for the average log-intensities A, the normalized log-ratios M, the location and scale normalization values, the layout of the arrays, and descriptions of the target samples hybridized to the arrays and probe sequences spotted onto the arrays.}

\section{Objects from the Class}{
Objects can be created by calls of the form 
\code{  new('marrayNorm',}\cr
\code{    maA        = ...., # Object of class matrix}\cr
\code{    maM        = ...., # Object of class matrix}\cr
\code{    maMloc     = ...., # Object of class matrix}\cr
\code{    maMscale   = ...., # Object of class matrix}\cr
\code{    maW        = ...., # Object of class matrix}\cr
\code{    maLayout   = ...., # Object of class marrayLayout}\cr
\code{    maGnames   = ...., # Object of class marrayInfo}\cr
\code{    maTargets  = ...., # Object of class marrayInfo}\cr
\code{    maNotes    = ...., # Object of class character}\cr
\code{    maNormCall = ...., # Object of class call}\cr
\code{  )}
}

\section{Slots}{
  \describe{
    \item{\code{maA}:}{Object of class \code{"matrix"}, average log-intensities (base 2) A, rows correspond to spotted probe sequences, columns to arrays in the batch.}
    \item{\code{maM}:}{Object of class \code{"matrix"}, intensity log-ratios (base 2) M, rows correspond to spotted probe sequences, columns to arrays in the batch.}
    \item{\code{maMloc}:}{Object of class \code{"matrix"}, location normalization values, rows correspond to spotted probe sequences, columns to arrays in the batch.}
    \item{\code{maMscale}:}{Object of class \code{"matrix"},  scale normalization values, rows correspond to spotted probe sequences, columns to arrays in the batch.}
    \item{\code{maW}:}{Object of class \code{"matrix"}, spot quality weights, rows correspond to spotted probe sequences, columns to arrays in the batch.}
    \item{\code{maLayout}:}{Object of class \code{"\link{marrayLayout}"}, layout parameters for cDNA microarrays.}
    \item{\code{maGnames}:}{Object of class \code{"\link{marrayInfo}"}, description of spotted probe sequences.}
    \item{\code{maTargets}:}{Object of class \code{"\link{marrayInfo}"}, description of target samples hybridized to the arrays.}
    \item{\code{maNotes}:}{Object of class \code{"character"}, any notes concerning the microarray experiments, e.g. hybridization or scanning conditions.}
    \item{\code{maNormCall}:}{Object of class \code{"call"}, function call for normalizing the batch of arrays.}
  }
}

\section{Methods}{
  \describe{
    \item{[}{\code{signature(x = "marrayNorm")}:  subsetting operator for spots on the array and arrays in the batch, ensures that all slots are subset properly.}
    \item{coerce}{\code{signature(from = "marrayRaw", to = "marrayNorm")}: coerce an object of class \code{"\link{marrayRaw}"} into an object of class \code{\link{marrayNorm}}. }
%    \item{coerce}{\code{signature(from = "marrayNorm", to = "ExpressionSet")}: coerce an object of class \code{"\link{marrayNorm}"} into an object of class \code{\link{ExpressionSet-class}}. }
% \item{coerce}{\code{signature(from = "marrayNorm", to = "marraySpots")}: coerce an object of class \code{"\link{marrayNorm}"} into an object of class \code{"\link{marraySpots}"}. }
    \item{maA}{\code{signature(object = "marrayNorm")}: slot accessor method.}
    \item{maA<-}{\code{signature(object = "marrayNorm", value = "matrix")}: slot assignment method.} 
    \item{maControls<-}{\code{signature(object = "marrayNorm")}: slot assignment method.} 
    \item{maControls}{\code{signature(object = "marrayNorm")}: slot accessor method.}
    \item{maGnames}{\code{signature(object = "marrayNorm")}: slot accessor method.}
    \item{maGnames<-}{\code{signature(object = "marrayNorm", value = "marrayInfo")}: slot assignment method.} 
    \item{maGridCol}{\code{signature(object = "marrayNorm")}: method which computes a vector of grid column coordinates for each spot.}
    \item{maGridRow}{\code{signature(object = "marrayNorm")}: method which computes a vector of grid row coordinates for each spot.}
    \item{maLayout}{\code{signature(object = "marrayNorm")}: slot accessor method.}
    \item{maLayout<-}{\code{signature(object = "marrayNorm", value = "marrayLayout")}: slot assignment method.} 
    \item{maM}{\code{signature(object = "marrayNorm")}: slot accessor method.}
    \item{maM<-}{\code{signature(object = "marrayNorm", value = "matrix")}: slot assignment method.} 
    \item{maMloc}{\code{signature(object = "marrayNorm")}: slot accessor method.}
    \item{maMloc<-}{\code{signature(object = "marrayNorm", value = "matrix")}: slot assignment method.} 
    \item{maMscale}{\code{signature(object = "marrayNorm")}: slot accessor method.}
    \item{maMscale<-}{\code{signature(object = "marrayNorm", value = "matrix")}: slot assignment method.} 
    \item{maNgc}{\code{signature(object = "marrayNorm")}: slot accessor method.}
 \item{maNgc<-}{\code{signature(object = "marrayNorm", value = "numeric")}: slot assignment method.} 
    \item{maNgr}{\code{signature(object = "marrayNorm")}: slot accessor method.}
\item{maNgr<-}{\code{signature(object = "marrayNorm", value = "numeric")}: slot assignment method.} 
    \item{maNormCall}{\code{signature(object = "marrayNorm")}: slot accessor method.}
    \item{maNotes}{\code{signature(object = "marrayNorm")}: slot accessor method.}
    \item{maNotes<-}{\code{signature(object = "marrayNorm", value = "character")}: slot assignment method.} 
    \item{maNsamples}{\code{signature(object = "marrayNorm")}: slot accessor method.}
    \item{maNsc}{\code{signature(object = "marrayNorm")}: slot accessor method.}
\item{maNsc<-}{\code{signature(object = "marrayNorm", value = "numeric")}: slot assignment method.} 
    \item{maNspots}{\code{signature(object = "marrayNorm")}: slot accessor method.}
\item{maNspots<-}{\code{signature(object = "marrayNorm", value = "numeric")}: slot assignment method.} 
    \item{maNsr}{\code{signature(object = "marrayNorm")}: slot accessor method.}
\item{maNsr<-}{\code{signature(object = "marrayNorm", value = "numeric")}: slot assignment method.} 
    \item{maPlate}{\code{signature(object = "marrayNorm")}: slot accessor method.}
    \item{maPlate<-}{\code{signature(object = "marrayNorm")}: slot assignment method.} 
    \item{maPrintTip}{\code{signature(object = "marrayNorm")}: method which computes a vector of print-tip-group indices for each spot.}
    \item{maSpotCol}{\code{signature(object = "marrayNorm")}: method which computes a vector of spot column coordinates for each spot.}
    \item{maSpotRow}{\code{signature(object = "marrayNorm")}: method which computes a vector of spot row coordinates for each spot.}
    \item{maSub}{\code{signature(object = "marrayNorm")}: slot accessor method.}
    \item{maSub<-}{\code{signature(object = "marrayNorm")}: slot assignment method.} 
    \item{maTargets}{\code{signature(object = "marrayNorm")}: slot accessor method.}
    \item{maTargets<-}{\code{signature(object = "marrayNorm", value = "marrayInfo")}: slot assignment method.}  
    \item{maW}{\code{signature(object = "marrayNorm")}: slot accessor method.}
    \item{maW<-}{\code{signature(object = "marrayNorm", value = "matrix")}: slot assignment method.} 
    \item{print}{\code{signature(x = "marrayNorm")}: print method for \code{"\link{marrayNorm}"} class.}
  }
}

\references{S. Dudoit and Y. H. Yang. (2002). Bioconductor R packages for exploratory analysis and normalization of cDNA microarray data. In G. Parmigiani, E. S. Garrett, R. A. Irizarry and S. L. Zeger, editors, \emph{The Analysis of Gene Expression Data: Methods and Software}, Springer, New York.
}

\author{Sandrine Dudoit, \url{http://www.stat.berkeley.edu/~sandrine}.}

\seealso{\code{\link{marrayLayout}}, \code{\link{marrayRaw}},
  \code{\link{marrayInfo}}}

\examples{
# Examples use swirl dataset, for description type ? swirl

data(swirl)

# Median normalization
mnorm<-maNorm(swirl[,2:3],norm="m")

# Object of class marrayNorm for the second and third swirl arrays
mnorm

# Function call
maNormCall(mnorm)

#  Object of class marrayInfo -- Probe sequences
maGnames(mnorm)

#  Object of class marrayInfo -- Target samples
 maTargets(mnorm)

# Density plot of log-ratios M for third array
plot(density(maM(mnorm[,2])), lwd=2, col=2, main="Density plots of log-ratios M")
lines(density(maM(swirl[,3])), lwd=2)
abline(v=0)
legend(2,1,c("Pre-normalization","Post-normalization"))
}

\keyword{classes}

\name{maNormLoess}
\alias{maNormLoess}
\title{Intensity dependent location normalization function}
\description{
This function is used for intensity dependent location normalization, using the robust local regression function \code{\link{loess}}.  It should be used as an argument to the main normalization function \code{\link{maNormMain}}.
}
\usage{
maNormLoess(x="maA", y="maM", z="maPrintTip", w=NULL, subset=TRUE,
span=0.4, ...)
}
\arguments{
  \item{x}{Name of accessor method for spot statistics, usually \code{maA}.}
  \item{y}{Name of accessor method for spot statistics, usually \code{maM}.}
  \item{z}{Name of accessor method for spot statistic used to stratify the data, usually a layout parameter, e.g. \code{maPrintTip} or \code{maPlate}. If \code{z} is not a character, e.g. NULL,  the data are not stratified.}
  \item{w}{An optional numeric vector of weights.}
  \item{subset}{A "logical" or "numeric" vector indicating the subset of points used to compute the fits.}
  \item{span}{The argument \code{span} which controls the degree of
    smoothing in the  \code{\link{loess}} function.}
  \item{...}{Misc arguments}
}

\value{
 A function with bindings for the above arguments. This latter function takes as argument an object of class \code{"\link{marrayRaw}"}  (or possibly \code{"\link{marrayNorm}"}), and returns a vector of fitted values to be substracted from the raw log-ratios. It calls the function \code{\link{maLoess}}, which is not specific to microarray objects.
}

\references{
S. Dudoit and Y. H. Yang. (2002). Bioconductor R packages for exploratory analysis and normalization of cDNA microarray data. In G. Parmigiani, E. S. Garrett, R. A. Irizarry and S. L. Zeger, editors, \emph{The Analysis of Gene Expression Data: Methods and Software}, Springer, New York.\cr \cr

Y. H. Yang, S. Dudoit, P. Luu, and T. P. Speed (2001). Normalization for cDNA microarray data. In M. L. Bittner, Y. Chen, A. N. Dorsel, and E. R. Dougherty (eds), \emph{Microarrays: Optical Technologies and Informatics}, Vol. 4266 of \emph{Proceedings of SPIE}.
\cr \cr

Y. H. Yang, S. Dudoit, P. Luu, D. M. Lin, V. Peng, J. Ngai, and T. P. Speed (2002). Normalization for cDNA microarray data: a robust composite method addressing single and multiple slide systematic variation. \emph{Nucleic Acids Research}, Vol. 30, No. 4.
}

\author{Sandrine Dudoit, \url{http://www.stat.berkeley.edu/~sandrine}.}


\seealso{\code{\link{maNormMain}}, \code{\link{maLoess}}, \code{\link{loess}}.}

\examples{
# See examples for maNormMain.

}

\keyword{smooth}

\name{swirl}
\alias{swirl}
\alias{swirl.1.spot}
\alias{swirl.2.spot}
\alias{swirl.3.spot}
\alias{swirl.4.spot}
\alias{fish.gal}
\alias{SwirlSample.txt}
\alias{SwirlSample}


\title{Gene expression data from Swirl zebrafish cDNA microarray experiment}

\description{
  The \code{swirlRaw} dataset consists of an object \code{swirl}
  of class \code{\link{marrayRaw}}, which represents
  pre-normalization intensity data for a batch of cDNA microarrays.
  
  This experiment was carried out
using zebrafish as a model organism to study early development in
vertebrates. Swirl is a point mutant in the BMP2 gene that affects the
dorsal/ventral body axis. Ventral fates such as blood are reduced, whereas
dorsal structures such as somites and notochord are expanded. A goal of
the Swirl experiment is to identify genes with altered expression in
the swirl mutant compared to wild-type zebrafish. Two sets of dye-swap experiments were performed, for a total of four replicate hybridizations. For
each of these hybridizations, target cDNA from the swirl mutant was labeled using
one of the Cy3 or Cy5 dyes and the target cDNA wild-type mutant was labeled using
the other dye. Target cDNA was hybridized to microarrays containing
8,448 cDNA probes, including 768 controls spots (e.g. negative, positive,
and normalization controls spots). Microarrays were printed using \eqn{4 \times 4} print-tips and are thus partitioned into a \eqn{4 \times 4} grid matrix. Each grid consists of a \eqn{22\times 24} spot matrix that was printed with a single print-tip. Here, spot row and plate coordinates should coincide, as each row of spots corresponds to probe sequences from the same 384 well-plate.\cr


Each of the four hybridizations produced
a pair of 16-bit images, which were processed using the image analysis software package \code{Spot}. Raw images of the Cy3 and Cy5 fluorescence intensities for all fourhybridizations are available at \url{http://fgl.lsa.berkeley.edu/Swirl/index.html}.the dataset includes four output files
\code{swirl.1.spot},
\code{swirl.2.spot},
\code{swirl.3.spot}, and
\code{swirl.4.spot}
from the \code{Spot} package.  Each of these files contains
8,448 rows and 30 columns; rows correspond to spots and columns to
different statistics from the \code{Spot} image analysis output. The file
\code{fish.gal} is a gal file generated by the \code{GenePix} program; it contains information on individual probe sequences, such as gene names, spot ID, spot coordinates.  Hybridization information for the mutant and wild-type target samples is stored in \code{SwirlSample.txt}.


}

\usage{data(swirl)}

\source{These data were provided by Katrin Wuennenberg-Stapleton from the Ngai Lab at UC Berkeley. The swirl embryos for this experiment were provided by David Kimelman and David
Raible at the University of Washington.}

\keyword{datasets}

\name{maCompLayout}
\alias{maCompLayout}

\title{Generate a marrayLayout object}
\description{
  Take a matrix of cooordiates and generate a marrayLayout object.
}
\usage{
maCompLayout(mat, ncolumns = 4)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{mat}{a matrix of coordinates, this can either be n by 3 matrix
    with columns (Block, Row, Column) or n by 4 matrix with columns
    (Grid.R, Grid.C, Spot.R, Spot.C)}
  \item{ncolumns}{For n by 3 matrix, the number of meta-grid columns.
    By default, it is set to 4.}
}
\value{
  An object of class \code{"\link{marrayLayout}"}.
}

\author{Jean Yang}

\examples{
X <- cbind(Block = c(1,1,2,2,3,3,4,4), Rows=c(1,2,1,2,1,2,1,2), Columns=rep(1,8))
maCompLayout(X, ncolumns=2)
}
\keyword{manip}% at least one, from doc/KEYWORDS

\name{dim}
\alias{dim.marrayRaw}
\alias{dim.marrayNorm}
\alias{dim.marrayInfo}

\title{Retrieve the Dimensions of an marrayRaw, marrayNorm or marrayInfo Object}
\description{
  Retrieve the number of rows (genes) and columns (arrays) for an
  marrayRaw, marrayNorm or marrayInfo object.
}
\usage{
\method{dim}{marrayRaw}(x)
}
\arguments{
  \item{x}{an object of class \code{marrayRaw}, \code{marrayNorm} or \code{marrayInfo}}
}
\details{
Microarray data objects share many analogies with ordinary matrices in
which the rows correspond to spots or genes and the columns to arrays. 
These methods allow one to extract the size of microarray data objects
in the same way that one would do for ordinary matrices. 

A consequence is that row and column commands \code{nrow(x)}, \code{ncol(x)} and so on also work.
}
\value{
Numeric vector of length 2.
The first element is the number of rows (genes) and the second is the number of columns (arrays).
}
\author{modified from Gordon Smyth's function}

\seealso{
  \code{\link[base]{dim}} in the base package.
}
\examples{
M <- A <- matrix(11:14,4,2)
rownames(M) <- rownames(A) <- c("a","b","c","d")
colnames(M) <- colnames(A) <- c("A1","A2")
MA <- new("marrayNorm", maM=M,maA=A)
dim(MA)
dim(M)
}
\keyword{array}

\name{maNorm2D}
\alias{maNorm2D}
%- Also NEED an `\alias' for EACH other topic documented here.
\title{2D spatial location normalization function}
\description{
This function is used for 2D spatial location normalization, using the robust local regression function \code{\link{loess}}.  It should be used as an argument to the main normalization function \code{\link{maNormMain}}.
}

\usage{
maNorm2D(x="maSpotRow", y="maSpotCol", z="maM", g="maPrintTip", w=NULL,
subset=TRUE, span=0.4, ...)
}
%- maybe also `usage' for other objects documented here.
\arguments{
  \item{x}{Name of accessor method for spot row coordinates, usually \code{maSpotRow}.}
  \item{y}{Name of accessor method for spot column coordinates, usually \code{maSpotCol}.}
  \item{z}{Name of accessor method for spot statistics, usually the log-ratio \code{maM}.}
  \item{g}{Name of accessor method for print-tip-group indices, usually \code{maPrintTip}.}
  \item{w}{An optional numeric vector of weights.}
  \item{subset}{A "logical" or "numeric" vector indicating the subset of points used to
  compute the fits. }
\item{span}{The argument \code{span} which controls the degree of
  smoothing in the  \code{\link{loess}} function.}
  \item{...}{Misc arguments}
}

\details{
The spot statistic named in \code{z} is regressed on spot row and column coordinates, separately within print-tip-group, using the \code{\link{loess}} function.
}

\value{
 A function with bindings for the above arguments. This latter function takes as argument an object of class \code{"\link{marrayRaw}"} (or possibly \code{"\link{marrayNorm}"}), and returns a vector of fitted values to be substracted from the raw log-ratios. It calls the function \code{\link{ma2D}}, which is not specific to microarray objects.
}

\references{S. Dudoit and Y. H. Yang. (2002). Bioconductor R packages for exploratory analysis and normalization of cDNA microarray data. In G. Parmigiani, E. S. Garrett, R. A. Irizarry and S. L. Zeger, editors, \emph{The Analysis of Gene Expression Data: Methods and Software}, Springer, New York.
}

\author{Sandrine Dudoit, \url{http://www.stat.berkeley.edu/~sandrine}.}


\seealso{\code{\link{maNormMain}}, \code{\link{ma2D}}, \code{\link{loess}}.}

\examples{
# See examples for maNormMain.

}
\keyword{smooth}% at least one, from doc/KEYWORDS

\name{maNormScale}
\alias{maNormScale}
\title{Simple scale normalization function}
\description{
This function is a simple wrapper function around the main normalization
function \code{\link{maNormMain}}. It allows the user to choose from a
set of two basic scale normalization procedures. The function operates
on an object of class \code{"\link{marrayRaw}"} (or possibly
\code{"\link{marrayNorm}"}, if normalization is performed in several
steps) and returns an object of class \code{"\link{marrayNorm}"}. This
function can be used to conormalize a batch of arrays 
(\code{norm="globalMAD"} option).
}
\usage{
maNormScale(mbatch, norm=c("globalMAD", "printTipMAD"), subset=TRUE, geo=TRUE,  Mscale=TRUE, echo=FALSE)
}

\arguments{
  \item{mbatch}{An object of class \code{"\link{marrayRaw}"}, containing 
intensity data for the batch of arrays to be normalized. An object of class 
\code{\link{marrayNorm}} may also be passed if normalization is performed 
in several steps.}
\item{norm}{A character string specifying the normalization
  procedures: 
\describe{
\item{globalMAD}{for global scale
  normalization using the median absolute deviation (MAD), this allows between 
slide scale normalization}
\item{printTipMAD}{for within-print-tip-group scale normalization
  using the median absolute deviation (MAD).}
This argument can be
  specified using the first letter of each method.}}
  \item{subset}{A "logical" or "numeric" vector indicating the subset of points used to
  compute the  normalization values.}
 \item{geo}{If \code{TRUE}, the MAD of each group is divided by the
  geometric mean of the MADs across groups (cf. Yang et al. (2002)). This allows observations to
  retain their original units.}
  \item{Mscale}{If \code{TRUE}, the scale normalization values are stored in the slot \code{maMscale} of the object of class \code{"\link{marrayNorm}"} returned by the function, if \code{FALSE}, these values are not retained.}
\item{echo}{If \code{TRUE}, the index of the array currently being
  normalized is printed.}
}
\details{
See \code{\link{maNormMain}} for details and more general procedures.
}
\value{
\item{mnorm}{An object of class \code{"\link{marrayNorm}"}, containing the normalized intensity data.}
}

\references{S. Dudoit and Y. H. Yang. (2002). Bioconductor R packages for exploratory analysis and normalization of cDNA microarray data. In G. Parmigiani, E. S. Garrett, R. A. Irizarry and S. L. Zeger, editors, \emph{The Analysis of Gene Expression Data: Methods and Software}, Springer, New York.\cr \cr

Y. H. Yang, S. Dudoit, P. Luu, and T. P. Speed (2001). Normalization for cDNA microarray data. In M. L. Bittner, Y. Chen, A. N. Dorsel, and E. R. Dougherty (eds), \emph{Microarrays: Optical Technologies and Informatics}, Vol. 4266 of \emph{Proceedings of SPIE}.\cr \cr

Y. H. Yang, S. Dudoit, P. Luu, D. M. Lin, V. Peng, J. Ngai, and T. P. Speed (2002). Normalization for cDNA microarray data: a robust composite method addressing single and multiple slide systematic variation. \emph{Nucleic Acids Research}, Vol. 30, No. 4.
}

\author{Sandrine Dudoit, \url{http://www.stat.berkeley.edu/~sandrine}.}

\seealso{\code{\link{maNormMain}}, \code{\link{maNorm}}.}


\examples{
# Examples use swirl dataset, for description type ? swirl
data(swirl)

# Global median normalization followed by global MAD normalization for 
# only arrays 2 and 3 in the batch swirl

mnorm1<-maNorm(swirl[,2:3], norm="m")
mnorm2<-maNormScale(mnorm1, norm="g")

}

\keyword{smooth}% at least one, from doc/KEYWORDS
\keyword{methods}% __ONLY ONE__ keyword per line

\name{maMAD}
\alias{maMAD}
%- Also NEED an `\alias' for EACH other topic documented here.
\title{Stratified MAD calculation}
\description{
This function computes the median absolute deviation (MAD) of values in \code{y} separately within values of \code{x}. It is used by \code{\link{maNormMAD}} for MAD scale normalization.
}
\usage{
maMAD(x, y, geo=TRUE, subset=TRUE)
}
%- maybe also `usage' for other objects documented here.
\arguments{
  \item{x}{Variables used to stratify the data.}
  \item{y}{A numeric vector.}
  \item{geo}{If \code{TRUE}, the MAD of each group is divided by the
  geometric mean of the MADs across groups (cf. Yang et al. (2002)). This allows observations to
  retain their original units.}
  \item{subset}{A "logical" or "numeric" vector indicating the subset of points used to compute the MAD. }
}

\value{
A numeric vector of MAD values. 
}


\references{S. Dudoit and Y. H. Yang. (2002). Bioconductor R packages for exploratory analysis and normalization of cDNA microarray data. In G. Parmigiani, E. S. Garrett, R. A. Irizarry and S. L. Zeger, editors, \emph{The Analysis of Gene Expression Data: Methods and Software}, Springer, New York.\cr \cr

Y. H. Yang, S. Dudoit, P. Luu, and T. P. Speed (2001). Normalization for cDNA microarray data. In M. L. Bittner, Y. Chen, A. N. Dorsel, and E. R. Dougherty (eds), \emph{Microarrays: Optical Technologies and Informatics}, Vol. 4266 of \emph{Proceedings of SPIE}.\cr \cr

Y. H. Yang, S. Dudoit, P. Luu, D. M. Lin, V. Peng, J. Ngai, and T. P. Speed (2002). Normalization for cDNA microarray data: a robust composite method addressing single and multiple slide systematic variation. \emph{Nucleic Acids Research}, Vol. 30, No. 4.
}

\author{Sandrine Dudoit, \url{http://www.stat.berkeley.edu/~sandrine}.}


\seealso{\code{\link{maNormMain}}, \code{\link{maNormMAD}}, \code{\link{mad}}.}

\examples{
# See examples for maNormMain.

}

\keyword{univar}% at least one, from doc/KEYWORDS
\keyword{robust}% __ONLY ONE__ keyword per line

\name{maNormMAD}
\alias{maNormMAD}
%- Also NEED an `\alias' for EACH other topic documented here.
\title{MAD scale normalization function}
\description{
This function is used for scale normalization using the median absolute deviation (MAD) of intensity log-ratios for a group of spots. It can be used for within or between array normalization. The function should be used as an argument to the main normalization function \code{\link{maNormMain}}.
}

\usage{
maNormMAD(x=NULL, y="maM", geo=TRUE, subset=TRUE)
}
%- maybe also `usage' for other objects documented here.

\arguments{
  \item{x}{Name of accessor function for spot statistic used to stratify the data, usually a layout parameter, e.g. \code{maPrintTip} or \code{maPlate}. If \code{x} is not a character, e.g. NULL, the data are not stratified.}
  \item{y}{Name of accessor function for spot statistics, usually
  \code{maM}.}
  \item{geo}{If \code{TRUE}, the MAD of each group is divided by the
  geometric mean of the MADs across groups (cf. Yang et al. (2002)). This allows observations to
  retain their original units.}
  \item{subset}{A "logical" or "numeric" vector indicating the subset of points used to compute the scale normalization values. }
}

\value{
A function with bindings for the above arguments. This latter function takes as argument an object of class \code{"\link{marrayRaw}"}  (or possibly \code{"\link{marrayNorm}"}), and returns a vector of values used to scale the location normalized log-ratios. It calls the function \code{\link{maMAD}}, which is not specific to microarray objects.
}


\references{S. Dudoit and Y. H. Yang. (2002). Bioconductor R packages for exploratory analysis and normalization of cDNA microarray data. In G. Parmigiani, E. S. Garrett, R. A. Irizarry and S. L. Zeger, editors, \emph{The Analysis of Gene Expression Data: Methods and Software}, Springer, New York.\cr \cr

Y. H. Yang, S. Dudoit, P. Luu, and T. P. Speed (2001). Normalization for cDNA microarray data. In M. L. Bittner, Y. Chen, A. N. Dorsel, and E. R. Dougherty (eds), \emph{Microarrays: Optical Technologies and Informatics}, Vol. 4266 of \emph{Proceedings of SPIE}.
\cr \cr

Y. H. Yang, S. Dudoit, P. Luu, D. M. Lin, V. Peng, J. Ngai, and T. P. Speed (2002). Normalization for cDNA microarray data: a robust composite method addressing single and multiple slide systematic variation. \emph{Nucleic Acids Research}, Vol. 30, No. 4.
}

\author{Sandrine Dudoit, \url{http://www.stat.berkeley.edu/~sandrine}.}


\seealso{\code{\link{maNormMain}}, \code{\link{maMAD}}, \code{\link{mad}}.}

\examples{
# See examples for maNormMain.

}

\keyword{univar}% at least one, from doc/KEYWORDS
\keyword{robust}% __ONLY ONE__ keyword per line

\name{ShowLargeObject-class}
\docType{class}

\alias{ShowLargeObject-class}
\alias{show,ShowLargeObject-method}
%\alias{print-methods}

\title{Show Large Data Object - class}
      
\description{
A virtual class including the data classes \code{marrayRaw},
\code{marrayNorm}, \code{marrayInfo}, \code{marrayLayout},
\code{PrinterInfo}, \code{RGData} and \code{MAData}, all of which typically contain large
quantities of numerical data in vector, matrices and data.frames. 
}

\section{Methods}{
A \code{show} method is defined for objects of class
\code{ShowLargeObject} which uses \code{printHead} to print only the
leading elements or rows of components or slots which contain large
quantities of data. 
}

\author{modifid from Gordon Smyth's function}

\keyword{classes}
\keyword{data}

\name{maCompCoord}
\alias{maCompCoord}
%- Also NEED an `\alias' for EACH other topic documented here.
\title{Generate grid and spot matrix coordinates}
\description{
This function generates grid and spot matrix coordinates from ranges of
rows and columns for the grid and spot matrices. Spots on the array are numbered consecutively starting
from the top left grid and the top left spot within each grid.
}
\usage{
maCompCoord(grows, gcols, srows, scols)
}
%- maybe also `usage' for other objects documented here.
\arguments{
  \item{grows}{numeric vector of grid rows.}
  \item{gcols}{numeric vector of grid columns.}
  \item{srows}{numeric vector of spot rows.}
  \item{scols}{numeric vector of spot columns.}
}

\value{

a matrix of spot four-coordinates, with rows corresponding to
  spots and columns to grid row, grid column, spot row, and spot column coordinates.


}

\author{Yee Hwa (Jean) Yang, Sandrine Dudoit, \url{http://www.stat.berkeley.edu/~sandrine}.}

\seealso{\code{\link{marrayLayout}}, \code{\link{maCoord2Ind}},
\code{\link{maInd2Coord}}, \code{\link{maCompInd}}.}


\examples{

maCompCoord(1:2,1,1:4,1:3)


}
\keyword{manip}
\keyword{methods}

\name{maMed}
\alias{maMed}
\title{Stratified median calculation}
\description{
This function computes the median of values in \code{y} separately within values of \code{x}. It is used by \code{\link{maNormMed}} for median location normalization.
}
\usage{
maMed(x, y, subset=TRUE)
}
\arguments{
  \item{x}{Variables used to stratify the data.}
  \item{y}{A numeric vector.}
  \item{subset}{A "logical" or "numeric" vector indicating the subset of points used to compute the median. }
}

\value{
A numeric vector of median values. 
}


\references{S. Dudoit and Y. H. Yang. (2002). Bioconductor R packages for exploratory analysis and normalization of cDNA microarray data. In G. Parmigiani, E. S. Garrett, R. A. Irizarry and S. L. Zeger, editors, \emph{The Analysis of Gene Expression Data: Methods and Software}, Springer, New York.\cr \cr

Y. H. Yang, S. Dudoit, P. Luu, and T. P. Speed (2001). Normalization for cDNA microarray data. In M. L. Bittner, Y. Chen, A. N. Dorsel, and E. R. Dougherty (eds), \emph{Microarrays: Optical Technologies and Informatics}, Vol. 4266 of \emph{Proceedings of SPIE}.\cr \cr

Y. H. Yang, S. Dudoit, P. Luu, D. M. Lin, V. Peng, J. Ngai, and T. P. Speed (2002). Normalization for cDNA microarray data: a robust composite method addressing single and multiple slide systematic variation. \emph{Nucleic Acids Research}, Vol. 30, No. 4.
}

\author{Sandrine Dudoit, \url{http://www.stat.berkeley.edu/~sandrine}.}


\seealso{\code{\link{maNormMain}}, \code{\link{maNormMed}}, \code{\link{median}}.}


\examples{
# See examples for maNormMain.

}

\keyword{univar}
\keyword{robust}

\name{maCompPlate}
\alias{maCompPlate}
%- Also NEED an `\alias' for EACH other topic documented here.
\title{Generate plate IDs}
\description{
This function generates plate IDs from the dimensions of the grid and
spot matrices. Note that this function only applies to arrays with a
regular plate layout, where the number of spots is a multiple of the
number of wells on a plate (usually 96 or 384) and each well contributes 
exactly one spot. It should thus be used with caution.}

\usage{
maCompPlate(x, n=384)
}
%- maybe also `usage' for other objects documented here.
\arguments{
  \item{x}{object of class \code{"\link{marrayLayout}"},
    \code{"\link{marrayRaw}"} and \code{"\link{marrayNorm}"}}
  \item{n}{object of class "numeric", number of wells in each plate,
    usually 384 or 96.} 
}

\details{
  Having plate IDs may be useful for the purpose of
  normalization. Normalization by plate can be done using the function
  \code{\link{maNormMain}}. 
}
\value{
a vector of plate IDs (\code{\link{factor}}).
}

\author{Yee Hwa (Jean) Yang, Sandrine Dudoit, \url{http://www.stat.berkeley.edu/~sandrine}.}

\seealso{\code{\link{marrayLayout}}, \code{\link{marrayRaw}},
  \code{\link{marrayNorm}}}

\examples{

L<-new("marrayLayout", maNgr=4, maNgc=4, maNsr=22, maNsc=24)
plate<-maCompPlate(L,384)
table(plate)
maPlate(L)<-plate

}
\keyword{methods}% at least one, from doc/KEYWORDS

\name{maPalette}
\alias{maPalette}
%- Also NEED an `\alias' for EACH other topic documented here.
\title{Microarray color palette}
\description{
This function returns a vector of color names corresponding to a range of colors specified in the arguments.  
}
\usage{
maPalette(low = "white", high = c("green", "red"), mid=NULL, k =50)
}
%- maybe also `usage' for other objects documented here.
\arguments{
  \item{low}{Color for the lower end of the color palette, specified using any of the three kinds of R colors, i.e., either a color name (an element of \code{colors}), a hexadecimal string of the form \code{"#rrggbb"}, or an integer \code{i} meaning \code{palette()[i]}.}
  \item{high}{Color for the upper end of the color palette, specified
  using any of the three kinds of R colors, i.e., either a color name
  (an element of \code{colors}), a hexadecimal string of the form
  \code{"#rrggbb"}, or an integer \code{i} meaning \code{palette()[i]}.}
 \item{mid}{Color for the middle portion of the color palette, specified using any of the three kinds of R colors, i.e., either a color name (an element of \code{colors}), a hexadecimal string of the form \code{"#rrggbb"}, or an integer \code{i} meaning \code{palette()[i]}.}
  \item{k}{Number of colors in the palette.}
}

\value{
  A "character" vector of color names. This can be used to create a user-defined color palette for subsequent graphics by \code{palette}, in a \code{col=} specification in graphics functions, or in \code{\link{par}}.
}

\references{
S. Dudoit and Y. H. Yang. (2002). Bioconductor R packages for exploratory analysis and normalization of cDNA microarray data. In G. Parmigiani, E. S. Garrett, R. A. Irizarry and S. L. Zeger, editors, \emph{The Analysis of Gene Expression Data: Methods and Software}, Springer, New York.
}

\author{Sandrine Dudoit, \url{http://www.stat.berkeley.edu/~sandrine}, Yee Hwa (Jean) Yang.}

\seealso{\code{\link{image}}, \code{\link{maColorBar}}, \code{\link{maImage}}, \code{\link{maImage.func}}.}

\examples{
par(mfrow=c(1,4))
pal <- maPalette(low="red", high="green")
maColorBar(seq(-2,2, 0.2), col=pal, horizontal=FALSE, k=21)
pal <- maPalette(low="red", high="green", mid="yellow")
maColorBar(seq(-2,2, 0.2), col=pal, horizontal=FALSE, k=21)
pal <- maPalette()
maColorBar(seq(-2,2, 0.2), col=pal, horizontal=FALSE, k=21)
pal <- maPalette(low="purple", high="purple",mid="white")
maColorBar(seq(-2,2, 0.2), col=pal, horizontal=FALSE, k=21)
}
\keyword{color}% at least one, from doc/KEYWORDS

\name{maDotsMatch}
\alias{maDotsMatch}

\title{Replace default arguments of a function by user supplied values}

\description{
  This function may be used to replace default arguements for any
  functions to user supplied parameters.}


\usage{
maDotsMatch(dots, defaults)
}

\arguments{
  \item{dots}{List of user supplied argements, e.g. from \code{list(...)}.}
  \item{defaults}{List of  formal arguments of a function, e.g. from the
    function \code{\link{formals}}.} 
}

\value{
  \item{args}{List of argument of a function.}
}

\author{Jean Yee Hwa Yang}

\seealso{\code{\link{maDefaultPar}}, \code{\link{maDotsDefaults}}}

\examples{
dots<-list(x=1:10, y=11:20)
argsfun <- maDotsMatch(dots, formals(args(plot)))
do.call("plot", argsfun)
}
\keyword{misc}% at least one, from doc/KEYWORDS

\name{maBoxplot}
\alias{maBoxplot}

%- Also NEED an `\alias' for EACH other topic documented here.
\title{Boxplots for cDNA microarray spot statistics}

\description{ The function \code{maBoxplot} produces boxplots of microarray spot
statistics for the classes  \code{\link{marrayRaw}} and
\code{\link{marrayNorm}}.We encourage users to use "boxplot"  rather than "maBoxplot".  The name of the arguments have changed.}

\usage{
maBoxplot(m, x="maPrintTip", y="maM", ...)
}
%- maybe also `usage' for other objects documented here.

\arguments{
  \item{m}{Microarray object of class \code{"\link{marrayRaw}"} and \code{"\link{marrayNorm}"}}
  \item{x}{Name of accessor method for the spot statistic used to stratify the data, typically a slot name for the microarray layout object (see \code{"\link{marrayLayout}"}) such as \code{maPlate} or a method such as \code{maPrintTip}. If \code{x} is NULL, the data are not stratified.}
  \item{y}{Name of accessor method for the spot statistic of interest, typically a slot name for the microarray object \code{m}, such as \code{maM}.}
  \item{\dots}{Optional graphical parameters, see \code{\link{par}}.}
}

\details{
If there are more than one array in the batch, the function produces a boxplot for each array in the batch. Such plots are useful when assessing the need for between array normalization, for example, to deal with scale differences among different arrays. Default graphical parameters are chosen for convenience using the function \code{\link{maDefaultPar}} (e.g. color palette,  axis labels,  plot title) but the user has the option to overwrite these parameters at any point.
}

\references{
S. Dudoit and Y. H. Yang. (2002). Bioconductor R packages for exploratory analysis and normalization of cDNA microarray data. In G. Parmigiani, E. S. Garrett, R. A. Irizarry and S. L. Zeger, editors, \emph{The Analysis of Gene Expression Data: Methods and Software}, Springer, New York.
}

\author{Sandrine Dudoit, \url{http://www.stat.berkeley.edu/~sandrine}.}

\seealso{\code{\link{boxplot}}, \code{\link{maDefaultPar}}.}

\examples{
## see example in boxplot
}
\keyword{hplot}% at least one, from doc/KEYWORDS

\name{write.list}
\alias{write.list}

\title{Data Output}

\description{
  Writes information from a list into a text file.
}

\usage{
write.list(x, filename = "data", append = FALSE, closefile = TRUE, outfile)
}
\arguments{
  \item{x}{the list object to be written.}
  \item{filename}{a character string representing the file name.}
  \item{append}{logical; if true, the data \code{x} is appended to file
    \code{filename}.}
  \item{closefile}{logical indicating if the file connection should be closed.}
  \item{outfile}{file name or connections.}
}
\details{
  This function may be called recursively if there exists list structure
  within a list.
}

\author{Jean Yee Hwa Yang}

\seealso{\code{\link{write.table}}, \code{\link{write}}}

\examples{
data(swirl)
test <- list(A = 1:10, B= maM(swirl)[1:10,], C=list(x=1:10, y=1:4),
             D = summary(maA(swirl[,1])))
write.list(test, filename="test.txt")
}
\keyword{file}

\name{maCompNormA}
\alias{maCompNormA}
\alias{maCompNormEq}
%- Also NEED an `\alias' for EACH other topic documented here.
\title{Weights for composite normalization}
\description{
This function is used for composite normalization with intensity dependent weights. The function should be used as an argument to the main normalization function \code{\link{maNormMain}}. It only applies when two normalization procedures are combined. 
}
\usage{
maCompNormA()
maCompNormEq()
}
%- maybe also `usage' for other objects documented here.

\value{
  A function which takes as arguments \code{x} and \code{n}, the spot average log-intensities A and the number of normalization procedures. This latter function returns a matrix of weights for combining two normalization procedures, rows correspond to spots and columns to normalization procedures. The weights for the first procedure are given by the empirical cumulative distribution function of the spot average log-intensities A. Note that when performing composite normalization as described in Yang et al. (2002), the first normalization procedure is the global fit and the second procedure is the within-print-tip-group fit.\cr \cr
For \code{maCompEq}, equal weights are given for each procedure.

}

\references{S. Dudoit and Y. H. Yang. (2002). Bioconductor R packages for exploratory analysis and normalization of cDNA microarray data. In G. Parmigiani, E. S. Garrett, R. A. Irizarry and S. L. Zeger, editors, \emph{The Analysis of Gene Expression Data: Methods and Software}, Springer, New York.\cr \cr

Y. H. Yang, S. Dudoit, P. Luu, D. M. Lin, V. Peng, J. Ngai, and T. P. Speed (2002). Normalization for cDNA microarray data: a robust composite method addressing single and multiple slide systematic variation. \emph{Nucleic Acids Research}, Vol. 30, No. 4.
}

\author{Sandrine Dudoit, \url{http://www.stat.berkeley.edu/~sandrine}, Yee Hwa (Jean) Yang.}

\seealso{\code{\link{maNormMain}}, \code{\link{maNormLoess}}, \code{\link{ecdf}}.}

\examples{
# See examples for maNormMain
}
\keyword{smooth}% at least one, from doc/KEYWORDS

\name{maImage.func}
\alias{maImage.func}

%- Also NEED an `\alias' for EACH other topic documented here.
\title{Color image for cDNA microarray spot statistics}
\description{
This function creates spatial images of shades of gray or colors that correspond to the values of a statistic for each spot on the array. The statistic can be the intensity log-ratio M, a spot quality measure (e.g. spot size or shape), or a test statistic. This function can be used to explore whether there are any spatial effects in the data, for example, print-tip or cover-slip effects. This function is called by \code{\link{maImage}}.
}

\usage{
maImage.func(x, L, subset=TRUE, col=heat.colors(12), contours=FALSE, overlay=NULL, ol.col=1,  ...)
}
%- maybe also `usage' for other objects documented here.

\arguments{
  \item{x}{A "numeric" vector of spot statistics.}
  \item{L}{An object of class \code{"\link{marrayLayout}"}, if L is
    missing we will assume the dimension of x.}
  \item{subset}{A "logical" or "numeric" vector indicating the subset of spots to display on the image.}
  \item{col}{A list of colors such as that generated by rainbow, heat.colors, topo.colors, terrain.colors, or similar functions. In addition to these color palette functions, a new function  \code{\link{maPalette}} was defined to generate color palettes from user supplied low, middle, and high color values.}
  \item{contours}{If \code{contours=TRUE}, contours are plotted,
    otherwise they are not shown.}
  \item{overlay}{A logical vector of spots to be highlighted on the image plots.}
  \item{ol.col}{Color of the overlay spots.}
  \item{\dots}{Optional graphical parameters, see \code{\link{par}}.}
}

\references{S. Dudoit and Y. H. Yang. (2002). Bioconductor R packages for exploratory analysis and normalization of cDNA microarray data. In G. Parmigiani, E. S. Garrett, R. A. Irizarry and S. L. Zeger, editors, \emph{The Analysis of Gene Expression Data: Methods and Software}, Springer, New York.
}


\author{Sandrine Dudoit, \url{http://www.stat.berkeley.edu/~sandrine}.}

\seealso{\code{\link{image}}, \code{\link{maImage}}, \code{\link{maColorBar}}, \code{\link{maPalette}}.}

\examples{
# See examples for image.
}
\keyword{hplot}% at least one, from doc/KEYWORDS

\name{maLoessLines}
\alias{maLoessLines}
\alias{maLowessLines}

%- Also NEED an `\alias' for EACH other topic documented here.
\title{Add smoothed fits to a plot}
\description{
This function may be used to compute and plot loess or lowess fits for an existing plot. The plot can be produced by \code{\link{plot}}, \code{\link{maPlot}}, or \code{\link{maPlot.func}}.
}

\usage{
maLoessLines(subset=TRUE, weights=NULL, loess.args=list(span = 0.4,
degree=1, family="symmetric", control=loess.control(trace.hat =
"approximate", iterations=5, surface="direct")), col=2, lty=1, lwd=2.5, ...)

maLowessLines(subset = TRUE, f = 0.3, col = 2, lty = 1, lwd = 2.5, ...)
}
%- maybe also `usage' for other objects documented here.
\arguments{
  \item{subset}{A "logical" or "numeric" vector indicating the subset of points used to compute the fits. }
  \item{weights}{Optional "numeric" vector of weights -- for \code{maLoessLines} only.} 
  \item{loess.args}{List of optional arguments for the \code{\link{loess}} functions -- for \code{maLoessLines} only.}
  \item{f}{The smoother span for the \code{\link{lowess}} function -- for \code{maLowessLines} only.}
  \item{col}{The fitted line colors.}
  \item{lty}{The fitted line types.}
  \item{lwd}{The fitted line widths.}
  \item{\dots}{Optional graphical parameters, see \code{\link{par}}.}
}

\value{
A function with bindings for \code{subset}, \code{weights}, \code{loess.args},  \code{col}, \code{lty}, \code{lwd}, and \code{\ldots}. This latter function takes as arguments \code{x} and \code{y}, the abscissa and ordinates of points on the plot, and \code{z} a vector of discrete values used to stratify the points. Loess (or lowess) fits are performed separately within values of \code{z}.
}

\references{S. Dudoit and Y. H. Yang. (2002). Bioconductor R packages for exploratory analysis and normalization of cDNA microarray data. In G. Parmigiani, E. S. Garrett, R. A. Irizarry and S. L. Zeger, editors, \emph{The Analysis of Gene Expression Data: Methods and Software}, Springer, New York.
}


\author{Sandrine Dudoit, \url{http://www.stat.berkeley.edu/~sandrine}.}

\seealso{\code{\link{loess}}, \code{\link{lowess}}, \code{\link{maPlot}}, \code{\link{maPlot.func}}.}

\examples{
# See examples for maPlot.

}

\keyword{aplot}% at least one, from doc/KEYWORDS
\keyword{smooth}% __ONLY ONE__ keyword per line

\name{maPlot.func}
\alias{maPlot.func}
%- Also NEED an `\alias' for EACH other topic documented here.
\title{Scatter-plots with fitted curves and text}
\description{
 This function produces scatter-plots of \code{x} vs. \code{y}. It also allows the user to highlight and annotate subsets of points on the plot, and display fitted curves from robust local regression or other smoothing procedures.
}

\usage{
maPlot.func(x, y, z, 
	lines.func = maLowessLines(subset = TRUE, f = 0.3, col = 1:length(unique(z)), lty = 1, lwd = 2.5),
    	text.func = maText(), 
	legend.func = maLegendLines(legend = as.character(unique(z)), col = 1:length(unique(z)), lty = 1, lwd = 2.5, ncol = 1),
	 ...)
}
%- maybe also `usage' for other objects documented here.

\arguments{
  \item{x}{A "numeric" vector for the abscissa.}
  \item{y}{A "numeric" vector for the ordinates.}
  \item{z}{A vector of statistic used to stratify the data, smoothed curves are fitted separately within values of \code{z}}
  \item{lines.func}{A function for computing and plotting smoothed fits of \code{y} as a function of \code{x}, separately within values of \code{z}, e.g. \code{\link{maLoessLines}}.}
  \item{text.func}{A function for highlighting a subset of points, e.g., \code{maText}.}
  \item{legend.func}{A function for adding a legend to the plot, e.g. \code{\link{maLegendLines}}.}
  \item{\dots}{Optional graphical parameters, see \code{\link{par}}.}
}

\references{S. Dudoit and Y. H. Yang. (2002). Bioconductor R packages for exploratory analysis and normalization of cDNA microarray data. In G. Parmigiani, E. S. Garrett, R. A. Irizarry and S. L. Zeger, editors, \emph{The Analysis of Gene Expression Data: Methods and Software}, Springer, New York.
}


\author{Sandrine Dudoit, \url{http://www.stat.berkeley.edu/~sandrine}.}

\seealso{\code{\link{maPlot}}, \code{\link{maLoessLines}}, \code{\link{maLegendLines}}, \code{\link{maText}}, \code{\link{plot}}, \code{\link{lowess}}, \code{\link{loess}}, \code{\link{legend}}.}

\examples{
# See examples for maPlot.


}
\keyword{hplot}% at least one, from doc/KEYWORDS

\name{rm.na}
\alias{rm.na}

\title{Remove missing values}
\description{
  Remove NA's, NAN's and INF's from a vector.
}

\usage{
rm.na(x)
}

\arguments{
  \item{x}{A numeric vector}
}

\value{
  A vector with all NA's remove.
}

\author{Jean Yang}

\examples{
x <- round(rnorm(10), 2)
x[c(2,4,5)] <- NA
x
rm.na(x)
}
\keyword{manip}% at least one, from doc/KEYWORDS

\name{marrayRaw-class}
\docType{class}

\alias{marrayRaw-class}
\alias{marrayRaw}

\alias{maRf}
\alias{maRf<-}
\alias{maGf}
\alias{maGf<-}
\alias{maRb}
\alias{maRb<-}
\alias{maGb}
\alias{maGb<-}
\alias{maW}
\alias{maW<-}
\alias{maLayout}
\alias{maLayout<-}

\alias{[,marrayRaw-method}
\alias{cbind,marrayRaw-method}
%\alias{coerce,marrayRaw,marrayNorm-method}
\alias{maA,marrayRaw-method}
\alias{maControls<-,marrayRaw-method}
\alias{maControls,marrayRaw-method}
\alias{maGb<-,marrayRaw,matrix-method}
\alias{maGb<-,marrayRaw,NULL-method}
\alias{maGb,marrayRaw-method}
\alias{maGf<-,marrayRaw,matrix-method}
\alias{maGf,marrayRaw-method}
%\alias{maGnames<-,marrayRaw,marrayInfo-method}
\alias{maGnames,marrayRaw-method}
\alias{maGridCol,marrayRaw-method}
\alias{maGridRow,marrayRaw-method}
%\alias{maLayout<-,marrayRaw,marrayLayout-method}
\alias{maLayout,marrayRaw-method}
\alias{maLG,marrayRaw-method}
\alias{maLR,marrayRaw-method}
\alias{maM,marrayRaw-method}
\alias{maNgc<-,marrayRaw,numeric-method}
\alias{maNgc,marrayRaw-method}
\alias{maNgr<-,marrayRaw,numeric-method}
\alias{maNgr,marrayRaw-method}
\alias{maNotes<-,marrayRaw,character-method}
\alias{maNotes,marrayRaw-method}
\alias{maNsamples,marrayRaw-method}
\alias{maNsc<-,marrayRaw,numeric-method}
\alias{maNsc,marrayRaw-method}
\alias{maNspots<-,marrayRaw,numeric-method}
\alias{maNspots,marrayRaw-method}
\alias{maNsr<-,marrayRaw,numeric-method}
\alias{maNsr,marrayRaw-method}
\alias{maPlate<-,marrayRaw-method}
\alias{maPlate,marrayRaw-method}
\alias{maPrintTip,marrayRaw-method}
\alias{maRb<-,marrayRaw,matrix-method}
\alias{maRb<-,marrayRaw,NULL-method}
\alias{maRb,marrayRaw-method}
\alias{maRf<-,marrayRaw,matrix-method}
\alias{maRf,marrayRaw-method}
\alias{maSpotCol,marrayRaw-method}
\alias{maSpotRow,marrayRaw-method}
\alias{maSub<-,marrayRaw-method}
\alias{maSub,marrayRaw-method}
%\alias{maTargets<-,marrayRaw,marrayInfo-method}
\alias{maTargets,marrayRaw-method}
\alias{maW<-,marrayRaw,matrix-method}
\alias{maW,marrayRaw-method}
\alias{print,marrayRaw-method}
\alias{show,marrayRaw-method}
\alias{summary,marrayRaw-method}

%\alias{maNgr}
%\alias{maNgc}
%\alias{maNsr}
%\alias{maNsc}
%\alias{maNspots}
%\alias{maSub}
%\alias{maPlate}
%\alias{maControls}

%\alias{maPrintTip}
%\alias{maGridRow}
%\alias{maGridCol
%\alias{maSpotRow}
%\alias{maSpotCol}

\alias{maGnames}
\alias{maGnames<-}
\alias{maTargets}
\alias{maTargets<-}
%\alias{maNotes}

\alias{maLR}
\alias{maLG}

%\alias{maA}
%\alias{maM}

\alias{maNsamples}

\title{Class "marrayRaw", classes and methods for pre-normalization cDNA microarray intensity data}

\description{This class represents pre-normalization intensity data for
  a batch of cDNA microarrays. A \emph{batch of arrays} consists of a
  collection of arrays with the same layout
  (\code{"\link{marrayLayout}"}). The class contains slots for the green
  (Cy3) and red (Cy5) foreground and background intensities, the layout
  of the arrays, and descriptions of the target samples hybridized to
  the arrays and probe sequences spotted onto the arrays.} 

\section{Objects from the Class}{
Objects can be created by calls of the form 
\code{  new('marrayRaw',}\cr
\code{    maRf      = ...., # Object of class matrix}\cr
\code{    maGf      = ...., # Object of class matrix}\cr
\code{    maRb      = ...., # Object of class matrix}\cr
\code{    maGb      = ...., # Object of class matrix}\cr
\code{    maW       = ...., # Object of class matrix}\cr
\code{    maLayout  = ...., # Object of class marrayLayout}\cr
\code{    maGnames  = ...., # Object of class marrayInfo}\cr
\code{    maTargets = ...., # Object of class marrayInfo}\cr
\code{    maNotes   = ...., # Object of class character}\cr
\code{  )}
}

\section{Slots}{
  \describe{
    \item{\code{maRf}:}{Object of class \code{"matrix"}, red foreground intensities, rows correspond to spotted probe sequences, columns to arrays in the batch.}
    \item{\code{maGf}:}{Object of class \code{"matrix"}, green foreground intensities, rows correspond to spotted probe sequences, columns to arrays in the batch. }
    \item{\code{maRb}:}{Object of class \code{"matrix"}, red background intensities, rows correspond to spotted probe sequences, columns to arrays in the batch. }
    \item{\code{maGb}:}{Object of class \code{"matrix"}, green background intensities, rows correspond to spotted probe sequences, columns to arrays in the batch. }
    \item{\code{maW}:}{Object of class \code{"matrix"}, spot quality weights, rows correspond to spotted probe sequences, columns to arrays in the batch.}
    \item{\code{maLayout}:}{Object of class \code{"\link{marrayLayout}"}, layout parameters for the cDNA microarrays.}
    \item{\code{maGnames}:}{Object of class \code{"\link{marrayInfo}"}, description of spotted probe sequences.}
    \item{\code{maTargets}:}{Object of class \code{"\link{marrayInfo}"},  description of target samples hybridized to the arrays.}
    \item{\code{maNotes}:}{Object of class \code{"character"},  any notes concerning the microarray experiments, e.g. hybridization or scanning conditions.}
  }
}

\section{Methods}{
  \describe{
    \item{[}{\code{signature(x = "marrayRaw")}: subsetting operator for spots on the array and arrays in the batch, ensures that all slots are subset properly.}
    \item{coerce}{\code{signature(from = "marrayRaw", to = "marrayNorm")}: coerce an object of class \code{"\link{marrayRaw}"} into an object of class \code{"\link{marrayNorm}"}. }
    \item{maA}{\code{signature(object = "marrayRaw")}: function which computes average log-intensities (base 2) A for an object of class \code{"\link{marrayRaw}"}.}
    \item{maControls<-}{\code{signature(object = "marrayRaw")}: slot assignment method.} 
    \item{maControls}{\code{signature(object = "marrayRaw")}: slot accessor method.}
    \item{maGb}{\code{signature(object = "marrayRaw")}: slot accessor method.}
    \item{maGb<-}{\code{signature(object = "marrayRaw", value = "matrix")}: slot assignment method.} 
    \item{maGb<-}{\code{signature(object = "marrayRaw", value = "NULL")}: slot assignment method.} 
    \item{maGf}{\code{signature(object = "marrayRaw")}: slot accessor method.}
    \item{maGf<-}{\code{signature(object = "marrayRaw", value = "matrix")}: slot assignment method.}
    \item{maGnames}{\code{signature(object = "marrayRaw")}: slot accessor method.}
    \item{maGnames<-}{\code{signature(object = "marrayRaw", value = "marrayInfo")}: slot assignment method.}
    \item{maGridCol}{\code{signature(object = "marrayRaw")}: method which computes a vector of grid column coordinates for each spot.}
    \item{maGridRow}{\code{signature(object = "marrayRaw")}: method which computes a vector of grid row coordinates for each spot.}
    \item{maLayout}{\code{signature(object = "marrayRaw")}: slot accessor method.}
    \item{maLayout<-}{\code{signature(object = "marrayRaw", value = "marrayLayout")}: slot assignment method. }
    \item{maLG}{\code{signature(object = "marrayRaw")}: method which computes green log-intensities (base 2) for an object of class \code{"\link{marrayRaw}"}.}
    \item{maLR}{\code{signature(object = "marrayRaw")}: method  which computes red log-intensities (base 2) for an object of class \code{"\link{marrayRaw}"}.}
    \item{maM}{\code{signature(object = "marrayRaw")}: method which computes intensity log-ratios (base 2) M for an object of class \code{"\link{marrayRaw}"}.}
    \item{maNgc}{\code{signature(object = "marrayRaw")}: slot accessor method.}
 \item{maNgc<-}{\code{signature(object = "marrayRaw", value = "numeric")}:  slot assignment method.}
    \item{maNgr}{\code{signature(object = "marrayRaw")}: slot accessor method.}
 \item{maNgr<-}{\code{signature(object = "marrayRaw", value = "numeric")}:  slot assignment method.}
    \item{maNotes}{\code{signature(object = "marrayRaw")}: slot accessor method.}
    \item{maNotes<-}{\code{signature(object = "marrayRaw", value = "character")}: slot assignment method.}
    \item{maNsamples}{\code{signature(object = "marrayRaw")}: slot accessor method.}
    \item{maNsc}{\code{signature(object = "marrayRaw")}: slot accessor method.}
\item{maNsc<-}{\code{signature(object = "marrayRaw", value = "numeric")}:  slot assignment method.}
    \item{maNspots}{\code{signature(object = "marrayRaw")}: slot accessor method.}
\item{maNspots<-}{\code{signature(object = "marrayRaw", value = "numeric")}:  slot assignment method.}
    \item{maNsr}{\code{signature(object = "marrayRaw")}: slot accessor method.}
\item{maNsr<-}{\code{signature(object = "marrayRaw", value = "numeric")}:  slot assignment method.}
    \item{maPlate}{\code{signature(object = "marrayRaw")}: slot accessor method.}
    \item{maPlate<-}{\code{signature(object = "marrayRaw")}: slot assignment method.}
    \item{maPrintTip}{\code{signature(object = "marrayRaw")}: method which computes a vector of print-tip-group indices for each spot.}
    \item{maRb}{\code{signature(object = "marrayRaw")}: slot accessor method.}
    \item{maRb<-}{\code{signature(object = "marrayRaw", value = "matrix")}: slot assignment method.}
    \item{maRb<-}{\code{signature(object = "marrayRaw", value = "NULL")}: slot assignment method. }
    \item{maRf}{\code{signature(object = "marrayRaw")}: slot accessor method. }
    \item{maRf<-}{\code{signature(object = "marrayRaw", value = "matrix")}: slot assignment method. }
    \item{maSpotCol}{\code{signature(object = "marrayRaw")}: method which computes a vector of spot column coordinates for each spot.}
    \item{maSpotRow}{\code{signature(object = "marrayRaw")}: method which computes a vector of spot row coordinates for each spot.}
    \item{maSub}{\code{signature(object = "marrayRaw")}: slot accessor method.}
    \item{maSub<-}{\code{signature(object = "marrayRaw")}: slot assignment method.}
    \item{maTargets}{\code{signature(object = "marrayRaw")}: slot accessor method.}
    \item{maTargets<-}{\code{signature(object = "marrayRaw", value = "marrayInfo")}: slot assignment method. }
    \item{maW}{\code{signature(object = "marrayRaw")}: slot accessor method.}
    \item{maW<-}{\code{signature(object = "marrayRaw", value = "matrix")}: slot assignment method.}
    \item{print}{\code{signature(x = "marrayRaw")}:  print method for \code{"\link{marrayRaw}"} class.}
  }
}

\references{S. Dudoit and Y. H. Yang. (2002). Bioconductor R packages for exploratory analysis and normalization of cDNA microarray data. In G. Parmigiani, E. S. Garrett, R. A. Irizarry and S. L. Zeger, editors, \emph{The Analysis of Gene Expression Data: Methods and Software}, Springer, New York.
}

\author{Sandrine Dudoit, \url{http://www.stat.berkeley.edu/~sandrine}.}

\seealso{\code{\link{marrayLayout}}, \code{\link{marrayNorm}}, \code{\link{marrayInfo}}.}

\examples{
# Examples use swirl dataset, for description type ? swirl
require(limma)
data(swirl)

# Object of class marrayRaw for the 4 swirl arrays
swirl

# Object of class marrayLayout
maLayout(swirl)

# Access only the first 100 spots of the third array
swirl[1:100,3]

# Accessor methods -- How many spots on the array
maNspots(swirl)

# Density plot of log-ratios M for third array
plot(density(maM(swirl[,3])))

# Assignment methods -- Replace maNotes slot
maNotes(swirl)
maNotes(swirl)<-"This is a zebrafish microarray"
maNotes(swirl)
}
\keyword{classes}

\name{maCompInd}
\alias{maCompInd}
%- Also NEED an `\alias' for EACH other topic documented here.
\title{Generate spot indices}
\description{
This function generates spot indices from ranges of rows and columns for
the grid and spot matrices. Spots on the array are numbered consecutively starting
from the top left grid and the top left spot within each grid.

}
\usage{
maCompInd(grows, gcols, srows, scols, L)
}
%- maybe also `usage' for other objects documented here.
\arguments{
  \item{grows}{numeric vector of grid rows.}
  \item{gcols}{numeric vector of grid columns.}
  \item{srows}{numeric vector of spot rows.}
  \item{scols}{numeric vector of spot columns.}
  \item{L}{object of class \code{"\link{marrayLayout}"}.}
}

\value{
a numeric vector of spot indices.
}

\author{Yee Hwa (Jean) Yang, Sandrine Dudoit, \url{http://www.stat.berkeley.edu/~sandrine}.}

\seealso{\code{\link{marrayLayout}}, \code{\link{maCoord2Ind}},
\code{\link{maInd2Coord}}, \code{\link{maCompCoord}}.}


\examples{
L <- new("marrayLayout", maNgr=4, maNgc=4, maNsr=22, maNsc=24)
maCompInd(1:2,1,1:4,1:3,L)


}
\keyword{manip}
\keyword{methods}

\name{write.xls}
\alias{write.xls}

\title{Data Output}
\description{
  Calls the function write.table with predefine argument.   The entries
  in each line (row) are separated by tab. 
}

\usage{
write.xls(res, file = "test.xls", ...)
}

\arguments{
  \item{res}{the object to be written, typically a data frame.  If not, it
          is attempted to coerce \code{x} to a data frame.}
  \item{file}{a character string representing the file name.}
  \item{\dots}{further arguments to \code{\link{write.table}}.}
}
\details{
  see \code{\link{write.table}}
}

\author{Jean Yee Hwa Yang}

\seealso{\code{\link{write.table}}, \code{\link{write.list}}}

\examples{
data(swirl)
write.xls(maM(swirl)[1:10,], "normM.xls")
}

\keyword{file}% at least one, from doc/KEYWORDS

\name{[-methods}
\docType{methods}

\alias{[-methods}
\alias{[}

\title{Subsetting methods for microarray objects}


\description{Subsetting methods were defined for the microarray classes,
  \code{\link{marrayInfo}},
  \code{\link{marrayLayout}},\code{\link{marrayRaw}} and
  \code{\link{marrayNorm}}. These methods create instances of the given
class, for a subset of spots and/or arrays in a batch.}

\section{Methods}{\describe{
\item{x = ANY}{generic method.}
\item{x = marrayInfo}{\code{x[i, j]} extract object of class \code{"\link{marrayInfo}"} for spots or arrays with indices i and labels with indices j.}
\item{x = marrayLayout}{\code{x[i]} extract object of class \code{"\link{marrayLayout}"} for spots with indices i.}
\item{x = marrayRaw}{\code{x[i, j]} extract object of class \code{"\link{marrayRaw}"} for spots with indices i and arrays with indices j.}
\item{x = marrayNorm}{\code{x[i, j]} extract object of class \code{"\link{marrayNorm}"} for spots with indices i and arrays with indices j.}
}}

\keyword{methods}

\name{maCoord2Ind}
\alias{maCoord2Ind}
%- Also NEED an `\alias' for EACH other topic documented here.
\title{Convert grid and spot matrix coordinates to spot indices}
\description{
This functions converts grid and spot matrix coordinates (four
coordinates) to spot
indices, where spots on the array are numbered consecutively starting
from the top left grid and the top left spot within each grid.
}
\usage{
maCoord2Ind(x, L)
}
%- maybe also `usage' for other objects documented here.
\arguments{
  \item{x}{a matrix of spot four-coordinates, with rows corresponding to
  spots and columns to grid row, grid column, spot row, and spot column coordinates.}
  \item{L}{an object of class \code{"\link{marrayLayout}"}.}
}

\value{
 a numeric vector of spot indices.
}

\author{Yee Hwa (Jean) Yang, Sandrine Dudoit, \url{http://www.stat.berkeley.edu/~sandrine}.}

\seealso{\code{\link{marrayLayout}}, \code{\link{maInd2Coord}},
\code{\link{maCompCoord}}, \code{\link{maCompInd}}.}

\examples{
L <- new("marrayLayout", maNgr=4, maNgc=4, maNsr=22, maNsc=24)
coord<-cbind(rep(2,4),rep(1,4),rep(1,4),1:4)
maCoord2Ind(coord, L)


}
\keyword{manip}
\keyword{methods}
